@@ -13,9 +13,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include <fstream>
@@ -24,53 +22,26 @@
 #include "Celeste.h"
 #include <sys/stat.h> 
 #include "hugin_config.h" 
-#include <hugin_version.h>
 #include <vigra_ext/impexalpha.hxx>
 #include <panodata/Panorama.h>
+#include <hugin_utils/utils.h>
 
 #ifdef _WINDOWS
-#include <windows.h>
 #include <getopt.h>
 #else
  #include <unistd.h>
 #endif
 
-#ifdef __APPLE__
-#include <hugin_config.h>
-#include <mach-o/dyld.h>	/* _NSGetExecutablePath */
-#include <limits.h>		/* PATH_MAX */
-#include <libgen.h>		/* dirname */
-#endif
-
 using namespace std;
 using namespace HuginBase;
 using namespace AppBase;
 
-bool fileexists(string strFilename) {
-
-	struct stat stFileInfo;
-	bool blnReturn;
-	int intStat;
-
-	// Attempt to get the file attributes
-	intStat = stat(strFilename.c_str(),&stFileInfo);
-	if(intStat == 0) {
-
-		blnReturn = true;
-  	}else{
-
-    		blnReturn = false;
-  	}
-  
-  	return(blnReturn);
-}
-
 static void usage(){
 
 	// Print usage and exit
 	cout << endl << "Celeste: Removes cloud-like control points from Hugin project files and creates image masks" << endl;
 	cout << "using Support Vector Machines." << endl;
-	cout << endl << "Version " << DISPLAY_VERSION << endl;
+    cout << endl << "Version " << hugin_utils::GetHuginVersion() << endl;
 	cout << endl << "Usage: celeste_standalone [options] image1 image2 [..]" << endl << endl;
 	cout << "Options:" << endl << endl;
 	cout << "  -i <filename>   Input Hugin PTO file. Control points over SVM threshold will" << endl;
@@ -290,44 +261,12 @@ int main(int argc, char* argv[])
 
     
 	// Check model file
-	if (!fileexists(model_file)){
-	
-#if _WINDOWS
-        char buffer[MAX_PATH];//always use MAX_PATH for filepaths
-        GetModuleFileName(NULL,buffer,sizeof(buffer));
-        string working_path=(buffer);
-        string install_path_model="";
-        //remove filename
-        std::string::size_type pos=working_path.rfind("\\");
-        if(pos!=std::string::npos)
-        {
-            working_path.erase(pos);
-            //remove last dir: should be bin
-            pos=working_path.rfind("\\");
-            if(pos!=std::string::npos)
-            {
-                working_path.erase(pos);
-                //append path delimiter and path
-                working_path.append("\\share\\hugin\\data\\");
-                install_path_model=working_path;
-            }
-        }
-#elif defined MAC_SELF_CONTAINED_BUNDLE
-		char path[PATH_MAX + 1];
-		uint32_t size = sizeof(path);
-		string install_path_model("");
-		if (_NSGetExecutablePath(path, &size) == 0)
-		{
-			install_path_model=dirname(path);
-			install_path_model.append("/../Resources/xrc/");
-		}
-#else
-        string install_path_model = (INSTALL_DATA_DIR);
-#endif
-
+    if (!hugin_utils::FileExists(model_file))
+    {
+        string install_path_model=hugin_utils::GetDataDir();
 		install_path_model.append(model_file);
 		
-		if (!fileexists(install_path_model)){
+		if (!hugin_utils::FileExists(install_path_model)){
 		
     			cout << endl << "Couldn't open SVM model file " << model_file << endl;
 			cout << "Also tried " << install_path_model << endl << endl; 
@@ -346,7 +285,7 @@ int main(int argc, char* argv[])
 	}
 
 	// Convert mask format to upper case
-	transform(mask_format.begin(), mask_format.end(), mask_format.begin(),(int(*)(int)) toupper);
+    mask_format=hugin_utils::toupper(mask_format);
 	if (mask_format == ("JPG")){
 		mask_format = ("JPEG");
 	}
@@ -357,7 +296,7 @@ int main(int argc, char* argv[])
 	// Print some stuff out
 	cout << endl << "Celeste: Removes cloud-like control points from Hugin project files and creates image masks" << endl;
 	cout << "using Support Vector Machines." << endl;
-	cout << endl << "Version " << DISPLAY_VERSION << endl << endl;
+    cout << endl << "Version " << hugin_utils::GetHuginVersion() << endl << endl;
 	cout << "Arguments:" << endl;
 	cout << "Input Hugin file:\t" << pto_file << endl;
 	cout << "Output Hugin file:\t" << output_pto << endl;
@@ -387,7 +326,7 @@ int main(int argc, char* argv[])
 	} 
 	
 	// Convert mask format to lower case
-	transform(mask_format.begin(), mask_format.end(), mask_format.begin(),(int(*)(int)) tolower);
+    mask_format=hugin_utils::tolower(mask_format);
 
 	// Vectors to store SVM responses and PTO file info etc
 	vector<string> images,pto_file_top,pto_file_cps,pto_file_end;
