@@ -18,16 +18,19 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
 #include "panoinc_WX.h"
+#include <wx/msgdlg.h>
 #include "panoinc.h"
 #include "LensTools.h"
 #include <algorithm>
 #include "panodata/ImageVariableTranslate.h"
+#include "panodata/StandardImageVariableGroups.h"
+#include "base_wx/PanoCommand.h"
 
 using namespace std;
 
@@ -46,11 +49,19 @@ void FillLensProjectionList(wxControlWithItems* list)
     list->SetSelection(0);
 };
 
-void SelectProjection(wxControlWithItems* list,size_t new_projection)
+void FillBlenderList(wxControlWithItems* list)
+{
+    list->Clear();
+    list->Append(_("enblend"), (void*)HuginBase::PanoramaOptions::ENBLEND_BLEND);
+    list->Append(_("builtin"), (void*)HuginBase::PanoramaOptions::INTERNAL_BLEND);
+    list->SetSelection(0);
+};
+
+void SelectListValue(wxControlWithItems* list, size_t newValue)
 {
     for(unsigned int i=0;i<list->GetCount();i++)
     {
-        if((size_t)list->GetClientData(i)==new_projection)
+        if((size_t)list->GetClientData(i)==newValue)
         {
             list->SetSelection(i);
             return;
@@ -59,11 +70,44 @@ void SelectProjection(wxControlWithItems* list,size_t new_projection)
     list->SetSelection(0);
 };
 
-size_t GetSelectedProjection(wxControlWithItems* list)
+size_t GetSelectedValue(wxControlWithItems* list)
 {
     return (size_t)(list->GetClientData(list->GetSelection()));
 };
 
+wxString getProjectionString(const HuginBase::SrcPanoImage& img)
+{
+    wxString ps;
+    switch (img.getProjection())
+    {
+    case HuginBase::SrcPanoImage::RECTILINEAR:          ps << _("Normal (rectilinear)"); break;
+    case HuginBase::SrcPanoImage::PANORAMIC:            ps << _("Panoramic (cylindrical)"); break;
+    case HuginBase::SrcPanoImage::CIRCULAR_FISHEYE:     ps << _("Circular fisheye"); break;
+    case HuginBase::SrcPanoImage::FULL_FRAME_FISHEYE:   ps << _("Full frame fisheye"); break;
+    case HuginBase::SrcPanoImage::EQUIRECTANGULAR:      ps << _("Equirectangular"); break;
+    case HuginBase::SrcPanoImage::FISHEYE_ORTHOGRAPHIC: ps << _("Orthographic"); break;
+    case HuginBase::SrcPanoImage::FISHEYE_STEREOGRAPHIC:ps << _("Stereographic"); break;
+    case HuginBase::SrcPanoImage::FISHEYE_EQUISOLID:    ps << _("Equisolid"); break;
+    case HuginBase::SrcPanoImage::FISHEYE_THOBY:        ps << _("Fisheye Thoby"); break;
+    }
+    return ps;
+};
+
+wxString getResponseString(const HuginBase::SrcPanoImage& img)
+{
+    wxString s;
+    switch (img.getResponseType())
+    {
+    case HuginBase::BaseSrcPanoImage::RESPONSE_EMOR:
+        s = _("custom (EMoR)");
+        break;
+    case HuginBase::BaseSrcPanoImage::RESPONSE_LINEAR:
+        s = _("Linear");
+        break;
+    };
+    return s;
+};
+
 void SaveLensParameters(const wxString filename, HuginBase::Panorama* pano, unsigned int imgNr)
 {
     HuginBase::StandardImageVariableGroups variable_groups(*pano);
@@ -126,7 +170,7 @@ void SaveLensParameters(const wxString filename, HuginBase::Panorama* pano, unsi
     free(old_locale);
 };
 
-bool ApplyLensParameters(wxWindow * parent, PT::Panorama *pano, HuginBase::UIntSet images, PT::PanoCommand*& cmd)
+bool ApplyLensParameters(wxWindow * parent, HuginBase::Panorama *pano, HuginBase::UIntSet images, PanoCommand::PanoCommand*& cmd)
 {
     HuginBase::StandardImageVariableGroups variable_groups(*pano);
     HuginBase::Lens lens=variable_groups.getLensForImage(*images.begin());
@@ -154,7 +198,7 @@ bool ApplyLensParameters(wxWindow * parent, PT::Panorama *pano, HuginBase::UIntS
         /** @todo I think the sensor size should be copied over,
             * but SrcPanoImage doesn't have such a variable yet.
             */
-        std::vector<PT::PanoCommand*> cmds;
+        std::vector<PanoCommand::PanoCommand*> cmds;
         // update links
         std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> linkedVariables;
         std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> unlinkedVariables;
@@ -177,32 +221,32 @@ bool ApplyLensParameters(wxWindow * parent, PT::Panorama *pano, HuginBase::UIntS
         }
         if (!linkedVariables.empty())
         {
-            cmds.push_back(new PT::ChangePartImagesLinkingCmd(*pano, images, linkedVariables,
+            cmds.push_back(new PanoCommand::ChangePartImagesLinkingCmd(*pano, images, linkedVariables,
                 true,HuginBase::StandardImageVariableGroups::getLensVariables()));
         }
         if (!unlinkedVariables.empty())
         {
-            cmds.push_back(new PT::ChangePartImagesLinkingCmd(*pano, images, unlinkedVariables,
+            cmds.push_back(new PanoCommand::ChangePartImagesLinkingCmd(*pano, images, unlinkedVariables,
                 false,HuginBase::StandardImageVariableGroups::getLensVariables()));
         }
         //update lens parameters
-        cmds.push_back(new PT::UpdateImagesVariablesCmd(*pano, images, vars));
+        cmds.push_back(new PanoCommand::UpdateImagesVariablesCmd(*pano, images, vars));
 
         // Set the lens projection type.
-        cmds.push_back(new PT::ChangeImageProjectionCmd(*pano, images, (HuginBase::SrcPanoImage::Projection) lens.getProjection()));
+        cmds.push_back(new PanoCommand::ChangeImageProjectionCmd(*pano, images, (HuginBase::SrcPanoImage::Projection) lens.getProjection()));
         // update crop factor
-        cmds.push_back(new PT::ChangeImageCropFactorCmd(*pano,images,lens.getCropFactor()));
+        cmds.push_back(new PanoCommand::ChangeImageCropFactorCmd(*pano,images,lens.getCropFactor()));
         // update the crop rect
-        cmds.push_back(new PT::ChangeImageAutoCenterCropCmd(*pano,images,autoCenterCrop));
+        cmds.push_back(new PanoCommand::ChangeImageAutoCenterCropCmd(*pano,images,autoCenterCrop));
         if(cropped)
         {
-            cmds.push_back(new PT::ChangeImageCropRectCmd(*pano,images,cropRect));
+            cmds.push_back(new PanoCommand::ChangeImageCropRectCmd(*pano,images,cropRect));
         }
         else
         {
-            cmds.push_back(new PT::ChangeImageCropModeCmd(*pano,images,HuginBase::BaseSrcPanoImage::NO_CROP));
+            cmds.push_back(new PanoCommand::ChangeImageCropModeCmd(*pano,images,HuginBase::BaseSrcPanoImage::NO_CROP));
         };
-        cmd=new PT::CombinedPanoCommand(*pano, cmds);
+        cmd=new PanoCommand::CombinedPanoCommand(*pano, cmds);
         return true;
     }
     else
@@ -336,7 +380,7 @@ bool LoadLensParametersChoose(wxWindow * parent, HuginBase::Lens & lens,
     };
 };
 
-void SaveLensParametersToIni(wxWindow * parent, PT::Panorama *pano, const HuginBase::UIntSet images)
+void SaveLensParametersToIni(wxWindow * parent, HuginBase::Panorama *pano, const HuginBase::UIntSet images)
 {
     if (images.size() == 1)
     {
@@ -345,7 +389,7 @@ void SaveLensParametersToIni(wxWindow * parent, PT::Panorama *pano, const HuginB
                          _("Save lens parameters file"),
                          wxConfigBase::Get()->Read(wxT("/lensPath"),wxT("")), wxT(""),
                          _("Lens Project Files (*.ini)|*.ini|All files (*)|*"),
-                         wxFD_SAVE, wxDefaultPosition);
+                         wxFD_SAVE | wxFD_OVERWRITE_PROMPT, wxDefaultPosition);
         dlg.SetDirectory(wxConfigBase::Get()->Read(wxT("/lensPath"),wxT("")));
         if (dlg.ShowModal() == wxID_OK)
         {
@@ -353,6 +397,18 @@ void SaveLensParametersToIni(wxWindow * parent, PT::Panorama *pano, const HuginB
             if(!filename.HasExt())
             {
                 filename.SetExt(wxT("ini"));
+#if wxCHECK_VERSION(3,0,0)
+                if (filename.Exists())
+#else
+                if (wxFile::Exists(filename.GetFullPath()))
+#endif
+                {
+                    int d = wxMessageBox(wxString::Format(_("File %s exists. Overwrite?"), filename.GetFullPath().c_str()),
+                        _("Save project"), wxYES_NO | wxICON_QUESTION);
+                    if (d != wxYES) {
+                        return;
+                    }
+                }
             }
             wxConfig::Get()->Write(wxT("/lensPath"), dlg.GetDirectory());  // remember for later
             SaveLensParameters(filename.GetFullPath(),pano,imgNr);
@@ -364,4 +420,58 @@ void SaveLensParametersToIni(wxWindow * parent, PT::Panorama *pano, const HuginB
     }
 }
 
+bool CheckLensStacks(HuginBase::Panorama* pano, bool allowCancel)
+{
+    if (pano->getNrOfImages() < 2)
+    {
+        return true;
+    };
+    const size_t nrImages = pano->getNrOfImages();
+    bool stacksCorrectLinked = true;
+    for (size_t i = 0; i < nrImages - 1; ++i)
+    {
+        const HuginBase::SrcPanoImage& image1 = pano->getImage(i);
+        if (image1.YawisLinked())
+        {
+            for (size_t j = i + 1; j < nrImages && stacksCorrectLinked; ++j)
+            {
+                const HuginBase::SrcPanoImage& image2 = pano->getImage(j);
+                if (image1.YawisLinkedWith(image2))
+                {
+                    stacksCorrectLinked = stacksCorrectLinked &
+                        image1.HFOVisLinkedWith(image2) &
+                        image1.RadialDistortionisLinkedWith(image2) &
+                        image1.RadialDistortionCenterShiftisLinkedWith(image2) &
+                        image1.ShearisLinkedWith(image2);
+                };
+            };
+        };
+    };
+    if (stacksCorrectLinked)
+    {
+        return true;
+    }
+    else
+    {
+        int flags = wxICON_EXCLAMATION | wxOK;
+        if (allowCancel)
+        {
+            flags = flags | wxCANCEL;
+        };
+        if (wxMessageBox(_("This project contains stacks with linked positions. But the lens parameters are not linked for these images.\nThis will result in unwanted results.\nPlease check and correct this before proceeding."),
+#ifdef _WINDOWS
+            _("Hugin"),
+#else
+            wxT(""),
+#endif
+            flags)==wxOK)
+        {
+            return true;
+        }
+        else
+        {
+            return false;
+        };
+    };
+};
 
