@@ -19,8 +19,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -30,6 +30,9 @@
 #include <hugin_shared.h>
 #include "panoinc_WX.h"
 #include <config.h>
+#if wxUSE_ON_FATAL_EXCEPTION
+#include <wx/debugrpt.h>
+#endif
 
 #ifdef __WXMSW__
 #define HUGIN_CONV_FILENAME (*wxConvCurrent)
@@ -74,4 +77,9 @@ WXIMPEX bool containsInvalidCharacters(const wxString stringToTest);
 /** shows a dialog about filename with invalid characters, all names in filelist will be show in list */
 WXIMPEX void ShowFilenameWarning(wxWindow* parent, const wxArrayString filelist);
 
+#if wxUSE_ON_FATAL_EXCEPTION
+/** shows debug report dialog box from wxWidgets */
+WXIMPEX void GenerateReport(wxDebugReport::Context ctx);
+#endif
+
 #endif
