@@ -20,8 +20,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -39,6 +39,7 @@
 #include <algorithms/optimizer/PTOptimizer.h>
 #include "lensdb/LensDB.h"
 #include "base_wx/wxLensDB.h"
+#include "panodata/StandardImageVariableGroups.h"
 
 using namespace HuginBase;
 
@@ -49,16 +50,6 @@ const unsigned int cps_per_line=10;
 #define DEFAULT_RESIZE_DIMENSION 1600
 #define DEFAULT_MINLINELENGTH 0.3
 
-// utility functions
-bool str2double(wxString s, double & d)
-{
-    if (!hugin_utils::stringToDouble(std::string(s.mb_str(wxConvLocal)), d)) 
-    {
-        return false;
-    }
-    return true;
-}
-
 /** file drag and drop handler method */
 bool FileDropTarget::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames)
 {
@@ -253,81 +244,20 @@ const wxString & LensCalFrame::GetXRCPath()
      return wxGetApp().GetXRCPath();
 };
 
-void LensCalFrame::resetProgress(double max)
-{
-    m_progressMax = max;
-    m_progress = 0;
-    m_progressMsg = wxT("");
-}
-
-bool LensCalFrame::increaseProgress(double delta)
-{
-    m_progress += delta;
-
-    // build the message:
-    int percentage = (int) floor(m_progress/m_progressMax*100);
-    if (percentage > 100) percentage = 100;
-
-    return displayProgress();
-}
-
-bool LensCalFrame::increaseProgress(double delta, const std::string & msg)
-{
-    m_progress += delta;
-    m_progressMsg = wxString(msg.c_str(), wxConvLocal);
-
-    return displayProgress();
-}
-
-
-void LensCalFrame::setMessage(const std::string & msg)
-{
-    m_progressMsg = wxString(msg.c_str(), wxConvLocal);
-}
-
-bool LensCalFrame::displayProgress()
-{
-    // build the message:
-    int percentage = (int) floor(m_progress/m_progressMax*100);
-    if (percentage > 100) percentage = 100;
-
-    wxStatusBar *statbar = GetStatusBar();
-    statbar->SetStatusText(wxString::Format(wxT("%s: %d%%"),m_progressMsg.c_str(), percentage),0);
-#ifdef __WXMSW__
-    UpdateWindow(NULL);
-#endif
-    return true;
-}
-
 /** update the display */
 void LensCalFrame::updateProgressDisplay()
 {
     wxString msg;
-    // build the message:
-    for (std::vector<AppBase::ProgressTask>::reverse_iterator it = tasks.rbegin();
-                 it != tasks.rend(); ++it)
+    if (!m_message.empty())
     {
-        wxString cMsg;
-        if (it->getProgress() > 0) {
-            cMsg.Printf(wxT("%s %s [%3.0f%%]"),
-                        wxString(it->getShortMessage().c_str(), wxConvLocal).c_str(),
-                        wxString(it->getMessage().c_str(), wxConvLocal).c_str(),
-                        100 * it->getProgress());
-        } else {
-            cMsg.Printf(wxT("%s %s"),wxString(it->getShortMessage().c_str(), wxConvLocal).c_str(),
-                        wxString(it->getMessage().c_str(), wxConvLocal).c_str());
-        }
-        // append to main message
-        if (it == tasks.rbegin()) {
-            msg = cMsg;
-        } else {
-            msg.Append(wxT(" | "));
-            msg.Append(cMsg);
-        }
-    }
-    wxStatusBar *m_statbar = GetStatusBar();
-    DEBUG_TRACE("Statusmb : " << msg.mb_str(wxConvLocal));
-    m_statbar->SetStatusText(msg,0);
+        msg = wxGetTranslation(wxString(m_message.c_str(), wxConvLocal));
+        if (!m_filename.empty())
+        {
+            msg.Append(wxT(" "));
+            msg.Append(wxString(ProgressDisplay::m_filename.c_str(), HUGIN_CONV_FILENAME));
+        };
+    };
+    GetStatusBar()->SetStatusText(msg, 0);
 
 #ifdef __WXMSW__
     UpdateWindow(NULL);
@@ -347,6 +277,21 @@ void LensCalFrame::AddImages(wxArrayString files)
     {
         ImageLineList* image=new ImageLineList(files[i]);
         //check input
+        {
+            // check for black/white images
+            const HuginBase::FileMetaData metadata = image->GetPanoImage()->getFileMetadata();
+            HuginBase::FileMetaData::const_iterator it = metadata.find("pixeltype");
+            if (it != metadata.end())
+            {
+                if (it->second == "BILEVEL")
+                {
+                    wxMessageBox(wxString::Format(_("File \"%s\" is a black/white image.\nHugin does not support this image type. Skipping this image.\nConvert image to grayscale image and try loading again."), files[i].c_str()),
+                        _("Warning"), wxOK | wxICON_EXCLAMATION, this);
+                    delete image;
+                    continue;
+                };
+            };
+        };
         if(m_images.size()>0)
         {
             const HuginBase::SrcPanoImage* image0=m_images[0]->GetPanoImage();
@@ -357,12 +302,16 @@ void LensCalFrame::AddImages(wxArrayString files)
                 delete image;
                 continue;
             };
-            if(image0->hasEXIFread() && image1->hasEXIFread())
+            if(!image0->getExifMake().empty() && !image1->getExifMake().empty() &&
+               !image0->getExifModel().empty() && !image1->getExifModel().empty() &&
+               image0->getExifFocalLength()>0 && image1->getExifFocalLength()>0 &&
+               image0->getCropFactor()>0 && image1->getCropFactor()>0                 
+              )
             {
                 if(image0->getExifMake()!=image1->getExifMake() ||
                     image0->getExifModel()!=image1->getExifModel() ||
                     image0->getExifFocalLength()!=image1->getExifFocalLength() ||
-                    image0->getExifCropFactor()!=image1->getExifCropFactor())
+                    image0->getCropFactor()!=image1->getCropFactor())
                 {
                     //only show a warning, but continue processing
                     wrongExif.push_back(files[i]);
@@ -371,16 +320,19 @@ void LensCalFrame::AddImages(wxArrayString files)
         };
         m_images.push_back(image);
         SetStatusText(wxString::Format(_("Added %s"),image->GetFilename().c_str()));
-        if(image->GetPanoImage()->hasEXIFread())
+        if (image->GetPanoImage()->getExifFocalLength() > 0)
         {
-            XRCCTRL(*this,"lenscal_focallength",wxTextCtrl)->SetValue( 
-                hugin_utils::doubleTowxString(image->GetPanoImage()->getExifFocalLength(),2)
+            XRCCTRL(*this, "lenscal_focallength", wxTextCtrl)->SetValue(
+                hugin_utils::doubleTowxString(image->GetPanoImage()->getExifFocalLength(), 2)
                 );
-            XRCCTRL(*this,"lenscal_cropfactor",wxTextCtrl)->SetValue(
-                hugin_utils::doubleTowxString(image->GetPanoImage()->getExifCropFactor(),2)
-                );
-            SelectProjection(m_choice_projection,image->GetPanoImage()->getProjection());
         };
+        if (image->GetPanoImage()->getExifCropFactor() > 0)
+        {
+            XRCCTRL(*this, "lenscal_cropfactor", wxTextCtrl)->SetValue(
+                hugin_utils::doubleTowxString(image->GetPanoImage()->getCropFactor(), 2)
+                );
+        };
+        SelectListValue(m_choice_projection,image->GetPanoImage()->getProjection());
     }
     UpdateList(false);
     m_images_list->SetSelection(m_images_list->GetCount()-1);
@@ -422,7 +374,7 @@ void LensCalFrame::OnAddImage(wxCommandEvent &e)
     wxFileDialog dlg(this,_("Add images"),
                      path, wxT(""),
                      HUGIN_WX_FILE_IMG_FILTER,
-                     wxFD_OPEN | wxFD_MULTIPLE, wxDefaultPosition);
+                     wxFD_OPEN | wxFD_MULTIPLE | wxFD_FILE_MUST_EXIST | wxFD_PREVIEW, wxDefaultPosition);
     dlg.SetDirectory(path);
 
     // remember the image extension
@@ -552,7 +504,7 @@ bool LensCalFrame::ReadInputs(bool readFocalLength,bool readOptions,bool readLen
 {
     if(readFocalLength)
     {
-        m_projection=(SrcPanoImage::Projection)GetSelectedProjection(m_choice_projection);
+        m_projection = (SrcPanoImage::Projection)GetSelectedValue(m_choice_projection);
         if(!str2double(XRCCTRL(*this,"lenscal_focallength",wxTextCtrl)->GetValue(),m_focallength))
             return false;
         if(m_focallength<1)
@@ -646,13 +598,13 @@ Panorama LensCalFrame::GetPanorama()
         SrcPanoImage image(*(m_images[i]->GetPanoImage()));
         image.setProjection(m_projection);
         image.setExifFocalLength(m_focallength);
-        image.setExifCropFactor(m_cropfactor);
+        image.setCropFactor(m_cropfactor);
         image.setVar("a",m_a);
         image.setVar("b",m_b);
         image.setVar("c",m_c);
         image.setVar("d",m_d);
         image.setVar("e",m_e);
-        double hfov=SrcPanoImage::calcHFOV(image.getProjection(),image.getExifFocalLength(),image.getExifCropFactor(),image.getSize());
+        double hfov=SrcPanoImage::calcHFOV(image.getProjection(),image.getExifFocalLength(),image.getCropFactor(),image.getSize());
         image.setHFOV(hfov);
         pano.addImage(image);
         std::set<std::string> imgopt;
@@ -739,7 +691,7 @@ void LensCalFrame::SaveLensToIni()
                         _("Save lens parameters file"),
                         wxConfigBase::Get()->Read(wxT("/lensPath"),wxT("")), wxT(""),
                         _("Lens Project Files (*.ini)|*.ini|All files (*)|*"),
-                        wxFD_SAVE, wxDefaultPosition);
+                        wxFD_SAVE | wxFD_OVERWRITE_PROMPT, wxDefaultPosition);
     dlg.SetDirectory(wxConfigBase::Get()->Read(wxT("/lensPath"),wxT("")));
     if (dlg.ShowModal() == wxID_OK)
     {
@@ -779,7 +731,7 @@ void LensCalFrame::OnSaveLens(wxCommandEvent &e)
 
     wxArrayString choices;
     choices.push_back(_("Save lens parameters to ini file"));
-    choices.push_back(_("Save lens parameters to lensfun database"));
+    choices.push_back(_("Save lens parameters to lens database"));
     wxSingleChoiceDialog save_dlg(this,_("Saving lens data"),_("Save lens"),choices);
     if(save_dlg.ShowModal()==wxID_OK)
     {
@@ -804,7 +756,7 @@ void LensCalFrame::OnSaveProject(wxCommandEvent &e)
     }
 
     wxFileDialog dlg(this,_("Save project file"),wxEmptyString,wxEmptyString,
-                     _("Project files (*.pto)|*.pto|All files (*)|*"), wxFD_SAVE, wxDefaultPosition);
+                     _("Project files (*.pto)|*.pto|All files (*)|*"), wxFD_SAVE | wxFD_OVERWRITE_PROMPT, wxDefaultPosition);
     dlg.SetDirectory(wxConfigBase::Get()->Read(wxT("/actualPath"),wxT("")));
     if (dlg.ShowModal() == wxID_OK)
     {
@@ -825,7 +777,7 @@ void LensCalFrame::OnSaveProject(wxCommandEvent &e)
         std::string path = getPathPrefix(std::string(filename.GetFullPath().mb_str(HUGIN_CONV_FILENAME)));
         std::ofstream script(filename.GetFullPath().mb_str(HUGIN_CONV_FILENAME));
         script.exceptions ( std::ofstream::eofbit | std::ofstream::failbit | std::ofstream::badbit );
-        PT::UIntSet all;
+        HuginBase::UIntSet all;
         fill_set(all, 0, pano.getNrOfImages()-1);
         pano.printPanoramaScript(script, pano.getOptimizeVector(), pano.getOptions(), all, false, path);
         script.close();
