@@ -19,8 +19,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -39,6 +39,10 @@ BEGIN_EVENT_TABLE(LensCalImageCtrl, wxPanel)
     EVT_MOUSE_EVENTS(LensCalImageCtrl::OnMouseEvent)
 END_EVENT_TABLE()
 
+#if !wxCHECK_VERSION(3,0,0)
+#define wxPENSTYLE_SOLID wxSOLID
+#endif
+
 // init some values
 LensCalImageCtrl::LensCalImageCtrl() : wxPanel()
 {
@@ -174,13 +178,13 @@ void LensCalImageCtrl::DrawView()
             switch(lines[i].status)
             {
                 case HuginLines::valid_line:
-                    memDC.SetPen(wxPen(wxColour(0,255,0), 1, wxSOLID));
+                    memDC.SetPen(wxPen(wxColour(0,255,0), 1, wxPENSTYLE_SOLID));
                     break;
                 case HuginLines::valid_line_disabled:
-                    memDC.SetPen(wxPen(wxColour(255,0,0), 1, wxSOLID));
+                    memDC.SetPen(wxPen(wxColour(255, 0, 0), 1, wxPENSTYLE_SOLID));
                     break;
                 default:
-                    memDC.SetPen(wxPen(wxColour(128,128,128), 1, wxSOLID));
+                    memDC.SetPen(wxPen(wxColour(128, 128, 128), 1, wxPENSTYLE_SOLID));
                     break;
             };
             for(unsigned int j=0;j<lines[i].line.size()-1;j++)
@@ -357,8 +361,7 @@ void LensCalImageCtrl::SetEdgeImage()
 
 void LensCalImageCtrl::GenerateRemappedImage(const unsigned int newWidth,const unsigned int newHeight)
 {
-    PT::RemappedPanoImage<vigra::BRGBImage,vigra::BImage>* remapped=new PT::RemappedPanoImage<vigra::BRGBImage,vigra::BImage>;
-    AppBase::MultiProgressDisplay* progress=new AppBase::DummyMultiProgressDisplay();
+    HuginBase::Nona::RemappedPanoImage<vigra::BRGBImage,vigra::BImage>* remapped=new HuginBase::Nona::RemappedPanoImage<vigra::BRGBImage,vigra::BImage>;
     //generate SrcPanoImage with current settings
     m_panoimage=*(m_imageLines->GetPanoImage());
     m_panoimage.setProjection(m_projection);
@@ -410,12 +413,10 @@ void LensCalImageCtrl::GenerateRemappedImage(const unsigned int newWidth,const u
     m_opts.setROI(vigra::Rect2D(0,0,m_opts.getWidth(),m_opts.getHeight()));
     //now remap image
     remapped->setPanoImage(m_panoimage,m_opts, m_opts.getROI());
-    remapped->remapImage(vigra::srcImageRange(*(img->get8BitImage())),vigra_ext::INTERP_CUBIC,*progress);
+    remapped->remapImage(vigra::srcImageRange(*(img->get8BitImage())),vigra_ext::INTERP_CUBIC, wxGetApp().GetLensCalFrame());
     m_remappedImage=remapped->m_image;
     m_remapped_img.SetData((unsigned char*)m_remappedImage.data(),m_remappedImage.width(),m_remappedImage.height(),true);
     delete remapped;
-    delete progress;
-
 };
 
 IMPLEMENT_DYNAMIC_CLASS(LensCalImageCtrl, wxPanel)
