@@ -16,8 +16,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -40,11 +40,12 @@
 #include "CelesteGlobals.h"
 #include "Utilities.h"
 
-#include <PT/Panorama.h>
+#include <panodata/Panorama.h>
 
 #include "CPImagesComboBox.h"
 
 #include "CPImageCtrl.h"
+#include <panotools/PanoToolsInterface.h>
 
 namespace vigra {
     class Diff2D;
@@ -60,7 +61,7 @@ namespace vigra_ext{
  *
  *  @todo support control lines
  */
-class CPEditorPanel : public wxPanel, public PT::PanoramaObserver
+class CPEditorPanel : public wxPanel, public HuginBase::PanoramaObserver
 {
 public:
     /** ctor.
@@ -69,7 +70,7 @@ public:
 
     bool Create(wxWindow* parent, wxWindowID id = wxID_ANY, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxTAB_TRAVERSAL, const wxString& name = wxT("panel"));
 
-    void Init(PT::Panorama * pano);
+    void Init(HuginBase::Panorama * pano);
 
     /** dtor.
      */
@@ -80,14 +81,14 @@ public:
     /// set right image
     void setRightImage(unsigned int imgNr);
 
-    void SetPano(PT::Panorama * panorama)
+    void SetPano(HuginBase::Panorama * panorama)
         { m_pano = panorama; };
 
     /** called when the panorama changes and we should
      *  update our display
      */
-    void panoramaChanged(PT::Panorama &pano);
-    void panoramaImagesChanged(PT::Panorama &pano, const PT::UIntSet & imgNr);
+    void panoramaChanged(HuginBase::Panorama &pano);
+    void panoramaImagesChanged(HuginBase::Panorama &pano, const HuginBase::UIntSet & imgNr);
 
 
     /** Select a point.
@@ -174,16 +175,11 @@ private:
     // local fine tune
     hugin_utils::FDiff2D LocalFineTunePoint(unsigned int srcNr,
                                const vigra::Diff2D & srcPnt,
+                               hugin_utils::FDiff2D & movedSrcPnt,
                                unsigned int moveNr,
                                const hugin_utils::FDiff2D & movePnt);
 
     /** Estimate position of point in the other image
-     *
-     *  simply average all point distances together
-     *  approximatly true for rectilinear images with roll and pitch
-     *   close to 0...
-     *  @todo use Pano Tools optimizer, to create better estimates,
-     *        that are correct in most cases..
      *
      *  @param p point to warp to other image
      *  @param left true if p is located in left image.
@@ -239,7 +235,7 @@ private:
     wxPanel *m_cp_ctrls;
 
     // my data
-    PT::Panorama * m_pano;
+    HuginBase::Panorama * m_pano;
     // the current images
     unsigned int m_leftImageNr;
     unsigned int m_rightImageNr;
@@ -248,10 +244,10 @@ private:
     bool m_listenToPageChange;
     double m_detailZoomFactor;
     // store transformation for drawing line control points
-    PTools::Transform m_leftTransform;
-    PTools::Transform m_leftInvTransform;
-    PTools::Transform m_rightTransform;
-    PTools::Transform m_rightInvTransform;
+    HuginBase::PTools::Transform m_leftTransform;
+    HuginBase::PTools::Transform m_leftInvTransform;
+    HuginBase::PTools::Transform m_rightTransform;
+    HuginBase::PTools::Transform m_rightInvTransform;
 
     unsigned int m_selectedPoint;
 
@@ -260,6 +256,7 @@ private:
     // this set contains all points that are mirrored (point 1 in right window,
     // point 2 in left window), in local point numbers
     std::set<unsigned int> mirroredPoints;
+    size_t m_countCP;
 
     CPImageCtrl::ImageRotation m_leftRot;
     CPImageCtrl::ImageRotation m_rightRot;
@@ -271,6 +268,12 @@ private:
 
 };
 
+/** function for fine-tune with remapping to stereographic projection */
+vigra_ext::CorrelationResult PointFineTuneProjectionAware(const HuginBase::SrcPanoImage& templ, const vigra::UInt8RGBImage& templImg,
+    vigra::Diff2D templPos, int templSize,
+    const SrcPanoImage& search, const vigra::UInt8RGBImage& searchImg,
+    vigra::Diff2D searchPos, int sWidth);
+
 /** xrc handler */
 class CPEditorPanelXmlHandler : public wxXmlResourceHandler
 {
