@@ -15,8 +15,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -68,6 +68,7 @@ class PreviewPanoMaskTool;
 class PreviewControlPointTool;
 class PreviewLayoutLinesTool;
 class PreviewColorPickerTool;
+class PreviewEditCPTool;
 
 class PanosphereOverviewProjectionGridTool;
 class PreviewProjectionGridTool;
@@ -98,7 +99,7 @@ class ImageToogleButtonEventHandler : public wxEvtHandler
 public:
     ImageToogleButtonEventHandler(unsigned int image_number,
                                   wxToolBarToolBase* identify_toolbutton_in,
-                                  PT::Panorama * m_pano);
+                                  HuginBase::Panorama * m_pano);
     void OnChange(wxCommandEvent &e);
     void AddIdentifyTool(PreviewIdentifyTool** identify_tool_in);
 protected:
@@ -109,13 +110,13 @@ private:
     unsigned int image_number;
     std::vector<PreviewIdentifyTool**> identify_tools;
     wxToolBarToolBase *identify_toolbutton;
-    PT::Panorama * m_pano;
+    HuginBase::Panorama * m_pano;
 };
 
 class ImageGroupButtonEventHandler : public wxEvtHandler
 {
 public:
-    ImageGroupButtonEventHandler(unsigned int image_number, GLPreviewFrame* frame_in, PT::Panorama* m_pano);
+    ImageGroupButtonEventHandler(unsigned int image_number, GLPreviewFrame* frame_in, HuginBase::Panorama* m_pano);
     void OnChange(wxCommandEvent &e);
     void AddDragTool(DragTool** drag_tool_in);
     void AddIdentifyTool(PreviewIdentifyTool** identify_tool_in);
@@ -128,7 +129,7 @@ private:
     std::vector<DragTool**> drag_tools;
     std::vector<PreviewIdentifyTool**> identify_tools;
     GLPreviewFrame* frame;
-    PT::Panorama * m_pano;
+    HuginBase::Panorama * m_pano;
 };
 
 
@@ -183,22 +184,20 @@ private:
  *  it is not created with XRC, because it is highly dynamic, buttons
  *  have to be added etc.
  */
-class GLPreviewFrame : public wxFrame, public PT::PanoramaObserver, public AppBase::MultiProgressDisplay
+class GLPreviewFrame : public wxFrame, public HuginBase::PanoramaObserver
 {
 public:
 
     /** ctor.
      */
-    GLPreviewFrame(wxFrame * frame, PT::Panorama &pano);
+    GLPreviewFrame(wxFrame * frame, HuginBase::Panorama &pano);
 
     /** dtor.
      */
     virtual ~GLPreviewFrame();
 
-    void panoramaChanged(PT::Panorama &pano);
-    void panoramaImagesChanged(PT::Panorama &pano, const PT::UIntSet &changed);
-    
-    void updateProgressDisplay();
+    virtual void panoramaChanged(HuginBase::Panorama &pano);
+    virtual void panoramaImagesChanged(HuginBase::Panorama &pano, const HuginBase::UIntSet &changed);
     
     void MakePreviewTools(PreviewToolHelper * helper);
     void MakePanosphereOverviewTools(PanosphereOverviewToolHelper * helper);
@@ -240,8 +239,8 @@ public:
     void ToggleImageInDragGroup(unsigned int image_nr, bool update_check_box = true);
     void RemoveImageFromDragGroup(unsigned int image_nr, bool update_check_box = true);
     void AddImageToDragGroup(unsigned int image_nr, bool update_check_box = true);
-    void SetDragGroupImages(PT::UIntSet imageDragGroup_in, bool update_check_box = true);
-    PT::UIntSet GetDragGroupImages();
+    void SetDragGroupImages(HuginBase::UIntSet imageDragGroup_in, bool update_check_box = true);
+    HuginBase::UIntSet GetDragGroupImages();
     void ClearDragGroupImages(bool update_check_box = true);
     /** updates the global white balance 
      * @param redFactor multiplies all WhiteBalanceRed of individuel images with this factor
@@ -319,6 +318,8 @@ protected:
     void OnLayoutScaleChange(wxScrollEvent &e);
     /** event handler when starting color picker */
     void OnColorPicker(wxCommandEvent &e);
+    /** event handler when starting edit cp tool */
+    void OnEditCPTool(wxCommandEvent &e);
     /** event handler when user changes background color */
     void OnPreviewBackgroundColorChanged(wxColourPickerEvent & e);
     /** event handler when user selects different guide */
@@ -333,6 +334,13 @@ protected:
     void OnFocalLengthChanged(wxCommandEvent & e);
     void OnCropFactorChanged(wxCommandEvent & e);
 
+    /** handler for creating cp in pano space */
+    void OnCreateCP(wxCommandEvent & e);
+    /** handler to remove cp */
+    void OnRemoveCP(wxCommandEvent & e);
+    /** handle menu close event */
+    void OnMenuClose(wxMenuEvent & e);
+
 private:
     /** changes the visibility of the group check boxes
      * @param isShown true if the group checkboxes should be visible
@@ -349,7 +357,7 @@ private:
     GLwxAuiManager * m_mgr;
 
     void SetMode(int newMode);
-    PT::Panorama & m_pano;
+    HuginBase::Panorama & m_pano;
 
     GLPreview * m_GLPreview;
     GLOverview * m_GLOverview;
@@ -361,6 +369,8 @@ private:
     int m_mode;
     int non_layout_blend_mode;
     wxToolBar* m_ToolBar_Identify;
+    wxToolBar* m_ToolBar_ColorPicker;
+    wxToolBar* m_ToolBar_editCP;
     wxNotebook* m_tool_notebook;
     wxPanel* m_projection_panel;
     wxSlider * m_HFOVSlider;
@@ -435,9 +445,10 @@ private:
     PreviewToolHelper *preview_helper;
     
     PreviewCropTool *crop_tool;
-    PT::UIntSet imageDragGroup;
+    HuginBase::UIntSet imageDragGroup;
     PreviewDragTool *drag_tool;
     PreviewColorPickerTool *color_picker_tool;
+    PreviewEditCPTool* edit_cp_tool;
 
     PreviewIdentifyTool *identify_tool;
     PreviewIdentifyTool *panosphere_overview_identify_tool;
@@ -482,12 +493,12 @@ private:
     bool loadedLayout;
 #endif
 
-    /** check, if panorama has non-zero translation parameters
+    /** check, if panorama has non-zero translation plane parameters
      *  @returns true, if panorama has non-zero translation parameters
      */
-    bool HasNonZeroTranslationParameters();
-    /** resets all translation parameters to zero */
-    void ResetTranslationParameters();
+    bool HasNonZeroTranslationPlaneParameters();
+    /** resets all translation plane parameters to zero */
+    void ResetTranslationPlaneParameters();
     void CleanButtonColours();
     /** Tell the user anything suspicious about the projection choice.
      * If nothing is suspicious, any previous message is removed.
