@@ -15,8 +15,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -188,6 +188,7 @@ BEGIN_EVENT_TABLE(GLViewer, wxGLCanvas)
     // mouse motion
     EVT_MOTION (GLViewer::MouseMotion)
     // mouse entered or left the preview
+    EVT_ENTER_WINDOW(GLViewer::MouseEnter)
     EVT_LEAVE_WINDOW(GLViewer::MouseLeave)
     // mouse buttons
     EVT_MOUSEWHEEL(GLViewer::MouseWheel)
@@ -200,14 +201,14 @@ END_EVENT_TABLE()
 
 GLViewer::GLViewer(
             wxWindow* parent, 
-            PT::Panorama &pano, 
+            HuginBase::Panorama &pano, 
             int args[], 
             GLPreviewFrame *frame_in,
             wxGLContext * shared_context
             ) :
 #if defined __WXGTK__ || wxCHECK_VERSION(2,9,0)
-          wxGLCanvas(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize,
-                     0, wxT("GLPreviewCanvas"), args, wxNullPalette)
+          wxGLCanvas(parent, wxID_ANY, args, wxDefaultPosition, wxDefaultSize,
+                     0, wxT("GLPreviewCanvas"), wxNullPalette)
 #else
           wxGLCanvas(parent,shared_context,wxID_ANY,wxDefaultPosition,
                      wxDefaultSize,0,wxT("GLPreviewCanvas"),args,wxNullPalette)
@@ -447,7 +448,9 @@ void GLViewer::Resized(wxSizeEvent& e)
    
     if (frame->CanResize()) {
         DEBUG_DEBUG("RESIZED_IN");
+#if !wxCHECK_VERSION(3,0,1)
         wxGLCanvas::OnSize(e);
+#endif
         if(!IsShown()) return;
         // if we have a render at this point, tell it the new size.
         DEBUG_DEBUG("RESIZED_IN_SHOWN");
@@ -513,6 +516,13 @@ void GLViewer::MouseMotion(wxMouseEvent& e)
                               (int) e.m_y - offset.y, e);
 }
 
+void GLViewer::MouseEnter(wxMouseEvent & e)
+{
+    if(m_renderer)
+        m_tool_helper->MouseEnter((int) e.m_x - offset.x,
+                              (int) e.m_y - offset.y, e);
+}
+
 void GLViewer::MouseLeave(wxMouseEvent & e)
 {
     if(m_renderer)
