@@ -19,8 +19,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -32,7 +32,9 @@
 #include <string>
 
 #include "hugin/huginApp.h"
-#include "CommandHistory.h"
+#include "base_wx/CommandHistory.h"
+#include "base_wx/PanoCommand.h"
+#include "panotools/PanoToolsInterface.h"
 
 using namespace HuginBase;
 
@@ -44,7 +46,7 @@ BEGIN_EVENT_TABLE(ImageVariableDialog,wxDialog)
     EVT_BUTTON(XRCID("image_show_response_graph"), ImageVariableDialog::OnShowResponseGraph)
 END_EVENT_TABLE()
 
-ImageVariableDialog::ImageVariableDialog(wxWindow *parent, PT::Panorama* pano, HuginBase::UIntSet imgs)
+ImageVariableDialog::ImageVariableDialog(wxWindow *parent, HuginBase::Panorama* pano, HuginBase::UIntSet imgs)
 {
     // load our children. some children might need special
     // initialization. this will be done later.
@@ -175,7 +177,7 @@ void ImageVariableDialog::SetGuiLevel(GuiLevel newLevel)
 
 bool ImageVariableDialog::ApplyNewVariables()
 {
-    std::vector<PT::PanoCommand*> commands;
+    std::vector<PanoCommand::PanoCommand*> commands;
     VariableMap varMap;
     for (const char** varname = m_varNames; *varname != 0; ++varname)
     {
@@ -234,20 +236,20 @@ bool ImageVariableDialog::ApplyNewVariables()
             img.setResponseType((SrcPanoImage::ResponseType)sel);
             SrcImgs.push_back(img);
         }
-        commands.push_back(new PT::UpdateSrcImagesCmd( *m_pano, m_images, SrcImgs ));
+        commands.push_back(new PanoCommand::UpdateSrcImagesCmd( *m_pano, m_images, SrcImgs ));
     }
     if(varMap.size()>0)
     {
         for(UIntSet::const_iterator it=m_images.begin();it!=m_images.end();++it)
         {
             commands.push_back(
-                new PT::UpdateImageVariablesCmd(*m_pano,*it,varMap)
+                new PanoCommand::UpdateImageVariablesCmd(*m_pano,*it,varMap)
                 );
         };
     };
     if(commands.size()>0)
     {
-        GlobalCmdHist::getInstance().addCommand(new PT::CombinedPanoCommand(*m_pano, commands));
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(new PanoCommand::CombinedPanoCommand(*m_pano, commands));
         return true;
     }
     else
@@ -323,7 +325,7 @@ void GraphPopupWindow::OnRightDown(wxMouseEvent &e)
                         _("Save graph"),
                         config->Read(wxT("/actualPath"),wxT("")), wxT(""),
                         _("Bitmap (*.bmp)|*.bmp|PNG-File (*.png)|*.png"),
-                        wxFD_SAVE, wxDefaultPosition);
+                        wxFD_SAVE | wxFD_OVERWRITE_PROMPT, wxDefaultPosition);
     dlg.SetDirectory(config->Read(wxT("/actualPath"),wxT("")));
     dlg.SetFilterIndex(config->Read(wxT("/lastImageTypeIndex"), 0l));
     if (dlg.ShowModal() == wxID_OK)
@@ -452,13 +454,13 @@ public:
     const wxBitmap GetGraph() const { return *m_bitmap; };
 private:
     //helper function to transform coordinates from real world to bitmap
-    int TransformX(float x)
+    int TransformX(double x)
     {
-        return (x-m_xmin)/(m_xmax-m_xmin)*(m_right-m_left)+m_left;
+        return hugin_utils::round((x-m_xmin)/(m_xmax-m_xmin)*(m_right-m_left)+m_left);
     };
-    int TransformY(float y)
+    int TransformY(double y)
     {
-        return m_bottom-(y-m_ymin)/(m_ymax-m_ymin)*(m_bottom-m_top);
+        return hugin_utils::round(m_bottom - (y - m_ymin) / (m_ymax - m_ymin)*(m_bottom - m_top));
     };
     // area to be drawn
     double m_xmin, m_xmax, m_ymin, m_ymax;
@@ -492,30 +494,50 @@ void ImageVariableDialog::OnShowDistortionGraph(wxCommandEvent & e)
     //create transformation
     SrcPanoImage srcImage;
 #define NRPOINTS 100
-    srcImage.setSize(vigra::Size2D(2*NRPOINTS, 2*NRPOINTS));
+    srcImage.setSize(m_pano->getImage(*(m_images.begin())).getSize());
+    // set projection to rectilinear, just in case, it should be the default value
+    srcImage.setProjection(SrcPanoImage::RECTILINEAR);
     srcImage.setRadialDistortion(radialDist);
     PanoramaOptions opts;
     opts.setHFOV(srcImage.getHFOV());
     opts.setProjection(PanoramaOptions::RECTILINEAR);
-    opts.setWidth(2*NRPOINTS);
-    opts.setHeight(2*NRPOINTS);
+    opts.setWidth(srcImage.getWidth());
+    opts.setHeight(srcImage.getHeight());
     HuginBase::PTools::Transform transform;
     transform.createTransform(srcImage, opts);
 
+    const double minLength = std::min<double>(srcImage.getWidth(), srcImage.getHeight()) / 2.0;
+    const double maxR = sqrt(srcImage.getWidth()*srcImage.getWidth() + srcImage.getHeight()*srcImage.getHeight()) / (2.0*minLength);
     //draw graph
     delete m_popup;
     Graph graph(300, 200, wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
     graph.SetChartArea(10, 10, 290, 190);
-    graph.SetChartDisplay(0, -0.1, 1, 0.1);
+    graph.SetChartDisplay(0, -0.1, maxR, 0.1);
     graph.DrawGrid(6, 7);
     std::vector<hugin_utils::FDiff2D> points;
+    // draw helper lines
+    // help line at r=1
+    points.push_back(hugin_utils::FDiff2D(1, -0.1));
+    points.push_back(hugin_utils::FDiff2D(1, 0.1));
+    graph.DrawLine(points, wxSystemSettings::GetColour(wxSYS_COLOUR_GRAYTEXT), 1);
+    points.clear();
+    // helper line at r=width/height (for landscape images), maximal radius in horizontal (or vertical) direction
+    // chart goes to r for diagonal
+    const double rMaxHorzVert = std::max<double>(srcImage.getWidth(), srcImage.getHeight()) / (2.0*minLength);
+    points.push_back(hugin_utils::FDiff2D(rMaxHorzVert, -0.1));
+    points.push_back(hugin_utils::FDiff2D(rMaxHorzVert, 0.1));
+    graph.DrawLine(points, wxSystemSettings::GetColour(wxSYS_COLOUR_GRAYTEXT), 1);
+    points.clear();
+    // now draw distortion graph
+    double r = 0;
     for(size_t i=0; i<NRPOINTS; i++)
     {
         double x,y;
-        if(transform.transformImgCoord(x, y, NRPOINTS, NRPOINTS-i))
+        if (transform.transform(x, y, r*minLength, 0))
         {
-            points.push_back(hugin_utils::FDiff2D(double(i)/NRPOINTS,(NRPOINTS-i-y)/(NRPOINTS)));
+            points.push_back(hugin_utils::FDiff2D(r, x/minLength - r));
         };
+        r += maxR / (NRPOINTS-1);
     };
     graph.DrawLine(points, wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT), 2);
 
