@@ -20,8 +20,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -34,9 +34,14 @@
 #include "base_wx/wxPlatform.h"
 #include <vector>
 #include <map>
+#ifdef HAVE_CXX11
+#include <functional>    // std::bind
+#else
+#include <boost/bind.hpp>
+#endif
 
 #include "hugin/ImagesPanel.h"
-#include "hugin/CommandHistory.h"
+#include "base_wx/CommandHistory.h"
 #include "hugin/TextKillFocusHandler.h"
 #include "hugin/CPEditorPanel.h"
 #include "hugin/ImagesList.h"
@@ -44,16 +49,15 @@
 #include "hugin/huginApp.h"
 #include "icpfind/AutoCtrlPointCreator.h"
 #include "hugin/config_defaults.h"
-#include "base_wx/MyProgressDialog.h"
 #include "base_wx/PTWXDlg.h"
-#include <PT/PTOptimise.h>
 #include "base_wx/LensTools.h"
 #include "hugin/ImagesTree.h"
 #include "panodata/OptimizerSwitches.h"
+#include "base_wx/PanoCommand.h"
 
 #include <panodata/StandardImageVariableGroups.h>
 
-using namespace PT;
+using namespace HuginBase;
 using namespace hugin_utils;
 using namespace vigra;
 using namespace vigra_ext;
@@ -209,7 +213,7 @@ void ImagesPanel::OnSize( wxSizeEvent & e )
     e.Skip();
 }
 
-void ImagesPanel::panoramaChanged(PT::Panorama & pano)
+void ImagesPanel::panoramaChanged(HuginBase::Panorama & pano)
 {
     //update optimizer choice selection
     int optSwitch=m_pano->getOptimizerSwitch();
@@ -224,8 +228,8 @@ void ImagesPanel::panoramaChanged(PT::Panorama & pano)
     };
     if(found==wxNOT_FOUND)
     {
-        GlobalCmdHist::getInstance().addCommand(
-            new PT::UpdateOptimizerSwitchCmd(*m_pano, 0)
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::UpdateOptimizerSwitchCmd(*m_pano, 0)
         );
     }
     else
@@ -246,20 +250,20 @@ void ImagesPanel::panoramaChanged(PT::Panorama & pano)
     };
     if(found==wxNOT_FOUND)
     {
-        GlobalCmdHist::getInstance().addCommand(
-            new PT::UpdatePhotometricOptimizerSwitchCmd(*m_pano, 0)
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::UpdatePhotometricOptimizerSwitchCmd(*m_pano, 0)
         );
     }
     else
     {
         m_optPhotoChoice->SetSelection(found);
     };
-    const PanoramaOptions opts=m_pano->getOptions();
+    const HuginBase::PanoramaOptions opts = m_pano->getOptions();
     m_overlap->SetValue(hugin_utils::doubleTowxString(opts.outputStacksMinOverlap,3));
     m_maxEv->SetValue(hugin_utils::doubleTowxString(opts.outputLayersExposureDiff,2));
 }
 
-void ImagesPanel::panoramaImagesChanged(PT::Panorama &pano, const PT::UIntSet & _imgNr)
+void ImagesPanel::panoramaImagesChanged(HuginBase::Panorama &pano, const HuginBase::UIntSet & _imgNr)
 {
     DEBUG_TRACE("");
 
@@ -329,12 +333,12 @@ void ImagesPanel::CPGenerate(wxCommandEvent & e)
     };
 
     AutoCtrlPointCreator matcher;
-    CPVector cps = matcher.automatch(cpdetector_config.settings[m_CPDetectorChoice->GetSelection()],
+    HuginBase::CPVector cps = matcher.automatch(cpdetector_config.settings[m_CPDetectorChoice->GetSelection()],
         *m_pano, selImg, nFeatures,this);
     wxString msg;
     wxMessageBox(wxString::Format(_("Added %lu control points"), (unsigned long) cps.size()), _("Control point detector result"),wxOK|wxICON_INFORMATION,this);
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::AddCtrlPointsCmd(*m_pano, cps)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::AddCtrlPointsCmd(*m_pano, cps)
                                            );
 
 };
@@ -379,7 +383,7 @@ void ImagesPanel::OnSelectionChanged(wxTreeEvent & e)
 
         if(identical_projection)
         {
-            SelectProjection(m_lenstype, proj);
+            SelectListValue(m_lenstype, proj);
         }
         else
         {
@@ -457,8 +461,13 @@ void ImagesPanel::UpdatePreviewImage()
         thumbnail_request = ImageCache::getInstance().requestAsyncSmallImage(
                 m_pano->getImage(m_showImgNr).getFilename());
         // When the image is ready, try this function again.
-        thumbnail_request->ready.connect(
-            boost::bind(&ImagesPanel::UpdatePreviewImage, this));
+        thumbnail_request->ready.push_back(
+#ifdef HAVE_CXX11
+            std::bind(&ImagesPanel::UpdatePreviewImage, this)
+#else
+            boost::bind(&ImagesPanel::UpdatePreviewImage, this)
+#endif
+            );
     } else {
         // forget any request now the image has loaded.
         thumbnail_request = ImageCache::RequestPtr();
@@ -499,17 +508,17 @@ void ImagesPanel::ReloadCPDetectorSettings()
 
 void ImagesPanel::OnLensTypeChanged (wxCommandEvent & e)
 {
-    size_t var = GetSelectedProjection(m_lenstype);
+    size_t var = GetSelectedValue(m_lenstype);
     UIntSet images=m_images_tree->GetSelectedImages();
     if(images.size()>0)
     {
         const SrcPanoImage & img=m_pano->getImage(*(images.begin()));
         double focal_length = SrcPanoImage::calcFocalLength(img.getProjection(),img.getHFOV(),img.getCropFactor(),img.getSize());
-        std::vector<PanoCommand*> commands;
-        commands.push_back(new PT::ChangeImageProjectionCmd(*m_pano, images,(HuginBase::SrcPanoImage::Projection) var));
-        commands.push_back(new PT::UpdateFocalLengthCmd(*m_pano, images, focal_length));
-        GlobalCmdHist::getInstance().addCommand(
-            new PT::CombinedPanoCommand(*m_pano, commands)
+        std::vector<PanoCommand::PanoCommand*> commands;
+        commands.push_back(new PanoCommand::ChangeImageProjectionCmd(*m_pano, images,(HuginBase::SrcPanoImage::Projection) var));
+        commands.push_back(new PanoCommand::UpdateFocalLengthCmd(*m_pano, images, focal_length));
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::CombinedPanoCommand(*m_pano, commands)
         );
     };
 };
@@ -560,8 +569,8 @@ void ImagesPanel::OnFocalLengthChanged(wxCommandEvent & e)
             };
         };
     };
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::UpdateFocalLengthCmd(*m_pano, images, val)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::UpdateFocalLengthCmd(*m_pano, images, val)
     );
 }
 
@@ -590,8 +599,8 @@ void ImagesPanel::OnCropFactorChanged(wxCommandEvent & e)
     };
 
     UIntSet images=m_images_tree->GetSelectedImages();
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::UpdateCropFactorCmd(*m_pano,images,val)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::UpdateCropFactorCmd(*m_pano,images,val)
     );
 }
 
@@ -607,7 +616,7 @@ void ImagesPanel::OnMinimumOverlapChanged(wxCommandEvent & e)
     {
         return;
     }
-    if(val<=0 || val>1)
+    if(fabs(val)<0.001 || val>1)
     {
         wxMessageBox(_("The minimum overlap has to be greater than 0 and smaller than 1."),
 #ifdef _WINDOWS
@@ -618,10 +627,14 @@ void ImagesPanel::OnMinimumOverlapChanged(wxCommandEvent & e)
             wxOK | wxICON_INFORMATION, this);
         return;
     };
-    PanoramaOptions opt=m_pano->getOptions();
+    if (val < 0)
+    {
+        val = -1;
+    };
+    HuginBase::PanoramaOptions opt = m_pano->getOptions();
     opt.outputStacksMinOverlap=val;
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetPanoOptionsCmd(*m_pano,opt)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetPanoOptionsCmd(*m_pano,opt)
     );
 };
 
@@ -648,10 +661,10 @@ void ImagesPanel::OnMaxEvDiffChanged(wxCommandEvent& e)
             wxOK | wxICON_INFORMATION, this);
         return;
     };
-    PanoramaOptions opt=m_pano->getOptions();
+    HuginBase::PanoramaOptions opt = m_pano->getOptions();
     opt.outputLayersExposureDiff=val;
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetPanoOptionsCmd(*m_pano,opt)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetPanoOptionsCmd(*m_pano,opt)
     );
 };
 
@@ -763,6 +776,11 @@ void ImagesPanel::FillOptimizerChoice()
     Refresh();
 };
 
+wxString ImagesPanel::GetCurrentOptimizerString()
+{
+    return m_optChoice->GetString(m_optChoice->GetSelection());
+};
+
 void ImagesPanel::OnGroupModeChanged(wxCommandEvent & e)
 {
     wxChoice* group=XRCCTRL(*this,"images_group_mode", wxChoice);
@@ -789,8 +807,8 @@ void ImagesPanel::OnOptimizerSwitchChanged(wxCommandEvent &e)
     int optSwitch=*static_cast<int*>(m_optChoice->GetClientData(m_optChoice->GetSelection()));
     if(optSwitch!=m_pano->getOptimizerSwitch())
     {
-        GlobalCmdHist::getInstance().addCommand(
-            new PT::UpdateOptimizerSwitchCmd(*m_pano,optSwitch)
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::UpdateOptimizerSwitchCmd(*m_pano,optSwitch)
         );
     };
 };
@@ -800,8 +818,8 @@ void ImagesPanel::OnPhotometricOptimizerSwitchChanged(wxCommandEvent &e)
     int optSwitch=*static_cast<int*>(m_optPhotoChoice->GetClientData(m_optPhotoChoice->GetSelection()));
     if(optSwitch!=m_pano->getPhotometricOptimizerSwitch())
     {
-        GlobalCmdHist::getInstance().addCommand(
-            new PT::UpdatePhotometricOptimizerSwitchCmd(*m_pano,optSwitch)
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::UpdatePhotometricOptimizerSwitchCmd(*m_pano,optSwitch)
         );
     };
 };
