@@ -18,8 +18,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -28,18 +28,20 @@
 #include "panoinc.h"
 
 #include "base_wx/wxPlatform.h"
+#include "base_wx/LensTools.h"
 #include "hugin/ImagesTree.h"
 #include "base_wx/wxImageCache.h"
 #include "base_wx/platform.h"
 #include "hugin_base/algorithms/basic/LayerStacks.h"
 #include <panodata/ImageVariableGroup.h>
 #include <panodata/StandardImageVariableGroups.h>
-#include "hugin/CommandHistory.h"
+#include "base_wx/CommandHistory.h"
+#include "base_wx/PanoCommand.h"
 #include <hugin_utils/stl_utils.h>
 #include "hugin/ImageVariableDialog.h"
 #include "hugin/huginApp.h"
 
-using namespace PT;
+using namespace HuginBase;
 using namespace hugin_utils;
 
 enum
@@ -203,7 +205,7 @@ void ImagesTreeCtrl::CreateColumns()
     }
 };
 
-void ImagesTreeCtrl::Init(PT::Panorama * panorama)
+void ImagesTreeCtrl::Init(HuginBase::Panorama * panorama)
 {
     m_pano = panorama;
     m_pano->addObserver(this);
@@ -224,7 +226,7 @@ ImagesTreeCtrl::~ImagesTreeCtrl(void)
     delete m_variable_groups;
 };
 
-void ImagesTreeCtrl::panoramaChanged(PT::Panorama & pano)
+void ImagesTreeCtrl::panoramaChanged(HuginBase::Panorama & pano)
 {
     if(m_optimizerMode)
     {
@@ -285,36 +287,20 @@ void ImagesTreeCtrl::panoramaImagesChanged(Panorama &pano, const UIntSet &change
     }
     else
     {
-        std::vector<UIntSet> imageGroups;
+        HuginBase::UIntSetVector imageGroups;
         switch (m_groupMode)
         {
             case GROUP_LENS:
-                {
-                    HuginBase::ImageVariableGroup& lenses=m_variable_groups->getLenses();
-                    imageGroups.resize(lenses.getNumberOfParts());
-                    for(size_t i=0;i<m_pano->getNrOfImages();i++)
-                    {
-                        imageGroups[lenses.getPartNumber(i)].insert(i);
-                    };
-                };
+                imageGroups = m_variable_groups->getLenses().getPartsSet();
                 break;
             case GROUP_STACK:
-                {
-                    HuginBase::ImageVariableGroup& stacks=m_variable_groups->getStacks();
-                    imageGroups.resize(stacks.getNumberOfParts());
-                    for(size_t i=0;i<m_pano->getNrOfImages();i++)
-                    {
-                        imageGroups[stacks.getPartNumber(i)].insert(i);
-                    };
-                };
+                imageGroups = m_variable_groups->getStacks().getPartsSet();
                 break;
             case GROUP_OUTPUTSTACK:
                 imageGroups=getHDRStacks(*m_pano,m_pano->getActiveImages(), m_pano->getOptions());
-                m_needsUpdate=false;
                 break;
             case GROUP_OUTPUTLAYERS:
                 imageGroups=getExposureLayers(*m_pano,m_pano->getActiveImages(), m_pano->getOptions());
-                m_needsUpdate=false;
                 break;
         };
 
@@ -351,6 +337,7 @@ void ImagesTreeCtrl::panoramaImagesChanged(Panorama &pano, const UIntSet &change
     };
 
     Thaw();
+    m_needsUpdate = false;
 
     // HACK! need to notify clients anyway... send dummy event..
     // lets hope our clients query for the selected images with GetSelected()
@@ -361,44 +348,11 @@ void ImagesTreeCtrl::panoramaImagesChanged(Panorama &pano, const UIntSet &change
     GetEventHandler()->ProcessEvent(e);
 }
 
-wxString getProjectionString(const SrcPanoImage& img)
-{
-    wxString ps;
-    switch(img.getProjection())
-    {
-        case SrcPanoImage::RECTILINEAR:          ps << _("Normal (rectilinear)"); break;
-        case SrcPanoImage::PANORAMIC:            ps << _("Panoramic (cylindrical)"); break;
-        case SrcPanoImage::CIRCULAR_FISHEYE:     ps << _("Circular fisheye"); break;
-        case SrcPanoImage::FULL_FRAME_FISHEYE:   ps << _("Full frame fisheye"); break;
-        case SrcPanoImage::EQUIRECTANGULAR:      ps << _("Equirectangular"); break;
-        case SrcPanoImage::FISHEYE_ORTHOGRAPHIC: ps << _("Orthographic"); break;
-        case SrcPanoImage::FISHEYE_STEREOGRAPHIC:ps << _("Stereographic"); break;
-        case SrcPanoImage::FISHEYE_EQUISOLID:    ps << _("Equisolid"); break;
-        case SrcPanoImage::FISHEYE_THOBY:        ps << _("Fisheye Thoby"); break;
-    }
-    return ps;
-};
-
-wxString getResponseString(const SrcPanoImage& img)
-{
-    wxString s;
-    switch(img.getResponseType())
-    {
-        case HuginBase::BaseSrcPanoImage::RESPONSE_EMOR:
-            s=_("custom (EMoR)");
-            break;
-        case HuginBase::BaseSrcPanoImage::RESPONSE_LINEAR:
-            s=_("Linear");
-            break;
-    };
-    return s;
-};
-
 void ImagesTreeCtrl::UpdateImageText(wxTreeItemId item)
 {
     ImagesTreeData* itemData=(ImagesTreeData*)GetItemData(item);
     wxString s;
-    size_t imgNr=itemData->GetImgNr();
+    const size_t imgNr=itemData->GetImgNr();
     const SrcPanoImage & img = m_pano->getImage(imgNr);
     wxFileName fn(wxString (img.getFilename().c_str(), HUGIN_CONV_FILENAME));
 
@@ -423,9 +377,9 @@ void ImagesTreeCtrl::UpdateImageText(wxTreeItemId item)
     s << cps.size();
     SetItemText(item, m_columnMap["cps"], s);
     s.Clear();
-    unsigned int stackNumber = m_variable_groups->getStacks().getPartNumber(imgNr);
+    const unsigned int stackNumber = m_variable_groups->getStacks().getPartNumber(imgNr);
     SetItemText(item, m_columnMap["stackNr"], wxString::Format(wxT("%u"), stackNumber));
-    unsigned int lensNr=m_variable_groups->getLenses().getPartNumber(imgNr);
+    const unsigned int lensNr=m_variable_groups->getLenses().getPartNumber(imgNr);
     SetItemText(item, m_columnMap["lensNr"], wxString::Format(wxT("%u"), lensNr));
 
     SetItemText(item, m_columnMap["maker"], wxString(img.getExifMake().c_str(), wxConvLocal));
@@ -1305,14 +1259,14 @@ void ImagesTreeCtrl::UnLinkImageVariables(bool linked)
         };
         if(m_groupMode==GROUP_LENS)
         {
-            GlobalCmdHist::getInstance().addCommand(
-                new PT::ChangePartImagesLinkingCmd(*m_pano, images, variables, linked, HuginBase::StandardImageVariableGroups::getLensVariables())
+            PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                new PanoCommand::ChangePartImagesLinkingCmd(*m_pano, images, variables, linked, HuginBase::StandardImageVariableGroups::getLensVariables())
             );
         }
         else
         {
-            GlobalCmdHist::getInstance().addCommand(
-                new PT::ChangePartImagesLinkingCmd(*m_pano, images, variables, linked, HuginBase::StandardImageVariableGroups::getStackVariables())
+            PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                new PanoCommand::ChangePartImagesLinkingCmd(*m_pano, images, variables, linked, HuginBase::StandardImageVariableGroups::getStackVariables())
             );
         };
     };
@@ -1428,8 +1382,8 @@ void ImagesTreeCtrl::OnEndDrag(wxMouseEvent &e)
             {
                 if(img1!=img2)
                 {
-                    GlobalCmdHist::getInstance().addCommand(
-                        new PT::MoveImageCmd(*m_pano, img1, img2)
+                    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                        new PanoCommand::MoveImageCmd(*m_pano, img1, img2)
                     );
                 };
             };
@@ -1458,14 +1412,14 @@ void ImagesTreeCtrl::OnEndDrag(wxMouseEvent &e)
                 {
                     if(m_groupMode==GROUP_LENS)
                     {
-                        GlobalCmdHist::getInstance().addCommand(
-                            new PT::ChangePartNumberCmd(*m_pano, m_draggingImages, groupNr, HuginBase::StandardImageVariableGroups::getLensVariables())
+                        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                            new PanoCommand::ChangePartNumberCmd(*m_pano, m_draggingImages, groupNr, HuginBase::StandardImageVariableGroups::getLensVariables())
                         );
                     }
                     else
                     {
-                        GlobalCmdHist::getInstance().addCommand(
-                            new PT::ChangePartNumberCmd(*m_pano, m_draggingImages, groupNr, HuginBase::StandardImageVariableGroups::getStackVariables())
+                        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                            new PanoCommand::ChangePartNumberCmd(*m_pano, m_draggingImages, groupNr, HuginBase::StandardImageVariableGroups::getStackVariables())
                         );
                     };
                 };
@@ -1474,14 +1428,14 @@ void ImagesTreeCtrl::OnEndDrag(wxMouseEvent &e)
             {
                 if(m_groupMode==GROUP_LENS)
                 {
-                    GlobalCmdHist::getInstance().addCommand(
-                        new PT::NewPartCmd(*m_pano, m_draggingImages, HuginBase::StandardImageVariableGroups::getLensVariables())
+                    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                        new PanoCommand::NewPartCmd(*m_pano, m_draggingImages, HuginBase::StandardImageVariableGroups::getLensVariables())
                     );
                 }
                 else
                 {
-                    GlobalCmdHist::getInstance().addCommand(
-                        new PT::NewPartCmd(*m_pano, m_draggingImages, HuginBase::StandardImageVariableGroups::getStackVariables())
+                    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                        new PanoCommand::NewPartCmd(*m_pano, m_draggingImages, HuginBase::StandardImageVariableGroups::getStackVariables())
                     );
                 }
             };
@@ -1598,8 +1552,8 @@ void ImagesTreeCtrl::OnLeftDown(wxMouseEvent &e)
                                 };
                             };
                         };
-                        GlobalCmdHist::getInstance().addCommand(
-                            new PT::UpdateOptimizeVectorCmd(*m_pano, optVec)
+                        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                            new PanoCommand::UpdateOptimizeVectorCmd(*m_pano, optVec)
                         );
                         return;
                     };
@@ -1681,7 +1635,7 @@ void ImagesTreeCtrl::SelectAllParameters(bool select, bool allImages)
         };
     };
 
-    OptimizeVector optVec=m_pano->getOptimizeVector();
+    HuginBase::OptimizeVector optVec = m_pano->getOptimizeVector();
     for(UIntSet::iterator img=imgs.begin(); img!=imgs.end(); ++img)
     {
         for(std::set<std::string>::const_iterator it=imgVars.begin(); it!=imgVars.end(); ++it)
@@ -1707,8 +1661,8 @@ void ImagesTreeCtrl::SelectAllParameters(bool select, bool allImages)
             };
         };
     };
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::UpdateOptimizeVectorCmd(*m_pano, optVec)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::UpdateOptimizeVectorCmd(*m_pano, optVec)
     );
 };
 
@@ -1747,8 +1701,8 @@ void ImagesTreeCtrl::OnChar(wxTreeEvent &e)
                 HuginBase::UIntSet imgs=GetSelectedImages();
                 if(imgs.size()>0)
                 {
-                    GlobalCmdHist::getInstance().addCommand(
-                        new PT::RemoveImagesCmd(*m_pano, imgs)
+                    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                        new PanoCommand::RemoveImagesCmd(*m_pano, imgs)
                     );
                 };
                 break;
@@ -1839,9 +1793,9 @@ void ImagesTreeCtrl::OnEndEdit(wxTreeEvent &e)
                         };
                     };
                 };
-                Variable var(m_columnVector[e.GetInt()], val);
-                GlobalCmdHist::getInstance().addCommand(
-                    new PT::SetVariableCmd(*m_pano, imgs, var)
+                HuginBase::Variable var(m_columnVector[e.GetInt()], val);
+                PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                    new PanoCommand::SetVariableCmd(*m_pano, imgs, var)
                 );
                 //we need to veto the event otherwise the internal
                 //function does update the text to the full number
@@ -1861,10 +1815,10 @@ void ImagesTreeCtrl::OnEndEdit(wxTreeEvent &e)
 void ImagesTreeCtrl::OnExecuteOperation(wxCommandEvent & e)
 {
     PanoOperation::PanoOperation* op=m_menuOperation[e.GetId()];
-    PT::PanoCommand* cmd=op->GetCommand(this,*m_pano, GetSelectedImages(), m_guiLevel);
+    PanoCommand::PanoCommand* cmd=op->GetCommand(this,*m_pano, GetSelectedImages(), m_guiLevel);
     if(cmd!=NULL)
     {
-        GlobalCmdHist::getInstance().addCommand(cmd);
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(cmd);
     };
 };
 
