@@ -16,8 +16,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -29,15 +29,14 @@
 #include <set>
 #include <map>
 
-#include "PT/Panorama.h"
+#include "panodata/Panorama.h"
 
 #include "wx/docview.h"
 #ifdef __WXMSW__
 #include "wx/msw/helpchm.h"
 #endif
 
-#include <appbase/ProgressDisplayOld.h>
-#include <appbase/ProgressReporterOld.h>
+#include <appbase/ProgressDisplay.h>
 
 #include "hugin/OptimizePanel.h"
 #include "hugin/PreferencesDialog.h"
@@ -46,8 +45,6 @@
 #include "Celeste.h"
 #include "GuiLevel.h"
 
-using namespace PT;
-
 // forward declarations, to save the #include statements
 class CPEditorPanel;
 class ImgPreview;
@@ -64,14 +61,14 @@ class CPListFrame;
 class PanoDropTarget : public wxFileDropTarget
 {
 public:
-    PanoDropTarget(PT::Panorama & p, bool imageOnly = false)
+    PanoDropTarget(HuginBase::Panorama & p, bool imageOnly = false)
         : pano(p)
     { m_imageOnly = imageOnly; }
 
     bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames);
 
 private:
-    PT::Panorama & pano;
+    HuginBase::Panorama & pano;
     bool m_imageOnly;
 };
 
@@ -83,13 +80,13 @@ private:
  *  it therefor also hold operations that determine the lifecycle
  *  of the panorama object (new, open, save, quit).
  */
-class MainFrame : public wxFrame, public PT::PanoramaObserver, public AppBase::MultiProgressDisplay, public AppBase::ProgressReporter
+class MainFrame : public wxFrame, public HuginBase::PanoramaObserver, public AppBase::ProgressDisplay
 {
 public:
 
     /** ctor.
      */
-    MainFrame(wxWindow* parent, PT::Panorama & pano);
+    MainFrame(wxWindow* parent, HuginBase::Panorama & pano);
 
     /** dtor.
      */
@@ -98,10 +95,10 @@ public:
     /** Enable or disable undo and redo.
      *  They should be enabled only when there is a command to act upon.
      */
-    virtual void panoramaChanged(PT::Panorama &pano);
-    void panoramaImagesChanged(PT::Panorama &pano, const PT::UIntSet & imgNr);
+    virtual void panoramaChanged(HuginBase::Panorama &pano);
+    void panoramaImagesChanged(HuginBase::Panorama &pano, const HuginBase::UIntSet & imgNr);
     /** returns panorama object */
-    const PT::Panorama & getPanorama() { return pano; };
+    const HuginBase::Panorama & getPanorama() { return pano; };
 
     // called when a control point in CPListFrame is selected
     void ShowCtrlPoint(unsigned int cpNr);
@@ -151,11 +148,6 @@ public:
     /** opens the stitcher tab */
     void ShowStitcherTab();
 
-    void resetProgress(double max);
-    bool increaseProgress(double delta);
-    bool increaseProgress(double delta, const std::string & msg);
-    void setMessage(const std::string & msg);
-
     void OnCPListFrameClosed();
     /** returns default cp detector setting */
     CPDetectorSetting& GetDefaultSetting();
@@ -173,6 +165,9 @@ public:
 
     wxFileHistory* GetFileHistory() { return &m_mruFiles; };
 
+    /** returns the string which describes the current selected optimizer setting */
+    wxString GetCurrentOptimizerString();
+
 protected:
     // called when a progress message should be displayed
     /** receive notification about progress. Should not be called directly.
@@ -226,9 +221,6 @@ private:
     void OnShowDonate(wxCommandEvent &e);
     void OnShowPanel(wxCommandEvent &e);
 
-    // update progress display
-    bool displayProgress();
-
     wxFileHistory m_mruFiles;
     wxNotebook * m_notebook;
     // tab panels
@@ -255,7 +247,7 @@ private:
     //PreferencesDialog * pref_dlg;
 
     // the model
-    PT::Panorama & pano;
+    HuginBase::Panorama & pano;
 
     // filename of the current project
     wxString m_filename;
@@ -266,10 +258,6 @@ private:
     wxMenu* m_menu_file_simple;
     wxMenu* m_menu_file_advanced;
 
-    // progress reporter
-    double m_progressMax;
-    double m_progress;
-    wxString m_progressMsg;
 #ifdef __WXMSW__
     wxCHMHelpController     m_msHtmlHelp;
 #endif
@@ -282,6 +270,4 @@ private:
     DECLARE_EVENT_TABLE()
 };
 
-bool getLensDataFromUser(wxWindow * parent, SrcPanoImage & srcImg);
-
 #endif // _MAINFRAME_H
