@@ -19,8 +19,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -36,7 +36,8 @@
 #include "base_wx/platform.h"
 #include "hugin/MainFrame.h"
 #include "hugin/config_defaults.h"
-#include "hugin/CommandHistory.h"
+#include "base_wx/CommandHistory.h"
+#include "base_wx/PanoCommand.h"
 #include "hugin/MaskEditorPanel.h"
 #include "hugin/MaskLoadDialog.h"
 #include <wx/clipbrd.h>
@@ -197,7 +198,7 @@ bool MaskEditorPanel::Create(wxWindow* parent, wxWindowID id,
     return true;
 }
 
-void MaskEditorPanel::Init(PT::Panorama * pano)
+void MaskEditorPanel::Init(HuginBase::Panorama * pano)
 {
     m_pano=pano;
     m_imagesListMask->Init(m_pano);
@@ -252,7 +253,7 @@ void MaskEditorPanel::setImage(unsigned int imgNr, bool updateListSelection)
     } 
     else 
     {
-        const SrcPanoImage& image=m_pano->getImage(imgNr);
+        const HuginBase::SrcPanoImage& image=m_pano->getImage(imgNr);
         updateImage=(m_File!=image.getFilename());
         if(updateImage)
             m_File=image.getFilename();
@@ -310,7 +311,7 @@ void MaskEditorPanel::UpdateMask()
     if(GetImgNr()<UINT_MAX)
     {
         m_currentMasks=m_editImg->getNewMask();
-        GlobalCmdHist::getInstance().addCommand(new PT::UpdateMaskForImgCmd(*m_pano,GetImgNr(),m_currentMasks));
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(new PanoCommand::UpdateMaskForImgCmd(*m_pano, GetImgNr(), m_currentMasks));
     };
 };
 
@@ -320,7 +321,7 @@ void MaskEditorPanel::AddMask()
     {
         m_currentMasks=m_editImg->getNewMask();
         m_currentMasks[m_currentMasks.size()-1].setMaskType(m_defaultMaskType);
-        GlobalCmdHist::getInstance().addCommand(new PT::UpdateMaskForImgCmd(*m_pano,GetImgNr(),m_currentMasks));
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(new PanoCommand::UpdateMaskForImgCmd(*m_pano, GetImgNr(), m_currentMasks));
         //select added mask
         SelectMask(m_currentMasks.size()-1);
         m_editImg->selectAllMarkers();
@@ -329,18 +330,27 @@ void MaskEditorPanel::AddMask()
 
 void MaskEditorPanel::SelectMask(unsigned int newMaskNr)
 {
-    if(GetImgNr()<UINT_MAX)
-        if(newMaskNr<m_currentMasks.size())
-            m_maskList->SetItemState(newMaskNr,wxLIST_STATE_SELECTED,wxLIST_STATE_SELECTED);
+    if (GetImgNr() < UINT_MAX)
+    {
+        if (newMaskNr < m_currentMasks.size())
+        {
+            m_maskList->SetItemState(newMaskNr, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
+        }
         else
-            m_maskList->SetItemState(m_MaskNr,0,wxLIST_STATE_SELECTED);
+        {
+            if (m_MaskNr < UINT_MAX)
+            {
+                m_maskList->SetItemState(m_MaskNr, 0, wxLIST_STATE_SELECTED);
+            };
+        };
+    };
 };
 
-void MaskEditorPanel::panoramaChanged(PT::Panorama &pano)
+void MaskEditorPanel::panoramaChanged(HuginBase::Panorama &pano)
 {
 };
 
-void MaskEditorPanel::panoramaImagesChanged(Panorama &pano, const UIntSet &changed)
+void MaskEditorPanel::panoramaImagesChanged(HuginBase::Panorama &pano, const HuginBase::UIntSet &changed)
 {
     unsigned int nrImages = pano.getNrOfImages();
     ImageCache::getInstance().softFlush();
@@ -406,7 +416,7 @@ void MaskEditorPanel::OnMaskTypeChange(wxCommandEvent &e)
     {
         m_currentMasks[m_MaskNr].setMaskType((HuginBase::MaskPolygon::MaskType)e.GetSelection());
         m_defaultMaskType=(HuginBase::MaskPolygon::MaskType)e.GetSelection();
-        GlobalCmdHist::getInstance().addCommand(new PT::UpdateMaskForImgCmd(*m_pano,GetImgNr(),m_currentMasks));
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(new PanoCommand::UpdateMaskForImgCmd(*m_pano, GetImgNr(), m_currentMasks));
     };
 };
 
@@ -430,84 +440,31 @@ void MaskEditorPanel::OnMaskSave(wxCommandEvent &e)
         wxFileDialog dlg(this, _("Save mask"),
                 wxConfigBase::Get()->Read(wxT("/actualPath"), wxT("")),
                 wxT(""), _("Mask files (*.msk)|*.msk|All files (*)|*"), 
-                wxFD_SAVE, wxDefaultPosition);
+                wxFD_SAVE | wxFD_OVERWRITE_PROMPT, wxDefaultPosition);
         if (dlg.ShowModal() == wxID_OK) 
         {
             wxString fn = dlg.GetPath();
-            if (fn.Right(4) != wxT(".msk")) 
+            if (fn.Right(4) != wxT(".msk"))
             {
                 fn.Append(wxT(".msk"));
-            }
-            if (wxFile::Exists(fn)) 
-            {
-                int d = wxMessageBox(wxString::Format(_("File %s exists. Overwrite?"), 
-                        fn.c_str()),_("Save mask"), 
+                if (wxFile::Exists(fn))
+                {
+                    int d = wxMessageBox(wxString::Format(_("File %s exists. Overwrite?"),
+                        fn.c_str()), _("Save mask"),
                         wxYES_NO | wxICON_QUESTION);
-                if (d != wxYES) {
-                    return;
+                    if (d != wxYES) {
+                        return;
+                    }
                 }
-            }
+            };
             wxFileName filename = fn;
             std::ofstream maskFile(filename.GetFullPath().mb_str(HUGIN_CONV_FILENAME));
-            SaveMaskToStream(maskFile);
+            SaveMaskToStream(maskFile, m_pano->getImage(GetImgNr()).getSize(), m_currentMasks[m_MaskNr], GetImgNr());
             maskFile.close();
         };
     }
 };
 
-void MaskEditorPanel::SaveMaskToStream(std::ostream& stream)
-{
-    vigra::Size2D imageSize = m_pano->getImage(GetImgNr()).getSize();
-    stream << "# w" << imageSize.width() << " h" << imageSize.height() << std::endl;
-    HuginBase::MaskPolygon maskToWrite = m_currentMasks[m_MaskNr];
-    maskToWrite.printPolygonLine(stream,GetImgNr());
-};
-
-void MaskEditorPanel::LoadMaskFromStream(std::istream& stream,vigra::Size2D& imageSize,HuginBase::MaskPolygonVector &newMasks)
-{
-    while (stream.good()) 
-    {
-        std::string line;
-        std::getline(stream,line);
-        switch (line[0]) 
-        {
-            case '#':
-            {
-                unsigned int w;
-                if (getIntParam(w, line, "w"))
-                    imageSize.setWidth(w);
-                unsigned int h;
-                if (getIntParam(h, line, "h"))
-                    imageSize.setHeight(h);
-                break;
-            }
-            case 'k':
-            {
-                HuginBase::MaskPolygon newPolygon;
-                //Ignore image number set in mask
-                newPolygon.setImgNr(GetImgNr());
-                unsigned int param;
-                if (getIntParam(param,line,"t"))
-                {
-                    newPolygon.setMaskType((HuginBase::MaskPolygon::MaskType)param);
-                }
-                std::string format;
-                if (getPTParam(format,line,"p"))
-                {
-                    if(newPolygon.parsePolygonString(format)) {
-                        newMasks.push_back(newPolygon);
-                    } 
-                }
-                break;
-            }
-            default:
-            {
-                break;
-            }
-        }
-    }
-};
-
 void MaskEditorPanel::OnMaskLoad(wxCommandEvent &e)
 {
     if (GetImgNr()<UINT_MAX)
@@ -525,7 +482,7 @@ void MaskEditorPanel::OnMaskLoad(wxCommandEvent &e)
         std::ifstream in(filename.GetFullPath().mb_str(HUGIN_CONV_FILENAME));
         vigra::Size2D maskImageSize;
         HuginBase::MaskPolygonVector loadedMasks;
-        LoadMaskFromStream(in,maskImageSize,loadedMasks);
+        LoadMaskFromStream(in, maskImageSize, loadedMasks, GetImgNr());
         in.close();
         if(maskImageSize.area()==0 || loadedMasks.size()==0)
         {
@@ -548,7 +505,7 @@ void MaskEditorPanel::OnMaskLoad(wxCommandEvent &e)
         for(unsigned int i=0;i<loadedMasks.size();i++)
             m_currentMasks.push_back(loadedMasks[i]);
         // Update the pano with the imported masks
-        GlobalCmdHist::getInstance().addCommand(new PT::UpdateMaskForImgCmd(*m_pano,GetImgNr(),m_currentMasks));
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(new PanoCommand::UpdateMaskForImgCmd(*m_pano, GetImgNr(), m_currentMasks));
     }
 };
 
@@ -557,7 +514,7 @@ void MaskEditorPanel::OnMaskCopy(wxCommandEvent &e)
     if(GetImgNr()<UINT_MAX && m_MaskNr<UINT_MAX && m_maskMode)
     {
         std::ostringstream stream;
-        SaveMaskToStream(stream);
+        SaveMaskToStream(stream, m_pano->getImage(GetImgNr()).getSize(), m_currentMasks[m_MaskNr], GetImgNr());
         if (wxTheClipboard->Open())
         {
             // This data objects are held by the clipboard,
@@ -581,7 +538,7 @@ void MaskEditorPanel::OnMaskPaste(wxCommandEvent &e)
                 wxTextDataObject data;
                 wxTheClipboard->GetData(data);
                 std::istringstream stream(std::string(data.GetText().mb_str()));
-                LoadMaskFromStream(stream,maskImageSize,loadedMasks);
+                LoadMaskFromStream(stream, maskImageSize, loadedMasks, GetImgNr());
             }
             wxTheClipboard->Close();
             if(maskImageSize.area()==0 || loadedMasks.size()==0)
@@ -605,7 +562,7 @@ void MaskEditorPanel::OnMaskPaste(wxCommandEvent &e)
             for(unsigned int i=0;i<loadedMasks.size();i++)
                 m_currentMasks.push_back(loadedMasks[i]);
             // Update the pano with the imported masks
-            GlobalCmdHist::getInstance().addCommand(new PT::UpdateMaskForImgCmd(*m_pano,GetImgNr(),m_currentMasks));
+            PanoCommand::GlobalCmdHist::getInstance().addCommand(new PanoCommand::UpdateMaskForImgCmd(*m_pano, GetImgNr(), m_currentMasks));
         };
     };
 };
@@ -617,7 +574,7 @@ void MaskEditorPanel::OnMaskDelete(wxCommandEvent &e)
         HuginBase::MaskPolygonVector editedMasks=m_currentMasks;
         editedMasks.erase(editedMasks.begin()+m_MaskNr);
         //setMask(UINT_MAX);
-        GlobalCmdHist::getInstance().addCommand(new PT::UpdateMaskForImgCmd(*m_pano,GetImgNr(),editedMasks));
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(new PanoCommand::UpdateMaskForImgCmd(*m_pano, GetImgNr(), editedMasks));
     };
 };
 
@@ -782,7 +739,7 @@ void MaskEditorPanel::OnShowActiveMasks(wxCommandEvent &e)
 
 void MaskEditorPanel::DisplayCrop(int imgNr)
 {
-    const SrcPanoImage & img = m_pano->getImage(imgNr);
+    const HuginBase::SrcPanoImage & img = m_pano->getImage(imgNr);
     m_cropMode=img.getCropMode();
     m_cropRect=img.getCropRect();
     m_autoCenterCrop=img.getAutoCenterCrop();
@@ -803,17 +760,17 @@ void MaskEditorPanel::UpdateCrop(bool updateFromImgCtrl)
     {
         m_cropRect=m_editImg->getCrop();
     };
-    vector<SrcPanoImage> imgs;
-    for (UIntSet::iterator it = m_selectedImages.begin(); it != m_selectedImages.end(); ++it)
+    vector<HuginBase::SrcPanoImage> imgs;
+    for (HuginBase::UIntSet::iterator it = m_selectedImages.begin(); it != m_selectedImages.end(); ++it)
     {
-        SrcPanoImage img=m_pano->getSrcImage(*it);
+        HuginBase::SrcPanoImage img=m_pano->getSrcImage(*it);
         img.setCropRect(m_cropRect);
         img.setAutoCenterCrop(m_autoCenterCrop);
         imgs.push_back(img);
     };
 
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::UpdateSrcImagesCmd(*m_pano, m_selectedImages, imgs)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::UpdateSrcImagesCmd(*m_pano, m_selectedImages, imgs)
     );
 }
 
@@ -917,7 +874,7 @@ void MaskEditorPanel::OnResetButton(wxCommandEvent & e)
     m_cropRect.setUpperLeft(vigra::Point2D(0,0));
     m_cropRect.setLowerRight(vigra::Point2D(0,0));
     m_autoCenterCrop = true;
-    m_cropMode=SrcPanoImage::NO_CROP;
+    m_cropMode=HuginBase::SrcPanoImage::NO_CROP;
     UpdateCropDisplay();
     UpdateCrop();
 }
