@@ -18,8 +18,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -31,7 +31,7 @@
 // Headers
 //-----------------------------------------------------------------------------
 
-#include <PT/Panorama.h>
+#include <panodata/Panorama.h>
 #include <wx/clrpicker.h>
 #include "MaskImageCtrl.h"
 #include "ImagesList.h"
@@ -41,7 +41,7 @@
  *  This panel is used to create/change/edit masks
  *
  */
-class MaskEditorPanel : public wxPanel, public PT::PanoramaObserver
+class MaskEditorPanel : public wxPanel, public HuginBase::PanoramaObserver
 {
 public:
     /** ctor.
@@ -50,13 +50,13 @@ public:
 
     bool Create(wxWindow* parent, wxWindowID id = wxID_ANY, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxTAB_TRAVERSAL, const wxString& name = wxT("panel"));
 
-    void Init(PT::Panorama * pano);
+    void Init(HuginBase::Panorama * pano);
 
     /** dtor.
      */
     virtual ~MaskEditorPanel();
 
-    void SetPano(PT::Panorama * panorama)
+    void SetPano(HuginBase::Panorama * panorama)
         { m_pano = panorama; };
     /** sets the image, which is currently edited 
          @param imgNr the image which should be shown, use UINT_MAX for no image selected 
@@ -85,8 +85,8 @@ public:
     /** called when the panorama changes and we should
      *  update our display
      */
-    void panoramaChanged(PT::Panorama &pano);
-    void panoramaImagesChanged(PT::Panorama &pano, const PT::UIntSet & imgNr);
+    void panoramaChanged(HuginBase::Panorama &pano);
+    void panoramaImagesChanged(HuginBase::Panorama &pano, const HuginBase::UIntSet & imgNr);
 
     /** called when user selected an other image */
     void OnImageSelect(wxListEvent &e);
@@ -131,10 +131,6 @@ private:
     unsigned int GetSelectedMask();
     /** called, when column with of mask list box was changed */
     void OnColumnWidthChange( wxListEvent & e );
-    /** load the mask from stream */
-    void LoadMaskFromStream(std::istream& stream,vigra::Size2D& imageSize,HuginBase::MaskPolygonVector &newMasks);
-    /** save the mask into stream */
-    void SaveMaskToStream(std::ostream& stream);
     /** determines, if the image should be rotated for display */
     MaskImageCtrl::ImageRotation GetRot(const unsigned int imgNr);
     /** copies the crop information from the Panorama object to GUI */
@@ -155,7 +151,7 @@ private:
     wxNotebook *m_maskCropCtrl;
 
     // my data
-    PT::Panorama * m_pano;
+    HuginBase::Panorama * m_pano;
     // current masks vector
     HuginBase::MaskPolygonVector m_currentMasks;
     HuginBase::MaskPolygon::MaskType m_defaultMaskType;
