@@ -19,8 +19,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -29,12 +29,15 @@
 
 #include "panoinc.h"
 
-#include <PT/RandomPointSampler.h>
-#include <PT/PhotometricOptimizer.h>
-#include <PT/PTOptimise.h>
+#include <algorithms/point_sampler/PointSampler.h>
+#include <algorithms/optimizer/PhotometricOptimizer.h>
+#include <algorithms/basic/CalculateMeanExposure.h>
 
+#include <vigra_ext/Pyramid.h>
+#include <vigra_ext/openmp_vigra.h>
 #include "hugin/OptimizePhotometricPanel.h"
-#include "hugin/CommandHistory.h"
+#include "base_wx/CommandHistory.h"
+#include "base_wx/PanoCommand.h"
 #include "hugin/MainFrame.h"
 #include "base_wx/MyProgressDialog.h"
 #include "hugin/config_defaults.h"
@@ -44,8 +47,6 @@
 #include "hugin/PanoOperation.h"
 
 using namespace std;
-using namespace PT;
-using namespace PTools;
 using namespace hugin_utils;
 using namespace vigra;
 using namespace vigra_ext;
@@ -165,14 +166,14 @@ void OptimizePhotometricPanel::OnOptimizeButton(wxCommandEvent & e)
     runOptimizer(imgs);
 }
 
-void OptimizePhotometricPanel::panoramaChanged(PT::Panorama & pano)
+void OptimizePhotometricPanel::panoramaChanged(HuginBase::Panorama & pano)
 {
     m_images_tree->Enable(m_pano->getPhotometricOptimizerSwitch()==0);
     m_lens_tree->Enable(m_pano->getPhotometricOptimizerSwitch()==0);
 }
 
-void OptimizePhotometricPanel::panoramaImagesChanged(PT::Panorama &pano,
-                                          const PT::UIntSet & imgNr)
+void OptimizePhotometricPanel::panoramaImagesChanged(HuginBase::Panorama &pano,
+                                          const HuginBase::UIntSet & imgNr)
 {
     XRCCTRL(*this, "optimize_photo_panel_optimize", wxButton)->Enable(pano.getNrOfImages()>1);
     XRCCTRL(*this, "optimize_photo_panel_reset", wxButton)->Enable(pano.getNrOfImages()>0);    
@@ -193,7 +194,7 @@ void OptimizePhotometricPanel::runOptimizer(const UIntSet & imgs)
             HuginBase::ImageVariableGroup::IVE_RadialVigCorrCenterShift
         };
     // keep a list of commands needed to fix it:
-    std::vector<PT::PanoCommand *> commands;
+    std::vector<PanoCommand::PanoCommand *> commands;
     HuginBase::ConstStandardImageVariableGroups variable_groups(*m_pano);
     HuginBase::ConstImageVariableGroup & lenses = variable_groups.getLenses();
     for (size_t i = 0; i < lenses.getNumberOfParts(); i++)
@@ -209,7 +210,7 @@ void OptimizePhotometricPanel::runOptimizer(const UIntSet & imgs)
         };
         if (!links_needed.empty())
         {
-            commands.push_back(new PT::LinkLensVarsCmd(*m_pano, i, links_needed));
+            commands.push_back(new PanoCommand::LinkLensVarsCmd(*m_pano, i, links_needed));
         }
     }
     // if the list of commands is empty, all is good and we don't need a warning.
@@ -218,15 +219,15 @@ void OptimizePhotometricPanel::runOptimizer(const UIntSet & imgs)
         if (ok == wxYES)
         {
             // perform all the commands we stocked up earilier.
-            for (std::vector<PT::PanoCommand *>::iterator it = commands.begin(); it != commands.end(); ++it)
+            for (std::vector<PanoCommand::PanoCommand *>::iterator it = commands.begin(); it != commands.end(); ++it)
             {
-                GlobalCmdHist::getInstance().addCommand(*it);
+                PanoCommand::GlobalCmdHist::getInstance().addCommand(*it);
             }
         }
         else
         {
             // free all the commands, the user doesn't want them used.
-            for (std::vector<PT::PanoCommand *>::iterator it = commands.begin(); it != commands.end(); ++it)
+            for (std::vector<PanoCommand::PanoCommand *>::iterator it = commands.begin(); it != commands.end(); ++it)
             {
                 delete *it;
             }
@@ -234,9 +235,9 @@ void OptimizePhotometricPanel::runOptimizer(const UIntSet & imgs)
     }
 
     Panorama optPano = m_pano->getSubset(imgs);
-    PanoramaOptions opts = optPano.getOptions();
+    HuginBase::PanoramaOptions opts = optPano.getOptions();
 
-    OptimizeVector optvars;
+    HuginBase::OptimizeVector optvars;
     if(mode==0)
     {
         optvars = optPano.getOptimizeVector();
@@ -262,7 +263,7 @@ void OptimizePhotometricPanel::runOptimizer(const UIntSet & imgs)
         long nPoints = 200;
         wxConfigBase::Get()->Read(wxT("/OptimizePhotometric/nRandomPointsPerImage"), &nPoints , HUGIN_PHOTOMETRIC_OPTIMIZER_NRPOINTS);
 
-        ProgressReporterDialog progress(5.0, _("Photometric alignment"), _("Loading images"));
+        ProgressReporterDialog progress(optPano.getNrOfImages()+2, _("Photometric alignment"), _("Loading images"));
         progress.Show();
 
         nPoints = nPoints * optPano.getNrOfImages();
@@ -280,7 +281,7 @@ void OptimizePhotometricPanel::runOptimizer(const UIntSet & imgs)
             if (e->image8 && e->image8->width() > 0)
             {
                 reduceToNextLevel(*(e->image8), *img);
-                transformImage(vigra::srcImageRange(*img), vigra::destImage(*img),
+                vigra::omp::transformImage(vigra::srcImageRange(*img), vigra::destImage(*img),
                                 vigra::functor::Arg1()/vigra::functor::Param(255.0));
             }
             else
@@ -288,7 +289,7 @@ void OptimizePhotometricPanel::runOptimizer(const UIntSet & imgs)
                 if (e->image16 && e->image16->width() > 0)
                 {
                     reduceToNextLevel(*(e->image16), *img);
-                    transformImage(vigra::srcImageRange(*img), vigra::destImage(*img),
+                    vigra::omp::transformImage(vigra::srcImageRange(*img), vigra::destImage(*img),
                                    vigra::functor::Arg1()/vigra::functor::Param(65535.0));
                 }
                 else
@@ -297,58 +298,73 @@ void OptimizePhotometricPanel::runOptimizer(const UIntSet & imgs)
                 }
             };
             srcImgs.push_back(img);
-        }
-        bool randomPoints = true;
-        extractPoints(optPano, srcImgs, nPoints, randomPoints, progress, points);
+            if (!progress.updateDisplayValue())
+            {
+                // check if user pressed cancel
+                return;
+            };
+        }   
+        HuginBase::PointSampler::extractPoints(optPano, srcImgs, nPoints, true, &progress, points);
 
+        if (!progress.updateDisplayValue())
+        {
+            return;
+        };
         if (points.size() == 0)
         {
             wxMessageBox(_("Error: no overlapping points found, Photometric optimization aborted"), _("Error"));
             return;
         }
 
+        progress.setMaximum(0);
+        progress.updateDisplay(_("Optimize..."));
         try
         {
             if (mode != 0)
             {
                 // run automatic optimisation
                 // ensure that we have a valid anchor.
-                PanoramaOptions opts = optPano.getOptions();
+                HuginBase::PanoramaOptions opts = optPano.getOptions();
                 if (opts.colorReferenceImage >= optPano.getNrOfImages())
                 {
                     opts.colorReferenceImage = 0;
                     optPano.setOptions(opts);
                 }
-                PhotometricOptimizeMode optMode;
+                HuginBase::SmartPhotometricOptimizer::PhotometricOptimizeMode optMode;
                 switch(mode)
                 {
                     case (HuginBase::OPT_EXPOSURE | HuginBase::OPT_VIGNETTING | HuginBase::OPT_RESPONSE):
-                        optMode=OPT_PHOTOMETRIC_LDR;
+                        optMode = HuginBase::SmartPhotometricOptimizer::OPT_PHOTOMETRIC_LDR;
                         break;
                     case (HuginBase::OPT_EXPOSURE | HuginBase::OPT_VIGNETTING | HuginBase::OPT_RESPONSE | HuginBase::OPT_WHITEBALANCE):
-                        optMode=OPT_PHOTOMETRIC_LDR_WB;
+                        optMode = HuginBase::SmartPhotometricOptimizer::OPT_PHOTOMETRIC_LDR_WB;
                         break;
                     case (HuginBase::OPT_VIGNETTING | HuginBase::OPT_RESPONSE):
-                        optMode=OPT_PHOTOMETRIC_HDR;
+                        optMode = HuginBase::SmartPhotometricOptimizer::OPT_PHOTOMETRIC_HDR;
                         break;
                     case (HuginBase::OPT_WHITEBALANCE | HuginBase::OPT_VIGNETTING | HuginBase::OPT_RESPONSE):
-                        optMode=OPT_PHOTOMETRIC_HDR_WB;
+                        optMode = HuginBase::SmartPhotometricOptimizer::OPT_PHOTOMETRIC_HDR_WB;
                         break;
                     default:
                         //invalid combination
                         return;
                 };
-                smartOptimizePhotometric(optPano, optMode, points, progress, error);
+                HuginBase::SmartPhotometricOptimizer::smartOptimizePhotometric(optPano, optMode, points, &progress, error);
             }
             else
             {
                 // optimize selected parameters
-                optimizePhotometric(optPano, optvars, points, progress, error);
+                HuginBase::PhotometricOptimizer::optimizePhotometric(optPano, optvars, points, &progress, error);
+            }
+            if (progress.wasCancelled())
+            {
+                return;
             }
         }
         catch (std::exception & error)
         {
             wxMessageBox(_("Internal error during photometric optimization:\n") + wxString(error.what(), wxConvLocal), _("Internal error"));
+            return;
         }
     }
     wxYield();
@@ -361,15 +377,15 @@ void OptimizePhotometricPanel::runOptimizer(const UIntSet & imgs)
     {
         DEBUG_DEBUG("Applying vignetting corr");
         // TODO: merge into a single update command
-        const VariableMapVector & vars = optPano.getVariables();
-        GlobalCmdHist::getInstance().addCommand(
-                new PT::UpdateImagesVariablesCmd(*m_pano, imgs, vars)
+        const HuginBase::VariableMapVector & vars = optPano.getVariables();
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                new PanoCommand::UpdateImagesVariablesCmd(*m_pano, imgs, vars)
                                                );
         //now update panorama exposure value
-        PanoramaOptions opts = m_pano->getOptions();
-        opts.outputExposureValue = calcMeanExposure(*m_pano);
-        GlobalCmdHist::getInstance().addCommand(
-                new PT::SetPanoOptionsCmd(*m_pano, opts)
+        HuginBase::PanoramaOptions opts = m_pano->getOptions();
+        opts.outputExposureValue = HuginBase::CalculateMeanExposure::calcMeanExposure(*m_pano);
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                new PanoCommand::SetPanoOptionsCmd(*m_pano, opts)
                                                );
     }
 }
@@ -394,10 +410,10 @@ void OptimizePhotometricPanel::OnClose(wxCloseEvent& event)
 void OptimizePhotometricPanel::OnReset(wxCommandEvent& e)
 {
     PanoOperation::ResetOperation op(PanoOperation::ResetOperation::RESET_DIALOG_PHOTOMETRICS);
-    PT::PanoCommand* cmd=op.GetCommand(this,*m_pano, m_images_tree->GetSelectedImages(), MainFrame::Get()->GetGuiLevel());
+    PanoCommand::PanoCommand* cmd=op.GetCommand(this,*m_pano, m_images_tree->GetSelectedImages(), MainFrame::Get()->GetGuiLevel());
     if(cmd!=NULL)
     {
-        GlobalCmdHist::getInstance().addCommand(cmd);
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(cmd);
     };
 };
 
