@@ -17,14 +17,15 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
 #include "hugin/PanoOperation.h"
 #include "hugin/config_defaults.h"
-#include "hugin/wxPanoCommand.h"
+#include "base_wx/PanoCommand.h"
+#include "base_wx/wxPanoCommand.h"
 #include "huginapp/ImageCache.h"
 #include "base_wx/MyProgressDialog.h"
 #include "base_wx/PTWXDlg.h"
@@ -37,6 +38,7 @@
 #include "base_wx/wxLensDB.h"
 #include "hugin/ResetDialog.h"
 #include "hugin/MainFrame.h"
+#include <vigra_ext/openmp_vigra.h>
 
 using namespace HuginBase;
 
@@ -48,12 +50,12 @@ wxString PanoOperation::GetLabel()
     return wxEmptyString;
 };
 
-bool PanoOperation::IsEnabled(PT::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
+bool PanoOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
 {
     return true;
 };
 
-PT::PanoCommand* PanoOperation::GetCommand(wxWindow* parent, PT::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
+PanoCommand::PanoCommand* PanoOperation::GetCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
 {
     //remember gui level, only used by some PanoOperation's
     m_guiLevel=guiLevel;
@@ -67,12 +69,12 @@ PT::PanoCommand* PanoOperation::GetCommand(wxWindow* parent, PT::Panorama& pano,
     };
 };
 
-bool PanoSingleImageOperation::IsEnabled(PT::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
+bool PanoSingleImageOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
 {
     return images.size()==1;
 };
 
-bool PanoMultiImageOperation::IsEnabled(PT::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
+bool PanoMultiImageOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
 {
     return images.size()>0;
 };
@@ -90,7 +92,7 @@ bool AddImageDialog(wxWindow* parent, std::vector<std::string>& files)
     wxFileDialog dlg(parent,_("Add images"),
                      path, wxT(""),
                      HUGIN_WX_FILE_IMG_FILTER,
-                     wxFD_OPEN | wxFD_MULTIPLE, wxDefaultPosition);
+                     wxFD_OPEN | wxFD_MULTIPLE | wxFD_FILE_MUST_EXIST | wxFD_PREVIEW, wxDefaultPosition);
     dlg.SetDirectory(path);
 
     // remember the image extension
@@ -168,14 +170,14 @@ wxString AddImageOperation::GetLabel()
     return _("Add individual images...");
 };
 
-PT::PanoCommand* AddImageOperation::GetInternalCommand(wxWindow* parent, PT::Panorama& pano, HuginBase::UIntSet images)
+PanoCommand::PanoCommand* AddImageOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
     std::vector<std::string> files;
     if(AddImageDialog(parent, files))
     {
         if(files.size()>0)
         {
-            return new PT::wxAddImagesCmd(pano,files);
+            return new PanoCommand::wxAddImagesCmd(pano,files);
         };
     };
     return NULL;
@@ -257,7 +259,7 @@ wxString AddImagesSeriesOperation::GetLabel()
     return _("Add time-series of images...");
 };
 
-PT::PanoCommand* AddImagesSeriesOperation::GetInternalCommand(wxWindow* parent, PT::Panorama& pano, HuginBase::UIntSet images)
+PanoCommand::PanoCommand* AddImagesSeriesOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
     //load image if pano contains no images
     std::vector<std::string> files;
@@ -362,7 +364,7 @@ PT::PanoCommand* AddImagesSeriesOperation::GetInternalCommand(wxWindow* parent,
         sortbytime spred(timeMap);
         sort(files.begin(), files.end(), spred);
         // Load all of the named files.
-        return new PT::wxAddImagesCmd(pano,files);
+        return new PanoCommand::wxAddImagesCmd(pano,files);
     }
     else
     {
@@ -383,14 +385,14 @@ wxString RemoveImageOperation::GetLabel()
     return _("Remove selected image(s)");
 };
 
-PT::PanoCommand* RemoveImageOperation::GetInternalCommand(wxWindow* parent, PT::Panorama& pano, HuginBase::UIntSet images)
+PanoCommand::PanoCommand* RemoveImageOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
     //remove images from cache
     for (UIntSet::iterator it = images.begin(); it != images.end(); ++it)
     {
         ImageCache::getInstance().removeImage(pano.getImage(*it).getFilename());
     }
-    return new PT::RemoveImagesCmd(pano, images);
+    return new PanoCommand::RemoveImagesCmd(pano, images);
 };
 
 wxString ChangeAnchorImageOperation::GetLabel()
@@ -398,11 +400,11 @@ wxString ChangeAnchorImageOperation::GetLabel()
     return _("Anchor this image for position");
 };
 
-PT::PanoCommand* ChangeAnchorImageOperation::GetInternalCommand(wxWindow* parent, PT::Panorama& pano, HuginBase::UIntSet images)
+PanoCommand::PanoCommand* ChangeAnchorImageOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
     PanoramaOptions opt = pano.getOptions();
     opt.optimizeReferenceImage = *(images.begin());
-    return new PT::SetPanoOptionsCmd(pano,opt);
+    return new PanoCommand::SetPanoOptionsCmd(pano,opt);
 };
 
 wxString ChangeColorAnchorImageOperation::GetLabel()
@@ -410,7 +412,7 @@ wxString ChangeColorAnchorImageOperation::GetLabel()
     return _("Anchor this image for exposure");
 };
 
-PT::PanoCommand* ChangeColorAnchorImageOperation::GetInternalCommand(wxWindow* parent, PT::Panorama& pano, HuginBase::UIntSet images)
+PanoCommand::PanoCommand* ChangeColorAnchorImageOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
     PanoramaOptions opt = pano.getOptions();
     opt.colorReferenceImage = *(images.begin());
@@ -419,10 +421,10 @@ PT::PanoCommand* ChangeColorAnchorImageOperation::GetInternalCommand(wxWindow* p
     {
         opt.colorCorrection = (PanoramaOptions::ColorCorrection) 1;
     }
-    return new PT::SetPanoOptionsCmd(pano, opt);
+    return new PanoCommand::SetPanoOptionsCmd(pano, opt);
 };
 
-bool NewLensOperation::IsEnabled(PT::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
+bool NewLensOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
 {
     if(pano.getNrOfImages()==0 || images.size()==0)
     {
@@ -440,12 +442,12 @@ wxString NewLensOperation::GetLabel()
     return _("New lens");
 };
 
-PT::PanoCommand* NewLensOperation::GetInternalCommand(wxWindow* parent, PT::Panorama& pano, HuginBase::UIntSet images)
+PanoCommand::PanoCommand* NewLensOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
-    return new PT::NewPartCmd(pano, images, HuginBase::StandardImageVariableGroups::getLensVariables());
+    return new PanoCommand::NewPartCmd(pano, images, HuginBase::StandardImageVariableGroups::getLensVariables());
 };
 
-bool ChangeLensOperation::IsEnabled(PT::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
+bool ChangeLensOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
 {
     if(pano.getNrOfImages()==0 || images.size()==0)
     {
@@ -464,7 +466,7 @@ wxString ChangeLensOperation::GetLabel()
     return _("Change lens...");
 };
 
-PT::PanoCommand* ChangeLensOperation::GetInternalCommand(wxWindow* parent, PT::Panorama& pano, HuginBase::UIntSet images)
+PanoCommand::PanoCommand* ChangeLensOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
     HuginBase::StandardImageVariableGroups variable_groups(pano);
     long nr = wxGetNumberFromUser(
@@ -476,7 +478,7 @@ PT::PanoCommand* ChangeLensOperation::GetInternalCommand(wxWindow* parent, PT::P
     if (nr >= 0)
     {
         // user accepted
-        return new PT::ChangePartNumberCmd(pano, images, nr, HuginBase::StandardImageVariableGroups::getLensVariables());
+        return new PanoCommand::ChangePartNumberCmd(pano, images, nr, HuginBase::StandardImageVariableGroups::getLensVariables());
     }
     else
     {
@@ -484,16 +486,16 @@ PT::PanoCommand* ChangeLensOperation::GetInternalCommand(wxWindow* parent, PT::P
     };
 };
 
-LoadLensOperation::LoadLensOperation(bool fromLensfunDB)
+LoadLensOperation::LoadLensOperation(bool fromDatabase)
 {
-    m_fromLensfunDB=fromLensfunDB;
+    m_fromDatabase = fromDatabase;
 };
 
 wxString LoadLensOperation::GetLabel()
 {
-    if(m_fromLensfunDB)
+    if (m_fromDatabase)
     {
-        return _("Load lens from Lensfun database");
+        return _("Load lens from lens database");
     }
     else
     {
@@ -501,7 +503,7 @@ wxString LoadLensOperation::GetLabel()
     };
 };
 
-PT::PanoCommand* LoadLensOperation::GetInternalCommand(wxWindow* parent, PT::Panorama& pano, HuginBase::UIntSet images)
+PanoCommand::PanoCommand* LoadLensOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
     HuginBase::StandardImageVariableGroups variable_groups(pano);
     if(images.size()==1)
@@ -533,9 +535,9 @@ PT::PanoCommand* LoadLensOperation::GetInternalCommand(wxWindow* parent, PT::Pan
             return NULL;
         };
     };
-    PT::PanoCommand* cmd=NULL;
+    PanoCommand::PanoCommand* cmd=NULL;
     bool isLoaded=false;
-    if(m_fromLensfunDB)
+    if (m_fromDatabase)
     {
         isLoaded=ApplyLensDBParameters(parent,&pano,images,cmd);
     }
@@ -553,43 +555,33 @@ PT::PanoCommand* LoadLensOperation::GetInternalCommand(wxWindow* parent, PT::Pan
     }
 };
 
-SaveLensOperation::SaveLensOperation(int lensInfo)
+SaveLensOperation::SaveLensOperation(bool toDatabase)
 {
-    m_lensInfo=lensInfo;
+    m_database = toDatabase;
 };
 
 wxString SaveLensOperation::GetLabel()
 {
-    switch(m_lensInfo)
+    if (m_database)
     {
-        case 0:
-            return _("Save lens to ini file");
-            break;
-        case 1:
-            return _("Save lens parameters to lensfun database");
-            break;
-        case 2:
-            return _("Save camera parameters to lensfun database");
-            break;
+        return _("Save lens parameters to lens database");
     }
-    return wxEmptyString;
+    else
+    {
+        return _("Save lens to ini file");
+    };
 };
 
-PT::PanoCommand* SaveLensOperation::GetInternalCommand(wxWindow* parent, PT::Panorama& pano, HuginBase::UIntSet images)
+PanoCommand::PanoCommand* SaveLensOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
     unsigned int imgNr = *(images.begin());
-    switch(m_lensInfo)
+    if (m_database)
     {
-        case 1:
-            SaveLensParameters(parent,pano.getImage(imgNr));
-            break;
-        case 2:
-            SaveCameraCropFactor(parent,pano.getImage(imgNr));
-            break;
-        case 0:
-        default:
-            SaveLensParametersToIni(parent, &pano, images);
-            break;
+        SaveLensParameters(parent, pano.getImage(imgNr));
+    }
+    else
+    {
+        SaveLensParametersToIni(parent, &pano, images);
     };
     return NULL;
 };
@@ -599,12 +591,12 @@ wxString RemoveControlPointsOperation::GetLabel()
     return _("Remove control points");
 };
 
-bool RemoveControlPointsOperation::IsEnabled(PT::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
+bool RemoveControlPointsOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
 {
     return pano.getNrOfImages()>0 && pano.getNrOfCtrlPoints()>0;
 };
 
-PT::PanoCommand* RemoveControlPointsOperation::GetInternalCommand(wxWindow* parent, PT::Panorama& pano, HuginBase::UIntSet images)
+PanoCommand::PanoCommand* RemoveControlPointsOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
     UIntSet selImages;
     if(images.size()==0)
@@ -641,7 +633,7 @@ PT::PanoCommand* RemoveControlPointsOperation::GetInternalCommand(wxWindow* pare
                         wxICON_QUESTION | wxYES_NO);
     if (r == wxYES)
     {
-        return new PT::RemoveCtrlPointsCmd(pano, cpsToDelete );
+        return new PanoCommand::RemoveCtrlPointsCmd(pano, cpsToDelete );
     }
     else
     {
@@ -654,18 +646,17 @@ wxString CleanControlPointsOperation::GetLabel()
     return _("Clean control points");
 };
 
-bool CleanControlPointsOperation::IsEnabled(PT::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
+bool CleanControlPointsOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
 {
     return pano.getNrOfCtrlPoints()>2;
 };
 
-PT::PanoCommand* CleanControlPointsOperation::GetInternalCommand(wxWindow* parent, PT::Panorama& pano, HuginBase::UIntSet images)
+PanoCommand::PanoCommand* CleanControlPointsOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
     deregisterPTWXDlgFcn();
-    // work around a flaw in wxProgresDialog that results in incorrect layout
-    // by pre-allocting sufficient horizontal space
-    ProgressReporterDialog progress(2, _("Cleaning Control points"), _("Checking pairwise")+wxString((wxChar)' ',10),parent, wxPD_AUTO_HIDE | wxPD_APP_MODAL | wxPD_ELAPSED_TIME);
-    UIntSet removedCPs=getCPoutsideLimit_pair(pano,2.0);
+    ProgressReporterDialog progress(0, _("Cleaning Control points"), _("Checking pairwise"), parent);
+
+    UIntSet removedCPs=getCPoutsideLimit_pair(pano, progress, 2.0);
 
     //create a copy to work with
     //we copy remaining control points to new pano object for running second step
@@ -690,11 +681,14 @@ PT::PanoCommand* CleanControlPointsOperation::GetInternalCommand(wxWindow* paren
     createCPGraph(newPano, graph);
     CPComponents comps;
     int n=findCPComponents(graph, comps);
-    progress.increaseProgress(1, std::wstring(wxString(_("Checking whole project")).wc_str(wxConvLocal)));
+    if (!progress.updateDisplayValue(_("Checking whole project")))
+    {
+        return NULL;
+    }
     if (n <= 1)
     {
         //now run the second step
-        UIntSet removedCP2=getCPoutsideLimit(newPano,2.0);
+        UIntSet removedCP2=getCPoutsideLimit(newPano, 2.0);
         if(removedCP2.size()>0)
         {
             for(UIntSet::const_iterator it=removedCP2.begin();it!=removedCP2.end();++it)
@@ -703,12 +697,15 @@ PT::PanoCommand* CleanControlPointsOperation::GetInternalCommand(wxWindow* paren
             };
         };
     }
-    progress.increaseProgress(1, std::wstring(wxString(_("Finished cleaning")).wc_str(wxConvLocal)));
-    registerPTWXDlgFcn(MainFrame::Get());
-    if(removedCPs.size()>0)
+    registerPTWXDlgFcn();
+    if (!progress.updateDisplay(_("Finished cleaning")))
     {
-        wxMessageBox(wxString::Format(_("Removed %lu control points"), removedCPs.size()), _("Cleaning"),wxOK|wxICON_INFORMATION,parent);
-        return new PT::RemoveCtrlPointsCmd(pano,removedCPs);
+        return NULL;
+    }
+    if (removedCPs.size()>0)
+    {
+        wxMessageBox(wxString::Format(_("Removed %lu control points"), (unsigned long int)removedCPs.size()), _("Cleaning"), wxOK | wxICON_INFORMATION, parent);
+        return new PanoCommand::RemoveCtrlPointsCmd(pano,removedCPs);
     };
     return NULL;
 };
@@ -718,16 +715,14 @@ wxString CelesteOperation::GetLabel()
     return _("Remove control points on clouds");
 };
 
-PT::PanoCommand* CelesteOperation::GetInternalCommand(wxWindow* parent, PT::Panorama& pano, HuginBase::UIntSet images)
+PanoCommand::PanoCommand* CelesteOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
-    ProgressReporterDialog progress(images.size()+2, _("Running Celeste"), _("Running Celeste"),parent);
-    MainFrame::Get()->SetStatusText(_("searching for cloud-like control points..."),0);
-    progress.increaseProgress(1.0, std::wstring(wxString(_("Loading model file")).wc_str(wxConvLocal)));
+    ProgressReporterDialog progress(images.size() + 2, _("Running Celeste"), _("Running Celeste"), parent);
+    progress.updateDisplay(_("Loading model file"));
 
     struct celeste::svm_model* model=MainFrame::Get()->GetSVMModel();
-    if(model==NULL)
+    if (model == NULL || !progress.updateDisplay(_("Loading images")))
     {
-        MainFrame::Get()->SetStatusText(wxT(""),0);
         return NULL;
     };
 
@@ -749,7 +744,10 @@ PT::PanoCommand* CelesteOperation::GetInternalCommand(wxWindow* parent, PT::Pano
         HuginBase::CPointVector cps=pano.getCtrlPointsVectorForImage(*it);
         if(cps.size()==0)
         {
-            progress.increaseProgress(1.0, std::wstring(wxString(_("Running Celeste")).wc_str(wxConvLocal)));
+            if (!progress.updateDisplayValue())
+            {
+                return NULL;
+            };
             continue;
         };
         ImageCache::EntryPtr img=ImageCache::getInstance().getImage(pano.getImage(*it).getFilename());
@@ -757,16 +755,24 @@ PT::PanoCommand* CelesteOperation::GetInternalCommand(wxWindow* parent, PT::Pano
         if(img->image16->width()>0)
         {
             in.resize(img->image16->size());
-            vigra::copyImage(srcImageRange(*(img->image16)),destImage(in));
+            vigra::omp::copyImage(srcImageRange(*(img->image16)),destImage(in));
         }
         else
         {
             ImageCache::ImageCacheRGB8Ptr im8=img->get8BitImage();
             in.resize(im8->size());
-            vigra::transformImage(srcImageRange(*im8),destImage(in),vigra::functor::Arg1()*vigra::functor::Param(65535/255));
+            vigra::omp::transformImage(srcImageRange(*im8),destImage(in),vigra::functor::Arg1()*vigra::functor::Param(65535/255));
+        };
+        if (!progress.updateDisplay(_("Running Celeste")))
+        {
+            return NULL;
         };
         UIntSet cloudCP=celeste::getCelesteControlPoints(model,in,cps,radius,threshold,800);
         in.resize(0,0);
+        if (!progress.updateDisplay())
+        {
+            return NULL;
+        };
         if(cloudCP.size()>0)
         {
             for(UIntSet::const_iterator it2=cloudCP.begin();it2!=cloudCP.end();it2++)
@@ -774,19 +780,23 @@ PT::PanoCommand* CelesteOperation::GetInternalCommand(wxWindow* parent, PT::Pano
                 cpsToRemove.insert(*it2);
             };
         };
-        progress.increaseProgress(1.0, std::wstring(wxString(_("Running Celeste")).wc_str(wxConvLocal)));
+        if (!progress.updateDisplayValue(_("Loading images")))
+        {
+            return NULL;
+        };
     };
 
-    progress.increaseProgress(1.0, std::wstring(wxString(_("Running Celeste")).wc_str(wxConvLocal)));
-    if(cpsToRemove.size()>0)
+    if (!progress.updateDisplayValue())
+    {
+        return NULL;
+    }
+    if (cpsToRemove.size()>0)
     {
         wxMessageBox(wxString::Format(_("Removed %lu control points"), (unsigned long int) cpsToRemove.size()), _("Celeste result"),wxOK|wxICON_INFORMATION);
-        MainFrame::Get()->SetStatusText(wxT(""),0);
-        return new PT::RemoveCtrlPointsCmd(pano,cpsToRemove);
+        return new PanoCommand::RemoveCtrlPointsCmd(pano,cpsToRemove);
     }
     else
     {
-        MainFrame::Get()->SetStatusText(wxT(""),0);
         return NULL;
     };
 };
@@ -835,7 +845,7 @@ wxString ResetOperation::GetLabel()
     return wxEmptyString;
 };
 
-bool ResetOperation::IsEnabled(PT::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
+bool ResetOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
 {
     switch(m_resetMode)
     {
@@ -847,7 +857,7 @@ bool ResetOperation::IsEnabled(PT::Panorama& pano, HuginBase::UIntSet images, Gu
     };
 };
 
-PT::PanoCommand* ResetOperation::GetInternalCommand(wxWindow* parent, PT::Panorama& pano, HuginBase::UIntSet images)
+PanoCommand::PanoCommand* ResetOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
     if(m_resetMode==RESET_DIALOG || m_resetMode==RESET_DIALOG_LENS || m_resetMode==RESET_DIALOG_PHOTOMETRICS)
     {
@@ -1009,14 +1019,14 @@ PT::PanoCommand* ResetOperation::GetInternalCommand(wxWindow* parent, PT::Panora
         };
         vars.push_back(ImgVars);
     };
-    std::vector<PT::PanoCommand *> reset_commands;
+    std::vector<PanoCommand::PanoCommand *> reset_commands;
     if (needs_unlink_exposure)
     {
         std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> variables;
         variables.insert(HuginBase::ImageVariableGroup::IVE_ExposureValue);
         
         reset_commands.push_back(
-                new ChangePartImagesLinkingCmd(
+                new PanoCommand::ChangePartImagesLinkingCmd(
                             pano,
                             images,
                             variables,
@@ -1030,7 +1040,7 @@ PT::PanoCommand* ResetOperation::GetInternalCommand(wxWindow* parent, PT::Panora
         variables.insert(HuginBase::ImageVariableGroup::IVE_WhiteBalanceRed);
         
         reset_commands.push_back(
-                new ChangePartImagesLinkingCmd(
+                new PanoCommand::ChangePartImagesLinkingCmd(
                             pano,
                             images,
                             variables,
@@ -1044,7 +1054,7 @@ PT::PanoCommand* ResetOperation::GetInternalCommand(wxWindow* parent, PT::Panora
         variables.insert(HuginBase::ImageVariableGroup::IVE_WhiteBalanceBlue);
         
         reset_commands.push_back(
-                new ChangePartImagesLinkingCmd(
+                new PanoCommand::ChangePartImagesLinkingCmd(
                             pano,
                             images,
                             variables,
@@ -1053,14 +1063,14 @@ PT::PanoCommand* ResetOperation::GetInternalCommand(wxWindow* parent, PT::Panora
                 );
     }
     reset_commands.push_back(
-                            new PT::UpdateImagesVariablesCmd(pano, images, vars)
+                            new PanoCommand::UpdateImagesVariablesCmd(pano, images, vars)
                                            );
     if(m_resetExposure>0)
     {
         //reset panorama output exposure value
-        reset_commands.push_back(new PT::ResetToMeanExposure(pano));
+        reset_commands.push_back(new PanoCommand::ResetToMeanExposure(pano));
     };
-    return new PT::CombinedPanoCommand(pano, reset_commands);
+    return new PanoCommand::CombinedPanoCommand(pano, reset_commands);
 };
 
 bool ResetOperation::ShowDialog(wxWindow* parent)
@@ -1136,7 +1146,7 @@ bool ResetOperation::ShowDialog(wxWindow* parent)
     };
 };
 
-bool NewStackOperation::IsEnabled(PT::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
+bool NewStackOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
 {
     if(pano.getNrOfImages()==0 || images.size()==0)
     {
@@ -1154,12 +1164,12 @@ wxString NewStackOperation::GetLabel()
     return _("New stack");
 };
 
-PT::PanoCommand* NewStackOperation::GetInternalCommand(wxWindow* parent, PT::Panorama& pano, HuginBase::UIntSet images)
+PanoCommand::PanoCommand* NewStackOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
-    return new PT::NewPartCmd(pano, images, HuginBase::StandardImageVariableGroups::getStackVariables());
+    return new PanoCommand::NewPartCmd(pano, images, HuginBase::StandardImageVariableGroups::getStackVariables());
 };
 
-bool ChangeStackOperation::IsEnabled(PT::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
+bool ChangeStackOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
 {
     if(pano.getNrOfImages()==0 || images.size()==0)
     {
@@ -1178,7 +1188,7 @@ wxString ChangeStackOperation::GetLabel()
     return _("Change stack...");
 };
 
-PT::PanoCommand* ChangeStackOperation::GetInternalCommand(wxWindow* parent, PT::Panorama& pano, HuginBase::UIntSet images)
+PanoCommand::PanoCommand* ChangeStackOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
     HuginBase::StandardImageVariableGroups variable_groups(pano);
     long nr = wxGetNumberFromUser(
@@ -1190,7 +1200,7 @@ PT::PanoCommand* ChangeStackOperation::GetInternalCommand(wxWindow* parent, PT::
     if (nr >= 0)
     {
         // user accepted
-        return new PT::ChangePartNumberCmd(pano, images, nr, HuginBase::StandardImageVariableGroups::getStackVariables());
+        return new PanoCommand::ChangePartNumberCmd(pano, images, nr, HuginBase::StandardImageVariableGroups::getStackVariables());
     }
     else
     {
@@ -1198,7 +1208,7 @@ PT::PanoCommand* ChangeStackOperation::GetInternalCommand(wxWindow* parent, PT::
     };
 };
 
-bool AssignStacksOperation::IsEnabled(PT::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
+bool AssignStacksOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
 {
     return pano.getNrOfImages()>1;
 };
@@ -1208,19 +1218,32 @@ wxString AssignStacksOperation::GetLabel()
     return _("Set stack size...");
 };
 
-PT::PanoCommand* AssignStacksOperation::GetInternalCommand(wxWindow* parent, PT::Panorama& pano, HuginBase::UIntSet images)
+PanoCommand::PanoCommand* AssignStacksOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
-    long stackSize = wxGetNumberFromUser(
-                            _("Enter image count per stack"),
-                            _("stack size"),
-                            _("Images per stack"), 3, 1,
-                            pano.getNrOfImages()
-                                 );
+    wxConfigBase* cfg = wxConfigBase::Get();
+    wxDialog dlg;
+    wxXmlResource::Get()->LoadDialog(&dlg, parent, wxT("stack_size_dialog"));
+    wxSpinCtrl* stackSpin = XRCCTRL(dlg, "stack_size_spinctrl", wxSpinCtrl);
+    stackSpin->SetRange(1, pano.getNrOfImages());
+    size_t oldStackSize = cfg->Read(wxT("/StackDialog/StackSize"), 3);
+    oldStackSize = std::min(oldStackSize, pano.getNrOfImages());
+    stackSpin->SetValue(oldStackSize);
+    wxCheckBox* linkCheckBox = XRCCTRL(dlg, "stack_size_link_checkbox", wxCheckBox);
+    linkCheckBox->SetValue(cfg->Read(wxT("/StackDialog/LinkPosition"), true));
+    if (dlg.ShowModal() != wxID_OK)
+    {
+        // user has canceled dialog
+        return NULL;
+    };
+    long stackSize = stackSpin->GetValue();
+    bool linkPosition = linkCheckBox->IsChecked();
+    cfg->Write(wxT("/StackDialog/StackSize"), stackSize);
+    cfg->Write(wxT("/StackDialog/LinkPosition"), linkPosition);
     if(stackSize<0)
     {
         return NULL;
     };
-    std::vector<PT::PanoCommand *> commands;
+    std::vector<PanoCommand::PanoCommand *> commands;
     HuginBase::StandardImageVariableGroups variable_groups(pano);
     if(variable_groups.getStacks().getNumberOfParts()<pano.getNrOfImages())
     {
@@ -1229,7 +1252,7 @@ PT::PanoCommand* AssignStacksOperation::GetInternalCommand(wxWindow* parent, PT:
         {
             UIntSet imgs;
             imgs.insert(i);
-            commands.push_back(new PT::NewPartCmd(pano, imgs, HuginBase::StandardImageVariableGroups::getStackVariables()));
+            commands.push_back(new PanoCommand::NewPartCmd(pano, imgs, HuginBase::StandardImageVariableGroups::getStackVariables()));
         };
     };
 
@@ -1245,11 +1268,28 @@ PT::PanoCommand* AssignStacksOperation::GetInternalCommand(wxWindow* parent, PT:
                 imgs.insert(imgNr);
                 imgNr++;
             };
-            commands.push_back(new PT::ChangePartNumberCmd(pano, imgs, stackNr, HuginBase::StandardImageVariableGroups::getStackVariables()));
+            commands.push_back(new PanoCommand::ChangePartNumberCmd(pano, imgs, stackNr, HuginBase::StandardImageVariableGroups::getStackVariables()));
             stackNr++;
         };
     };
-    return new PT::CombinedPanoCommand(pano, commands);
+
+    if (!linkPosition && stackSize > 1)
+    {
+        // unlink image position
+        UIntSet imgs;
+        fill_set(imgs, 0, pano.getNrOfImages() - 1);
+        std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> variables;
+        variables.insert(HuginBase::ImageVariableGroup::IVE_Yaw);
+        variables.insert(HuginBase::ImageVariableGroup::IVE_Pitch);
+        variables.insert(HuginBase::ImageVariableGroup::IVE_Roll);
+        variables.insert(HuginBase::ImageVariableGroup::IVE_X);
+        variables.insert(HuginBase::ImageVariableGroup::IVE_Y);
+        variables.insert(HuginBase::ImageVariableGroup::IVE_Z);
+        variables.insert(HuginBase::ImageVariableGroup::IVE_TranslationPlaneYaw);
+        variables.insert(HuginBase::ImageVariableGroup::IVE_TranslationPlanePitch);
+        commands.push_back(new PanoCommand::ChangePartImagesLinkingCmd(pano, imgs, variables, false, HuginBase::StandardImageVariableGroups::getStackVariables()));
+    };
+    return new PanoCommand::CombinedPanoCommand(pano, commands);
 };
 
 static PanoOperationVector PanoOpImages;
@@ -1295,9 +1335,8 @@ void GeneratePanoOperationVector()
     PanoOpLens.push_back(new ChangeLensOperation());
     PanoOpLens.push_back(new LoadLensOperation(false));
     PanoOpLens.push_back(new LoadLensOperation(true));
-    PanoOpLens.push_back(new SaveLensOperation(0));
-    PanoOpLens.push_back(new SaveLensOperation(1));
-    PanoOpLens.push_back(new SaveLensOperation(2));
+    PanoOpLens.push_back(new SaveLensOperation(false));
+    PanoOpLens.push_back(new SaveLensOperation(true));
 
     PanoOpStacks.push_back(new NewStackOperation());
     PanoOpStacks.push_back(new ChangeStackOperation());
