@@ -20,8 +20,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -31,18 +31,18 @@
 #include "panoinc_WX.h"
 #include "panoinc.h"
 #include "base_wx/platform.h"
-
+#include "base_wx/PanoCommand.h"
 
 #include <hugin/config_defaults.h>
 
-#include "PT/Stitcher.h"
+#include "nona/Stitcher.h"
 #include "base_wx/wxPlatform.h"
 
 extern "C" {
 #include <pano13/queryfeature.h>
 }
 
-#include "hugin/CommandHistory.h"
+#include "base_wx/CommandHistory.h"
 #include "hugin/CPImageCtrl.h"
 #include "hugin/CPImagesComboBox.h"
 #include "hugin/PanoPanel.h"
@@ -54,11 +54,15 @@ extern "C" {
 #include "hugin/config_defaults.h"
 #include "base_wx/platform.h"
 #include "base_wx/huginConfig.h"
+#include "base_wx/LensTools.h"
 #include "algorithms/basic/LayerStacks.h"
+#include "algorithms/basic/CalculateOptimalScale.h"
+#include "algorithms/basic/CalculateOptimalROI.h"
+#include "algorithms/nona/FitPanorama.h"
+#include "lensdb/LensDB.h"
 
 #define WX_BROKEN_SIZER_UNKNOWN
 
-using namespace PT;
 using namespace std;
 using namespace hugin_utils;
 
@@ -216,6 +220,7 @@ bool PanoPanel::Create(wxWindow *parent, wxWindowID id, const wxPoint& pos, cons
     DEBUG_ASSERT(m_HDRMergeChoice);
     m_BlenderChoice = XRCCTRL(*this, "pano_choice_blender", wxChoice);
     DEBUG_ASSERT(m_BlenderChoice);
+    FillBlenderList(m_BlenderChoice);
 
     m_StitchButton = XRCCTRL(*this, "pano_button_stitch", wxButton);
     DEBUG_ASSERT(m_StitchButton);
@@ -284,7 +289,7 @@ PanoPanel::~PanoPanel(void)
 }
 
 
-void PanoPanel::panoramaChanged (PT::Panorama &pano)
+void PanoPanel::panoramaChanged (HuginBase::Panorama &pano)
 {
     DEBUG_TRACE("");
 
@@ -294,7 +299,7 @@ void PanoPanel::panoramaChanged (PT::Panorama &pano)
     const bool hasStacks = false;
 #endif
 
-    PanoramaOptions opt = pano.getOptions();
+    HuginBase::PanoramaOptions opt = pano.getOptions();
 
     // update all options for dialog and notebook tab
     UpdateDisplay(opt,hasStacks);
@@ -303,10 +308,10 @@ void PanoPanel::panoramaChanged (PT::Panorama &pano)
 }
 
 
-bool PanoPanel::StackCheck(PT::Panorama &pano)
+bool PanoPanel::StackCheck(HuginBase::Panorama &pano)
 {
     DEBUG_TRACE("");
-    PanoramaOptions opt = pano.getOptions();
+    HuginBase::PanoramaOptions opt = pano.getOptions();
 
     // Determine if there are stacks in the pano.
     UIntSet activeImages = pano.getActiveImages();
@@ -353,7 +358,7 @@ bool PanoPanel::StackCheck(PT::Panorama &pano)
 }
 
 
-void PanoPanel::UpdateDisplay(const PanoramaOptions & opt, const bool hasStacks)
+void PanoPanel::UpdateDisplay(const HuginBase::PanoramaOptions & opt, const bool hasStacks)
 {
 
 //    m_HFOVSpin->SetRange(1,opt.getMaxHFOV());
@@ -467,7 +472,9 @@ void PanoPanel::UpdateDisplay(const PanoramaOptions & opt, const bool hasStacks)
 
     m_BlenderChoice->Enable(blenderEnabled);
     m_BlenderChoice->Show(m_guiLevel>GUI_SIMPLE);
-    XRCCTRL(*this, "pano_button_blender_opts", wxButton)->Enable(blenderEnabled);
+    // select correct blending mechanism
+    SelectListValue(m_BlenderChoice, opt.blendMode);
+    XRCCTRL(*this, "pano_button_blender_opts", wxButton)->Enable(blenderEnabled && opt.blendMode!=HuginBase::PanoramaOptions::INTERNAL_BLEND);
     XRCCTRL(*this, "pano_button_blender_opts", wxButton)->Show(m_guiLevel>GUI_SIMPLE);
     XRCCTRL(*this, "pano_text_blender", wxStaticText)->Enable(blenderEnabled);
     XRCCTRL(*this, "pano_text_blender", wxStaticText)->Show(m_guiLevel>GUI_SIMPLE);
@@ -584,16 +591,16 @@ void PanoPanel::UpdateDisplay(const PanoramaOptions & opt, const bool hasStacks)
 void PanoPanel::ProjectionChanged ( wxCommandEvent & e )
 {
     if (updatesDisabled) return;
-    PanoramaOptions opt = pano->getOptions();
+    HuginBase::PanoramaOptions opt = pano->getOptions();
 //    PanoramaOptions::ProjectionFormat oldP = opt.getProjection();
 
-    PanoramaOptions::ProjectionFormat newP = (PanoramaOptions::ProjectionFormat) m_ProjectionChoice->GetSelection();
+    HuginBase::PanoramaOptions::ProjectionFormat newP = (HuginBase::PanoramaOptions::ProjectionFormat) m_ProjectionChoice->GetSelection();
 //    int w = opt.getWidth();
 //    int h = opt.getHeight();
     opt.setProjection(newP);
 
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetPanoOptionsCmd( *pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetPanoOptionsCmd( *pano, opt )
         );
     DEBUG_DEBUG ("Projection changed: "  << newP)
 }
@@ -601,7 +608,7 @@ void PanoPanel::ProjectionChanged ( wxCommandEvent & e )
 void PanoPanel::HFOVChanged ( wxCommandEvent & e )
 {
     if (updatesDisabled) return;
-    PanoramaOptions opt = pano->getOptions();
+    HuginBase::PanoramaOptions opt = pano->getOptions();
 
 
     wxString text = m_HFOVText->GetValue();
@@ -623,8 +630,8 @@ void PanoPanel::HFOVChanged ( wxCommandEvent & e )
     }
     opt.setHFOV(hfov);
     // recalculate panorama height...
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetPanoOptionsCmd( *pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetPanoOptionsCmd( *pano, opt )
         );
 
     DEBUG_INFO ( "new hfov: " << hfov )
@@ -633,7 +640,7 @@ void PanoPanel::HFOVChanged ( wxCommandEvent & e )
 void PanoPanel::VFOVChanged ( wxCommandEvent & e )
 {
     if (updatesDisabled) return;
-    PanoramaOptions opt = pano->getOptions();
+    HuginBase::PanoramaOptions opt = pano->getOptions();
 
     wxString text = m_VFOVText->GetValue();
     DEBUG_INFO ("VFOV = " << text.mb_str(wxConvLocal) );
@@ -655,8 +662,8 @@ void PanoPanel::VFOVChanged ( wxCommandEvent & e )
     }
     opt.setVFOV(vfov);
     // recalculate panorama height...
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetPanoOptionsCmd( *pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetPanoOptionsCmd( *pano, opt )
         );
 
     DEBUG_INFO ( "new vfov: " << vfov )
@@ -673,7 +680,7 @@ void PanoPanel::VFOVChanged ( wxCommandEvent & e )
     if (vfov != opt.getVFOV()) {
         opt.setVFOV(vfov);
         GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( pano, opt )
+            new PanoCommand::SetPanoOptionsCmd( pano, opt )
             );
         DEBUG_INFO ( "new vfov: " << vfov << " => height: " << opt.getHeight() );
     } else {
@@ -685,13 +692,13 @@ void PanoPanel::VFOVChanged ( wxCommandEvent & e )
 void PanoPanel::WidthChanged ( wxCommandEvent & e )
 {
     if (updatesDisabled) return;
-    PanoramaOptions opt = pano->getOptions();
+    HuginBase::PanoramaOptions opt = pano->getOptions();
     long nWidth;
     if (m_WidthTxt->GetValue().ToLong(&nWidth)) {
         if (nWidth <= 0) return;
         opt.setWidth((unsigned int) nWidth, m_keepViewOnResize);
-        GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( *pano, opt )
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( *pano, opt )
             );
         DEBUG_INFO(nWidth );
     } else {
@@ -702,13 +709,13 @@ void PanoPanel::WidthChanged ( wxCommandEvent & e )
 void PanoPanel::HeightChanged ( wxCommandEvent & e )
 {
     if (updatesDisabled) return;
-    PanoramaOptions opt = pano->getOptions();
+    HuginBase::PanoramaOptions opt = pano->getOptions();
     long nHeight;
     if (m_HeightTxt->GetValue().ToLong(&nHeight)) {
         if(nHeight <= 0) return;
         opt.setHeight((unsigned int) nHeight);
-        GlobalCmdHist::getInstance().addCommand(
-                new PT::SetPanoOptionsCmd( *pano, opt )
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                new PanoCommand::SetPanoOptionsCmd( *pano, opt )
                                                );
         DEBUG_INFO(nHeight);
     } else {
@@ -719,7 +726,7 @@ void PanoPanel::HeightChanged ( wxCommandEvent & e )
 void PanoPanel::ROIChanged ( wxCommandEvent & e )
 {
     if (updatesDisabled) return;
-    PanoramaOptions opt = pano->getOptions();
+    HuginBase::PanoramaOptions opt = pano->getOptions();
     long left, right, top, bottom;
     if (!m_ROITopTxt->GetValue().ToLong(&top)) {
         wxLogError(_("Top needs to be an integer bigger than 0"));
@@ -752,8 +759,8 @@ void PanoPanel::ROIChanged ( wxCommandEvent & e )
         return;
     }
 
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( *pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( *pano, opt )
                                            );
 }
 
@@ -775,22 +782,22 @@ void PanoPanel::RemapperChanged(wxCommandEvent & e)
     int remapper = m_RemapperChoice->GetSelection();
     DEBUG_DEBUG("changing remapper to " << remapper);
 
-    PanoramaOptions opt = pano->getOptions();
+    HuginBase::PanoramaOptions opt = pano->getOptions();
     if (remapper == 1) {
-        opt.remapper = PanoramaOptions::PTMENDER;
+        opt.remapper = HuginBase::PanoramaOptions::PTMENDER;
     } else {
-        opt.remapper = PanoramaOptions::NONA;
+        opt.remapper = HuginBase::PanoramaOptions::NONA;
     }
 
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( *pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( *pano, opt )
             );
 }
 
 void PanoPanel::OnRemapperOptions(wxCommandEvent & e)
 {
-    PanoramaOptions opt = pano->getOptions();
-    if (opt.remapper == PanoramaOptions::NONA) {
+    HuginBase::PanoramaOptions opt = pano->getOptions();
+    if (opt.remapper == HuginBase::PanoramaOptions::NONA) {
         wxDialog dlg;
         wxXmlResource::Get()->LoadDialog(&dlg, this, wxT("nona_options_dialog"));
         wxChoice * interpol_choice = XRCCTRL(dlg, "nona_choice_interpolator", wxChoice);
@@ -805,8 +812,8 @@ void PanoPanel::OnRemapperOptions(wxCommandEvent & e)
                 opt.interpolator = (vigra_ext::Interpolator) interpol;
             }
             opt.tiff_saveROI = cropped_cb->GetValue();
-            GlobalCmdHist::getInstance().addCommand(
-                new PT::SetPanoOptionsCmd( *pano, opt )
+            PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                new PanoCommand::SetPanoOptionsCmd( *pano, opt )
                 );
         }
     } else {
@@ -816,35 +823,18 @@ void PanoPanel::OnRemapperOptions(wxCommandEvent & e)
 
 void PanoPanel::BlenderChanged(wxCommandEvent & e)
 {
-    int blender = m_BlenderChoice->GetSelection();
-    DEBUG_DEBUG("changing stitcher to " << blender);
+    HuginBase::PanoramaOptions opt = pano->getOptions();
+    opt.blendMode = static_cast<HuginBase::PanoramaOptions::BlendingMechanism>(GetSelectedValue(m_BlenderChoice));
 
-    PanoramaOptions opt = pano->getOptions();
-    switch (blender) {
-        case 1:
-            opt.blendMode = PanoramaOptions::NO_BLEND;
-            break;
-        case 2:
-            opt.blendMode = PanoramaOptions::PTMASKER_BLEND;
-            break;
-        case 3:
-            opt.blendMode = PanoramaOptions::PTBLENDER_BLEND;
-            break;
-        default:
-        case 0:
-            opt.blendMode = PanoramaOptions::ENBLEND_BLEND;
-            break;
-    }
-
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( *pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( *pano, opt )
             );
 }
 
 void PanoPanel::OnBlenderOptions(wxCommandEvent & e)
 {
-    PanoramaOptions opt = pano->getOptions();
-    if (opt.blendMode == PanoramaOptions::ENBLEND_BLEND) {
+    HuginBase::PanoramaOptions opt = pano->getOptions();
+    if (opt.blendMode == HuginBase::PanoramaOptions::ENBLEND_BLEND) {
         wxDialog dlg;
         wxXmlResource::Get()->LoadDialog(&dlg, this, wxT("enblend_options_dialog"));
         wxTextCtrl * enblend_opts_text = XRCCTRL(dlg, "blender_arguments_text", wxTextCtrl);
@@ -859,8 +849,8 @@ void PanoPanel::OnBlenderOptions(wxCommandEvent & e)
             {
                 opt.enblendOptions = wxConfigBase::Get()->Read(wxT("Enblend/Args"),wxT(HUGIN_ENBLEND_ARGS)).mb_str(wxConvLocal);
             };
-            GlobalCmdHist::getInstance().addCommand(
-                new PT::SetPanoOptionsCmd( *pano, opt )
+            PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                new PanoCommand::SetPanoOptionsCmd( *pano, opt )
                 );
         }
     } else {
@@ -876,7 +866,7 @@ void PanoPanel::FusionChanged(wxCommandEvent & e)
 
 void PanoPanel::OnFusionOptions(wxCommandEvent & e)
 {
-    PanoramaOptions opt = pano->getOptions();
+    HuginBase::PanoramaOptions opt = pano->getOptions();
     wxDialog dlg;
     wxXmlResource::Get()->LoadDialog(&dlg, this, wxT("enfuse_options_dialog"));
     wxTextCtrl * enfuse_opts_text = XRCCTRL(dlg, "enfuse_arguments_text", wxTextCtrl);
@@ -891,8 +881,8 @@ void PanoPanel::OnFusionOptions(wxCommandEvent & e)
         {
             opt.enfuseOptions = wxConfigBase::Get()->Read(wxT("Enfuse/Args"),wxT(HUGIN_ENFUSE_ARGS)).mb_str(wxConvLocal);
         };
-        GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( *pano, opt )
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( *pano, opt )
             );
     }
 }
@@ -906,15 +896,15 @@ void PanoPanel::HDRMergeChanged(wxCommandEvent & e)
 
 void PanoPanel::OnHDRMergeOptions(wxCommandEvent & e)
 {
-    PanoramaOptions opt = pano->getOptions();
-    if (opt.hdrMergeMode == PanoramaOptions::HDRMERGE_AVERAGE) {
+    HuginBase::PanoramaOptions opt = pano->getOptions();
+    if (opt.hdrMergeMode == HuginBase::PanoramaOptions::HDRMERGE_AVERAGE) {
         HDRMergeOptionsDialog dlg(this);
         dlg.SetCommandLineArgument(wxString(opt.hdrmergeOptions.c_str(), wxConvLocal));
         if (dlg.ShowModal() == wxOK) 
         {
             opt.hdrmergeOptions=dlg.GetCommandLineArgument().mb_str(wxConvLocal);
-            GlobalCmdHist::getInstance().addCommand(
-                new PT::SetPanoOptionsCmd( *pano, opt )
+            PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                new PanoCommand::SetPanoOptionsCmd( *pano, opt )
                 );
         }
     } else {
@@ -929,19 +919,19 @@ void PanoPanel::DoCalcFOV(wxCommandEvent & e)
     DEBUG_TRACE("");
     if (pano->getActiveImages().size() == 0) return;
 
-    double hfov, height;
-    pano->fitPano(hfov, height);
-    PanoramaOptions opt = pano->getOptions();
-    opt.setHFOV(hfov);
-    opt.setHeight(roundi(height));
+    HuginBase::PanoramaOptions opt = pano->getOptions();
+    HuginBase::CalculateFitPanorama fitPano(*pano);
+    fitPano.run();
+    opt.setHFOV(fitPano.getResultHorizontalFOV());
+    opt.setHeight(roundi(fitPano.getResultHeight()));
 
     DEBUG_INFO ( "hfov: " << opt.getHFOV() << "  w: " << opt.getWidth() << " h: " << opt.getHeight() << "  => vfov: " << opt.getVFOV()  << "  before update");
 
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetPanoOptionsCmd( *pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetPanoOptionsCmd( *pano, opt )
         );
 
-    PanoramaOptions opt2 = pano->getOptions();
+    HuginBase::PanoramaOptions opt2 = pano->getOptions();
     DEBUG_INFO ( "hfov: " << opt2.getHFOV() << "  w: " << opt2.getWidth() << " h: " << opt2.getHeight() << "  => vfov: " << opt2.getVFOV()  << "  after update");
 
 }
@@ -950,12 +940,13 @@ void PanoPanel::DoCalcOptimalWidth(wxCommandEvent & e)
 {
     if (pano->getActiveImages().size() == 0) return;
 
-    PanoramaOptions opt = pano->getOptions();
-    unsigned width = pano->calcOptimalWidth();
+    HuginBase::PanoramaOptions opt = pano->getOptions();
+    unsigned width = hugin_utils::roundi(HuginBase::CalculateOptimalScale::calcOptimalScale(*pano) * opt.getWidth());
+
     if (width > 0) {
         opt.setWidth( width );
-        GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( *pano, opt )
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( *pano, opt )
             );
     }
     DEBUG_INFO ( "new optimal width: " << opt.getWidth() );
@@ -971,25 +962,23 @@ void PanoPanel::DoCalcOptimalROI(wxCommandEvent & e)
     };
 
     vigra::Rect2D newROI;
-    vigra::Size2D newSize;
     {
-        ProgressReporterDialog progress(2, _("Autocrop"), _("Calculating optimal crop"), this, wxPD_AUTO_HIDE | wxPD_APP_MODAL | wxPD_ELAPSED_TIME);
-        progress.increaseProgress(1);
-        progress.Pulse();
-        pano->calcOptimalROI(newROI,newSize);
+        ProgressReporterDialog progress(0, _("Autocrop"), _("Calculating optimal crop"), this);
+        HuginBase::CalculateOptimalROI cropPano(*pano, &progress);
+        cropPano.run();
+        if (cropPano.hasRunSuccessfully())
+        {
+            newROI = cropPano.getResultOptimalROI();
+        };
     };
-#ifdef __WXMSW__
-    //try to workaround an issue that the main window lost it focus after wxProgressDialog is destroyed
-    MainFrame::Get()->Raise();
-#endif
 
-    PanoramaOptions opt = pano->getOptions();
+    HuginBase::PanoramaOptions opt = pano->getOptions();
     //set the ROI - fail if the right/bottom is zero, meaning all zero
     if(newROI.right() != 0 && newROI.bottom() != 0)
     {
         opt.setROI(newROI);
-        GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( *pano, opt )
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( *pano, opt )
             );
     };
 };
@@ -1025,7 +1014,7 @@ void PanoPanel::DoStitch()
     DEBUG_DEBUG("tmp PTO file: " << (const char *)currentPTOfn.mb_str(wxConvLocal));
     // copy is not enough, need to adjust image path names...
     ofstream script(currentPTOfn.mb_str(HUGIN_CONV_FILENAME));
-    PT::UIntSet all;
+    HuginBase::UIntSet all;
     if (pano->getNrOfImages() > 0) {
         fill_set(all, 0, pano->getNrOfImages()-1);
     }
@@ -1078,6 +1067,27 @@ void PanoPanel::DoStitch()
         if(dlg.ShowModal()!=wxID_OK)
             return;
     };
+    wxFileName prefix(dlg.GetPath());
+    while (!prefix.IsDirWritable())
+    {
+        wxMessageBox(wxString::Format(_("You have no permissions to write in folder \"%s\".\nPlease select another folder for the final output."), prefix.GetPath().c_str()),
+#ifdef __WXMSW__
+            wxT("Hugin"),
+#else
+            wxT(""),
+#endif
+            wxOK | wxICON_INFORMATION);
+        if (dlg.ShowModal() != wxID_OK)
+        {
+            return;
+        };
+        prefix = dlg.GetPath();
+    };
+    // check free space
+    if (!CheckFreeSpace(prefix.GetPath()))
+    {
+        return;
+    };
 
     wxString switches(wxT(" --delete -o "));
     if(wxConfigBase::Get()->Read(wxT("/Processor/overwrite"), HUGIN_PROCESSOR_OVERWRITE) == 1)
@@ -1097,7 +1107,7 @@ void PanoPanel::DoStitch()
 #else
     wxExecute(command);
 #endif
-
+    HuginBase::LensDB::SaveLensDataFromPano(*pano);
 }
 
 void PanoPanel::DoSendToBatch()
@@ -1123,10 +1133,6 @@ void PanoPanel::DoSendToBatch()
     {
         switches += wxT("-b ");
     }
-    if (wxConfigBase::Get()->Read(wxT("/Processor/parallel"), HUGIN_PROCESSOR_PARALLEL) != 0)
-    {
-        switches += wxT("-p ");
-    }
     if (wxConfigBase::Get()->Read(wxT("/Processor/overwrite"), HUGIN_PROCESSOR_OVERWRITE) != 0)
     {
         switches += wxT("-o ");
@@ -1181,6 +1187,27 @@ void PanoPanel::DoSendToBatch()
             if(dlg.ShowModal()!=wxID_OK)
                 return;
         };
+        wxFileName prefix(dlg.GetPath());
+        while (!prefix.IsDirWritable())
+        {
+            wxMessageBox(wxString::Format(_("You have no permissions to write in folder \"%s\".\nPlease select another folder for the final output."), prefix.GetPath().c_str()),
+#ifdef __WXMSW__
+                wxT("Hugin"),
+#else
+                wxT(""),
+#endif
+                wxOK | wxICON_INFORMATION);
+            if (dlg.ShowModal() != wxID_OK)
+            {
+                return;
+            };
+            prefix = dlg.GetPath();
+        };
+        // check free space
+        if (!CheckFreeSpace(prefix.GetPath()))
+        {
+            return;
+        };
 
 #if defined __WXMAC__ && defined MAC_SELF_CONTAINED_BUNDLE
 		wxString cmd = MacGetPathToMainExecutableFileOfRegisteredBundle(CFSTR("net.sourceforge.hugin.PTBatcherGUI"));
@@ -1201,13 +1228,10 @@ void PanoPanel::DoSendToBatch()
 		}
 		
 #else
-#ifdef __WINDOWS__
-        wxString huginPath = getExePath(wxGetApp().argv[0])+wxFileName::GetPathSeparator();
-#else
-        wxString huginPath = wxT(""); //we call the batch processor directly without path on linux
-#endif
-        wxExecute(huginPath+wxT("PTBatcherGUI")+switches+wxQuoteFilename(projectFile)+wxT(" ")+wxQuoteFilename(dlg.GetPath()));
+        const wxFileName exePath(wxStandardPaths::Get().GetExecutablePath());
+        wxExecute(exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR) + wxT("PTBatcherGUI ")+switches+wxQuoteFilename(projectFile)+wxT(" ")+wxQuoteFilename(dlg.GetPath()));
 #endif
+        HuginBase::LensDB::SaveLensDataFromPano(*pano);
     }
 };
 
@@ -1249,7 +1273,7 @@ void PanoPanel::FileFormatChanged(wxCommandEvent & e)
     int fmt = m_FileFormatChoice->GetSelection();
     DEBUG_DEBUG("changing file format to " << fmt);
 
-    PanoramaOptions opt = pano->getOptions();
+    HuginBase::PanoramaOptions opt = pano->getOptions();
     switch (fmt) {
         case 1:
             opt.outputImageType ="jpg";
@@ -1266,8 +1290,8 @@ void PanoPanel::FileFormatChanged(wxCommandEvent & e)
             break;
     }
 
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( *pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( *pano, opt )
             );
 }
 
@@ -1277,7 +1301,7 @@ void PanoPanel::HDRFileFormatChanged(wxCommandEvent & e)
     int fmt = m_HDRFileFormatChoice->GetSelection();
     DEBUG_DEBUG("changing file format to " << fmt);
 
-    PanoramaOptions opt = pano->getOptions();
+    HuginBase::PanoramaOptions opt = pano->getOptions();
     switch (fmt) {
         case 1:
             opt.outputImageTypeHDR ="tif";
@@ -1288,28 +1312,28 @@ void PanoPanel::HDRFileFormatChanged(wxCommandEvent & e)
             break;
     }
 
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( *pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( *pano, opt )
             );
 }
 
 void PanoPanel::OnJPEGQualityText(wxCommandEvent & e)
 {
-    PanoramaOptions opt = pano->getOptions();
+    HuginBase::PanoramaOptions opt = pano->getOptions();
     long l = 100;
     m_FileFormatJPEGQualityText->GetValue().ToLong(&l);
     if (l < 0) l=1;
     if (l > 100) l=100;
     DEBUG_DEBUG("Setting jpeg quality to " << l);
     opt.quality = l;
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( *pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( *pano, opt )
             );
 }
 
 void PanoPanel::OnNormalTIFFCompression(wxCommandEvent & e)
 {
-    PanoramaOptions opt = pano->getOptions();
+    HuginBase::PanoramaOptions opt = pano->getOptions();
     switch(e.GetSelection()) {
         case 0:
         default:
@@ -1329,14 +1353,14 @@ void PanoPanel::OnNormalTIFFCompression(wxCommandEvent & e)
             opt.tiffCompression = "DEFLATE";
             break;
     }
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( *pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( *pano, opt )
             );
 }
 
 void PanoPanel::OnHDRTIFFCompression(wxCommandEvent & e)
 {
-    PanoramaOptions opt = pano->getOptions();
+    HuginBase::PanoramaOptions opt = pano->getOptions();
     switch(e.GetSelection()) {
         case 0:
         default:
@@ -1352,15 +1376,15 @@ void PanoPanel::OnHDRTIFFCompression(wxCommandEvent & e)
             opt.outputImageTypeHDRCompression = "DEFLATE";
             break;
     }
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( *pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( *pano, opt )
             );
 }
 
 void PanoPanel::OnOutputFilesChanged(wxCommandEvent & e)
 {
     int id = e.GetId();
-    PanoramaOptions opts = pano->getOptions();
+    HuginBase::PanoramaOptions opts = pano->getOptions();
 
     if (id == XRCID("pano_cb_ldr_output_blended") ) {
         opts.outputLDRBlended = e.IsChecked();
@@ -1384,17 +1408,30 @@ void PanoPanel::OnOutputFilesChanged(wxCommandEvent & e)
         opts.outputHDRLayers = e.IsChecked();
     }
     
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( *pano, opts )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( *pano, opts )
         );
 }
 
 bool PanoPanel::CheckGoodSize()
 {
     vigra::Rect2D cropped_region = pano->getOptions().getROI();
+    wxString message;
     unsigned long long int area = ((unsigned long int) cropped_region.width()) * ((unsigned long int) cropped_region.height());
     // Argh, more than half a gigapixel!
     if (area > 500000000)
+    {
+        message = wxString::Format(_("The panorama you are trying to stitch is %.1f gigapixels.\nIf this is too big, reduce the panorama Canvas Size and the cropped region and stitch from the Stitcher tab. Stitching a panorama this size could take a long time and a large amount of memory."),
+            area / 1000000000.0);
+    }
+    else
+    {
+        if (cropped_region.width() > 32700 || cropped_region.height() > 32700)
+        {
+            message = _("The width or the height of the final panorama exceeds 32700 pixel.\nSome programs have problems to open or display such big images.\n\nIf this is too big, reduce the panorama Canvas Size or the cropped region.");
+        };
+    };
+    if (!message.empty())
     {
         // Tell the user the stitch will be really big, and give them a
         // chance to reduce the size.
@@ -1407,9 +1444,7 @@ bool PanoPanel::CheckGoodSize()
                 wxT(""),
 #endif
                 wxICON_EXCLAMATION | wxYES_NO);
-        dialog.SetExtendedMessage(
-                wxString::Format(_("The panorama you are trying to stitch is %.1f gigapixels.\nIf this is too big, reduce the panorama Canvas Size and the cropped region and stitch from the Stitcher tab. Stitching a panorama this size could take a long time and a large amount of memory."),
-                        area / 1000000000.0));
+        dialog.SetExtendedMessage(message);
         dialog.SetYesNoLabels(_("Stitch anyway"), _("Let me fix that"));
 #else // replacement for old wxWidgets versions.
         // wxMessageDialog derives from wxDialog, but I don't understand
@@ -1426,9 +1461,9 @@ bool PanoPanel::CheckGoodSize()
         /** @todo (Possibly) make a dialog manually with properly labelled
          * buttons.
          */
-        wxMessageDialog dialog(this,
-                wxString::Format(_("Are you sure you want to stitch such a large panorama?\n\nThe panorama you are trying to stitch is %.1f gigapixels.\nIf this is too big, reduce the panorama Canvas Size and the cropped region and stitch from the Stitcher tab. Stitching a panorama this size could take a long time and a large amount of memory."),
-                        area / 1000000000.0),
+        message.Prepend(wxT("\n\n"));
+        message=_("Are you sure you want to stitch such a large panorama?")+message;
+        wxMessageDialog dialog(this, message,
 #ifdef _WINDOWS
                 _("Hugin"),
 #else
@@ -1445,7 +1480,12 @@ bool PanoPanel::CheckGoodSize()
                 break;
             default:
                 // bring the user towards the approptiate controls.
-                MainFrame::Get()->ShowStitcherTab();
+                MainFrame* mainframe = MainFrame::Get();
+                if (!mainframe->IsShown())
+                {
+                    mainframe->Show();
+                }
+                mainframe->ShowStitcherTab();
                 return false;
         }
     }
@@ -1469,6 +1509,50 @@ bool PanoPanel::CheckHasImages()
     return images.size()>0;
 };
 
+bool PanoPanel::CheckFreeSpace(const wxString& folder)
+{
+    wxLongLong freeSpace;
+    if (wxGetDiskSpace(folder, NULL, &freeSpace))
+    {
+        // 4 channels, 16 bit per channel, assuming the we need the 10 fold space for all temporary space
+        if (pano->getOptions().getROI().area() * 80 > freeSpace)
+        {
+#if wxCHECK_VERSION(2,9,0)
+            wxMessageDialog dialog(this,
+                wxString::Format(_("The folder \"%s\" has only %.1f MiB free. This is not enough for stitching the current panorama. Decrease the output size or select another output folder.\nAre you sure that you still want to stitch it?"), folder.c_str(), freeSpace / 1048576.0f),
+#ifdef _WINDOWS
+                _("Hugin"),
+#else
+                wxT(""),
+#endif
+                wxICON_EXCLAMATION | wxYES_NO);
+            dialog.SetYesNoLabels(_("Stitch anyway"), _("Let me fix that"));
+#else // replacement for old wxWidgets versions.
+            wxMessageDialog dialog(this,
+                wxString::Format(_("The folder \"%s\" has only %.1f MiB free. This is not enough for stitching the current panorama. Decrease the output size or select another output folder.\nAre you sure that you still want to stitch it?"), folder.c_str(), freeSpace / 1048576.0f),
+#ifdef _WINDOWS
+                _("Hugin"),
+#else
+                wxT(""),
+#endif
+                wxICON_EXCLAMATION | wxYES_NO);
+#endif
+            if (dialog.ShowModal() == wxID_NO)
+            {
+                // bring the user towards the approptiate controls.
+                MainFrame* mainframe = MainFrame::Get();
+                if (!mainframe->IsShown())
+                {
+                    mainframe->Show();
+                }
+                mainframe->ShowStitcherTab();
+                return false;
+            };
+        };
+    };
+    return true;
+};
+
 void PanoPanel::SetGuiLevel(GuiLevel newGuiLevel)
 {
     m_guiLevel=newGuiLevel;
