@@ -19,8 +19,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -31,6 +31,7 @@
 #include "panoinc.h"
 
 #include "base_wx/wxPlatform.h"
+#include "base_wx/LensTools.h"
 
 #include "hugin/huginApp.h"
 #include "hugin/config_defaults.h"
@@ -65,6 +66,8 @@ BEGIN_EVENT_TABLE(PreferencesDialog, wxDialog)
     EVT_BUTTON(XRCID("prefs_enblend_enfuse_select"), PreferencesDialog::OnEnfuseExe)
     EVT_BUTTON(XRCID("pref_exiftool_argfile_choose"), PreferencesDialog::OnExifArgfile)
     EVT_BUTTON(XRCID("pref_exiftool_argfile_edit"), PreferencesDialog::OnExifArgfileEdit)
+    EVT_BUTTON(XRCID("pref_exiftool_argfile2_choose"), PreferencesDialog::OnExifArgfile2)
+    EVT_BUTTON(XRCID("pref_exiftool_argfile2_edit"), PreferencesDialog::OnExifArgfileEdit2)
     EVT_CHECKBOX(XRCID("pref_exiftool_metadata"), PreferencesDialog::OnExifTool)
     EVT_CHECKBOX(XRCID("prefs_ft_RotationSearch"), PreferencesDialog::OnRotationCheckBox)
     EVT_CHECKBOX(XRCID("prefs_enblend_Custom"), PreferencesDialog::OnCustomEnblend)
@@ -122,10 +125,10 @@ PreferencesDialog::PreferencesDialog(wxWindow* parent)
     lp = new long;
     *lp = wxLANGUAGE_BULGARIAN;
     lang_choice->Append(_("Bulgarian"), lp);
+    */
     lp = new long;
     *lp = wxLANGUAGE_CATALAN;
     lang_choice->Append(_("Catalan"), lp);
-    */
     lp = new long;
     *lp = wxLANGUAGE_CHINESE_SIMPLIFIED;
     lang_choice->Append(_("Chinese (Simplified)"), lp);
@@ -195,13 +198,27 @@ PreferencesDialog::PreferencesDialog(wxWindow* parent)
     lp = new long;
     *lp = wxLANGUAGE_FINNISH;
     lang_choice->Append(_("Finnish"), lp);
+    lp = new long;
+    *lp = wxLANGUAGE_VALENCIAN;
+    lang_choice->Append(_("Valencian (Southern Catalan)"), lp);
     lang_choice->SetSelection(0);
 
+    // default blender settings
+    FillBlenderList(XRCCTRL(*this, "pref_default_blender", wxChoice));
+
 #if wxCHECK_VERSION(2,9,1)
     wxStaticText* preview=XRCCTRL(*this, "prefs_project_filename_preview", wxStaticText);
     preview->SetWindowStyle(preview->GetWindowStyle() | wxST_ELLIPSIZE_START);
     preview=XRCCTRL(*this, "prefs_output_filename_preview", wxStaticText);
     preview->SetWindowStyle(preview->GetWindowStyle() | wxST_ELLIPSIZE_START);
+#endif
+#if wxCHECK_VERSION(2,9,3)
+    // enable auto-completion
+    XRCCTRL(*this, "prefs_misc_tempdir", wxTextCtrl)->AutoCompleteDirectories();
+    XRCCTRL(*this, "pref_exiftool_argfile", wxTextCtrl)->AutoCompleteFileNames();
+    XRCCTRL(*this, "pref_exiftool_argfile2", wxTextCtrl)->AutoCompleteFileNames();
+    XRCCTRL(*this, "prefs_enblend_EnblendExe", wxTextCtrl)->AutoCompleteFileNames();
+    XRCCTRL(*this, "prefs_enblend_EnfuseExe", wxTextCtrl)->AutoCompleteFileNames();
 #endif
     // load autopano settings
     wxConfigBase* cfg = wxConfigBase::Get();
@@ -241,11 +258,6 @@ PreferencesDialog::PreferencesDialog(wxWindow* parent)
     cfg->Write(wxT("/Enfuse/Custom"), HUGIN_ENFUSE_EXE_CUSTOM);
 #endif
 
-#ifndef EXIFTOOL_GPANO_SUPPORT
-    //hide GPano tags checkbox if ExifTool is too old
-    XRCCTRL(*this, "pref_exiftool_gpano", wxCheckBox)->Hide();
-#endif
-
     RestoreFramePosition(this, wxT("PreferencesDialog"));
 }
 
@@ -386,7 +398,7 @@ void PreferencesDialog::OnExifArgfileEdit(wxCommandEvent & e)
         wxFileDialog dlg(this,_("Select new ExifTool argfile"),
             wxStandardPaths::Get().GetUserConfigDir(), wxT(""),
                          _("ExifTool Argfiles (*.arg)|*.arg|All Files(*)|*"),
-                         wxFD_SAVE, wxDefaultPosition);
+                         wxFD_SAVE | wxFD_OVERWRITE_PROMPT, wxDefaultPosition);
         if (dlg.ShowModal() != wxID_OK)
         {
             return;
@@ -410,13 +422,87 @@ void PreferencesDialog::OnExifArgfileEdit(wxCommandEvent & e)
     };
 };
 
+void PreferencesDialog::OnExifArgfile2(wxCommandEvent & e)
+{
+    wxFileDialog dlg(this, _("Select ExifTool argfile"),
+        wxT(""), XRCCTRL(*this, "pref_exiftool_argfile2", wxTextCtrl)->GetValue(),
+        _("ExifTool Argfiles (*.arg)|*.arg|All Files(*)|*"),
+        wxFD_OPEN, wxDefaultPosition);
+    if (dlg.ShowModal() == wxID_OK)
+    {
+        XRCCTRL(*this, "pref_exiftool_argfile2", wxTextCtrl)->SetValue(
+            dlg.GetPath());
+    }
+}
+
+void PreferencesDialog::OnExifArgfileEdit2(wxCommandEvent & e)
+{
+    wxString filename = XRCCTRL(*this, "pref_exiftool_argfile2", wxTextCtrl)->GetValue();
+    if (!filename.empty())
+    {
+        if (!wxFileName::FileExists(filename))
+        {
+            if (wxMessageBox(wxString::Format(_("File %s does not exist.\nShould an empty argfile be created?"), filename.c_str()),
+                _("Exiftool argfile"), wxYES_NO | wxICON_EXCLAMATION, this) != wxYES)
+            {
+                return;
+            };
+            wxTextFile newFile(filename);
+            newFile.Create();
+            newFile.Write();
+            newFile.Close();
+        };
+    }
+    else
+    {
+        if (wxMessageBox(_("No file selected.\nShould an empty argfile be created?"),
+            _("Exiftool argfile"), wxYES_NO | wxICON_EXCLAMATION, this) != wxYES)
+        {
+            return;
+        };
+        wxFileDialog dlg(this, _("Select new ExifTool argfile"),
+            wxStandardPaths::Get().GetUserConfigDir(), wxT(""),
+            _("ExifTool Argfiles (*.arg)|*.arg|All Files(*)|*"),
+            wxFD_SAVE | wxFD_OVERWRITE_PROMPT, wxDefaultPosition);
+        if (dlg.ShowModal() != wxID_OK)
+        {
+            return;
+        };
+        filename = dlg.GetPath();
+        wxTextFile newFile(filename);
+        newFile.Create();
+        newFile.Write();
+        newFile.Close();
+    };
+    XRCCTRL(*this, "pref_exiftool_argfile2", wxTextCtrl)->SetValue(filename);
+    wxDialog * edit_dlg = wxXmlResource::Get()->LoadDialog(this, wxT("pref_edit_argfile_placeholders"));
+    RestoreFramePosition(edit_dlg, wxT("EditArgfilePlaceholders"));
+    wxTextCtrl* argfileControl = XRCCTRL(*edit_dlg, "pref_edit_textcontrol", wxTextCtrl);
+    argfileControl->LoadFile(filename);
+    if (edit_dlg->ShowModal() == wxID_OK)
+    {
+        if (!argfileControl->SaveFile(filename))
+        {
+            wxMessageBox(wxString::Format(_("Could not save file \"%s\"."), filename.c_str()),
+                _("Error"), wxOK | wxICON_ERROR);
+        };
+        StoreFramePosition(edit_dlg, wxT("EditArgfilePlaceholders"));
+    };
+};
+
 void PreferencesDialog::OnExifTool(wxCommandEvent & e)
 {
     bool copyMetadata=XRCCTRL(*this, "pref_exiftool_metadata", wxCheckBox)->GetValue();
+    XRCCTRL(*this, "pref_exiftool_argfile_intermediate_label", wxStaticText)->Enable(copyMetadata);
     XRCCTRL(*this, "pref_exiftool_argfile_label", wxStaticText)->Enable(copyMetadata);
     XRCCTRL(*this, "pref_exiftool_argfile", wxTextCtrl)->Enable(copyMetadata);
     XRCCTRL(*this, "pref_exiftool_argfile_choose", wxButton)->Enable(copyMetadata);
     XRCCTRL(*this, "pref_exiftool_argfile_edit", wxButton)->Enable(copyMetadata);
+    XRCCTRL(*this, "pref_exiftool_argfile_final_label", wxStaticText)->Enable(copyMetadata);
+    XRCCTRL(*this, "pref_exiftool_argfile2_label", wxStaticText)->Enable(copyMetadata);
+    XRCCTRL(*this, "pref_exiftool_argfile2", wxTextCtrl)->Enable(copyMetadata);
+    XRCCTRL(*this, "pref_exiftool_argfile2_choose", wxButton)->Enable(copyMetadata);
+    XRCCTRL(*this, "pref_exiftool_argfile2_edit", wxButton)->Enable(copyMetadata);
     XRCCTRL(*this, "pref_exiftool_gpano", wxCheckBox)->Enable(copyMetadata);
 };
 
@@ -452,15 +538,6 @@ void PreferencesDialog::UpdateDisplayData(int panel)
 #endif
         MY_SPIN_VAL("prefs_cache_UpperBound", mem >> 20);
 
-        // number of threads
-        int nThreads = wxThread::GetCPUCount();
-        if (nThreads < 1)
-        {
-            nThreads = 1;
-        }
-        nThreads = cfg->Read(wxT("/Nona/NumberOfThreads"), nThreads);
-        MY_SPIN_VAL("prefs_nona_NumberOfThreads", nThreads);
-
         // language
         // check if current language is in list and activate it then.
         wxChoice* lang_choice = XRCCTRL(*this, "prefs_gui_language", wxChoice);
@@ -496,10 +573,6 @@ void PreferencesDialog::UpdateDisplayData(int panel)
         t = cfg->Read(wxT("CopyLogToClipboard"), 0l) == 1;
         MY_BOOL_VAL("prefs_copy_log", t);
 
-        // save makefile 
-        t = cfg->Read(wxT("SaveMakefile"), 0l) == 1;
-        MY_BOOL_VAL("prefs_makefile", t);
-
         t = cfg->Read(wxT("/GLPreviewFrame/ShowProjectionHints"), HUGIN_SHOW_PROJECTION_HINTS) == 1;
         MY_BOOL_VAL("pref_show_projection_hints", t)
     };
@@ -590,7 +663,7 @@ void PreferencesDialog::UpdateDisplayData(int panel)
         cpdetector_config_edit.FillControl(m_CPDetectorList,true,true);
     }
 
-    if (panel==0 || panel == 6)
+    if (panel == 0 || panel == 6)
     {
         /////
         /// DEFAULT OUTPUT FORMAT
@@ -601,33 +674,38 @@ void PreferencesDialog::UpdateDisplayData(int panel)
         MY_SPIN_VAL("pref_jpeg_quality", cfg->Read(wxT("/output/jpeg_quality"), HUGIN_JPEG_QUALITY));
         UpdateFileFormatControls();
 
+        // default blender
+        SelectListValue(XRCCTRL(*this, "pref_default_blender", wxChoice), cfg->Read(wxT("/default_blender"), HUGIN_DEFAULT_BLENDER));
+
         /////
         /// PROCESSOR
         MY_CHOICE_VAL("pref_processor_gui", cfg->Read(wxT("/Processor/gui"), HUGIN_PROCESSOR_GUI));
         t = cfg->Read(wxT("/Processor/start"), HUGIN_PROCESSOR_START) == 1;
         MY_BOOL_VAL("pref_processor_start", t);
-        t = cfg->Read(wxT("/Processor/parallel"), HUGIN_PROCESSOR_PARALLEL) == 1;
-        MY_BOOL_VAL("pref_processor_parallel", t);
         t = cfg->Read(wxT("/Processor/overwrite"), HUGIN_PROCESSOR_OVERWRITE) == 1;
         MY_BOOL_VAL("pref_processor_overwrite", t);
         t = cfg->Read(wxT("/Processor/verbose"), HUGIN_PROCESSOR_VERBOSE) == 1;
         MY_BOOL_VAL("pref_processor_verbose", t);
         UpdateProcessorControls();
+    }
 
+    if (panel == 0 || panel == 7)
+    {
+        // stitching (2) panel
         t = cfg->Read(wxT("/output/useExiftool"), HUGIN_USE_EXIFTOOL) == 1;
         MY_BOOL_VAL("pref_exiftool_metadata", t);
         MY_STR_VAL("pref_exiftool_argfile", cfg->Read(wxT("/output/CopyArgfile"), wxT("")));
+        MY_STR_VAL("pref_exiftool_argfile2", cfg->Read(wxT("/output/FinalArgfile"), wxT("")));
         t = cfg->Read(wxT("/output/writeGPano"), HUGIN_EXIFTOOL_CREATE_GPANO) == 1;
         MY_BOOL_VAL("pref_exiftool_gpano", t);
         wxCommandEvent dummy;
         OnExifTool(dummy);
-
+        // number of threads
         int nThreads = cfg->Read(wxT("/output/NumberOfThreads"), 0l);
         MY_SPIN_VAL("prefs_output_NumberOfThreads", nThreads);
-
     }
 
-    if (panel==0 || panel == 7)
+    if (panel==0 || panel == 8)
     {
 
         /////
@@ -662,7 +740,7 @@ void PreferencesDialog::UpdateDisplayData(int panel)
                    wxT(HUGIN_ENFUSE_ARGS)));
     }
 
-    if (panel==0 || panel == 8)
+    if (panel==0 || panel == 9)
     {
         // Celeste settings
         d=HUGIN_CELESTE_THRESHOLD;
@@ -672,7 +750,12 @@ void PreferencesDialog::UpdateDisplayData(int panel)
         MY_CHOICE_VAL("prefs_celeste_filter", cfg->Read(wxT("/Celeste/Filter"), HUGIN_CELESTE_FILTER));
         // photometric optimizer settings
         MY_SPIN_VAL("prefs_photo_optimizer_nr_points", cfg->Read(wxT("/OptimizePhotometric/nRandomPointsPerImage"), HUGIN_PHOTOMETRIC_OPTIMIZER_NRPOINTS));
-
+        // warnings
+        t = cfg->Read(wxT("/ShowSaveMessage"), 1l) == 1;
+        MY_BOOL_VAL("prefs_warning_save", t);
+        t = cfg->Read(wxT("/ShowExposureWarning"), 1l) == 1;
+        MY_BOOL_VAL("prefs_warning_exposure", t);
+        MY_CHOICE_VAL("pref_editcp_action", cfg->Read(wxT("/EditCPAfterAction"), 0l));
     }
 }
 
@@ -699,19 +782,11 @@ void PreferencesDialog::OnRestoreDefaults(wxCommandEvent& e)
             #endif
             */
             cfg->Write(wxT("/ImageCache/UpperBound"), HUGIN_IMGCACHE_UPPERBOUND);
-            // number of threads
-            int cpucount = wxThread::GetCPUCount();
-            if (cpucount < 1)
-            {
-                cpucount = 1;
-            }
-            cfg->Write(wxT("/Nona/NumberOfThreads"), cpucount);
             // locale
             cfg->Write(wxT("language"), int(HUGIN_LANGUAGE));
             // smart undo
             cfg->Write(wxT("smartUndo"), HUGIN_SMART_UNDO);
             cfg->Write(wxT("CopyLogToClipboard"), 0l);
-            cfg->Write(wxT("SaveMakefile"), 0l);
             // projection hints
             cfg->Write(wxT("/GLPreviewFrame/ShowProjectionHints"), HUGIN_SHOW_PROJECTION_HINTS);
         }
@@ -763,19 +838,23 @@ void PreferencesDialog::OnRestoreDefaults(wxCommandEvent& e)
             // cfg->Write(wxT("/output/hdr_format"), HUGIN_HDR_OUTPUT_FORMAT);
             cfg->Write(wxT("/output/tiff_compression"), HUGIN_TIFF_COMPRESSION);
             cfg->Write(wxT("/output/jpeg_quality"), HUGIN_JPEG_QUALITY);
+            // default blender
+            cfg->Write(wxT("/default_blender"), static_cast<long>(HUGIN_DEFAULT_BLENDER));
             // stitching engine
             cfg->Write(wxT("/Processor/gui"), HUGIN_PROCESSOR_GUI);
             cfg->Write(wxT("/Processor/start"), HUGIN_PROCESSOR_START);
-            cfg->Write(wxT("/Processor/parallel"), HUGIN_PROCESSOR_PARALLEL);
             cfg->Write(wxT("/Processor/overwrite"), HUGIN_PROCESSOR_OVERWRITE);
             cfg->Write(wxT("/Processor/verbose"), HUGIN_PROCESSOR_VERBOSE);
-            cfg->Write(wxT("/output/useExiftool"), HUGIN_USE_EXIFTOOL);
-            cfg->Write(wxT("/output/CopyArgfile"), wxT(""));
-            cfg->Write(wxT("/output/writeGPano"), HUGIN_EXIFTOOL_CREATE_GPANO);
-            cfg->Write(wxT("/output/NumberOfThreads"), 0l);
-
         }
         if (noteb->GetSelection() == 6)
+        {
+            cfg->Write(wxT("/output/useExiftool"), HUGIN_USE_EXIFTOOL);
+            cfg->Write(wxT("/output/CopyArgfile"), wxT(""));
+            cfg->Write(wxT("/output/FinalArgfile"), wxT(""));
+            cfg->Write(wxT("/output/writeGPano"), HUGIN_EXIFTOOL_CREATE_GPANO);
+            cfg->Write(wxT("/output/NumberOfThreads"), 0l);
+        }
+        if (noteb->GetSelection() == 7)
         {
             /// ENBLEND
             cfg->Write(wxT("/Enblend/Exe"), wxT(HUGIN_ENBLEND_EXE));
@@ -787,12 +866,15 @@ void PreferencesDialog::OnRestoreDefaults(wxCommandEvent& e)
             cfg->Write(wxT("/Enfuse/Args"), wxT(HUGIN_ENFUSE_ARGS));
         }
 
-        if (noteb->GetSelection() == 7)
+        if (noteb->GetSelection() == 8)
         {
             /// Celeste
             cfg->Write(wxT("/Celeste/Threshold"), HUGIN_CELESTE_THRESHOLD);
             cfg->Write(wxT("/Celeste/Filter"), HUGIN_CELESTE_FILTER);
             cfg->Write(wxT("/OptimizePhotometric/nRandomPointsPerImage"), HUGIN_PHOTOMETRIC_OPTIMIZER_NRPOINTS);
+            cfg->Write(wxT("/ShowSaveMessage"), 1l);
+            cfg->Write(wxT("/ShowExposureWarning"), 1l);
+            cfg->Write(wxT("/EditCPAfterAction"), 0l);
         }
 
         /*
@@ -849,9 +931,6 @@ void PreferencesDialog::UpdateConfigData()
     cfg->Write(wxT("/ImageCache/UpperBoundHigh"), (long) MY_G_SPIN_VAL("prefs_cache_UpperBound") >> 12);
 #endif
     cfg->Write(wxT("/ImageCache/UpperBound"), (long) MY_G_SPIN_VAL("prefs_cache_UpperBound") << 20);
-    // number of threads
-    cfg->Write(wxT("/Nona/NumberOfThreads"), MY_G_SPIN_VAL("prefs_nona_NumberOfThreads"));
-
     // locale
     // language
     wxChoice* lang = XRCCTRL(*this, "prefs_gui_language", wxChoice);
@@ -866,7 +945,6 @@ void PreferencesDialog::UpdateConfigData()
     // smart undo
     cfg->Write(wxT("smartUndo"), MY_G_BOOL_VAL("prefs_smart_undo"));
     cfg->Write(wxT("CopyLogToClipboard"), MY_G_BOOL_VAL("prefs_copy_log"));
-    cfg->Write(wxT("SaveMakefile"), MY_G_BOOL_VAL("prefs_makefile"));
     // show projections hints
     cfg->Write(wxT("/GLPreviewFrame/ShowProjectionHints"), MY_G_BOOL_VAL("pref_show_projection_hints"));
     // tempdir
@@ -894,16 +972,18 @@ void PreferencesDialog::UpdateConfigData()
     cfg->Write(wxT("/output/tiff_compression"), MY_G_CHOICE_VAL("pref_tiff_compression"));
     cfg->Write(wxT("/output/jpeg_quality"), MY_G_SPIN_VAL("pref_jpeg_quality"));
 
+    cfg->Write(wxT("/default_blender"), static_cast<long>(GetSelectedValue(XRCCTRL(*this, "pref_default_blender", wxChoice))));
+
     /////
     /// PROCESSOR
     cfg->Write(wxT("/Processor/gui"), MY_G_CHOICE_VAL("pref_processor_gui"));
     cfg->Write(wxT("/Processor/start"), MY_G_BOOL_VAL("pref_processor_start"));
-    cfg->Write(wxT("/Processor/parallel"), MY_G_BOOL_VAL("pref_processor_parallel"));
     cfg->Write(wxT("/Processor/overwrite"), MY_G_BOOL_VAL("pref_processor_overwrite"));
     cfg->Write(wxT("/Processor/verbose"), MY_G_BOOL_VAL("pref_processor_verbose"));
 
     cfg->Write(wxT("/output/useExiftool"), MY_G_BOOL_VAL("pref_exiftool_metadata"));
     cfg->Write(wxT("/output/CopyArgfile"), MY_G_STR_VAL("pref_exiftool_argfile"));
+    cfg->Write(wxT("/output/FinalArgfile"), MY_G_STR_VAL("pref_exiftool_argfile2"));
     cfg->Write(wxT("/output/writeGPano"), MY_G_BOOL_VAL("pref_exiftool_gpano"));
     cfg->Write(wxT("/output/NumberOfThreads"), MY_G_SPIN_VAL("prefs_output_NumberOfThreads"));
     /////
@@ -930,6 +1010,9 @@ void PreferencesDialog::UpdateConfigData()
     cfg->Write(wxT("/Celeste/Filter"), MY_G_CHOICE_VAL("prefs_celeste_filter"));
     //photometric optimizer
     cfg->Write(wxT("/OptimizePhotometric/nRandomPointsPerImage"), MY_G_SPIN_VAL("prefs_photo_optimizer_nr_points"));
+    cfg->Write(wxT("/ShowSaveMessage"), MY_G_BOOL_VAL("prefs_warning_save"));
+    cfg->Write(wxT("/ShowExposureWarning"), MY_G_BOOL_VAL("prefs_warning_exposure"));
+    cfg->Write(wxT("/EditCPAfterAction"), MY_G_CHOICE_VAL("pref_editcp_action"));
 
     cfg->Flush();
     UpdateDisplayData(0);
@@ -1093,7 +1176,6 @@ void PreferencesDialog::UpdateProcessorControls()
 {
     int i=MY_G_CHOICE_VAL("pref_processor_gui");
     XRCCTRL(*this,"pref_processor_start",wxCheckBox)->Enable(i==0);
-    XRCCTRL(*this,"pref_processor_parallel",wxCheckBox)->Enable(i==0);
     XRCCTRL(*this,"pref_processor_verbose",wxCheckBox)->Enable(i==0);
     switch(i)
     {
@@ -1102,14 +1184,12 @@ void PreferencesDialog::UpdateProcessorControls()
             {
                 wxConfigBase* config=wxConfigBase::Get();
                 XRCCTRL(*this,"pref_processor_start",wxCheckBox)->SetValue(config->Read(wxT("/Processor/start"), HUGIN_PROCESSOR_START) == 1);
-                XRCCTRL(*this,"pref_processor_parallel",wxCheckBox)->SetValue(config->Read(wxT("/Processor/parallel"), HUGIN_PROCESSOR_PARALLEL) == 1);
                 XRCCTRL(*this,"pref_processor_verbose",wxCheckBox)->SetValue(config->Read(wxT("/Processor/verbose"), HUGIN_PROCESSOR_VERBOSE) == 1);
             }
             break;
         case 1:
             //Hugin_stitch_project
             XRCCTRL(*this,"pref_processor_start",wxCheckBox)->SetValue(true);
-            XRCCTRL(*this,"pref_processor_parallel",wxCheckBox)->SetValue(false);
             XRCCTRL(*this,"pref_processor_verbose",wxCheckBox)->SetValue(true);
             break;
     };
