@@ -16,8 +16,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -65,6 +65,8 @@ protected:
     void OnExifTool(wxCommandEvent & e);
     void OnExifArgfile(wxCommandEvent & e);
     void OnExifArgfileEdit(wxCommandEvent & e);
+    void OnExifArgfile2(wxCommandEvent & e);
+    void OnExifArgfileEdit2(wxCommandEvent & e);
     void OnRestoreDefaults(wxCommandEvent & e);
     void OnCustomEnblend(wxCommandEvent & e);
     void OnCustomEnfuse(wxCommandEvent & e);
