@@ -18,8 +18,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -66,7 +66,7 @@ void PreviewColorPickerTool::CalcCorrection(hugin_utils::FDiff2D pos)
     m_red=0;
     m_blue=0;
     m_count=0;
-    PT::Panorama* pano=helper->GetPanoramaPtr();
+    HuginBase::Panorama* pano=helper->GetPanoramaPtr();
     UIntSet activeImages=pano->getActiveImages();
     if(activeImages.size()>0)
     {
