@@ -19,8 +19,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -43,8 +43,12 @@
 #include "hugin/huginApp.h"
 #include "hugin/PreviewPanel.h"
 #include "hugin/ImagesPanel.h"
-#include "hugin/CommandHistory.h"
+#include "base_wx/CommandHistory.h"
+#include "base_wx/PanoCommand.h"
 #include "hugin/TextKillFocusHandler.h"
+#include "algorithms/nona/FitPanorama.h"
+#include "algorithms/basic/CalculateMeanExposure.h"
+#include "base_wx/wxPlatform.h"
 
 #include <vigra_ext/ImageTransforms.h>
 
@@ -116,7 +120,7 @@ END_EVENT_TABLE()
 
 #define PF_STYLE (wxMAXIMIZE_BOX | wxMINIMIZE_BOX | wxRESIZE_BORDER | wxSYSTEM_MENU | wxCAPTION | wxCLOSE_BOX | wxCLIP_CHILDREN)
 
-PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
+PreviewFrame::PreviewFrame(wxFrame * frame, HuginBase::Panorama &pano)
     : wxFrame(frame,-1, _("Panorama preview"), wxDefaultPosition, wxDefaultSize,
               PF_STYLE),
       m_pano(pano)
@@ -144,9 +148,9 @@ PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
     m_ButtonPanel->SetAutoLayout(true);
 	m_ButtonPanel->SetSizer(m_ButtonSizer);
 						
-	m_ToggleButtonSizer->Add(m_ButtonPanel, 1, wxEXPAND | wxADJUST_MINSIZE, 0);
+	m_ToggleButtonSizer->Add(m_ButtonPanel, 1, wxEXPAND, 0);
 
-    m_topsizer->Add(m_ToggleButtonSizer, 0, wxEXPAND | wxADJUST_MINSIZE | wxALL, 5);
+    m_topsizer->Add(m_ToggleButtonSizer, 0, wxEXPAND | wxALL, 5);
 
     wxFlexGridSizer * flexSizer = new wxFlexGridSizer(2,0,5,5);
     flexSizer->AddGrowableCol(0);
@@ -172,7 +176,11 @@ PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
                                 _("VFOV"));
     m_VFOVSlider->SetLineSize(2);
     m_VFOVSlider->SetPageSize(10);
+#if wxCHECK_VERSION(3,0,1)
+    m_VFOVSlider->SetTickFreq(5);
+#else
     m_VFOVSlider->SetTickFreq(5,0);
+#endif
     m_VFOVSlider->SetToolTip(_("drag to change the vertical field of view"));
 
     flexSizer->Add(m_VFOVSlider, 0, wxEXPAND);
@@ -185,7 +193,11 @@ PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
                                 _("HFOV"));
     m_HFOVSlider->SetPageSize(10);
     m_HFOVSlider->SetLineSize(2);
+#if wxCHECK_VERSION(3,0,1)
+    m_HFOVSlider->SetTickFreq(5);
+#else
     m_HFOVSlider->SetTickFreq(5,0);
+#endif
 
     m_HFOVSlider->SetToolTip(_("drag to change the horizontal field of view"));
 
@@ -431,8 +443,8 @@ void PreviewFrame::OnChangeDisplayedImgs(wxCommandEvent & e)
             activeImages.erase(id);
         }
 //        m_PreviewPanel->SetDisplayedImages(m_displayedImgs);
-    	GlobalCmdHist::getInstance().addCommand(
-            new PT::SetActiveImagesCmd(m_pano, activeImages)
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetActiveImagesCmd(m_pano, activeImages)
         );
     } else {
         DEBUG_ERROR("invalid Togglebutton ID");
@@ -441,14 +453,14 @@ void PreviewFrame::OnChangeDisplayedImgs(wxCommandEvent & e)
 
 void PreviewFrame::panoramaChanged(Panorama &pano)
 {
-    const PanoramaOptions & opts = pano.getOptions();
+    const HuginBase::PanoramaOptions & opts = pano.getOptions();
 
     wxString projection;
     m_ProjectionChoice->SetSelection(opts.getProjection());
     m_VFOVSlider->Enable( opts.fovCalcSupported(opts.getProjection()) );
 
     m_outputModeChoice->SetSelection(opts.outputMode);
-    if (opts.outputMode == PanoramaOptions::OUTPUT_HDR) {
+    if (opts.outputMode == HuginBase::PanoramaOptions::OUTPUT_HDR) {
         /*
         m_exposureTextCtrl->Hide();
         m_defaultExposureBut->Hide();
@@ -570,7 +582,7 @@ void PreviewFrame::panoramaImagesChanged(Panorama &pano, const UIntSet &changed)
                 but->SetValue(true);
                 m_ButtonSizer->Add(but,
                                    0,
-                                   wxLEFT | wxTOP | wxADJUST_MINSIZE,
+                                   wxLEFT | wxTOP,
                                    5);
                 m_ToggleButtons.push_back(but);
                 dirty = true;
@@ -587,7 +599,11 @@ void PreviewFrame::panoramaImagesChanged(Panorama &pano, const UIntSet &changed)
     }
 
     if (dirty) {
+#if wxCHECK_VERSION(3,0,0)
+        m_ButtonSizer->FitInside(m_ButtonPanel);
+#else
 		m_ButtonSizer->SetVirtualSizeHints(m_ButtonPanel);
+#endif
 		// Layout();
 		DEBUG_INFO("New m_ButtonPanel width: " << (m_ButtonPanel->GetSize()).GetWidth());
 		DEBUG_INFO("New m_ButtonPanel Height: " << (m_ButtonPanel->GetSize()).GetHeight());
@@ -631,8 +647,8 @@ void PreviewFrame::OnProjectionChanged()
     case PanoramaOptions::EQUIRECTANGULAR:   Ip = _("Equirectangular"); break;
     }
     opt.projectionFormat = (PanoramaOptions::ProjectionFormat) lt;
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetPanoOptionsCmd( pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetPanoOptionsCmd( pano, opt )
         );
     DEBUG_DEBUG ("Projection changed: "  << lt << ":" << Ip )
 
@@ -644,8 +660,8 @@ void PreviewFrame::OnCenterHorizontally(wxCommandEvent & e)
 {
     if (m_pano.getActiveImages().size() == 0) return;
 
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::CenterPanoCmd(m_pano)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::CenterPanoCmd(m_pano)
         );
 }
 
@@ -653,8 +669,8 @@ void PreviewFrame::OnStraighten(wxCommandEvent & e)
 {
     if (m_pano.getNrOfImages() == 0) return;
 
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::StraightenPanoCmd(m_pano)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::StraightenPanoCmd(m_pano)
         );
 }
 
@@ -675,15 +691,14 @@ void PreviewFrame::OnFitPano(wxCommandEvent & e)
     if (m_pano.getActiveImages().size() == 0) return;
 
     DEBUG_TRACE("");
-    PanoramaOptions opt = m_pano.getOptions();
+    HuginBase::PanoramaOptions opt = m_pano.getOptions();
+    HuginBase::CalculateFitPanorama fitPano(m_pano);
+    fitPano.run();
+    opt.setHFOV(fitPano.getResultHorizontalFOV());
+    opt.setHeight(roundi(fitPano.getResultHeight()));
 
-    double hfov, height;
-    m_pano.fitPano(hfov, height);
-    opt.setHFOV(hfov);
-    opt.setHeight(roundi(height));
-
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetPanoOptionsCmd( m_pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetPanoOptionsCmd( m_pano, opt )
         );
 
     DEBUG_INFO ( "new fov: [" << opt.getHFOV() << " "<< opt.getVFOV() << "] => height: " << opt.getHeight() );
@@ -699,8 +714,8 @@ void PreviewFrame::OnShowAll(wxCommandEvent & e)
     for (unsigned int i=0; i < m_pano.getNrOfImages(); i++) {
         displayedImgs.insert(i);
     }
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetActiveImagesCmd(m_pano, displayedImgs)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetActiveImagesCmd(m_pano, displayedImgs)
         );
     updatePano();
 }
@@ -714,8 +729,8 @@ void PreviewFrame::OnShowNone(wxCommandEvent & e)
         m_ToggleButtons[i]->SetValue(false);
     }
     UIntSet displayedImgs;
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetActiveImagesCmd(m_pano, displayedImgs)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetActiveImagesCmd(m_pano, displayedImgs)
         );
     updatePano();
 }
@@ -746,8 +761,8 @@ void PreviewFrame::OnNumTransform(wxCommandEvent & e)
             wxLogError(_("Roll value must be numeric."));
             return;
         }
-        GlobalCmdHist::getInstance().addCommand(
-            new PT::RotatePanoCmd(m_pano, y, p, r)
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::RotatePanoCmd(m_pano, y, p, r)
             );
         // update preview panel
         updatePano();
@@ -758,7 +773,7 @@ void PreviewFrame::OnNumTransform(wxCommandEvent & e)
 
 void PreviewFrame::OnTextCtrlChanged(wxCommandEvent & e)
 {
-    PanoramaOptions opts = m_pano.getOptions();
+    HuginBase::PanoramaOptions opts = m_pano.getOptions();
     if (e.GetEventObject() == m_exposureTextCtrl) {
         // exposure
         wxString text = m_exposureTextCtrl->GetValue();
@@ -790,8 +805,8 @@ void PreviewFrame::OnTextCtrlChanged(wxCommandEvent & e)
         }
         opts.setProjectionParameters(para);
     }
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( m_pano, opts )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( m_pano, opts )
                                            );
     // update preview panel
     updatePano();
@@ -800,10 +815,10 @@ void PreviewFrame::OnTextCtrlChanged(wxCommandEvent & e)
 
 void PreviewFrame::OnProjParameterReset(wxCommandEvent &e)
 {
-    PanoramaOptions opts=m_pano.getOptions();
+    HuginBase::PanoramaOptions opts = m_pano.getOptions();
     opts.resetProjectionParameters();
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetPanoOptionsCmd(m_pano, opts)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetPanoOptionsCmd(m_pano, opts)
         );
 };
 
@@ -811,7 +826,7 @@ void PreviewFrame::OnChangeFOV(wxScrollEvent & e)
 {
     DEBUG_TRACE("");
 
-    PanoramaOptions opt = m_pano.getOptions();
+    HuginBase::PanoramaOptions opt = m_pano.getOptions();
 
     if (e.GetEventObject() == m_HFOVSlider) {
         DEBUG_DEBUG("HFOV changed (slider): " << e.GetInt() << " == " << m_HFOVSlider->GetValue());
@@ -833,8 +848,8 @@ void PreviewFrame::OnChangeFOV(wxScrollEvent & e)
 		opt.setVFOV(m_VFOVSlider->GetValue());
     }
 
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetPanoOptionsCmd( m_pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetPanoOptionsCmd( m_pano, opt )
         );
 }
 
@@ -861,10 +876,10 @@ void PreviewFrame::OnBlendChoice(wxCommandEvent & e)
 void PreviewFrame::OnDefaultExposure( wxCommandEvent & e )
 {
     if (m_pano.getNrOfImages() > 0) {
-        PanoramaOptions opt = m_pano.getOptions();
-        opt.outputExposureValue = calcMeanExposure(m_pano);
-        GlobalCmdHist::getInstance().addCommand(
-                new PT::SetPanoOptionsCmd( m_pano, opt )
+        HuginBase::PanoramaOptions opt = m_pano.getOptions();
+        opt.outputExposureValue = HuginBase::CalculateMeanExposure::calcMeanExposure(m_pano);
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                new PanoCommand::SetPanoOptionsCmd( m_pano, opt )
                                                );
         updatePano();
     }
@@ -872,20 +887,20 @@ void PreviewFrame::OnDefaultExposure( wxCommandEvent & e )
 
 void PreviewFrame::OnIncreaseExposure( wxSpinEvent & e )
 {
-    PanoramaOptions opt = m_pano.getOptions();
+    HuginBase::PanoramaOptions opt = m_pano.getOptions();
     opt.outputExposureValue = opt.outputExposureValue + 1.0/3;
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( m_pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( m_pano, opt )
                                             );
     updatePano();
 }
 
 void PreviewFrame::OnDecreaseExposure( wxSpinEvent & e )
 {
-    PanoramaOptions opt = m_pano.getOptions();
+    HuginBase::PanoramaOptions opt = m_pano.getOptions();
     opt.outputExposureValue = opt.outputExposureValue - 1.0/3;
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( m_pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( m_pano, opt )
                                            );
     updatePano();
 }
@@ -893,12 +908,12 @@ void PreviewFrame::OnDecreaseExposure( wxSpinEvent & e )
 void PreviewFrame::OnProjectionChoice( wxCommandEvent & e )
 {
     if (e.GetEventObject() == m_ProjectionChoice) {
-        PanoramaOptions opt = m_pano.getOptions();
+        HuginBase::PanoramaOptions opt = m_pano.getOptions();
         int lt = m_ProjectionChoice->GetSelection();
         wxString Ip;
-        opt.setProjection( (PanoramaOptions::ProjectionFormat) lt );
-        GlobalCmdHist::getInstance().addCommand(
-                new PT::SetPanoOptionsCmd( m_pano, opt )
+        opt.setProjection((HuginBase::PanoramaOptions::ProjectionFormat) lt);
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                new PanoCommand::SetPanoOptionsCmd( m_pano, opt )
                                             );
         DEBUG_DEBUG ("Projection changed: "  << lt);
         updatePano();
@@ -911,12 +926,12 @@ void PreviewFrame::OnProjectionChoice( wxCommandEvent & e )
 void PreviewFrame::OnOutputChoice( wxCommandEvent & e)
 {
     if (e.GetEventObject() == m_outputModeChoice) {
-        PanoramaOptions opt = m_pano.getOptions();
+        HuginBase::PanoramaOptions opt = m_pano.getOptions();
         int lt = m_outputModeChoice->GetSelection();
         wxString Ip;
-        opt.outputMode = ( (PanoramaOptions::OutputMode) lt );
-        GlobalCmdHist::getInstance().addCommand(
-                new PT::SetPanoOptionsCmd( m_pano, opt )
+        opt.outputMode = ((HuginBase::PanoramaOptions::OutputMode) lt);
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                new PanoCommand::SetPanoOptionsCmd( m_pano, opt )
                                                );
         updatePano();
 
@@ -929,38 +944,19 @@ void PreviewFrame::OnOutputChoice( wxCommandEvent & e)
 void PreviewFrame::updateProgressDisplay()
 {
     wxString msg;
-    // build the message:
-    for (std::vector<AppBase::ProgressTask>::iterator it = tasks.begin();
-         it != tasks.end(); ++it)
+    if (!m_message.empty())
     {
-        wxString cMsg;
-        if (it->getProgress() > 0) {
-            cMsg.Printf(wxT("%s [%3.0f%%]: %s "),
-                        wxString(it->getShortMessage().c_str(), wxConvLocal).c_str(),
-                        100 * it->getProgress(),
-                        wxString(it->getMessage().c_str(), wxConvLocal).c_str());
-        } else {
-            cMsg.Printf(wxT("%s %s"),wxString(it->getShortMessage().c_str(), wxConvLocal).c_str(),
-                        wxString(it->getMessage().c_str(), wxConvLocal).c_str());
-        }
-        // append to main message
-        if (it == tasks.begin()) {
-            msg = cMsg;
-        } else {
-            msg.Append(wxT(" | "));
-            msg.Append(cMsg);
-        }
-    }
-//    wxStatusBar *m_statbar = GetStatusBar();
-    //DEBUG_TRACE("Statusmb : " << msg.mb_str(wxConvLocal));
-    //m_statbar->SetStatusText(msg,0);
+        msg = wxGetTranslation(wxString(m_message.c_str(), wxConvLocal));
+        if (!m_filename.empty())
+        {
+            msg.Append(wxT(" "));
+            msg.Append(wxString(ProgressDisplay::m_filename.c_str(), HUGIN_CONV_FILENAME));
+        };
+    };
+    GetStatusBar()->SetStatusText(msg, 0);
 
 #ifdef __WXMSW__
     UpdateWindow(NULL);
-#else
-    // This is a bad call.. we just want to repaint the window, instead we will
-    // process user events as well :( Unfortunately, there is not portable workaround...
-//    wxYield();
 #endif
 }
 
@@ -976,7 +972,7 @@ void PreviewFrame::OnFullScreen(wxCommandEvent & e)
 
 void PreviewFrame::OnUndo(wxCommandEvent &e)
 {
-    if(GlobalCmdHist::getInstance().canUndo())
+    if(PanoCommand::GlobalCmdHist::getInstance().canUndo())
     {
         wxCommandEvent dummy(wxEVT_COMMAND_MENU_SELECTED, XRCID("ID_EDITUNDO"));
         m_parent->GetEventHandler()->AddPendingEvent(dummy);
@@ -989,7 +985,7 @@ void PreviewFrame::OnUndo(wxCommandEvent &e)
 
 void PreviewFrame::OnRedo(wxCommandEvent &e)
 {
-    if(GlobalCmdHist::getInstance().canRedo())
+    if(PanoCommand::GlobalCmdHist::getInstance().canRedo())
     {
         wxCommandEvent dummy(wxEVT_COMMAND_MENU_SELECTED, XRCID("ID_EDITREDO"));
         m_parent->GetEventHandler()->AddPendingEvent(dummy);
