@@ -14,8 +14,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -54,8 +54,8 @@ public:
      * When we next redraw, we recalculate the statistics for the lines if the
      * flag is set.
      */
-    void panoramaChanged(HuginBase::PanoramaData &pano);
-    void panoramaImagesChanged(HuginBase::PanoramaData&, const HuginBase::UIntSet&);
+    void panoramaChanged(HuginBase::Panorama &pano);
+    void panoramaImagesChanged(HuginBase::Panorama&, const HuginBase::UIntSet&);
     
     /// Start using the PreviewLayoutLinesTool.
     void Activate();
