@@ -19,8 +19,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -30,22 +30,19 @@
 #include "panoinc.h"
 
 #include <vigra/basicimageview.hxx>
-#include "vigra_ext/blend.h"
-#include "PT/Stitcher.h"
-
+#include "nona/Stitcher.h"
 
 #include "base_wx/wxImageCache.h"
 #include "hugin/PreviewPanel.h"
 #include "hugin/PreviewFrame.h"
 #include "hugin/MainFrame.h"
-#include "hugin/CommandHistory.h"
+#include "base_wx/CommandHistory.h"
+#include "base_wx/PanoCommand.h"
 #include "hugin/config_defaults.h"
 #include "hugin/huginApp.h"
-//#include "hugin/ImageProcessing.h"
 
 #include <math.h>
 
-using namespace PT;
 using namespace std;
 using namespace vigra;
 using namespace vigra_ext;
@@ -54,10 +51,6 @@ using namespace hugin_utils;
 typedef RGBValue<unsigned char> BRGBValue;
 
 BEGIN_EVENT_TABLE(PreviewPanel, wxPanel)
-//    EVT_PAINT(CPImageCtrl::OnPaint)
-//    EVT_LEFT_DOWN(CPImageCtrl::mousePressEvent)
-//    EVT_MOTION(CPImageCtrl::mouseMoveEvent)
-//    EVT_LEFT_UP(CPImageCtrl::mouseReleaseEvent)
     EVT_SIZE(PreviewPanel::OnResize)
     EVT_LEFT_DOWN(PreviewPanel::mousePressLMBEvent)
     EVT_RIGHT_DOWN(PreviewPanel::mousePressRMBEvent)
@@ -70,7 +63,6 @@ PreviewPanel::PreviewPanel()
     m_panoBitmap(0), 
     m_pano2erect(0), m_blendMode(BLEND_COPY), 
     m_state_rendering(false), m_rerender(false), m_imgsDirty(true)
-
 {
 }
 
@@ -98,7 +90,7 @@ bool PreviewPanel::Create(wxWindow* parent, wxWindowID id,
 }
 
 
-void PreviewPanel::Init(PreviewFrame *parent, PT::Panorama * panorama )
+void PreviewPanel::Init(PreviewFrame *parent, HuginBase::Panorama * panorama )
 {
     pano = panorama;
     parentWindow = parent;
@@ -117,17 +109,16 @@ PreviewPanel::~PreviewPanel()
     if (m_pano2erect) {
         delete m_pano2erect;
     }
-	delete & vigra_ext::ThreadManager::get();
     DEBUG_TRACE("dtor end");
 }
 
-void PreviewPanel::panoramaChanged(Panorama &pano)
+void PreviewPanel::panoramaChanged(HuginBase::Panorama &pano)
 {
     // avoid recursive calls.. don't know if they can happen at all,
     // but they might lead to crashes.
     bool dirty = false;
 
-    const PanoramaOptions & newOpts = pano.getOptions();
+    const HuginBase::PanoramaOptions & newOpts = pano.getOptions();
 
     // check if an important options has been changed
     if (newOpts.getHFOV() != opts.getHFOV()) {
@@ -170,40 +161,12 @@ void PreviewPanel::panoramaChanged(Panorama &pano)
     }
 }
 
-void PreviewPanel::panoramaImagesChanged(Panorama &pano, const UIntSet &changed)
+void PreviewPanel::panoramaImagesChanged(HuginBase::Panorama &pano, const HuginBase::UIntSet &changed)
 {
     DEBUG_TRACE("");
-
-    /*
-    for(PT::UIntSet::const_iterator it = changed.begin(); it != changed.end();
-        ++it)
-    {
-        // TODO: need to check if just the active flag changed and skip invalidate in that case
-        if (pano->getSrcImage(*it) != m_remapCache.getSrcDescription()) {
-            m_remapCache.invalidate(*it);
-        }
-    }
-    */
-
     m_imgsDirty = true;
-    /*
-    if (m_autoPreview) {
-        DEBUG_DEBUG("updating preview after image change");
-        updatePreview();
-    }
-    */
 }
 
-/*
-void PreviewPanel::SetDisplayedImages(const UIntSet & imgs)
-{
-    m_displayedImages = imgs;
-    if (m_autoPreview) {
-        updatePreview();
-    }
-}
-*/
-
 void PreviewPanel::SetBlendMode(BlendMode b)
 {
     m_blendMode = b;
@@ -260,11 +223,6 @@ void PreviewPanel::updatePreview()
     m_state_rendering = true;
     m_rerender = false;
 
-    long nthreads = wxConfigBase::Get()->Read(wxT("/Nona/NumberOfThreads"), wxThread::GetCPUCount());
-    if (nthreads < 1) nthreads = 1;
-    vigra_ext::ThreadManager::get().setNThreads(nthreads);
-
-
 	{
 	  // Even though the frame is hidden, the panel is not
 	  // so check the parent instead
@@ -278,14 +236,6 @@ void PreviewPanel::updatePreview()
 		}
 	  }
 	}
-//    bool seaming = wxConfigBase::Get()->Read("/PreviewPanel/UseSeaming",0l) != 0;
-
-    // temporary bitmap for our remapped image
-    // calculate the image size from panel widht, height from vfov
-
-//    long cor = wxConfigBase::Get()->Read("/PreviewPanel/correctDistortion",0l);
-//    bool corrLens = cor != 0;
-
     wxBusyCursor wait;
     double finalWidth = pano->getOptions().getWidth();
     double finalHeight = pano->getOptions().getHeight();
@@ -307,12 +257,11 @@ void PreviewPanel::updatePreview()
         DEBUG_DEBUG("landscape: " << m_panoImgSize);
     }
 
-    PanoramaOptions opts = pano->getOptions();
+    HuginBase::PanoramaOptions opts = pano->getOptions();
     //don't use GPU for preview
     opts.remapUsingGPU = false;
     opts.setWidth(m_panoImgSize.x, false);
     opts.setHeight(m_panoImgSize.y);
-    //m_panoImgSize.y = opts.getHeight();
     // always use bilinear for preview.
 
     // reset ROI. The preview needs to draw the parts outside the ROI, too!
@@ -325,32 +274,19 @@ void PreviewPanel::updatePreview()
         vigra::BasicImageView<RGBValue<unsigned char> > panoImg8((RGBValue<unsigned char> *)panoImage.GetData(), panoImage.GetWidth(), panoImage.GetHeight());
         FRGBImage panoImg(m_panoImgSize);
         BImage alpha(m_panoImgSize);
-        // the empty panorama roi
-//        Rect2D panoROI;
+
         DEBUG_DEBUG("about to stitch images, pano size: " << m_panoImgSize);
-        UIntSet displayedImages = pano->getActiveImages();
+        HuginBase::UIntSet displayedImages = pano->getActiveImages();
         if (displayedImages.size() > 0) {
-            if (opts.outputMode == PanoramaOptions::OUTPUT_HDR) {
+            if (opts.outputMode == HuginBase::PanoramaOptions::OUTPUT_HDR) {
                 DEBUG_DEBUG("HDR output merge");
 
                 ReduceToHDRFunctor<RGBValue<float> > hdrmerge;
-                ReduceStitcher<FRGBImage, BImage> stitcher(*pano, *parentWindow);
+                HuginBase::Nona::ReduceStitcher<FRGBImage, BImage> stitcher(*pano, parentWindow);
                 stitcher.stitch(opts, displayedImages,
                                 destImageRange(panoImg), destImage(alpha),
                                 m_remapCache,
                                 hdrmerge);
-                /*
-                std::vector<RemappedPanoImage<FRGBImage, BImage> *> remapped;
-                // get all remapped images
-                for (UIntSet::const_iterator it = displayedImages.begin();
-                     it != displayedImages.end(); ++it)
-                {
-                    remapped.push_back(m_remapCache.getRemapped(pano, opts, *it, *parentWindow));
-                }
-                reduceROIImages(remapped,
-                                destImageRange(panoImg), destImage(alpha),
-                                hdrmerge);
-                */
 #ifdef DEBUG_REMAP
 {
     vigra::ImageExportInfo exi( DEBUG_FILE_PREFIX "hugin04_preview_HDR_Reduce.tif"); \
@@ -369,30 +305,16 @@ void PreviewPanel::updatePreview()
                 double min = std::max(minmax.min, 1e-6f);
                 double max = minmax.max;
 
-#if 0
-                for (int i=0; i<3; i++) {
-                    if (minmax.min[i]> 1e-6 && minmax.min[i] < min)
-                        min = minmax.min[i];
-                }
-                double max = DBL_MIN;
-                for (int i=0; i<3; i++) {
-                    if (minmax.max[i]> 1e-6 && minmax.max[i] > max)
-                        max = minmax.max[i];
-                }
-#endif
-
                 int mapping = wxConfigBase::Get()->Read(wxT("/ImageCache/MappingFloat"), HUGIN_IMGCACHE_MAPPING_FLOAT);
                 applyMapping(srcImageRange(panoImg), destImage(panoImg8), min, max, mapping);
 
             } else {
                     // LDR output
-    //            FileRemapper<BRGBImage, BImage> m;
                 switch (m_blendMode) {
                 case BLEND_COPY:
                 {
-                    StackingBlender blender;
-    //                SimpleStitcher<BRGBImage, BImage> stitcher(pano, *(MainFrame::Get()));
-                    SimpleStitcher<FRGBImage, BImage> stitcher(*pano, *parentWindow);
+                    HuginBase::Nona::StackingBlender blender;
+                    HuginBase::Nona::SimpleStitcher<FRGBImage, BImage> stitcher(*pano, parentWindow);
                     stitcher.stitch(opts, displayedImages,
                                     destImageRange(panoImg), destImage(alpha),
                                     m_remapCache,
@@ -401,34 +323,14 @@ void PreviewPanel::updatePreview()
                 }
                 case BLEND_DIFFERENCE:
                 {
-                    ReduceToDifferenceFunctor<RGBValue<float> > func;
-                    ReduceStitcher<FRGBImage, BImage> stitcher(*pano, *parentWindow);
+                    HuginBase::Nona::ReduceToDifferenceFunctor<RGBValue<float> > func;
+                    HuginBase::Nona::ReduceStitcher<FRGBImage, BImage> stitcher(*pano, parentWindow);
                     stitcher.stitch(opts, displayedImages,
                                     destImageRange(panoImg), destImage(alpha),
                                     m_remapCache,
                                     func);
                     break;
-    /*
-    
-                    WeightedStitcher<BRGBImage, BImage> stitcher(pano, *(MainFrame::Get()));
-                    stitcher.stitch(opts, m_displayedImages,
-                                    destImageRange(panoImg), destImage(alpha),
-                                    m_remapCache);
-                    break;
-    */
                 }
-    /*
-                case BLEND_DIFFERENCE:
-                {
-                    DifferenceBlender blender;
-                    SimpleStitcher<BRGBImage, BImage> stitcher(pano, *(MainFrame::Get()));
-                    stitcher.stitch(opts, m_displayedImages,
-                                    destImageRange(panoImg), destImage(alpha),
-                                    m_remapCache,
-                                    blender);
-                    break;
-                }
-    */
                 }
 
                 
@@ -444,7 +346,7 @@ void PreviewPanel::updatePreview()
 #endif
 
                 // apply default exposure and convert to 8 bit
-                SrcPanoImage src = pano->getSrcImage(0);
+                HuginBase::SrcPanoImage src = pano->getSrcImage(0);
 
                 // apply the exposure
                 double scale = 1.0/pow(2.0,opts.outputExposureValue);
@@ -453,7 +355,7 @@ void PreviewPanel::updatePreview()
                                       vigra::functor::Arg1()*vigra::functor::Param(scale));
 
                 DEBUG_DEBUG("LDR output, with response: " << src.getResponseType());
-                if (src.getResponseType() == SrcPanoImage::RESPONSE_LINEAR) {
+                if (src.getResponseType() == HuginBase::SrcPanoImage::RESPONSE_LINEAR) {
                     vigra::transformImage(srcImageRange(panoImg), destImage(panoImg8),
                                           vigra::functor::Arg1()*vigra::functor::Param(255));
                 } else {
@@ -462,10 +364,10 @@ void PreviewPanel::updatePreview()
                     LUT lut;
                     switch(src.getResponseType())
                     {
-                        case SrcPanoImage::RESPONSE_EMOR:
+                        case HuginBase::SrcPanoImage::RESPONSE_EMOR:
                             EMoR::createEMoRLUT(src.getEMoRParams(), lut);
                             break;
-                        case SrcPanoImage::RESPONSE_GAMMA:
+                        case HuginBase::SrcPanoImage::RESPONSE_GAMMA:
                             lut.resize(256);
                             createGammaLUT(1/src.getGamma(), lut);
                             break;
@@ -502,11 +404,11 @@ void PreviewPanel::updatePreview()
 
     // update the transform for pano -> erect coordinates
     if (m_pano2erect) delete m_pano2erect;
-    SrcPanoImage src;
-    src.setProjection(SrcPanoImage::EQUIRECTANGULAR);
+    HuginBase::SrcPanoImage src;
+    src.setProjection(HuginBase::SrcPanoImage::EQUIRECTANGULAR);
     src.setHFOV(360);
     src.setSize(Size2D(360,180));
-    m_pano2erect = new PTools::Transform;
+    m_pano2erect = new HuginBase::PTools::Transform;
     m_pano2erect->createTransform(src, opts);
 
     if (m_panoBitmap) {
@@ -532,8 +434,6 @@ void PreviewPanel::DrawPreview(wxDC & dc)
     }
     DEBUG_TRACE("");
 
-//    bool drawOutlines = wxConfigBase::Get()->Read("/PreviewPanel/drawOutlines",1l) != 0;
-
     int offsetX = 0;
     int offsetY = 0;
 
@@ -545,8 +445,13 @@ void PreviewPanel::DrawPreview(wxDC & dc)
         offsetY = (sz.GetHeight() - m_panoImgSize.y) / 2;
     }
 
-    dc.SetPen(wxPen(GetBackgroundColour(),1,wxSOLID));
+#if wxCHECK_VERSION(3,0,0)
+    dc.SetPen(wxPen(GetBackgroundColour(), 1, wxPENSTYLE_SOLID));
+    dc.SetBrush(wxBrush(GetBackgroundColour(), wxBRUSHSTYLE_SOLID));
+#else
+    dc.SetPen(wxPen(GetBackgroundColour(), 1, wxSOLID));
     dc.SetBrush(wxBrush(GetBackgroundColour(),wxSOLID));
+#endif
     dc.DrawRectangle(0, 0, offsetX, sz.GetHeight());
     dc.DrawRectangle(offsetX, 0, sz.GetWidth(), offsetY);
     dc.DrawRectangle(offsetX, sz.GetHeight() - offsetY,
@@ -559,8 +464,13 @@ void PreviewPanel::DrawPreview(wxDC & dc)
     dc.SetClippingRegion(offsetX, offsetY,
                          m_panoImgSize.x, m_panoImgSize.y);
 
-    dc.SetPen(wxPen(wxT("BLACK"),1,wxSOLID));
+#if wxCHECK_VERSION(3,0,0)
+    dc.SetPen(wxPen(wxT("BLACK"), 1, wxPENSTYLE_SOLID));
+    dc.SetBrush(wxBrush(wxT("BLACK"), wxBRUSHSTYLE_SOLID));
+#else
+    dc.SetPen(wxPen(wxT("BLACK"), 1, wxSOLID));
     dc.SetBrush(wxBrush(wxT("BLACK"),wxSOLID));
+#endif
     dc.DrawRectangle(offsetX, offsetY, m_panoImgSize.x, m_panoImgSize.y);
 
 
@@ -635,7 +545,11 @@ void PreviewPanel::DrawPreview(wxDC & dc)
                          m_panoImgSize.x, m_panoImgSize.y);
 
             // draw boundaries
+#if wxCHECK_VERSION(3,0,0)
+            dc.SetPen(wxPen(wxT("WHITE"), 1, wxPENSTYLE_SOLID));
+#else
             dc.SetPen(wxPen(wxT("WHITE"), 1, wxSOLID));
+#endif
             dc.SetLogicalFunction(wxINVERT);
 
             DEBUG_DEBUG("ROI scale factor: " << scale << " screen ROI: " << screenROI);
@@ -653,24 +567,16 @@ void PreviewPanel::DrawPreview(wxDC & dc)
         }
     }
 
-#if 0
-    // currently disabled
-    if (drawOutlines) {
-        for (UIntSet::iterator it = m_displayedImages.begin();
-             it != m_displayedImages.end();
-             ++it)
-        {
-            dc.SetPen(wxPen(wxT("GREY"), 1, wxSOLID));
-            DrawOutline(m_remapped[*it]->getOutline(), dc, offsetX, offsetY);
-        }
-    }
-#endif
     dc.DestroyClippingRegion();
     dc.SetClippingRegion(offsetX, offsetY,
                     m_panoImgSize.x, m_panoImgSize.y);
 
     // draw center lines over display
+#if wxCHECK_VERSION(3,0,0)
+    dc.SetPen(wxPen(wxT("WHITE"), 1, wxPENSTYLE_SOLID));
+#else
     dc.SetPen(wxPen(wxT("WHITE"), 1, wxSOLID));
+#endif
     dc.SetLogicalFunction(wxINVERT);
     dc.DrawLine(offsetX + w/2, offsetY,
                 offsetX + w/2, offsetY + h);
@@ -717,8 +623,8 @@ void PreviewPanel::mousePressLMBEvent(wxMouseEvent & e)
     r = RAD_TO_DEG(r);
     DEBUG_DEBUG("rotation angles pitch*yaw: " << y << " " << p << " " << r);
 
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::RotatePanoCmd(*pano, y, p, r)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::RotatePanoCmd(*pano, y, p, r)
         );
     if (!m_autoPreview) {
         ForceUpdate();
@@ -743,8 +649,8 @@ void PreviewPanel::mousePressRMBEvent(wxMouseEvent & e)
 
     DEBUG_DEBUG("roll correction: " << roll);
 
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::RotatePanoCmd(*pano, 0, 0, roll)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::RotatePanoCmd(*pano, 0, 0, roll)
         );
     if (!m_autoPreview) {
         ForceUpdate();
@@ -756,25 +662,6 @@ void PreviewPanel::OnMouse(wxMouseEvent & e)
 {
     double yaw, pitch;
     mouse2erect(e.m_x, e.m_y,  yaw, pitch);
-    
-    /*
-    wxSize sz = GetClientSize();
-    int offsetX = 0;
-    int offsetY = 0;
-    if (sz.GetWidth() > m_panoImgSize.x) {
-        offsetX = (sz.GetWidth() - m_panoImgSize.x) / 2;
-    }
-    if (sz.GetHeight() > m_panoImgSize.y) {
-        offsetY = (sz.GetHeight() - m_panoImgSize.y) / 2;
-    }
-    double x = e.m_x - offsetX - m_panoImgSize.x/2;
-    double y = e.m_y - offsetY - m_panoImgSize.y/2;
-
-    int w = pano->getOptions().getWidth();
-    double scale = w/(double)m_panoImgSize.x;
-    x *= scale;
-    y *= scale;
-    */
 
     parentWindow->SetStatusText(_("Left click to define new center point, right click to move point to horizon."),0);
     parentWindow->SetStatusText(wxString::Format(wxT("%.1f %.1f"), yaw, pitch), 1);
