@@ -16,8 +16,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */ 
 
@@ -29,7 +29,7 @@
 #include "GLViewer.h"
 #include "MeshManager.h"
 
-ToolHelper::ToolHelper(PT::Panorama *pano_in,
+ToolHelper::ToolHelper(HuginBase::Panorama *pano_in,
                                      VisualizationState *visualization_state_in,
                                      GLPreviewFrame *frame_in)
 {
@@ -77,7 +77,7 @@ void ToolHelper::MouseMoved(int x, int y, wxMouseEvent & e)
     // now tell tools that want notification.
     std::set<Tool *>::iterator iterator;
     for (iterator = mouse_move_notified_tools.begin();
-         iterator != mouse_move_notified_tools.end(); iterator++)
+         iterator != mouse_move_notified_tools.end(); ++iterator)
     {
         (*iterator)->MouseMoveEvent(mouse_screen_x, mouse_screen_y, e);
     }
@@ -101,7 +101,7 @@ void ToolHelper::InvalidateImagesUnderMouse()
             std::set<Tool *>::iterator iterator;
             for (iterator = images_under_mouse_notified_tools.begin();
                  iterator != images_under_mouse_notified_tools.end();
-                 iterator++)
+                 ++iterator)
             {
                 (*iterator)->ImagesUnderMouseChangedEvent();
             }
@@ -118,7 +118,7 @@ void ToolHelper::MouseButtonEvent(wxMouseEvent &e)
 //    }
     std::set<Tool *>::iterator iterator;
     for (iterator = mouse_button_notified_tools.begin();
-         iterator != mouse_button_notified_tools.end(); iterator++)
+         iterator != mouse_button_notified_tools.end(); ++iterator)
     {
         (*iterator)->MouseButtonEvent(e);
     }
@@ -129,7 +129,7 @@ void ToolHelper::MouseWheelEvent(wxMouseEvent &e)
 {
     std::set<Tool *>::iterator iterator;
     for (iterator = mouse_wheel_notified_tools.begin();
-         iterator != mouse_wheel_notified_tools.end(); iterator++)
+         iterator != mouse_wheel_notified_tools.end(); ++iterator)
     {
         (*iterator)->MouseWheelEvent(e);
     }
@@ -145,7 +145,7 @@ void ToolHelper::KeypressEvent(int keycode, int modifiers, bool pressed)
 //    }
     std::set<Tool *>::iterator iterator;
     for (iterator = keypress_notified_tools.begin();
-         iterator != keypress_notified_tools.end(); iterator++)
+         iterator != keypress_notified_tools.end(); ++iterator)
     {
         (*iterator)->KeypressEvent(keycode, modifiers, pressed);
     }
@@ -157,7 +157,7 @@ void ToolHelper::BeforeDrawImages()
     // let all tools that want to draw under the images do so.
     std::set<Tool *>::iterator iterator;
     for (iterator = draw_under_notified_tools.begin();
-         iterator != draw_under_notified_tools.end(); iterator++)
+         iterator != draw_under_notified_tools.end(); ++iterator)
     {
         (*iterator)->BeforeDrawImagesEvent();
     }
@@ -176,7 +176,7 @@ void ToolHelper::AfterDrawImages()
     // let all tools that want to draw on top of the images do so.
     std::set<Tool *>::iterator iterator;
     for (iterator = draw_over_notified_tools.begin();
-         iterator != draw_over_notified_tools.end(); iterator++)
+         iterator != draw_over_notified_tools.end(); ++iterator)
     {
 //        std::cerr << "tool after draw" << std::endl;
         (*iterator)->AfterDrawImagesEvent();
@@ -184,7 +184,7 @@ void ToolHelper::AfterDrawImages()
 //    std::cerr << "after draw images" << std::endl;
     // The overlays are done separetly to avoid errors with blending order.
     for (iterator = really_draw_over_notified_tools.begin();
-         iterator != really_draw_over_notified_tools.end(); iterator++)
+         iterator != really_draw_over_notified_tools.end(); ++iterator)
     {
         (*iterator)->ReallyAfterDrawImagesEvent();
     }
@@ -199,7 +199,7 @@ bool ToolHelper::BeforeDrawImageNumber(unsigned int image)
         {
             bool result = true;
             std::set<Tool *>::iterator it;
-            for(it = image_draw_begin_tools[image].begin() ; it != image_draw_begin_tools[image].end() ; it++) {
+            for(it = image_draw_begin_tools[image].begin() ; it != image_draw_begin_tools[image].end() ; ++it) {
                 result &= (*it)->BeforeDrawImageEvent(image);
             }
             return result;
@@ -216,7 +216,7 @@ void ToolHelper::AfterDrawImageNumber(unsigned int image)
         if (image_draw_end_tools[image].size() > 0)
         {
             std::set<Tool *>::iterator it;
-            for(it = image_draw_end_tools[image].begin() ; it != image_draw_end_tools[image].end() ; it++) {
+            for(it = image_draw_end_tools[image].begin() ; it != image_draw_end_tools[image].end() ; ++it) {
                 (*it)->BeforeDrawImageEvent(image);
             }
 //            image_draw_end_tools[image]->AfterDrawImageEvent(image);
@@ -224,6 +224,21 @@ void ToolHelper::AfterDrawImageNumber(unsigned int image)
     }
 }
 
+void ToolHelper::MouseEnter(int x, int y, wxMouseEvent &e)
+{
+    // You might expect that this is redundant, because any MouseEnter will be
+    // accompanied by a MouseMove, which will achieve the same ends.  However,
+    // this is not so.  It's possible for the mouse to enter without moving.
+    // Morevover, some environments will trigger a MouseLeave+MouseEnter just
+    // before a MouseButton event; handling this is particularly critical for
+    // such cases.
+    mouse_over_pano = true;
+    mouse_screen_x = x;
+    mouse_screen_y = y;
+    // If the mouse has moved, then we don't know what is underneath it anoymore
+    InvalidateImagesUnderMouse();
+}
+
 void ToolHelper::MouseLeave()
 {
     // if the mouse leaves the preview, there are no images under the mouse
@@ -237,7 +252,7 @@ void ToolHelper::MouseLeave()
         std::set<Tool *>::iterator iterator;
         for (iterator = images_under_mouse_notified_tools.begin();
              iterator != images_under_mouse_notified_tools.end();
-             iterator++)
+             ++iterator)
         {
             (*iterator)->ImagesUnderMouseChangedEvent();
         }        
@@ -274,7 +289,7 @@ ViewState *ToolHelper::GetViewStatePtr()
     return visualization_state->getViewState();
 }
 
-PT::Panorama *ToolHelper::GetPanoramaPtr()
+HuginBase::Panorama *ToolHelper::GetPanoramaPtr()
 {
     return pano;
 }
@@ -467,7 +482,7 @@ void PreviewToolHelper::MouseMoved(int x, int y, wxMouseEvent &e)
     // now tell tools that want notification.
     std::set<Tool *>::iterator iterator;
     for (iterator = mouse_move_notified_tools.begin();
-         iterator != mouse_move_notified_tools.end(); iterator++)
+         iterator != mouse_move_notified_tools.end(); ++iterator)
     {
         (*iterator)->MouseMoveEvent(mouse_pano_x, mouse_pano_y, e);
     }
@@ -637,7 +652,7 @@ void PanosphereOverviewToolHelper::MouseMoved(int x, int y, wxMouseEvent & e)
     ToolHelper::MouseMoved(x,y,e);
 }
 
-PanosphereOverviewToolHelper::PanosphereOverviewToolHelper(PT::Panorama *pano,
+PanosphereOverviewToolHelper::PanosphereOverviewToolHelper(HuginBase::Panorama *pano,
                   VisualizationState *visualization_state,
                   GLPreviewFrame * frame) : OverviewToolHelper(pano, visualization_state, frame) {}
 
@@ -681,7 +696,7 @@ void PanosphereOverviewToolHelper::BeforeDrawImagesBack()
 {
     std::set<Tool *>::iterator iterator;
     for (iterator = draw_under_notified_tools_back.begin();
-         iterator != draw_under_notified_tools_back.end(); iterator++)
+         iterator != draw_under_notified_tools_back.end(); ++iterator)
     {
         ((PanosphereOverviewTool*)(*iterator))->BeforeDrawImagesBackEvent();
     }
@@ -691,7 +706,7 @@ void PanosphereOverviewToolHelper::BeforeDrawImagesFront()
 {
     std::set<Tool *>::iterator iterator;
     for (iterator = draw_under_notified_tools_front.begin();
-         iterator != draw_under_notified_tools_front.end(); iterator++)
+         iterator != draw_under_notified_tools_front.end(); ++iterator)
     {
         ((PanosphereOverviewTool*)(*iterator))->BeforeDrawImagesFrontEvent();
     }
@@ -701,7 +716,7 @@ void PanosphereOverviewToolHelper::AfterDrawImagesBack()
 {
     std::set<Tool *>::iterator iterator;
     for (iterator = draw_over_notified_tools_back.begin();
-         iterator != draw_over_notified_tools_back.end(); iterator++)
+         iterator != draw_over_notified_tools_back.end(); ++iterator)
     {
         ((PanosphereOverviewTool*)(*iterator))->AfterDrawImagesBackEvent();
     }
@@ -711,7 +726,7 @@ void PanosphereOverviewToolHelper::AfterDrawImagesFront()
 {
     std::set<Tool *>::iterator iterator;
     for (iterator = draw_over_notified_tools_front.begin();
-         iterator != draw_over_notified_tools_front.end(); iterator++)
+         iterator != draw_over_notified_tools_front.end(); ++iterator)
     {
         ((PanosphereOverviewTool*)(*iterator))->AfterDrawImagesFrontEvent();
     }
@@ -728,7 +743,7 @@ void PanosphereOverviewToolHelper::DeactivateTool(Tool *tool)
 }
 
 
-PlaneOverviewToolHelper::PlaneOverviewToolHelper(PT::Panorama *pano,
+PlaneOverviewToolHelper::PlaneOverviewToolHelper(HuginBase::Panorama *pano,
                   VisualizationState *visualization_state,
                   GLPreviewFrame * frame) : OverviewToolHelper(pano, visualization_state, frame) {}
 
