@@ -16,8 +16,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -31,7 +31,7 @@
 
 
 
-ViewState::ViewState(PT::Panorama *pano, bool supportMultiTexture)
+ViewState::ViewState(HuginBase::Panorama *pano, bool supportMultiTexture)
 {
 
 
@@ -61,7 +61,7 @@ ViewState::~ViewState()
 }
 
 
-void ViewState::panoramaChanged(HuginBase::PanoramaData &pano)
+void ViewState::panoramaChanged(HuginBase::Panorama &pano)
 {
 
     bool require_draw=false;
@@ -101,7 +101,7 @@ void ViewState::panoramaChanged(HuginBase::PanoramaData &pano)
 
     if (require_draw) {
         //refresh function is called in the respective VisualizationState callback
-        for (std::map<VisualizationState*,bool>::iterator it = vis_states.begin() ; it != vis_states.end() ; it++) {
+        for (std::map<VisualizationState*,bool>::iterator it = vis_states.begin() ; it != vis_states.end() ; ++it) {
             DEBUG_DEBUG("PanoChanged - iterator before");
             if (it->second) {
                 DEBUG_DEBUG("PanoChanged - iterator after");
@@ -112,7 +112,7 @@ void ViewState::panoramaChanged(HuginBase::PanoramaData &pano)
 
 }
 
-void ViewState::panoramaImagesChanged(HuginBase::PanoramaData&,
+void ViewState::panoramaImagesChanged(HuginBase::Panorama&,
                                       const HuginBase::UIntSet&)
 {
     // actually this stuff is handled by panoramaChanged.
@@ -123,7 +123,7 @@ void ViewState::SetOptions(const HuginBase::PanoramaOptions *new_opts)
 
     bool dirty_projection = false;
 
-    for (std::map<VisualizationState*,bool>::iterator it = vis_states.begin() ; it != vis_states.end() ; it++) {
+    for (std::map<VisualizationState*,bool>::iterator it = vis_states.begin() ; it != vis_states.end() ; ++it) {
 
         if (!(it->second)) continue;
         
@@ -249,7 +249,7 @@ void ViewState::SetSrcImage(unsigned int image_nr, HuginBase::SrcPanoImage *new_
     img_states[image_nr] = *new_img;
 
 
-    for (std::map<VisualizationState*,bool>::iterator it = vis_states.begin() ; it != vis_states.end() ; it++) {
+    for (std::map<VisualizationState*,bool>::iterator it = vis_states.begin() ; it != vis_states.end() ; ++it) {
         if (!(it->second)) continue;
         if (dirty_draw) it->first->ForceRequireRedraw();
         if (dirty_mesh) it->first->SetDirtyMesh(image_nr);
@@ -263,7 +263,7 @@ void ViewState::ForceRequireRedraw()
     // this is generally called by preview tools. We let them manage themselves.
     // often they give some user interface thing that doesn't reflect a change
     // in the panorama at all, so we let them force a redraw.
-    for (std::map<VisualizationState*,bool>::iterator it = vis_states.begin() ; it != vis_states.end() ; it++) {
+    for (std::map<VisualizationState*,bool>::iterator it = vis_states.begin() ; it != vis_states.end() ; ++it) {
         if (!(it->second)) continue;
         it->first->ForceRequireRedraw();
     }
@@ -345,7 +345,7 @@ void ViewState::DoUpdates()
 void ViewState::Redraw()
 {
 
-    for (std::map<VisualizationState*,bool>::iterator it = vis_states.begin() ; it != vis_states.end() ; it++) {
+    for (std::map<VisualizationState*,bool>::iterator it = vis_states.begin() ; it != vis_states.end() ; ++it) {
         
         if (!(it->second)) continue;
         
@@ -482,7 +482,7 @@ HuginBase::SrcPanoImage * VisualizationState::GetSrcImage(unsigned int image_nr)
 
 
 
-PanosphereOverviewVisualizationState::PanosphereOverviewVisualizationState(PT::Panorama* pano, ViewState* view_state, GLViewer * viewer, void (*RefreshFunction)(void*), void *arg)
+PanosphereOverviewVisualizationState::PanosphereOverviewVisualizationState(HuginBase::Panorama* pano, ViewState* view_state, GLViewer * viewer, void (*RefreshFunction)(void*), void *arg)
         : OverviewVisualizationState(pano, view_state, viewer, RefreshFunction, arg, (PanosphereOverviewMeshManager*) NULL) 
 {
     scale = 1;
@@ -538,7 +538,7 @@ void PanosphereOverviewVisualizationState::setAngY(double angy_in)
 }
 
 
-PlaneOverviewVisualizationState::PlaneOverviewVisualizationState(PT::Panorama* pano, ViewState* view_state, GLViewer * viewer, void (*RefreshFunction)(void*), void *arg)
+PlaneOverviewVisualizationState::PlaneOverviewVisualizationState(HuginBase::Panorama* pano, ViewState* view_state, GLViewer * viewer, void(*RefreshFunction)(void*), void *arg)
         : OverviewVisualizationState(pano, view_state, viewer, RefreshFunction, arg, (PlaneOverviewMeshManager*) NULL) 
 {
 
