@@ -19,8 +19,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -35,7 +35,6 @@
 
 #include "panoinc.h"
 
-
 #include "hugin/config_defaults.h"
 #include "hugin/huginApp.h"
 #include "hugin/ImagesPanel.h"
@@ -44,11 +43,12 @@
 #include "hugin/OptimizePhotometricPanel.h"
 #include "hugin/PanoPanel.h"
 #include "hugin/ImagesList.h"
+#include "hugin/CPListFrame.h"
 #include "hugin/PreviewPanel.h"
 #include "hugin/GLPreviewFrame.h"
 #include "base_wx/PTWXDlg.h"
-#include "hugin/CommandHistory.h"
-#include "hugin/wxPanoCommand.h"
+#include "base_wx/CommandHistory.h"
+#include "base_wx/wxPanoCommand.h"
 #include "hugin/HtmlWindow.h"
 #include "hugin/treelistctrl.h"
 #include "hugin/ImagesTree.h"
@@ -56,15 +56,18 @@
 #include "base_wx/platform.h"
 #include "base_wx/huginConfig.h"
 #ifdef __WXMSW__
-#include "wx/dir.h"
-#include "wx/cshelp.h"
+#include <wx/dir.h>
+#include <wx/cshelp.h>
+#include <wx/stdpaths.h>
+#if wxCHECK_VERSION(3,1,0)
+#include <wx/taskbarbutton.h>
+#endif
 #endif
 
 #include <tiffio.h>
 
 #include "AboutDialog.h"
 
-#include <hugin_version.h>
 //for natural sorting
 #include "hugin_utils/alphanum.h"
 #include "lensdb/LensDB.h"
@@ -76,22 +79,12 @@ bool checkVersion(wxString v1, wxString v2)
 
 using namespace hugin_utils;
 
-// utility functions
-bool str2double(wxString s, double & d)
-{
-    if (!stringToDouble(std::string(s.mb_str(wxConvLocal)), d)) {
-        wxLogError(_("Value must be numeric."));
-        return false;
-    }
-    return true;
-}
-
-wxString Components2Str(const CPComponents & comp)
+wxString Components2Str(const HuginBase::CPComponents & comp)
 {
     wxString ret;
     for (unsigned i=0; i < comp.size(); i++) {
         ret = ret + wxT("[");
-        CPComponents::value_type::const_iterator it;
+        HuginBase::CPComponents::value_type::const_iterator it;
         size_t c=0;
         for (it = comp[i].begin();
             it != comp[i].end();
@@ -128,6 +121,9 @@ huginApp::huginApp()
 {
     DEBUG_TRACE("ctor");
     m_this=this;
+#if wxUSE_ON_FATAL_EXCEPTION
+    wxHandleFatalExceptions();
+#endif
 }
 
 huginApp::~huginApp()
@@ -170,20 +166,15 @@ bool huginApp::OnInit()
     wxHelpControllerHelpProvider* provider = new wxHelpControllerHelpProvider;
     wxHelpProvider::Set(provider);
 
-    wxString huginExeDir = getExePath(argv[0]);
-
-    wxString huginRoot;
-    wxFileName::SplitPath( huginExeDir, &huginRoot, NULL, NULL );
-
-    m_xrcPrefix = huginRoot + wxT("/share/hugin/xrc/");
-	m_DataDir = huginRoot + wxT("/share/hugin/data/");
-    m_utilsBinDir = huginRoot + wxT("/bin/");
+    wxFileName exePath(wxStandardPaths::Get().GetExecutablePath());
+    m_utilsBinDir = exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR);
+    exePath.RemoveLastDir();
+    const wxString huginRoot=exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR);
+    m_xrcPrefix = huginRoot + wxT("share\\hugin\\xrc\\");
+    m_DataDir = huginRoot + wxT("share\\hugin\\data\\");
 
     // locale setup
-    locale.AddCatalogLookupPathPrefix(huginRoot + wxT("/share/locale"));
-    // lensfun database init
-    wxString lensfunDBPath=huginRoot + wxT("/share/lensfun");
-    HuginBase::LensDB::LensDB::GetSingleton().SetMainDBPath(std::string(lensfunDBPath.mb_str(HUGIN_CONV_FILENAME)));
+    locale.AddCatalogLookupPathPrefix(huginRoot + wxT("share\\locale"));
 
 #elif defined __WXMAC__ && defined MAC_SELF_CONTAINED_BUNDLE
     // initialize paths
@@ -223,16 +214,6 @@ bool huginApp::OnInit()
         }
     }
 	
-	{
-		wxString thePath = MacGetPathToBundledResourceFile(CFSTR("lensfun"));
-		if (thePath == wxT("")) {
-			wxMessageBox(_("lensfun directory not found in bundle"),
-						 _("Fatal Error"));
-			return false;
-		}
-		HuginBase::LensDB::LensDB::GetSingleton().SetMainDBPath(std::string(thePath.mb_str(HUGIN_CONV_FILENAME)));
-	}
-
 #else
     // add the locale directory specified during configure
     m_xrcPrefix = wxT(INSTALL_XRC_DIR);
@@ -300,6 +281,7 @@ bool huginApp::OnInit()
     wxXmlResource::Get()->AddHandler(new wxTreeListCtrlXmlHandler());
 #endif
     wxXmlResource::Get()->AddHandler(new ImagesTreeCtrlXmlHandler());
+    wxXmlResource::Get()->AddHandler(new CPListCtrlXmlHandler());
 
     // load XRC files
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("cp_list_frame.xrc"));
@@ -339,6 +321,22 @@ bool huginApp::OnInit()
     provider->SetHelpController(&frame->GetHelpController());
     frame->GetHelpController().Initialize(m_xrcPrefix+wxT("data/hugin_help_en_EN.chm"));
     frame->SendSizeEvent();
+#if wxCHECK_VERSION(3,1,0)
+    wxTaskBarJumpList jumpList;
+    wxFileName exeFile(wxStandardPaths::Get().GetExecutablePath());
+    exeFile.SetName("PTBatcherGUI");
+    wxTaskBarJumpListItem *item1 = new wxTaskBarJumpListItem(
+        NULL, wxTASKBAR_JUMP_LIST_TASK, _("Open Batch Processor"), exeFile.GetFullPath(), wxEmptyString,
+        _("Opens PTBatcher, the batch processor for Hugin's project files"),
+        exeFile.GetFullPath(), 0);
+    jumpList.GetTasks().Append(item1);
+    exeFile.SetName("calibrate_lens_gui");
+    wxTaskBarJumpListItem *item2 = new wxTaskBarJumpListItem(
+        NULL, wxTASKBAR_JUMP_LIST_TASK, _("Open Lens calibrate tool"), exeFile.GetFullPath(), wxEmptyString,
+        _("Opens Calibrate_lens_gui, a simple GUI for lens calibration"),
+        exeFile.GetFullPath(), 0);
+    jumpList.GetTasks().Append(item2);
+#endif
 #endif
 
     // we are closing Hugin, if the top level window is deleted
@@ -393,8 +391,8 @@ bool huginApp::OnInit()
     DEBUG_DEBUG("using temp dir: " << m_workDir.mb_str(wxConvLocal));
 
     // set some suitable defaults
-    GlobalCmdHist::getInstance().addCommand(new wxNewProjectCmd(pano));
-    GlobalCmdHist::getInstance().clear();
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(new PanoCommand::wxNewProjectCmd(pano));
+    PanoCommand::GlobalCmdHist::getInstance().clear();
 
     // suppress tiff warnings
     TIFFSetWarningHandler(0);
@@ -477,11 +475,11 @@ bool huginApp::OnInit()
             }
             if(filesv.size()>0)
             {
-                std::vector<PT::PanoCommand*> cmds;
-                cmds.push_back(new PT::wxAddImagesCmd(pano,filesv));
-                cmds.push_back(new PT::DistributeImagesCmd(pano));
-                cmds.push_back(new PT::CenterPanoCmd(pano));
-                GlobalCmdHist::getInstance().addCommand(new PT::CombinedPanoCommand(pano, cmds));
+                std::vector<PanoCommand::PanoCommand*> cmds;
+                cmds.push_back(new PanoCommand::wxAddImagesCmd(pano,filesv));
+                cmds.push_back(new PanoCommand::DistributeImagesCmd(pano));
+                cmds.push_back(new PanoCommand::CenterPanoCmd(pano));
+                PanoCommand::GlobalCmdHist::getInstance().addCommand(new PanoCommand::CombinedPanoCommand(pano, cmds));
             }
         }
     }
@@ -565,6 +563,13 @@ void huginApp::MacOpenFile(const wxString &fileName)
 }
 #endif
 
+#if wxUSE_ON_FATAL_EXCEPTION
+void huginApp::OnFatalException()
+{
+    GenerateReport(wxDebugReport::Context_Exception);
+};
+#endif
+
 huginApp * huginApp::m_this = 0;
 
 
@@ -580,7 +585,7 @@ void RestoreFramePosition(wxTopLevelWindow * frame, const wxString & basename)
     int dx,dy;
     wxDisplaySize(&dx,&dy);
 
-#if ( __WXGTK__ ) &&  wxCHECK_VERSION(2,6,0)
+#if ( __WXGTK__ )
 // restoring the splitter positions properly when maximising doesn't work.
 // Disabling maximise on wxWidgets >= 2.6.0 and gtk
         //size
@@ -631,7 +636,7 @@ void StoreFramePosition(wxTopLevelWindow * frame, const wxString & basename)
 
     wxConfigBase * config = wxConfigBase::Get();
 
-#if ( __WXGTK__ ) &&  wxCHECK_VERSION(2,6,0)
+#if ( __WXGTK__ )
 // restoring the splitter positions properly when maximising doesn't work.
 // Disabling maximise on wxWidgets >= 2.6.0 and gtk
     
