@@ -17,8 +17,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -32,21 +32,25 @@
 #include <ext/stdio_filebuf.h>
 #endif
 
-#include "PT/Panorama.h"
-#include "PT/ImageGraph.h"
+#include "algorithms/optimizer/ImageGraph.h"
 
 #include <hugin_utils/platform.h>
 #include <wx/app.h>
 #include "hugin/config_defaults.h"
+#include "panodata/StandardImageVariableGroups.h"
 #include "icpfind/AutoCtrlPointCreator.h"
 #include <algorithms/optimizer/PTOptimizer.h>
 #include <algorithms/basic/CalculateOverlap.h>
+#include <algorithms/basic/CalculateOptimalScale.h>
 
 #include "base_wx/MyExternalCmdExecDialog.h"
 #include "base_wx/platform.h"
 #include "base_wx/huginConfig.h"
 #include "base_wx/wxPlatform.h"
 #include <wx/utils.h>
+#ifdef __WXMSW__
+#include <wx/stdpaths.h>
+#endif
 
 // somewhere SetDesc gets defined.. this breaks wx/cmdline.h on OSX
 #ifdef SetDesc
@@ -61,7 +65,7 @@
 #endif
 
 using namespace std;
-using namespace PT;
+using namespace HuginBase;
 using namespace hugin_utils;
 
 void CPMessage(const wxString message,const wxString caption, wxWindow *parent)
@@ -89,62 +93,17 @@ int CPExecute(wxString prog, wxString args, wxString caption, wxWindow *parent)
     };
 };
 
-CPVector AutoCtrlPointCreator::readUpdatedControlPoints(const std::string & file,
-                                                    PT::Panorama & pano)
-{
-    ifstream stream(file.c_str());
-    if (! stream.is_open()) {
-        DEBUG_ERROR("Could not open autopano output: " << file);
-        return CPVector();
-    }
-
-    Panorama tmpp;
-    PanoramaMemento newPano;
-    int ptoVersion = 0;
-    newPano.loadPTScript(stream, ptoVersion, "");
-    tmpp.setMemento(newPano);
-
-    // create mapping between the panorama images.
-    map<unsigned int, unsigned int> imgMapping;
-    for (unsigned int ni = 0; ni < tmpp.getNrOfImages(); ni++) {
-        std::string nname = stripPath(tmpp.getImage(ni).getFilename());
-        for (unsigned int oi=0; oi < pano.getNrOfImages(); oi++) {
-            std::string oname = stripPath(pano.getImage(oi).getFilename());
-            if (nname == oname) {
-                // insert image
-                imgMapping[ni] = oi;
-                break;
-            }
-        }
-        if (! set_contains(imgMapping, ni)) {
-            DEBUG_ERROR("Could not find image " << ni << ", name: " << tmpp.getImage(ni).getFilename() << " in autopano output");
-            return CPVector();
-        }
-    }
-
-
-    // get control points
-    CPVector ctrlPoints = tmpp.getCtrlPoints();
-    // make sure they are in correct order
-    for (CPVector::iterator it= ctrlPoints.begin(); it != ctrlPoints.end(); ++it) {
-        (*it).image1Nr = imgMapping[(*it).image1Nr];
-        (*it).image2Nr = imgMapping[(*it).image2Nr];
-    }
-
-    return ctrlPoints;
-}
-
-CPVector AutoCtrlPointCreator::readUpdatedControlPoints(const std::string & file,
-                                                    PT::Panorama & pano, const PT::UIntSet & imgs)
+HuginBase::CPVector AutoCtrlPointCreator::readUpdatedControlPoints(const std::string & file,
+                                                    HuginBase::Panorama & pano, const HuginBase::UIntSet & imgs, bool reordered)
 {
     ifstream stream(file.c_str());
     if (! stream.is_open()) {
         DEBUG_ERROR("Could not open control point detector output: " << file);
-        return CPVector();
+        return HuginBase::CPVector();
     }
 
     Panorama tmpp;
-    PanoramaMemento newPano;
+    HuginBase::PanoramaMemento newPano;
     int ptoVersion = 0;
     newPano.loadPTScript(stream, ptoVersion, "");
     tmpp.setMemento(newPano);
@@ -152,21 +111,41 @@ CPVector AutoCtrlPointCreator::readUpdatedControlPoints(const std::string & file
     //check if sizes matches
     if(tmpp.getNrOfImages()!=imgs.size())
     {
-        return CPVector();
+        return HuginBase::CPVector();
     };
 
     // create mapping between the panorama images.
-    vector<size_t> imgMapping(imgs.size());
-    size_t i=0;
-    for(UIntSet::const_iterator it=imgs.begin();it!=imgs.end();++it)
-    {
-        imgMapping[i++]=*it;
-    };
+    map<unsigned int, unsigned int> imgMapping;
 
+    // create mapping between the panorama images.
+    if (reordered) {
+        for (unsigned int ni = 0; ni < tmpp.getNrOfImages(); ni++) {
+            std::string nname = stripPath(tmpp.getImage(ni).getFilename());
+            for(UIntSet::const_iterator it=imgs.begin();it!=imgs.end();++it) {
+                std::string oname = stripPath(pano.getImage(*it).getFilename());
+                if (nname == oname) {
+                    // insert image
+                    imgMapping[ni] = *it;
+                    break;
+                }
+            }
+            if (! set_contains(imgMapping, ni)) {
+                DEBUG_ERROR("Could not find image " << ni << ", name: " << tmpp.getImage(ni).getFilename() << " in autopano output");
+                return HuginBase::CPVector();
+            }
+        }
+    } else {
+        size_t i=0;
+        for(UIntSet::const_iterator it=imgs.begin();it!=imgs.end();++it)
+        {
+            imgMapping[i++]=*it;
+        };
+    }
+    
     // get control points
-    CPVector ctrlPoints = tmpp.getCtrlPoints();
+    HuginBase::CPVector ctrlPoints = tmpp.getCtrlPoints();
     // make sure they are in correct order
-    for (CPVector::iterator it= ctrlPoints.begin(); it != ctrlPoints.end(); ++it) {
+    for (HuginBase::CPVector::iterator it = ctrlPoints.begin(); it != ctrlPoints.end(); ++it) {
         (*it).image1Nr = imgMapping[(*it).image1Nr];
         (*it).image2Nr = imgMapping[(*it).image2Nr];
     }
@@ -203,7 +182,8 @@ wxString GetProgPath(wxString progName)
     else
     {
         wxPathList pathlist;
-        pathlist.Add(getExePath(wxTheApp->argv[0]));
+        const wxFileName exePath(wxStandardPaths::Get().GetExecutablePath());
+        pathlist.Add(exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR));
         pathlist.AddEnvList(wxT("PATH"));
         return pathlist.FindAbsoluteValidPath(progName);
     };
@@ -228,7 +208,8 @@ bool CanStartProg(wxString progName,wxWindow* parent)
     {
         wxPathList pathlist;
 #ifdef __WXMSW__
-        pathlist.Add(getExePath(wxTheApp->argv[0]));
+        const wxFileName exePath(wxStandardPaths::Get().GetExecutablePath());
+        pathlist.Add(exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR));
 #endif
         pathlist.AddEnvList(wxT("PATH"));
         wxString path = pathlist.FindAbsoluteValidPath(progName);
@@ -247,21 +228,21 @@ bool CanStartProg(wxString progName,wxWindow* parent)
     return canStart;
 };
 
-CPVector AutoCtrlPointCreator::automatch(CPDetectorSetting &setting, PT::Panorama & pano, const PT::UIntSet & imgs,
+HuginBase::CPVector AutoCtrlPointCreator::automatch(CPDetectorSetting &setting, HuginBase::Panorama & pano, const HuginBase::UIntSet & imgs,
                            int nFeatures, wxWindow *parent)
 {
     int return_value;
     return automatch(setting,pano,imgs,nFeatures,return_value,parent);
 };
 
-CPVector AutoCtrlPointCreator::automatch(CPDetectorSetting &setting, 
+HuginBase::CPVector AutoCtrlPointCreator::automatch(CPDetectorSetting &setting,
                                          Panorama & pano,
                                          const UIntSet & imgs,
                                          int nFeatures,
                                          int & ret_value, 
                                          wxWindow *parent)
 {
-    CPVector cps;
+    HuginBase::CPVector cps;
     CPDetectorType t = setting.GetType();
     //check, if the cp generators exists
     if(!CanStartProg(setting.GetProg(),parent))
@@ -328,7 +309,7 @@ CPVector AutoCtrlPointCreator::automatch(CPDetectorSetting &setting,
     return cps;
 }
 
-void AutoCtrlPointCreator::Cleanup(CPDetectorSetting &setting, PT::Panorama & pano, const PT::UIntSet & imgs,
+void AutoCtrlPointCreator::Cleanup(CPDetectorSetting &setting, HuginBase::Panorama & pano, const HuginBase::UIntSet & imgs,
                            std::vector<wxString> &keyFiles, wxWindow *parent)
 {
     if(setting.IsTwoStepDetector())
@@ -379,10 +360,10 @@ void AutoCtrlPointCreator::Cleanup(CPDetectorSetting &setting, PT::Panorama & pa
     };
 };
         
-CPVector AutoPanoSift::automatch(CPDetectorSetting &setting, Panorama & pano, const UIntSet & imgs,
+HuginBase::CPVector AutoPanoSift::automatch(CPDetectorSetting &setting, Panorama & pano, const UIntSet & imgs,
                                      int nFeatures, int & ret_value, wxWindow *parent)
 {
-    CPVector cps;
+    HuginBase::CPVector cps;
     if (imgs.size() == 0) {
         return cps;
     }
@@ -462,7 +443,7 @@ CPVector AutoPanoSift::automatch(CPDetectorSetting &setting, Panorama & pano, co
         //delete all existing control points in temp project
         //otherwise the existing control points will be loaded again
         Panorama tempPano=pano.duplicate();
-        CPVector emptyCPV;
+        HuginBase::CPVector emptyCPV;
         tempPano.setCtrlPoints(emptyCPV);
         tempPano.printPanoramaScript(ptoinstream, tempPano.getOptimizeVector(), tempPano.getOptions(), imgs, false);
     }
@@ -508,14 +489,7 @@ CPVector AutoPanoSift::automatch(CPDetectorSetting &setting, Panorama & pano, co
     }
 
     // read and update control points
-    if(use_inputscript)
-    {
-        cps = readUpdatedControlPoints((const char*)ptofile.mb_str(HUGIN_CONV_FILENAME), pano, imgs);
-    }
-    else
-    {
-        cps = readUpdatedControlPoints((const char *)ptofile.mb_str(HUGIN_CONV_FILENAME), pano);
-    };
+    cps = readUpdatedControlPoints((const char*)ptofile.mb_str(HUGIN_CONV_FILENAME), pano, imgs, !use_inputscript);
 
     if (namefile_name != wxString(wxT(""))) {
         namefile.Close();
@@ -533,10 +507,10 @@ CPVector AutoPanoSift::automatch(CPDetectorSetting &setting, Panorama & pano, co
     return cps;
 }
 
-CPVector AutoPanoSift::automatch(CPDetectorSetting &setting, PT::Panorama & pano, const PT::UIntSet & imgs,
+HuginBase::CPVector AutoPanoSift::automatch(CPDetectorSetting &setting, HuginBase::Panorama & pano, const HuginBase::UIntSet & imgs,
                            int nFeatures, vector<wxString> &keyFiles, int & ret_value, wxWindow *parent)
 {
-    CPVector cps;
+    HuginBase::CPVector cps;
     if (imgs.size() == 0) 
     {
         return cps;
@@ -675,7 +649,7 @@ CPVector AutoPanoSift::automatch(CPDetectorSetting &setting, PT::Panorama & pano
     }
 
     // read and update control points
-    cps = readUpdatedControlPoints((const char *)ptofile.mb_str(HUGIN_CONV_FILENAME), pano);
+    cps = readUpdatedControlPoints((const char *)ptofile.mb_str(HUGIN_CONV_FILENAME), pano, imgs, true);
 
     if (!wxRemoveFile(ptofile)) {
         DEBUG_DEBUG("could not remove temporary file: " << ptofile.c_str());
@@ -684,10 +658,10 @@ CPVector AutoPanoSift::automatch(CPDetectorSetting &setting, PT::Panorama & pano
     return cps;
 };
 
-CPVector AutoPanoKolor::automatch(CPDetectorSetting &setting, Panorama & pano, const UIntSet & imgs,
+HuginBase::CPVector AutoPanoKolor::automatch(CPDetectorSetting &setting, Panorama & pano, const UIntSet & imgs,
                               int nFeatures, int & ret_value, wxWindow *parent)
 {
-    CPVector cps;
+    HuginBase::CPVector cps;
     wxString autopanoExe = setting.GetProg();
 
     // write default autopano.kolor.com flags
@@ -759,7 +733,7 @@ CPVector AutoPanoKolor::automatch(CPDetectorSetting &setting, Panorama & pano, c
         return cps;
     }
     // read and update control points
-    cps = readUpdatedControlPoints((const char *)ptofile.mb_str(HUGIN_CONV_FILENAME), pano);
+    cps = readUpdatedControlPoints((const char *)ptofile.mb_str(HUGIN_CONV_FILENAME), pano, imgs, true);
 
     if (!wxRemoveFile(ptofile)) {
         DEBUG_DEBUG("could not remove temporary file: " << ptofile.c_str());
@@ -779,7 +753,7 @@ struct stack_img
 };
 bool sort_img_ev (img_ev i1, img_ev i2) { return (i1.ev<i2.ev); };
 
-void AddControlPointsWithCheck(CPVector &cpv, CPVector &new_cp, Panorama *pano=NULL)
+void AddControlPointsWithCheck(HuginBase::CPVector &cpv, HuginBase::CPVector &new_cp, Panorama *pano = NULL)
 {
     for(unsigned int i=0;i<new_cp.size();i++)
     {
@@ -802,10 +776,10 @@ void AddControlPointsWithCheck(CPVector &cpv, CPVector &new_cp, Panorama *pano=N
     };
 };
 
-CPVector AutoPanoSiftStack::automatch(CPDetectorSetting &setting, Panorama & pano, const UIntSet & imgs,
+HuginBase::CPVector AutoPanoSiftStack::automatch(CPDetectorSetting &setting, Panorama & pano, const UIntSet & imgs,
                                      int nFeatures, int & ret_value, wxWindow *parent)
 {
-    CPVector cps;
+    HuginBase::CPVector cps;
     if (imgs.size() == 0) {
         return cps;
     };
@@ -876,7 +850,7 @@ CPVector AutoPanoSiftStack::automatch(CPDetectorSetting &setting, Panorama & pan
             if(images_stack.size()>1)
             {
                 AutoPanoSift matcher;
-                CPVector new_cps=matcher.automatch(stack_setting, pano, images_stack, nFeatures, ret_value, parent);
+                HuginBase::CPVector new_cps = matcher.automatch(stack_setting, pano, images_stack, nFeatures, ret_value, parent);
                 if(new_cps.size()>0)
                     AddControlPointsWithCheck(cps,new_cps);
                 if(ret_value!=0)
@@ -887,10 +861,10 @@ CPVector AutoPanoSiftStack::automatch(CPDetectorSetting &setting, Panorama & pan
     return cps;
 };
 
-CPVector AutoPanoSiftMultiRow::automatch(CPDetectorSetting &setting, Panorama & pano, const UIntSet & imgs,
+HuginBase::CPVector AutoPanoSiftMultiRow::automatch(CPDetectorSetting &setting, Panorama & pano, const UIntSet & imgs,
                                      int nFeatures, int & ret_value, wxWindow *parent)
 {
-    CPVector cps;
+    HuginBase::CPVector cps;
     if (imgs.size() < 2) 
     {
         return cps;
@@ -909,7 +883,7 @@ CPVector AutoPanoSiftMultiRow::automatch(CPDetectorSetting &setting, Panorama &
         ++it;
         ImagePair.insert(*it);
         AutoPanoSift matcher;
-        CPVector new_cps;
+        HuginBase::CPVector new_cps;
         new_cps.clear();
         if(setting.IsTwoStepDetector())
             new_cps=matcher.automatch(setting, pano, ImagePair, nFeatures, keyFiles, ret_value, parent);
@@ -928,13 +902,13 @@ CPVector AutoPanoSiftMultiRow::automatch(CPDetectorSetting &setting, Panorama &
     UIntSet allImgs;
     fill_set(allImgs, 0, pano.getNrOfImages()-1);
     Panorama optPano=pano.getSubset(allImgs);
-    for (CPVector::const_iterator it=cps.begin();it!=cps.end();++it)
+    for (HuginBase::CPVector::const_iterator it = cps.begin(); it != cps.end(); ++it)
         optPano.addCtrlPoint(*it);
 
-    CPGraph graph;
-    createCPGraph(optPano, graph);
-    CPComponents comps;
-    int n = findCPComponents(graph, comps);
+    HuginBase::CPGraph graph;
+    HuginBase::createCPGraph(optPano, graph);
+    HuginBase::CPComponents comps;
+    int n = HuginBase::findCPComponents(graph, comps);
     if(n>1)
     {
         UIntSet ImagesGroups;
@@ -945,7 +919,7 @@ CPVector AutoPanoSiftMultiRow::automatch(CPDetectorSetting &setting, Panorama &
                 ImagesGroups.insert(*(comps[i].rbegin()));
         };
         AutoPanoSift matcher;
-        CPVector new_cps;
+        HuginBase::CPVector new_cps;
         if(setting.IsTwoStepDetector())
             new_cps=matcher.automatch(setting, optPano, ImagesGroups, nFeatures, keyFiles, ret_value, parent);
         else
@@ -957,28 +931,29 @@ CPVector AutoPanoSiftMultiRow::automatch(CPDetectorSetting &setting, Panorama &
             Cleanup(setting, pano, imgs, keyFiles, parent);
             return cps;
         };
-        createCPGraph(optPano,graph);
-        n=findCPComponents(graph, comps);
+        HuginBase::createCPGraph(optPano,graph);
+        n=HuginBase::findCPComponents(graph, comps);
     };
     if(n==1 && setting.GetOption())
     {
         //next steps happens only when all images are connected;
         //now optimize panorama
-        PanoramaOptions opts = pano.getOptions();
-        opts.setProjection(PanoramaOptions::EQUIRECTANGULAR);
+        HuginBase::PanoramaOptions opts = pano.getOptions();
+        opts.setProjection(HuginBase::PanoramaOptions::EQUIRECTANGULAR);
         // calculate proper scaling, 1:1 resolution.
         // Otherwise optimizer distances are meaningless.
         opts.setWidth(30000, false);
         opts.setHeight(15000);
 
         optPano.setOptions(opts);
-        int w = optPano.calcOptimalWidth();
+        int w = hugin_utils::roundi(HuginBase::CalculateOptimalScale::calcOptimalScale(optPano) * optPano.getOptions().getWidth());
+
         opts.setWidth(w);
         opts.setHeight(w/2);
         optPano.setOptions(opts);
 
         //generate optimize vector, optimize only yaw and pitch
-        OptimizeVector optvars;
+        HuginBase::OptimizeVector optvars;
         const SrcPanoImage & anchorImage = optPano.getImage(opts.optimizeReferenceImage);
         for (unsigned i=0; i < optPano.getNrOfImages(); i++) 
         {
@@ -1002,10 +977,10 @@ CPVector AutoPanoSiftMultiRow::automatch(CPDetectorSetting &setting, Panorama &
         optPano.setOptimizeVector(optvars);
 
         // remove vertical and horizontal control points
-        CPVector backupOldCPS = optPano.getCtrlPoints();
-        CPVector backupNewCPS;
-        for (CPVector::const_iterator it = backupOldCPS.begin(); it != backupOldCPS.end(); ++it) {
-            if (it->mode == ControlPoint::X_Y)
+        HuginBase::CPVector backupOldCPS = optPano.getCtrlPoints();
+        HuginBase::CPVector backupNewCPS;
+        for (HuginBase::CPVector::const_iterator it = backupOldCPS.begin(); it != backupOldCPS.end(); ++it) {
+            if (it->mode == HuginBase::ControlPoint::X_Y)
             {
                 backupNewCPS.push_back(*it);
             }
@@ -1027,7 +1002,7 @@ CPVector AutoPanoSiftMultiRow::automatch(CPDetectorSetting &setting, Panorama &
             newSetting.SetArgsMatcher(setting.GetArgsMatcher());
         };
         newSetting.SetOption(true);
-        CPVector new_cps;
+        HuginBase::CPVector new_cps;
         if(setting.IsTwoStepDetector())
             new_cps=matcher.automatch(newSetting, optPano, imgs, nFeatures, keyFiles, ret_value, parent);
         else
@@ -1039,10 +1014,10 @@ CPVector AutoPanoSiftMultiRow::automatch(CPDetectorSetting &setting, Panorama &
     return cps;
 };
 
-CPVector AutoPanoSiftMultiRowStack::automatch(CPDetectorSetting &setting, Panorama & pano, const UIntSet & imgs,
+HuginBase::CPVector AutoPanoSiftMultiRowStack::automatch(CPDetectorSetting &setting, Panorama & pano, const UIntSet & imgs,
                                      int nFeatures, int & ret_value, wxWindow *parent)
 {
-    CPVector cps;
+    HuginBase::CPVector cps;
     if (imgs.size() == 0) {
         return cps;
     };
@@ -1105,7 +1080,7 @@ CPVector AutoPanoSiftMultiRowStack::automatch(CPDetectorSetting &setting, Panora
             if(images_stack.size()>1)
             {
                 AutoPanoSift matcher;
-                CPVector new_cps=matcher.automatch(stack_setting, pano, images_stack, nFeatures, ret_value, parent);
+                HuginBase::CPVector new_cps = matcher.automatch(stack_setting, pano, images_stack, nFeatures, ret_value, parent);
                 if(new_cps.size()>0)
                     AddControlPointsWithCheck(cps,new_cps);
                 if(ret_value!=0)
@@ -1124,28 +1099,28 @@ CPVector AutoPanoSiftMultiRowStack::automatch(CPDetectorSetting &setting, Panora
         fill_set(allImgs, 0, pano.getNrOfImages()-1);
         Panorama newPano=pano.getSubset(allImgs);
         if(cps.size()>0)
-            for (CPVector::const_iterator it=cps.begin();it!=cps.end();++it)
+            for (HuginBase::CPVector::const_iterator it = cps.begin(); it != cps.end(); ++it)
                 newPano.addCtrlPoint(*it);
 
         AutoPanoSiftMultiRow matcher;
-        CPVector new_cps=matcher.automatch(setting, newPano, images_layer, nFeatures, ret_value, parent);
+        HuginBase::CPVector new_cps = matcher.automatch(setting, newPano, images_layer, nFeatures, ret_value, parent);
         if(new_cps.size()>0)
             AddControlPointsWithCheck(cps,new_cps);
     };
     return cps;
 };
 
-CPVector AutoPanoSiftPreAlign::automatch(CPDetectorSetting &setting, Panorama & pano, const UIntSet & imgs,
+HuginBase::CPVector AutoPanoSiftPreAlign::automatch(CPDetectorSetting &setting, Panorama & pano, const UIntSet & imgs,
                                      int nFeatures, int & ret_value, wxWindow *parent)
 {
     std::vector<wxString> keyFiles(pano.getNrOfImages());
     return automatch(setting, pano, imgs, nFeatures, keyFiles, ret_value, parent);
 };
 
-CPVector AutoPanoSiftPreAlign::automatch(CPDetectorSetting &setting, Panorama & pano, const UIntSet & imgs,
+HuginBase::CPVector AutoPanoSiftPreAlign::automatch(CPDetectorSetting &setting, Panorama & pano, const UIntSet & imgs,
                                          int nFeatures, std::vector<wxString> &keyFiles, int & ret_value, wxWindow *parent)
 {
-    CPVector cps;
+    HuginBase::CPVector cps;
     if (imgs.size()<2) 
         return cps;
     DEBUG_ASSERT(keyFiles.size()==pano.getNrOfImages());
@@ -1155,10 +1130,10 @@ CPVector AutoPanoSiftPreAlign::automatch(CPDetectorSetting &setting, Panorama &
     if(setting.GetOption())
     {
         //only work on not connected image pairs
-        CPVector oldCps=pano.getCtrlPoints();
+        HuginBase::CPVector oldCps = pano.getCtrlPoints();
         for(unsigned i=0;i<oldCps.size();i++)
         {
-            if(oldCps[i].mode==ControlPoint::X_Y)
+            if (oldCps[i].mode == HuginBase::ControlPoint::X_Y)
             {
                 usedImages[oldCps[i].image1Nr].insert(oldCps[i].image2Nr);
                 usedImages[oldCps[i].image2Nr].insert(oldCps[i].image1Nr);
@@ -1191,7 +1166,7 @@ CPVector AutoPanoSiftPreAlign::automatch(CPDetectorSetting &setting, Panorama &
             for(UIntSet::const_iterator img_it2=images.begin();img_it2!=images.end();++img_it2)
                 usedImages[*img_it].insert(*img_it2);
         AutoPanoSift matcher;
-        CPVector new_cps;
+        HuginBase::CPVector new_cps;
         if(setting.IsTwoStepDetector())
             new_cps=matcher.automatch(setting, pano, images, nFeatures, keyFiles, ret_value, parent);
         else
