@@ -19,12 +19,11 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
-#include <hugin_version.h>
 #include "icpfind.h"
 
 #include <fstream>
@@ -36,6 +35,11 @@
 #include "hugin/config_defaults.h"
 #include <vigra/impex.hxx>
 
+extern "C"
+{
+    #include <pano13/filter.h>  // for PT_setProgressFcn, PT_setInfoDlgFcn
+}
+
 using namespace std;
  
 void iCPApp::ReadDetectorConfig()
@@ -102,7 +106,7 @@ int iCPApp::OnRun()
 {
     ReadDetectorConfig();
     //read input project
-    PT::PanoramaMemento newPano;
+    HuginBase::PanoramaMemento newPano;
     int ptoVersion = 0;
     wxFileName file(m_input);
     file.MakeAbsolute();
