@@ -19,13 +19,17 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
 #include "Batch.h"
 #include <wx/stdpaths.h>
+#ifdef __WXMSW__
+#include <powrprof.h>
+#pragma comment(lib, "PowrProf.lib")
+#endif
 
 BEGIN_EVENT_TABLE(Batch, wxFrame)
     EVT_END_PROCESS(-1, Batch::OnProcessTerminate)
@@ -41,12 +45,11 @@ DEFINE_EVENT_TYPE(EVT_INFORMATION)
 DEFINE_EVENT_TYPE(EVT_UPDATE_PARENT)
 #endif
 
-Batch::Batch(wxFrame* parent, wxString path) : wxFrame(parent,wxID_ANY,_T("Batch"))
+Batch::Batch(wxFrame* parent) : wxFrame(parent, wxID_ANY, _T("Batch"))
 {
     //default flag settings
-    parallel = false;
     deleteFiles = false;
-    shutdown = false;
+    atEnd = DO_NOTHING;
     overwrite = true;
     verbose = false;
     autoremove = false;
@@ -61,22 +64,6 @@ Batch::Batch(wxFrame* parent, wxString path) : wxFrame(parent,wxID_ANY,_T("Batch
     // Required to access the preferences of hugin
     //SetAppName(wxT("hugin"));
 
-    // setup the environment for the different operating systems
-    wxConfigBase* config=wxConfigBase::Get();
-#if defined __WXMSW__
-    wxString huginExeDir = getExePath(path);
-
-    wxString huginRoot;
-    wxFileName::SplitPath(huginExeDir, &huginRoot, NULL, NULL);
-
-    progs = getPTProgramsConfig(huginExeDir, config);
-    progsAss = getAssistantProgramsConfig(huginExeDir, config);
-#else
-    // add the locale directory specified during configure
-    progs = getPTProgramsConfig(wxT(""), config);
-    progsAss = getAssistantProgramsConfig(wxT(""), config);
-#endif
-
 }
 
 void Batch::AddAppToBatch(wxString app)
@@ -137,13 +124,44 @@ void Batch::AppendBatchFile(wxString file)
         //TO-DO: batch file error checking?
         //first line in file is idGenerator, we save it a temp variable, cause it gets set when adding projects
         long idGenTemp = 1;
+        if (fileStream.Eof())
+        {
+            return;
+        };
         textStream.ReadLine().ToLong(&idGenTemp);
         //then for each project: project path, prefix, id, status, skip
+        if (fileStream.Eof())
+        {
+            return;
+        };
         while((projectName = textStream.ReadLine()).Cmp(wxT(""))!=0)
         {
-            //we add project to internal list
+            // read all line, check before reading
+            if (fileStream.Eof())
+            {
+                break;
+            };
             wxString line=textStream.ReadLine();
-            if(line.IsEmpty())
+            if (fileStream.Eof())
+            {
+                break;
+            };
+            long id;
+            textStream.ReadLine().ToLong(&id);
+            long status;
+            if (fileStream.Eof())
+            {
+                break;
+            };
+            textStream.ReadLine().ToLong(&status);
+            if (fileStream.Eof())
+            {
+                break;
+            };
+            const bool skip = textStream.ReadLine().StartsWith(_T("T"));
+
+            //we add project to internal list
+            if (line.IsEmpty())
             {
                 AddProjectToBatch(projectName,wxT(""),Project::DETECTING);
             }
@@ -151,19 +169,22 @@ void Batch::AppendBatchFile(wxString file)
             {
                 AddProjectToBatch(projectName,line);
             };
-            textStream.ReadLine().ToLong(&m_projList.Last().id);
-            long status;
-            textStream.ReadLine().ToLong(&status);
             //if status was RUNNING or PAUSED, we set it to FAILED
             if(status==(long)Project::RUNNING || status==(long)Project::PAUSED)
             {
                 status=(long)Project::FAILED;
             }
+            m_projList.Last().id = id;
             m_projList.Last().status = (Project::Status)status;
-            if(textStream.ReadLine().StartsWith(_T("T")))
+            if(skip)
             {
                 m_projList.Last().skip = true;
-            }
+            };
+
+            if (fileStream.Eof())
+            {
+                break;
+            };
         }
         //we set the id generator we got from file
         Project::idGenerator = idGenTemp;
@@ -207,7 +228,7 @@ int Batch::ClearBatch()
 #else
                                 wxT(""),
 #endif
-                                wxYES | wxCANCEL | wxICON_INFORMATION);
+                                wxYES_NO | wxICON_INFORMATION);
         if(message.ShowModal()==wxID_YES)
         {
             CancelBatch();
@@ -513,59 +534,80 @@ void Batch::OnProcessTerminate(wxProcessEvent& event)
                 else
                 {
                     ((wxFrame*)GetParent())->SetStatusText(_("Batch completed with errors."));
-                    if(!shutdown)
+                    if(atEnd==DO_NOTHING)
                     {
                         //notify parent, that at least one project failed
+                        // show dialog only if we don't shutdown the computer or end PTBatcherGUI
                         wxCommandEvent e(EVT_BATCH_FAILED,wxID_ANY);
                         GetParent()->GetEventHandler()->AddPendingEvent(e);
                     };
-                }
-                if(shutdown)	//after we are finished we turn off the computer if checked
+                };
+                if (atEnd != DO_NOTHING)	//after we are finished we turn off the computer if checked
                 {
-                    wxProgressDialog progress(_("Initializing shutdown..."), _("Shutting down..."),49,this,
-                                                wxPD_AUTO_HIDE | wxPD_SMOOTH | wxPD_APP_MODAL | wxPD_CAN_ABORT | wxPD_CAN_SKIP);
-                    progress.Fit();
-                    int i = 0;
-                    bool skip = false;
-                    while(progress.Update(i, _("Shutting down..."),&skip))
+                    switch (atEnd)
                     {
-                        if(skip || i==50)
-                        {
-                            /*wxMessageDialog message(this,_T("Krneksa"));
-                            message.ShowModal();
-                            break;*/
-                            wxShutdown(wxSHUTDOWN_POWEROFF);
-                        }
-                        i++;
-#if defined __WXMSW__
-                        Sleep(200);
-#else
-                        sleep(200);
+                        case CLOSE_PTBATCHERGUI:
+                            GetParent()->Close();
+                            break;
+                        case SHUTDOWN:
+                            {
+                                wxProgressDialog progress(_("Initializing shutdown..."), _("Shutting down..."), 49, this,
+                                    wxPD_AUTO_HIDE | wxPD_SMOOTH | wxPD_APP_MODAL | wxPD_CAN_ABORT | wxPD_CAN_SKIP);
+                                progress.Fit();
+                                int i = 0;
+                                bool skip = false;
+                                while (progress.Update(i, _("Shutting down..."), &skip))
+                                {
+                                    if (skip || i == 50)
+                                    {
+                                        wxShutdown(wxSHUTDOWN_POWEROFF);
+                                    }
+                                    i++;
+    #if defined __WXMSW__
+                                    Sleep(200);
+    #else
+                                    sleep(200);
+    #endif
+                                }
+                                progress.Close();
+                            }
+                            break;
+                        case SUSPEND:
+                        case HIBERNATE:
+#ifdef __WXMSW__
+                            {
+                                wxString progressCaption(_("Prepare to hibernate..."));
+                                wxString progressLabel(_("Initializing hibernating..."));
+                                if (atEnd == SUSPEND)
+                                {
+                                    progressCaption = wxString(_("Prepare to suspend..."));
+                                    progressLabel = wxString(_("Initializing suspend mode..."));
+                                };
+                                wxProgressDialog progress(progressLabel, progressCaption, 49, this,
+                                    wxPD_AUTO_HIDE | wxPD_SMOOTH | wxPD_APP_MODAL | wxPD_CAN_ABORT | wxPD_CAN_SKIP);
+                                progress.Fit();
+                                int i = 0;
+                                bool skip = false;
+                                while (progress.Update(i, progressCaption, &skip))
+                                {
+                                    if (skip || i == 50)
+                                    {
+                                        SetSuspendState(atEnd==HIBERNATE, false, false);
+                                        break;
+                                    }
+                                    i++;
+                                    Sleep(200);
+                                }
+                                progress.Close();
+                            };
 #endif
-                    }
-                    progress.Close();
-                }
+                            break;
+                    };
+                };
             }
             else
             {
-                if(parallel)	//if we are running in parallel
-                {
-                    //the last executed process in parallel runs next
-                    if(GetRunningCount() == 0)
-                    {
-                        //SetStatusText(_T("Project \""+m_projList.Item(i).path)+_T("\" finished. Running next project..."));
-                        RunNextInBatch();
-                    }
-                    else
-                    {
-                        //SetStatusText(_T("Project \""+m_projList.Item(i).path)+_T("\" finished. Waiting for all in parallel to complete..."));
-                    }
-                }
-                else
-                {
-                    //SetStatusText(_T("Project \""+m_projList.Item(i).path)+_T("\" finished. Running next project..."));
-                    RunNextInBatch();
-                }
+                RunNextInBatch();
             }
         }
         else
@@ -582,19 +624,23 @@ void Batch::OnProcessTerminate(wxProcessEvent& event)
 
 bool Batch::OnStitch(wxString scriptFile, wxString outname, int id)
 {
+    // delete the existing wxConfig to force reloading of settings from file/registy
+    delete wxConfigBase::Set((wxConfigBase*)NULL);
+    wxConfigBase* config = wxConfigBase::Get();
     if(wxIsEmpty(scriptFile))
     {
-        wxString defaultdir = wxConfigBase::Get()->Read(wxT("/actualPath"),wxT(""));
+        wxString defaultdir = config->Read(wxT("/actualPath"),wxT(""));
         wxFileDialog dlg(0,
                          _("Specify project source project file"),
                          defaultdir, wxT(""),
-                         _("Project files (*.pto,*.ptp,*.pts,*.oto)|*.pto;*.ptp;*.pts;*.oto;|All files (*)|*"),
+                         _("Project files (*.pto)|*.pto|All files (*)|*"),
                          wxFD_OPEN, wxDefaultPosition);
 
         dlg.SetDirectory(wxConfigBase::Get()->Read(wxT("/actualPath"),wxT("")));
         if (dlg.ShowModal() == wxID_OK)
         {
-            wxConfig::Get()->Write(wxT("/actualPath"), dlg.GetDirectory());  // remember for later
+            config->Write(wxT("/actualPath"), dlg.GetDirectory());  // remember for later
+            config->Flush();
             wxFileDialog dlg2(0,_("Specify output prefix"),
                               wxConfigBase::Get()->Read(wxT("/actualPath"),wxT("")),
                               wxT(""), wxT(""),
@@ -646,7 +692,7 @@ bool Batch::OnStitch(wxString scriptFile, wxString outname, int id)
         stitchFrame->m_stitchPanel->SetOverwrite(true);
     }
 
-    bool n = stitchFrame->StitchProject(scriptFile, outname, progs);
+    bool n = stitchFrame->StitchProject(scriptFile, outname);
     if(n)
     {
         m_stitchFrames.Add(stitchFrame);
@@ -661,7 +707,9 @@ bool Batch::OnStitch(wxString scriptFile, wxString outname, int id)
 
 bool Batch::OnDetect(wxString scriptFile, int id)
 {
-    RunStitchFrame* stitchFrame = new RunStitchFrame(this, wxT("Hugin Assistant"), wxDefaultPosition, wxSize(640,600));
+    // delete the existing wxConfig to force reloading of settings from file/registy
+    delete wxConfigBase::Set((wxConfigBase*)NULL);
+    RunStitchFrame* stitchFrame = new RunStitchFrame(this, wxT("Hugin Assistant"), wxDefaultPosition, wxSize(640, 600));
     stitchFrame->SetProjectId(id);
     if(verbose)
     {
@@ -672,7 +720,7 @@ bool Batch::OnDetect(wxString scriptFile, int id)
     wxFileName basename(scriptFile);
     stitchFrame->SetTitle(wxString::Format(_("%s - Assistant"), basename.GetName().c_str()));
 
-    bool n = stitchFrame->DetectProject(scriptFile, progsAss);
+    bool n = stitchFrame->DetectProject(scriptFile);
     if(n)
     {
         m_stitchFrames.Add(stitchFrame);
@@ -780,7 +828,7 @@ void Batch::RunNextInBatch()
         //execute command line instructions
         if(m_projList.Item(i).id<0)
         {
-            SetStatusText(_("Running command \"")+m_projList.Item(i).path+_T("\""));
+            SetStatusText(wxString::Format(_("Running command \"%s\""), m_projList.Item(i).path.c_str()));
             m_projList.Item(i).status=Project::RUNNING;
             //we create a fake stitchFrame, so program waits for app to complete
             if(wxExecute(m_projList.Item(i).path, wxEXEC_SYNC)==0)
@@ -794,67 +842,29 @@ void Batch::RunNextInBatch()
         }
         else
         {
-            //we run in sequence
-            if(!parallel)
+            m_projList.Item(i).status=Project::RUNNING;
+            m_running = true;
+            if(m_projList.Item(i).target==Project::STITCHING)
             {
-                m_projList.Item(i).status=Project::RUNNING;
-                m_running = true;
-                if(m_projList.Item(i).target==Project::STITCHING)
-                {
-                    wxCommandEvent e(EVT_INFORMATION,wxID_ANY);
-                    e.SetString(wxString::Format(_("Now stitching: %s"),m_projList.Item(i).path.c_str()));
-                    GetParent()->GetEventHandler()->AddPendingEvent(e);
-                    value = OnStitch(m_projList.Item(i).path, m_projList.Item(i).prefix, m_projList.Item(i).id);
-                }
-                else
-                {
-                    wxCommandEvent e(EVT_INFORMATION,wxID_ANY);
-                    e.SetString(wxString::Format(_("Now detecting: %s"),m_projList.Item(i).path.c_str()));
-                    GetParent()->GetEventHandler()->AddPendingEvent(e);
-                    value = OnDetect(m_projList.Item(i).path,m_projList.Item(i).id);
-                };
-                if(!value)
-                {
-                    m_projList.Item(i).status=Project::FAILED;
-                }
-                else
-                {
-                    repeat = false;
-                }
+                wxCommandEvent e(EVT_INFORMATION,wxID_ANY);
+                e.SetString(wxString::Format(_("Now stitching: %s"),m_projList.Item(i).path.c_str()));
+                GetParent()->GetEventHandler()->AddPendingEvent(e);
+                value = OnStitch(m_projList.Item(i).path, m_projList.Item(i).prefix, m_projList.Item(i).id);
             }
             else
             {
-                while((i = GetFirstAvailable())!=-1)
-                {
-                    if(m_projList.Item(i).id<0)
-                    {
-                        break;
-                    }
-                    m_projList.Item(i).status=Project::RUNNING;
-                    m_running = true;
-                    if(m_projList.Item(i).target==Project::STITCHING)
-                    {
-                        wxCommandEvent e(EVT_INFORMATION,wxID_ANY);
-                        e.SetString(wxString::Format(_("Now stitching: %s"),m_projList.Item(i).path.c_str()));
-                        GetParent()->GetEventHandler()->AddPendingEvent(e);
-                        value = OnStitch(m_projList.Item(i).path, m_projList.Item(i).prefix, m_projList.Item(i).id);
-                    }
-                    else
-                    {
-                        wxCommandEvent e(EVT_INFORMATION,wxID_ANY);
-                        e.SetString(wxString::Format(_("Now detecting: %s"),m_projList.Item(i).path.c_str()));
-                        GetParent()->GetEventHandler()->AddPendingEvent(e);
-                        value = OnDetect(m_projList.Item(i).path,m_projList.Item(i).id);
-                    };
-                    if(!value)
-                    {
-                        m_projList.Item(i).status=Project::FAILED;
-                    }
-                    else
-                    {
-                        repeat = false;
-                    }
-                }
+                wxCommandEvent e(EVT_INFORMATION,wxID_ANY);
+                e.SetString(wxString::Format(_("Now detecting: %s"),m_projList.Item(i).path.c_str()));
+                GetParent()->GetEventHandler()->AddPendingEvent(e);
+                value = OnDetect(m_projList.Item(i).path,m_projList.Item(i).id);
+            };
+            if(!value)
+            {
+                m_projList.Item(i).status=Project::FAILED;
+            }
+            else
+            {
+                repeat = false;
             }
         }
     }
