@@ -19,8 +19,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -29,6 +29,13 @@
 #include "PTBatcherGUI.h"
 #include "FindPanoDialog.h"
 #include "FailedProjectsDialog.h"
+#ifdef __WXMSW__
+#include <powrprof.h>
+#pragma comment(lib, "PowrProf.lib")
+#if wxCHECK_VERSION(3,1,0)
+#include <wx/taskbarbutton.h>
+#endif
+#endif
 
 /* file drag and drop handler method */
 bool BatchDropTarget::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames)
@@ -103,9 +110,8 @@ BEGIN_EVENT_TABLE(BatchFrame, wxFrame)
     EVT_BUTTON(XRCID("button_edit"),BatchFrame::OnButtonOpenWithHugin)
     EVT_BUTTON(XRCID("button_move_up"),BatchFrame::OnButtonMoveUp)
     EVT_BUTTON(XRCID("button_move_down"),BatchFrame::OnButtonMoveDown)
-    EVT_CHECKBOX(XRCID("cb_parallel"), BatchFrame::OnCheckParallel)
     EVT_CHECKBOX(XRCID("cb_overwrite"), BatchFrame::OnCheckOverwrite)
-    EVT_CHECKBOX(XRCID("cb_shutdown"), BatchFrame::OnCheckShutdown)
+    EVT_CHOICE(XRCID("choice_end"), BatchFrame::OnChoiceEnd)
     EVT_CHECKBOX(XRCID("cb_verbose"), BatchFrame::OnCheckVerbose)
     EVT_CHECKBOX(XRCID("cb_autoremove"), BatchFrame::OnCheckAutoRemove)
     EVT_CHECKBOX(XRCID("cb_autostitch"), BatchFrame::OnCheckAutoStitch)
@@ -117,6 +123,7 @@ BEGIN_EVENT_TABLE(BatchFrame, wxFrame)
     EVT_COMMAND(wxID_ANY, EVT_BATCH_FAILED, BatchFrame::OnBatchFailed)
     EVT_COMMAND(wxID_ANY, EVT_INFORMATION, BatchFrame::OnBatchInformation)
     EVT_COMMAND(wxID_ANY, EVT_UPDATE_PARENT, BatchFrame::OnRefillListBox)
+    EVT_COMMAND(wxID_ANY, EVT_QUEUE_PROGRESS, BatchFrame::OnProgress)
     EVT_ICONIZE(BatchFrame::OnMinimize)
 END_EVENT_TABLE()
 
@@ -141,7 +148,9 @@ BatchFrame::BatchFrame(wxLocale* locale, wxString xrc)
     // create tool bar
     SetToolBar(wxXmlResource::Get()->LoadToolBar(this, wxT("batch_toolbar")));
 
-    CreateStatusBar(1);
+    int widths[2] = { -1, 150 };
+    CreateStatusBar(2);
+    SetStatusWidths(2, widths);
     SetStatusText(_("Not doing much..."));
 
     // set the minimize icon
@@ -179,7 +188,7 @@ BatchFrame::BatchFrame(wxLocale* locale, wxString xrc)
 #else
     m_tray=NULL;
 #endif
-    m_batch = new Batch(this,wxTheApp->argv[0]);
+    m_batch = new Batch(this);
 #if wxCHECK_VERSION(2,9,4)
     if(wxGetKeyState(WXK_COMMAND))
 #else
@@ -213,6 +222,29 @@ BatchFrame::BatchFrame(wxLocale* locale, wxString xrc)
         m_batch->SaveTemp();
     }
     projListBox = XRCCTRL(*this,"project_listbox",ProjectListBox);
+    // fill at end list box, check which options are available
+    m_endChoice = XRCCTRL(*this, "choice_end", wxChoice);
+    m_endChoice->Clear();
+    m_endChoice->Append(_("Do nothing"), (void*)Batch::DO_NOTHING);
+    m_endChoice->Append(_("Close PTBatcherGUI"), (void*)Batch::CLOSE_PTBATCHERGUI);
+#if !defined __WXMAC__ && !defined __WXOSX_COCOA__
+    // there is no wxShutdown for wxMac
+    m_endChoice->Append(_("Shutdown computer"), (void*)Batch::SHUTDOWN);
+#endif
+#ifdef __WXMSW__
+    SYSTEM_POWER_CAPABILITIES pwrCap;
+    if (GetPwrCapabilities(&pwrCap))
+    {
+        if (pwrCap.SystemS3)
+        {
+            m_endChoice->Append(_("Suspend computer"), (void*)Batch::SUSPEND);
+        };
+        if (pwrCap.SystemS4 && pwrCap.HiberFilePresent)
+        {
+            m_endChoice->Append(_("Hibernate computer"), (void*)Batch::HIBERNATE);
+        };
+    };
+#endif
 
     //wxThreadHelper::Create is deprecated in wxWidgets 2.9+, but its
     // replacement, CreateThread, does not exist in 2.8. Pick one
@@ -226,6 +258,7 @@ BatchFrame::BatchFrame(wxLocale* locale, wxString xrc)
     //TO-DO: include a batch or project progress gauge?
     projListBox->Fill(m_batch);
     SetDropTarget(new BatchDropTarget());
+    UpdateTaskBarProgressBar();
 }
 
 void* BatchFrame::Entry()
@@ -238,8 +271,14 @@ void* BatchFrame::Entry()
         wxFileName aFile(m_batch->GetLastFile());
         if(!aFile.FileExists())
         {
+#if wxCHECK_VERSION(3,0,0)
+            // for thread safety wxWidgets recommends using wxQueueEvent
+            // but this function is not available for older wxWidgets versions
+            wxQueueEvent(this, new wxCommandEvent(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_RELOAD_BATCH));
+#else
             wxCommandEvent evt(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_RELOAD_BATCH);
-            wxPostEvent(this,evt);
+            wxPostEvent(this, evt);
+#endif
         }
         else
         {
@@ -247,18 +286,33 @@ void* BatchFrame::Entry()
             aFile.GetTimes(NULL,NULL,&create);
             if(create.IsLaterThan(m_batch->GetLastFileDate()))
             {
+#if wxCHECK_VERSION(3,0,0)
+                wxQueueEvent(this, new wxCommandEvent(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_RELOAD_BATCH));
+#else
                 wxCommandEvent evt(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_RELOAD_BATCH);
-                wxPostEvent(this,evt);
+                wxPostEvent(this, evt);
+#endif
             };
         };
         //update project list box
+#if wxCHECK_VERSION(3,0,0)
+        wxQueueEvent(this, new wxCommandEvent(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_UPDATE_LISTBOX));
+#else
         wxCommandEvent evt(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_UPDATE_LISTBOX);
-        wxPostEvent(this,evt);
+        wxPostEvent(this, evt);
+#endif
         GetThread()->Sleep(1000);
     }
     return 0;
 }
 
+wxStatusBar* BatchFrame::OnCreateStatusBar(int number, long style, wxWindowID id, const wxString& name)
+{
+    m_progStatusBar = new ProgressStatusBar(this, id, style, name);
+    m_progStatusBar->SetFieldsCount(number);
+    return m_progStatusBar;
+}
+
 bool BatchFrame::IsRunning()
 {
     return m_batch->IsRunning();
@@ -378,7 +432,7 @@ void BatchFrame::OnButtonAddToStitchingQueue(wxCommandEvent& event)
     wxFileDialog dlg(0,
                      _("Specify project source file(s)"),
                      defaultdir, wxT(""),
-                     _("Project files (*.pto,*.ptp,*.pts,*.oto)|*.pto;*.ptp;*.pts;*.oto;|All files (*)|*"),
+                     _("Project files (*.pto)|*.pto|All files (*)|*"),
                      wxFD_OPEN | wxFD_MULTIPLE, wxDefaultPosition);
     dlg.SetDirectory(defaultdir);
 
@@ -406,7 +460,7 @@ void BatchFrame::OnButtonAddToAssistantQueue(wxCommandEvent& event)
     wxFileDialog dlg(0,
                      _("Specify project source file(s)"),
                      defaultdir, wxT(""),
-                     _("Project files (*.pto,*.ptp,*.pts,*.oto)|*.pto;*.ptp;*.pts;*.oto;|All files (*)|*"),
+                     _("Project files (*.pto)|*.pto|All files (*)|*"),
                      wxFD_OPEN | wxFD_MULTIPLE, wxDefaultPosition);
     dlg.SetDirectory(defaultdir);
 
@@ -447,7 +501,7 @@ void BatchFrame::AddDirToList(wxString aDir)
         projListBox->AppendProject(proj);
     };
     m_batch->SaveTemp();
-    SetStatusText(_("Added projects from dir ")+aDir);
+    SetStatusText(wxString::Format(_("Added projects from dir %s"), aDir.c_str()));
 };
 
 void BatchFrame::AddToList(wxString aFile,Project::Target target)
@@ -502,6 +556,23 @@ void BatchFrame::OnButtonChangePrefix(wxCommandEvent& event)
                         return;
                     }
                 };
+                wxFileName prefix(dlg.GetPath());
+                while (!prefix.IsDirWritable())
+                {
+                    wxMessageBox(wxString::Format(_("You have no permissions to write in folder \"%s\".\nPlease select another folder for the final output."), prefix.GetPath().c_str()),
+#ifdef __WXMSW__
+                        wxT("PTBatcherGUI"),
+#else
+                        wxT(""),
+#endif
+                        wxOK | wxICON_INFORMATION);
+                    if (dlg.ShowModal() != wxID_OK)
+                    {
+                        return;
+                    };
+                    prefix = dlg.GetPath();
+                };
+
                 wxString outname(dlg.GetPath());
                 ChangePrefix(selIndex,outname);
                 //SetStatusText(_T("Changed prefix for "+projListBox->GetSelectedProject()));
@@ -644,17 +715,13 @@ void BatchFrame::OnButtonOpenBatch(wxCommandEvent& event)
 
 void BatchFrame::OnButtonOpenWithHugin(wxCommandEvent& event)
 {
-#ifdef __WINDOWS__
-    wxString huginPath = getExePath(wxGetApp().argv[0])+wxFileName::GetPathSeparator();
-#else
-    wxString huginPath = _T("");	//we call hugin directly without path on linux
-#endif
+    const wxFileName exePath(wxStandardPaths::Get().GetExecutablePath());
     if(projListBox->GetSelectedIndex()!=-1)
         if(projListBox->GetText(projListBox->GetSelectedIndex(),0).Cmp(_T(""))!=0)
 #ifdef __WXMAC__
             wxExecute(_T("open -b net.sourceforge.hugin.hugin \"" + projListBox->GetSelectedProject()+_T("\"")));
 #else
-            wxExecute(huginPath+_T("hugin \"" + projListBox->GetSelectedProject()+_T("\" -notips")));
+            wxExecute(exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR)+_T("hugin \"" + projListBox->GetSelectedProject()+_T("\" -notips")));
 #endif
         else
         {
@@ -669,13 +736,13 @@ void BatchFrame::OnButtonOpenWithHugin(wxCommandEvent& event)
 #else
                                 wxT(""),
 #endif
-                                wxYES | wxNO | wxICON_INFORMATION );
+                                wxYES_NO | wxICON_INFORMATION );
         if(message.ShowModal() == wxID_YES)
         {
 #ifdef __WXMAC__
             wxExecute(_T("open -b net.sourceforge.hugin.hugin"));
 #else
-            wxExecute(huginPath+_T("hugin"));
+            wxExecute(exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR)+_T("hugin"));
 #endif
         }
     }
@@ -715,7 +782,7 @@ void BatchFrame::OnButtonRemoveComplete(wxCommandEvent& event)
 #else
                                 wxT(""),
 #endif
-                                wxYES | wxNO | wxICON_INFORMATION );
+                                wxYES_NO | wxICON_INFORMATION );
         if(message.ShowModal()==wxID_YES)
         {
             removeErrors=true;
@@ -747,7 +814,7 @@ void BatchFrame::OnButtonRemoveFromList(wxCommandEvent& event)
 #else
                                     wxT(""),
 #endif
-                                    wxYES | wxCANCEL | wxICON_INFORMATION);
+                                    wxYES_NO | wxICON_INFORMATION);
             if(message.ShowModal()==wxID_YES)
             {
                 OnButtonSkip(event);
@@ -755,7 +822,7 @@ void BatchFrame::OnButtonRemoveFromList(wxCommandEvent& event)
         }
         else
         {
-            SetStatusText(_("Removed project ")+projListBox->GetSelectedProject());
+            SetStatusText(wxString::Format(_("Removed project %s"), projListBox->GetSelectedProject().c_str()));
             projListBox->Deselect(selIndex);
             projListBox->DeleteItem(selIndex);
             m_batch->RemoveProjectAtIndex(selIndex);
@@ -782,7 +849,7 @@ void BatchFrame::OnButtonReset(wxCommandEvent& event)
 #else
                                     wxT(""),
 #endif
-                                    wxYES | wxCANCEL | wxICON_INFORMATION);
+                                    wxYES_NO | wxICON_INFORMATION);
             if(message.ShowModal()==wxID_YES)
             {
                 OnButtonSkip(event);
@@ -791,7 +858,7 @@ void BatchFrame::OnButtonReset(wxCommandEvent& event)
         else
         {
             m_batch->SetStatus(selIndex,Project::WAITING);
-            SetStatusText(_("Reset project ")+projListBox->GetSelectedProject());
+            SetStatusText(wxString::Format(_("Reset project %s"), projListBox->GetSelectedProject().c_str()));
         }
     }
     else
@@ -811,7 +878,7 @@ void BatchFrame::OnButtonResetAll(wxCommandEvent& event)
 #else
                                 wxT(""),
 #endif
-                                wxYES | wxCANCEL | wxICON_INFORMATION);
+                                wxYES_NO | wxICON_INFORMATION);
         if(message.ShowModal()==wxID_YES)
         {
             OnButtonCancel(event);
@@ -847,7 +914,7 @@ void BatchFrame::OnButtonSaveBatch(wxCommandEvent& event)
                      _("Specify batch file to save"),
                      defaultdir, wxT(""),
                      _("Batch file (*.ptb)|*.ptb;|All files (*)|*"),
-                     wxFD_SAVE, wxDefaultPosition);
+                     wxFD_SAVE | wxFD_OVERWRITE_PROMPT, wxDefaultPosition);
     if (dlg.ShowModal() == wxID_OK)
     {
         wxConfig::Get()->Write(wxT("/BatchFrame/batchPath"), dlg.GetDirectory());  // remember for later
@@ -898,23 +965,53 @@ void BatchFrame::OnButtonSkip(wxCommandEvent& event)
     }
 }
 
+void SelectEndTask(wxControlWithItems* list, const Batch::EndTask endTask)
+{
+    for (unsigned int i = 0; i<list->GetCount(); i++)
+    {
+        if (static_cast<Batch::EndTask>((size_t)list->GetClientData(i)) == endTask)
+        {
+            list->SetSelection(i);
+            return;
+        };
+    };
+    list->SetSelection(0);
+};
+
 void BatchFrame::SetCheckboxes()
 {
     wxConfigBase* config=wxConfigBase::Get();
     int i;
-    i=config->Read(wxT("/BatchFrame/ParallelCheck"), 0l);
-    XRCCTRL(*this,"cb_parallel",wxCheckBox)->SetValue(i!=0);
-    m_batch->parallel=(i!=0);
+    // read older version
+#if defined __WXMAC__ || defined __WXOSX_COCOA__
+    i = 0;
+#else
     i=config->Read(wxT("/BatchFrame/ShutdownCheck"), 0l);
-    XRCCTRL(*this,"cb_shutdown",wxCheckBox)->SetValue(i!=0);
-    m_batch->shutdown=(i!=0);
+#endif
+    if (i != 0)
+    {
+        SelectEndTask(m_endChoice, Batch::SHUTDOWN);
+        m_batch->atEnd = Batch::SHUTDOWN;
+    };
+    config->DeleteEntry(wxT("/BatchFrame/ShutdownCheck"));
+    // now read current version
+    i = config->Read(wxT("/BatchFrame/AtEnd"), 0l);
+#if defined __WXMAC__ || defined __WXOSX_COCOA__
+    // wxWidgets for MacOS does not support wxShutdown
+    if (i == Batch::SHUTDOWN)
+    {
+        i = 0;
+    };
+#endif
+    m_batch->atEnd = static_cast<Batch::EndTask>(i);
+    SelectEndTask(m_endChoice, m_batch->atEnd);
     i=config->Read(wxT("/BatchFrame/OverwriteCheck"), 0l);
     XRCCTRL(*this,"cb_overwrite",wxCheckBox)->SetValue(i!=0);
     m_batch->overwrite=(i!=0);
     i=config->Read(wxT("/BatchFrame/VerboseCheck"), 0l);
     XRCCTRL(*this,"cb_verbose",wxCheckBox)->SetValue(i!=0);
     m_batch->verbose=(i!=0);
-    i=config->Read(wxT("/BatchFrame/AutoRemoveCheck"), 0l);
+    i=config->Read(wxT("/BatchFrame/AutoRemoveCheck"), 1l);
     XRCCTRL(*this,"cb_autoremove",wxCheckBox)->SetValue(i!=0);
     m_batch->autoremove=(i!=0);
     i=config->Read(wxT("/BatchFrame/AutoStitchCheck"), 0l);
@@ -925,14 +1022,9 @@ void BatchFrame::SetCheckboxes()
     m_batch->saveLog=(i!=0);
 };
 
-bool BatchFrame::GetCheckParallel()
+Batch::EndTask BatchFrame::GetEndTask()
 {
-    return XRCCTRL(*this,"cb_parallel",wxCheckBox)->IsChecked();
-};
-
-bool BatchFrame::GetCheckShutdown()
-{
-    return XRCCTRL(*this,"cb_shutdown",wxCheckBox)->IsChecked();
+    return static_cast<Batch::EndTask>((size_t)m_endChoice->GetClientData(m_endChoice->GetSelection()));
 };
 
 bool BatchFrame::GetCheckOverwrite()
@@ -974,32 +1066,10 @@ void BatchFrame::OnCheckOverwrite(wxCommandEvent& event)
     }
 }
 
-void BatchFrame::OnCheckParallel(wxCommandEvent& event)
+void BatchFrame::OnChoiceEnd(wxCommandEvent& event)
 {
-    if(event.IsChecked())
-    {
-        m_batch->parallel = true;
-        wxConfigBase::Get()->Write(wxT("/BatchFrame/ParallelCheck"), 1l);
-    }
-    else
-    {
-        m_batch->parallel = false;
-        wxConfigBase::Get()->Write(wxT("/BatchFrame/ParallelCheck"), 0l);
-    }
-}
-
-void BatchFrame::OnCheckShutdown(wxCommandEvent& event)
-{
-    if(event.IsChecked())
-    {
-        m_batch->shutdown = true;
-        wxConfigBase::Get()->Write(wxT("/BatchFrame/ShutdownCheck"), 1l);
-    }
-    else
-    {
-        m_batch->shutdown = false;
-        wxConfigBase::Get()->Write(wxT("/BatchFrame/ShutdownCheck"), 0l);
-    }
+    m_batch->atEnd = static_cast<Batch::EndTask>((size_t)m_endChoice->GetClientData(event.GetSelection()));
+    wxConfigBase::Get()->Write(wxT("/BatchFrame/AtEnd"), static_cast<long>(m_batch->atEnd));
 }
 
 void BatchFrame::OnCheckVerbose(wxCommandEvent& event)
@@ -1092,21 +1162,24 @@ void BatchFrame::OnClose(wxCloseEvent& event)
     }
     config->Flush();
     m_closeThread = true;
-    this->GetThread()->Wait();
+    if (this->GetThread() && this->GetThread()->IsRunning())
+    {
+        this->GetThread()->Wait();
+    };
 #ifndef __WXMSW__
     delete m_help;
 #endif
     if(m_tray!=NULL)
     {
         delete m_tray;
+        m_tray = NULL;
     }
     this->Destroy();
 }
 
 void BatchFrame::PropagateDefaults()
 {
-    m_batch->parallel=GetCheckParallel();
-    m_batch->shutdown=GetCheckShutdown();
+    m_batch->atEnd = GetEndTask();
     m_batch->overwrite=GetCheckOverwrite();
     m_batch->verbose=GetCheckVerbose();
     m_batch->autoremove=GetCheckAutoRemove();
@@ -1214,6 +1287,36 @@ void BatchFrame::SetStatusInformation(wxString status,bool showBalloon)
     };
 };
 
+void BatchFrame::OnProgress(wxCommandEvent& e)
+{
+    m_progStatusBar->SetProgress(e.GetInt());
+    UpdateTaskBarProgressBar();
+};
+
+void BatchFrame::UpdateTaskBarProgressBar()
+{
+#if defined __WXMSW__ && wxCHECK_VERSION(3,1,0)
+    // provide also a feedback in task bar if available
+    if (IsShown())
+    {
+        wxTaskBarButton* taskBarButton = MSWGetTaskBarButton();
+        if (taskBarButton != NULL)
+        {
+            if (m_progStatusBar->GetProgress() < 0)
+            {
+                taskBarButton->SetProgressValue(wxTASKBAR_BUTTON_NO_PROGRESS);
+            }
+            else
+            {
+                taskBarButton->SetProgressRange(100);
+                taskBarButton->SetProgressState(wxTASKBAR_BUTTON_NORMAL);
+                taskBarButton->SetProgressValue(m_progStatusBar->GetProgress());
+            };
+        };
+    };
+#endif
+};
+
 void BatchFrame::OnMinimize(wxIconizeEvent& e)
 {
     //hide/show window in taskbar when minimizing
@@ -1234,11 +1337,21 @@ void BatchFrame::OnMinimize(wxIconizeEvent& e)
         else
         {
             m_batch->verbose=XRCCTRL(*this,"cb_verbose",wxCheckBox)->IsChecked();
+            UpdateTaskBarProgressBar();
         };
         m_batch->ShowOutput(m_batch->verbose);
     }
     else //don't hide window if no tray icon
     {
+#if wxCHECK_VERSION(2,9,0)
+        if (!e.IsIconized())
+#else
+        if (!e.Iconized())
+#endif
+        {
+            // window is restored, update progress indicators
+            UpdateTaskBarProgressBar();
+        };
         e.Skip();
     };
 };
