@@ -19,8 +19,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -36,6 +36,7 @@
 #endif
 #include "BatchTrayIcon.h"
 //#include <wx/app.h>
+#include "ProgressStatusBar.h"
 
 /** Simple class that forward the drop to the mainframe */
 class BatchDropTarget : public wxFileDropTarget
@@ -50,13 +51,15 @@ public:
     bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames);
 };
 
-class BatchFrame : public wxFrame, wxThreadHelper
+class BatchFrame : public wxFrame, public wxThreadHelper
 {
 public:
     //Main constructor
     BatchFrame(wxLocale* locale, wxString xrc);
     //Main thread for all file polling - checking for new projects and updating modified ones.
     void* Entry();
+    // create statusbar with progress control
+    wxStatusBar* OnCreateStatusBar(int number, long style, wxWindowID id, const wxString& name);
 
     void OnUserExit(wxCommandEvent& event);
     void OnButtonAddCommand(wxCommandEvent& event);
@@ -84,8 +87,7 @@ public:
     void OnButtonSkip(wxCommandEvent& event);
 
     void OnCheckOverwrite(wxCommandEvent& event);
-    void OnCheckParallel(wxCommandEvent& event);
-    void OnCheckShutdown(wxCommandEvent& event);
+    void OnChoiceEnd(wxCommandEvent& event);
     void OnCheckVerbose(wxCommandEvent& event);
     /** event handler called when auto remove checkbox was changed */
     void OnCheckAutoRemove(wxCommandEvent& event);
@@ -93,7 +95,8 @@ public:
     void OnCheckAutoStitch(wxCommandEvent& event);
     /** event handler called when always save log checkbox was changed */
     void OnCheckSaveLog(wxCommandEvent& event);
-
+    /** event handler for update progress controls */
+    void OnProgress(wxCommandEvent& event);
 
     //Called on window close to take care of the child thread
     void OnClose(wxCloseEvent& event);
@@ -110,10 +113,8 @@ public:
     //Swaps the project entry at index in the list with the next (at index+1).
     void SwapProject(int index);
     //PanoramaOptions readOptions(wxString projectFile);
-    /** return if parallel checkbox is checked */
-    bool GetCheckParallel();
-    /** return if shutdown checkbox is checked */
-    bool GetCheckShutdown();
+    /** return which task should be executed at end*/
+    Batch::EndTask GetEndTask();
     /** return if overwrite checkbox is checked */
     bool GetCheckOverwrite();
     /** return if verbose checkbox is checked */
@@ -145,6 +146,8 @@ public:
     };
     /** sets the current verbose status, does not update the checkbox */
     void SetInternalVerbose(bool newVerbose);
+    /** update the progress bar in the task bar */
+    void UpdateTaskBarProgressBar();
 
 #ifdef __WXMSW__
     /** return help controller for open help */
@@ -164,6 +167,7 @@ private:
     bool m_cancelled;
     bool m_paused;
     bool m_closeThread; //included to signal the thread to finish execution
+    wxChoice* m_endChoice;
     //TO-DO: include a batch or project progress gauge? Test initialization commented out in constructor
     //wxGauge* m_gauge;
 #ifdef __WXMSW__
@@ -172,6 +176,7 @@ private:
     wxHtmlHelpController* m_help;
 #endif
     BatchTaskBarIcon* m_tray;
+    ProgressStatusBar* m_progStatusBar;
     bool m_startedMinimized;
 
     void OnProcessTerminate(wxProcessEvent& event);
