@@ -19,8 +19,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -31,12 +31,21 @@
 #include "PTBatcherGUI.h"
 #include "hugin_utils/alphanum.h"
 #include "hugin/config_defaults.h"
+#include "wx/mstream.h"
+#include "exiv2/exiv2.hpp"
+#include "exiv2/preview.hpp"
+#ifdef _WIN32
+#include <CommCtrl.h>
+#endif
+#include "base_wx/LensTools.h"
+#include "panodata/StandardImageVariableGroups.h"
 
 BEGIN_EVENT_TABLE(FindPanoDialog,wxDialog)
     EVT_BUTTON(XRCID("find_pano_close"), FindPanoDialog::OnButtonClose)
     EVT_BUTTON(XRCID("find_pano_select_dir"), FindPanoDialog::OnButtonChoose)
     EVT_BUTTON(XRCID("find_pano_start_stop"), FindPanoDialog::OnButtonStart)
     EVT_BUTTON(XRCID("find_pano_add_queue"), FindPanoDialog::OnButtonSend)
+    EVT_LISTBOX(XRCID("find_pano_list"), FindPanoDialog::OnSelectPossiblePano)
     EVT_CLOSE(FindPanoDialog::OnClose)
 END_EVENT_TABLE()
 
@@ -45,6 +54,9 @@ bool SortFilename::operator()(const SrcPanoImage* img1, const SrcPanoImage* img2
     return doj::alphanum_comp(img1->getFilename(),img2->getFilename())<0;
 };
 
+// thumbnail size currently set to 80x80
+#define THUMBSIZE 80
+
 FindPanoDialog::FindPanoDialog(BatchFrame* batchframe, wxString xrcPrefix)
 {
     // load our children. some children might need special
@@ -66,6 +78,9 @@ FindPanoDialog::FindPanoDialog(BatchFrame* batchframe, wxString xrcPrefix)
     m_button_send=XRCCTRL(*this,"find_pano_add_queue",wxButton);
     m_button_close=XRCCTRL(*this,"find_pano_close",wxButton);
     m_textctrl_dir=XRCCTRL(*this,"find_pano_dir",wxTextCtrl);
+#if wxCHECK_VERSION(2,9,3)
+    m_textctrl_dir->AutoCompleteDirectories();
+#endif
     m_cb_subdir=XRCCTRL(*this,"find_pano_subdir",wxCheckBox);
     m_statustext=XRCCTRL(*this,"find_pano_label",wxStaticText);
     m_list_pano=XRCCTRL(*this,"find_pano_list",wxCheckListBox);
@@ -75,6 +90,8 @@ FindPanoDialog::FindPanoDialog(BatchFrame* batchframe, wxString xrcPrefix)
     m_cb_loadVignetting=XRCCTRL(*this,"find_pano_load_vignetting",wxCheckBox);
     m_sc_minNumberImages=XRCCTRL(*this, "find_pano_min_number_images", wxSpinCtrl);
     m_sc_maxTimeDiff=XRCCTRL(*this, "find_pano_max_time_diff", wxSpinCtrl);
+    m_ch_blender = XRCCTRL(*this, "find_pano_default_blender", wxChoice);
+    FillBlenderList(m_ch_blender);
 
     //set parameters
     wxConfigBase* config = wxConfigBase::Get();
@@ -111,6 +128,11 @@ FindPanoDialog::FindPanoDialog(BatchFrame* batchframe, wxString xrcPrefix)
             this->Move(0, 44);
         }
     }
+    long splitterPos = config->Read(wxT("/FindPanoDialog/splitterPos"), -1l);
+    if (splitterPos != -1)
+    {
+        XRCCTRL(*this, "find_pano_splitter", wxSplitterWindow)->SetSashPosition(splitterPos);
+    };
     wxString path=config->Read(wxT("/FindPanoDialog/actualPath"),wxEmptyString);
     if(!path.IsEmpty())
     {
@@ -131,7 +153,17 @@ FindPanoDialog::FindPanoDialog(BatchFrame* batchframe, wxString xrcPrefix)
     m_sc_minNumberImages->SetValue(i);
     i=config->Read(wxT("/FindPanoDialog/MaxTimeDiff"), 30l);
     m_sc_maxTimeDiff->SetValue(i);
+    i = config->Read(wxT("/FindPanoDialog/DefaultBlender"), static_cast<long>(HuginBase::PanoramaOptions::ENBLEND_BLEND));
+    SelectListValue(m_ch_blender, i);
     m_button_send->Disable();
+    m_thumbs = new wxImageList(THUMBSIZE, THUMBSIZE, true, 0);
+    wxListCtrl* thumbs_list = XRCCTRL(*this, "find_pano_selected_thumbslist", wxListCtrl);
+    thumbs_list->SetImageList(m_thumbs, wxIMAGE_LIST_NORMAL);
+#ifdef _WIN32
+    // default image spacing is too big, wxWidgets does not provide direct 
+    // access to the spacing, so using the direct API function
+    ListView_SetIconSpacing(thumbs_list->GetHandle(), THUMBSIZE + 20, THUMBSIZE + 20);
+#endif
 };
 
 FindPanoDialog::~FindPanoDialog()
@@ -151,6 +183,7 @@ FindPanoDialog::~FindPanoDialog()
     {
         config->Write(wxT("/FindPanoDialog/maximized"), 1l);
     };
+    config->Write(wxT("/FindPanoDialog/splitterPos"), XRCCTRL(*this, "find_pano_splitter", wxSplitterWindow)->GetSashPosition());
     config->Write(wxT("/FindPanoDialog/actualPath"),m_textctrl_dir->GetValue());
     config->Write(wxT("/FindPanoDialog/includeSubDirs"),m_cb_subdir->GetValue());
     config->Write(wxT("/FindPanoDialog/Naming"),m_ch_naming->GetSelection());
@@ -159,7 +192,9 @@ FindPanoDialog::~FindPanoDialog()
     config->Write(wxT("/FindPanoDialog/loadVignetting"),m_cb_loadDistortion->GetValue());
     config->Write(wxT("/FindPanoDialog/MinNumberImages"), m_sc_minNumberImages->GetValue());
     config->Write(wxT("/FindPanoDialog/MaxTimeDiff"), m_sc_maxTimeDiff->GetValue());
+    config->Write(wxT("/FindPanoDialog/DefaultBlender"), static_cast<long>(GetSelectedValue(m_ch_blender)));
     CleanUpPanolist();
+    delete m_thumbs;
 };
 
 void FindPanoDialog::CleanUpPanolist()
@@ -240,6 +275,8 @@ void FindPanoDialog::OnButtonStart(wxCommandEvent& e)
             m_button_start->SetLabel(_("Stop"));
             CleanUpPanolist();
             m_list_pano->Clear();
+            wxCommandEvent dummy;
+            OnSelectPossiblePano(dummy);
             EnableButtons(false);
             SearchInDir(m_start_dir,m_cb_subdir->GetValue(), m_cb_loadDistortion->GetValue(), m_cb_loadVignetting->GetValue(), 
                 m_sc_minNumberImages->GetValue(), m_sc_maxTimeDiff->GetValue());
@@ -277,7 +314,8 @@ void FindPanoDialog::OnButtonSend(wxCommandEvent& e)
     {
         if(m_list_pano->IsChecked(i))
         {
-            wxString filename=m_panos[i]->GeneratePanorama((PossiblePano::NamingConvention)(m_ch_naming->GetSelection()),createLinks);
+            wxString filename=m_panos[i]->GeneratePanorama((PossiblePano::NamingConvention)(m_ch_naming->GetSelection()),createLinks, 
+                static_cast<HuginBase::PanoramaOptions::BlendingMechanism>(GetSelectedValue(m_ch_blender)));
             if(!filename.IsEmpty())
             {
                 m_batchframe->AddToList(filename,Project::DETECTING);
@@ -306,6 +344,28 @@ void FindPanoDialog::EnableButtons(const bool state)
     m_button_send->Enable(state);
 };
 
+void FindPanoDialog::OnSelectPossiblePano(wxCommandEvent &e)
+{
+    int selected = m_list_pano->GetSelection();
+    if (selected != wxNOT_FOUND)
+    {
+        XRCCTRL(*this, "find_pano_selected_cam", wxStaticText)->SetLabel(m_panos[selected]->GetCameraName());
+        XRCCTRL(*this, "find_pano_selected_lens", wxStaticText)->SetLabel(m_panos[selected]->GetLensName());
+        XRCCTRL(*this, "find_pano_selected_focallength", wxStaticText)->SetLabel(m_panos[selected]->GetFocalLength());
+        XRCCTRL(*this, "find_pano_selected_date_time", wxStaticText)->SetLabel(m_panos[selected]->GetStartString() + wxT(" (")+ m_panos[selected]->GetDuration() + wxT(")"));
+        m_panos[selected]->PopulateListCtrl(XRCCTRL(*this, "find_pano_selected_thumbslist", wxListCtrl), m_thumbs);
+    }
+    else
+    {
+        XRCCTRL(*this, "find_pano_selected_cam", wxStaticText)->SetLabel(wxEmptyString);
+        XRCCTRL(*this, "find_pano_selected_lens", wxStaticText)->SetLabel(wxEmptyString);
+        XRCCTRL(*this, "find_pano_selected_focallength", wxStaticText)->SetLabel(wxEmptyString);
+        XRCCTRL(*this, "find_pano_selected_date_time", wxStaticText)->SetLabel(wxEmptyString);
+        XRCCTRL(*this, "find_pano_selected_thumbslist", wxListCtrl)->DeleteAllItems();
+        m_thumbs->RemoveAll();
+    };
+};
+
 int SortWxFilenames(const wxString& s1,const wxString& s2)
 {
     return doj::alphanum_comp(std::string(s1.mb_str(wxConvLocal)),std::string(s2.mb_str(wxConvLocal)));
@@ -335,6 +395,17 @@ void FindPanoDialog::SearchInDir(wxString dirstring, const bool includeSubdir, c
             SrcPanoImage* img=new SrcPanoImage;
             img->setFilename(filenamestr);
             img->readEXIF();
+            // check for black/white images, if so skip
+            const HuginBase::FileMetaData& metadata = img->getFileMetadata();
+            HuginBase::FileMetaData::const_iterator it = metadata.find("pixeltype");
+            if (it != metadata.end())
+            {
+                if (it->second == "BILEVEL")
+                { 
+                    wxGetApp().Yield(true);
+                    continue;
+                };
+            };
             img->applyEXIFValues();
             if(!img->getExifMake().empty() && !img->getExifModel().empty() && 
                 img->getExifFocalLength()!=0 && img->getCropFactor()!=0)
@@ -416,7 +487,7 @@ void FindPanoDialog::SearchInDir(wxString dirstring, const bool includeSubdir, c
         m_button_send->Enable(m_panos.size()>0);
         if(m_panos.size()>0)
         {
-            m_statustext->SetLabel(wxString::Format(_("Found %d possible panoramas."),m_panos.size()));
+            m_statustext->SetLabel(wxString::Format(_("Found %d possible panoramas."), static_cast<int>(m_panos.size())));
         }
         else
         {
@@ -466,79 +537,6 @@ bool PossiblePano::BelongsTo(SrcPanoImage* img, const wxTimeSpan max_time_diff)
     return true;
 };
 
-double PossiblePano::GetMaxExposureDifference()
-{
-    if(m_images.empty())
-    {
-        return 0;
-    }
-    double minEv=1000;
-    double maxEv=-1000;
-    for(ImageSet::const_iterator it=m_images.begin(); it!=m_images.end(); ++it)
-    {
-        double ev=(*it)->getExposureValue();
-        minEv=std::min(minEv,ev);
-        maxEv=std::max(maxEv,ev);
-    };
-    return maxEv-minEv;
-};
-
-bool PossiblePano::IsStacked()
-{
-    if(m_images.empty())
-    {
-        return false;
-    }
-    // this algorithm is based on panostart by Bruno Postle
-    // bracketed pano has at least a dynamic range of 1.2 ev values (corresponds to bracket with +-2/3)
-    if(GetMaxExposureDifference()<1.2)
-    {
-        return false;
-    }
-    //if image is shooted in auto exposure mode then it is not a bracket pano
-    if((*m_images.begin())->getExifExposureMode()==0)
-    {
-        return false;
-    }
-    //now collect all unique exposure values
-    std::set<int> evValues;
-    for(ImageSet::const_iterator it=m_images.begin(); it!=m_images.end(); ++it)
-    {
-        //we multiply with 10 to don't get fooled by rounded double values
-        evValues.insert(int((*it)->getExposureValue()*10));
-    };
-    //if there is only one unique exposure value then there are no stacks
-    if(evValues.size()<2)
-    {
-        return false;
-    }
-    //if number of unique exposure values is equal the number of images then there are no stacks
-    if(evValues.size()==m_images.size())
-    {
-        return false;
-    }
-    //if number of images is not a multiple of number of unique exposure values
-    //then the stacks are incomplete, skipping
-    if(m_images.size() % evValues.size()!=0)
-    {
-        return false;
-    }
-    //check if exposure value is repeated with step size of bracket size
-    ImageSet::const_iterator it=m_images.begin();
-    for(unsigned int i=0; i<evValues.size(); i++)
-    {
-        ++it;
-    };
-    if(abs((*m_images.begin())->getExposureValue()-(*it)->getExposureValue())<0.1)
-    {
-        return true;
-    }
-    else
-    {
-        return false;
-    };
-};
-
 const wxDateTime PossiblePano::GetDateTime(const SrcPanoImage* img)
 {
     struct tm exifdatetime;
@@ -600,7 +598,7 @@ const wxString PossiblePano::GetFilestring(const wxString BasePath, const bool s
 
 const wxString PossiblePano::GetItemString(const wxString BasePath) const
 {
-    return wxString::Format(_("%d images: %s"),m_images.size(),GetFilestring(BasePath).c_str());
+    return wxString::Format(_("%d images: %s"), static_cast<int>(m_images.size()), GetFilestring(BasePath).c_str());
 };
 
 bool PossiblePano::GetNewProjectFilename(NamingConvention nc,const wxString basePath, wxFileName& projectFile)
@@ -677,7 +675,7 @@ bool PossiblePano::GetNewProjectFilename(NamingConvention nc,const wxString base
     return true;
 };
 
-wxString PossiblePano::GeneratePanorama(NamingConvention nc,bool createLinks)
+wxString PossiblePano::GeneratePanorama(NamingConvention nc, bool createLinks, HuginBase::PanoramaOptions::BlendingMechanism defaultBlender)
 {
     if(m_images.empty())
     {
@@ -726,29 +724,9 @@ wxString PossiblePano::GeneratePanorama(NamingConvention nc,bool createLinks)
             pano.setSrcImage(i, img);
         };
     };
-    if(IsStacked())
+    if (pano.hasPossibleStacks())
     {
-        //if it is a stacked pano, create the stacks and link position if desired
-        unsigned int imgNr=0;
-        double ev=pano.getImage(imgNr).getExposureValue();
-        for(unsigned int i=1; i<pano.getNrOfImages(); i++)
-        {
-            if(abs(pano.getImage(i).getExposureValue()-ev)<0.1)
-            {
-                imgNr=i;
-                ev=pano.getImage(imgNr).getExposureValue();
-            }
-            else
-            {
-                pano.linkImageVariableStack(imgNr,i);
-                if(createLinks)
-                {
-                    pano.linkImageVariableYaw(imgNr,i);
-                    pano.linkImageVariablePitch(imgNr,i);
-                    pano.linkImageVariableRoll(imgNr,i);
-                };
-            };
-        };
+        pano.linkPossibleStacks(createLinks);
     };
     // Setup pano with options from preferences
     PanoramaOptions opts = pano.getOptions();
@@ -793,7 +771,7 @@ wxString PossiblePano::GeneratePanorama(NamingConvention nc,bool createLinks)
             break;
     }
     opts.outputFormat = PanoramaOptions::TIFF_m;
-    opts.blendMode = PanoramaOptions::ENBLEND_BLEND;
+    opts.blendMode = defaultBlender;
     opts.enblendOptions = config->Read(wxT("Enblend/Args"),wxT(HUGIN_ENBLEND_ARGS)).mb_str(wxConvLocal);
     opts.enfuseOptions = config->Read(wxT("Enfuse/Args"),wxT(HUGIN_ENFUSE_ARGS)).mb_str(wxConvLocal);
     opts.interpolator = (vigra_ext::Interpolator)config->Read(wxT("Nona/Interpolator"),HUGIN_NONA_INTERPOLATOR);
@@ -811,7 +789,7 @@ wxString PossiblePano::GeneratePanorama(NamingConvention nc,bool createLinks)
     {
         return wxEmptyString;
     };
-    PT::UIntSet all;
+    HuginBase::UIntSet all;
     fill_set(all, 0, pano.getNrOfImages()-1);
     try
     {
@@ -826,3 +804,112 @@ wxString PossiblePano::GeneratePanorama(NamingConvention nc,bool createLinks)
     return projectFile.GetFullPath();
 };
 
+wxString PossiblePano::GetCameraName()
+{
+    return wxString(m_camera.c_str(), wxConvLocal);
+}
+
+wxString PossiblePano::GetLensName()
+{
+    return wxString(m_lens.c_str(), wxConvLocal);
+};
+
+wxString PossiblePano::GetFocalLength()
+{
+    return wxString::Format(wxT("%0.1f mm"), m_focallength);
+};
+
+wxString PossiblePano::GetStartString()
+{
+    return m_dt_start.Format();
+};
+
+wxString PossiblePano::GetDuration()
+{
+    wxTimeSpan diff = m_dt_end.Subtract(m_dt_start);
+    if (diff.GetSeconds() > 60)
+    {
+        return diff.Format(_("%M:%S min"));
+    }
+    else
+    {
+        return diff.Format(_("%S s"));
+    };
+};
+
+void PossiblePano::PopulateListCtrl(wxListCtrl* list, wxImageList* thumbs)
+{
+    list->DeleteAllItems();
+    thumbs->RemoveAll();
+    wxBusyCursor cursor;
+    for (ImageSet::iterator it = m_images.begin(); it != m_images.end(); ++it)
+    {
+        Exiv2::Image::AutoPtr image;
+        bool opened = false;
+        try
+        {
+            image = Exiv2::ImageFactory::open((*it)->getFilename().c_str());
+            opened = true;
+        }
+        catch (...)
+        {
+            std::cerr << __FILE__ << " " << __LINE__ << " Error opening file" << std::endl;
+        }
+        int index = -1;
+        if (opened)
+        {
+            image->readMetadata();
+            // read all thumbnails
+            Exiv2::PreviewManager previews(*image);
+            Exiv2::PreviewPropertiesList lists = previews.getPreviewProperties();
+            if (!lists.empty())
+            {
+                // select a preview with matching size
+                int previewIndex = 0;
+                while (previewIndex < lists.size() - 1 && lists[previewIndex].width_ < THUMBSIZE && lists[previewIndex].height_ < THUMBSIZE)
+                {
+                    ++previewIndex;
+                };
+                // load preview image to wxImage
+                wxImage rawImage;
+                Exiv2::PreviewImage previewImage = previews.getPreviewImage(lists[previewIndex]);
+                wxMemoryInputStream stream(previewImage.pData(), previewImage.size());
+                rawImage.LoadFile(stream, wxString(previewImage.mimeType().c_str(), wxConvLocal), -1);
+                int x = 0;
+                int y = 0;
+                if (previewImage.width() > previewImage.height())
+                {
+                    //landscape format
+                    int newHeight = THUMBSIZE*previewImage.height() / previewImage.width();
+                    rawImage.Rescale(THUMBSIZE, newHeight);
+                    x = 0;
+                    y = (THUMBSIZE - newHeight) / 2;
+                }
+                else
+                {
+                    //portrait format
+                    int newWidth = THUMBSIZE*previewImage.width() / previewImage.height();
+                    rawImage.Rescale(newWidth, THUMBSIZE);
+                    x = (THUMBSIZE - newWidth) / 2;
+                    y = 0;
+                }
+                // create final bitmap with centered thumbnail
+                wxBitmap bitmap(THUMBSIZE, THUMBSIZE);
+                wxMemoryDC dc(bitmap);
+                dc.SetBackground(list->GetBackgroundColour());
+                dc.Clear();
+                dc.DrawBitmap(rawImage, x, y);
+                dc.SelectObject(wxNullBitmap);
+                // create mask bitmap
+                wxImage mask(THUMBSIZE, THUMBSIZE);
+                mask.SetRGB(wxRect(0, 0, THUMBSIZE, THUMBSIZE), 0, 0, 0);
+                mask.SetRGB(wxRect(x, y, THUMBSIZE - 2 * x, THUMBSIZE - 2 * y), 255, 255, 255);
+                // add to wxImageList
+                index = thumbs->Add(bitmap, wxBitmap(mask, 1));
+            };
+        };
+        // create item in thumb list
+        wxFileName fn(wxString((*it)->getFilename().c_str(), HUGIN_CONV_FILENAME));
+        list->InsertItem(list->GetItemCount(), fn.GetFullName(), index);
+    };
+};
