@@ -19,14 +19,15 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
 #include "PTBatcherGUI.h"
 #ifdef __WXMSW__
-#include "wx/cshelp.h"
+#include <wx/cshelp.h>
+#include <wx/stdpaths.h>
 #endif
 #include "lensdb/LensDB.h"
 
@@ -41,6 +42,9 @@ END_EVENT_TABLE()
 bool PTBatcherGUI::OnInit()
 {
     // Required to access the preferences of hugin
+#if wxUSE_ON_FATAL_EXCEPTION
+    wxHandleFatalExceptions();
+#endif
     SetAppName(wxT("hugin"));
 
 #if defined __WXMSW__
@@ -56,18 +60,13 @@ bool PTBatcherGUI::OnInit()
 
     // setup the environment for the different operating systems
 #if defined __WXMSW__
-    wxString huginExeDir = getExePath(argv[0]);
-
-    wxString huginRoot;
-    wxFileName::SplitPath(huginExeDir, &huginRoot, NULL, NULL);
-    m_xrcPrefix = wxString(huginRoot + wxT("/share/hugin/xrc/"));
-    // lensfun database init
-    wxString lensfunDBPath=huginRoot + wxT("/share/lensfun");
-    HuginBase::LensDB::LensDB::GetSingleton().SetMainDBPath(std::string(lensfunDBPath.mb_str(HUGIN_CONV_FILENAME)));
+    wxFileName exePath(wxStandardPaths::Get().GetExecutablePath());
+    exePath.RemoveLastDir();
+    const wxString huginRoot(exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR));
+    m_xrcPrefix = huginRoot + wxT("share\\hugin\\xrc\\");
 
     // locale setup
-    m_locale.AddCatalogLookupPathPrefix(huginRoot + wxT("/share/locale"));
-
+    m_locale.AddCatalogLookupPathPrefix(huginRoot + wxT("share\\locale"));
 #elif defined __WXMAC__ && defined MAC_SELF_CONTAINED_BUNDLE
     {
         wxString exec_path = MacGetPathToBundledResourceFile(CFSTR("xrc"));
@@ -91,13 +90,6 @@ bool PTBatcherGUI::OnInit()
             wxMessageBox(_("Translations not found in bundle"), _("Fatal Error"));
             return false;
         }
-        thePath = MacGetPathToBundledResourceFile(CFSTR("lensfun"));
-        if (thePath == wxT("")) {
-            wxMessageBox(_("lensfun directory not found in bundle"),
-                            _("Fatal Error"));
-            return false;
-        }
-        HuginBase::LensDB::LensDB::GetSingleton().SetMainDBPath(std::string(thePath.mb_str(HUGIN_CONV_FILENAME)));
     }
 #else
     // add the locale directory specified during configure
@@ -144,7 +136,6 @@ bool PTBatcherGUI::OnInit()
             wxCMD_LINE_VAL_NONE, wxCMD_LINE_OPTION_HELP
         },
         { wxCMD_LINE_SWITCH, "b", "batch",  "run batch immediately" },
-        { wxCMD_LINE_SWITCH, "p", "parallel",  "run batch projects in parallel" },
         { wxCMD_LINE_SWITCH, "o", "overwrite",  "overwrite previous files without asking" },
         { wxCMD_LINE_SWITCH, "s", "shutdown",  "shutdown computer after batch is complete" },
         { wxCMD_LINE_SWITCH, "v", "verbose",  "show verbose output when processing projects" },
@@ -160,7 +151,6 @@ bool PTBatcherGUI::OnInit()
             wxCMD_LINE_VAL_NONE, wxCMD_LINE_OPTION_HELP
         },
         { wxCMD_LINE_SWITCH, wxT("b"), wxT("batch"),  wxT("run batch immediately") },
-        { wxCMD_LINE_SWITCH, wxT("p"), wxT("parallel"),  wxT("run batch projects in parallel") },
         { wxCMD_LINE_SWITCH, wxT("o"), wxT("overwrite"),  wxT("overwrite previous files without asking") },
         { wxCMD_LINE_SWITCH, wxT("s"), wxT("shutdown"),  wxT("shutdown computer after batch is complete") },
         { wxCMD_LINE_SWITCH, wxT("v"), wxT("verbose"),  wxT("show verbose output when processing projects") },
@@ -346,13 +336,13 @@ bool PTBatcherGUI::OnInit()
     if(IsFirstInstance)
     {
         wxConfigBase* config=wxConfigBase::Get();
-        if (parser.Found(wxT("p")))
-        {
-            config->Write(wxT("/BatchFrame/ParallelCheck"), 1l);
-        }
         if (parser.Found(wxT("s")))
         {
-            config->Write(wxT("/BatchFrame/ShutdownCheck"), 1l);
+            config->DeleteEntry(wxT("/BatchFrame/ShutdownCheck"));
+#if !defined __WXMAC__ && !defined __WXOSX_COCOA__
+            // wxMac has not wxShutdown
+            config->Write(wxT("/BatchFrame/AtEnd"), static_cast<long>(Batch::SHUTDOWN));
+#endif
         }
         if (parser.Found(wxT("o")))
         {
@@ -366,13 +356,12 @@ bool PTBatcherGUI::OnInit()
     }
     else
     {
-        if (parser.Found(wxT("p")))
-        {
-            conn->Request(wxT("SetParallelCheck"));
-        }
         if (parser.Found(wxT("s")))
         {
+#if !defined __WXMAC__ && !defined __WXOSX_COCOA__
+            // wxMac has not wxShutdown
             conn->Request(wxT("SetShutdownCheck"));
+#endif
         }
         if (parser.Found(wxT("o")))
         {
@@ -414,6 +403,13 @@ int PTBatcherGUI::OnExit()
     return 0;
 }
 
+#if wxUSE_ON_FATAL_EXCEPTION
+void PTBatcherGUI::OnFatalException()
+{
+    GenerateReport(wxDebugReport::Context_Exception);
+};
+#endif
+
 void PTBatcherGUI::OnItemActivated(wxListEvent& event)
 {
     wxCommandEvent dummy;
@@ -491,18 +487,17 @@ wxChar* BatchIPCConnection::OnRequest(const wxString& topic, const wxString& ite
     };
     wxCommandEvent event;
     event.SetInt(1);
-    if(item==wxT("SetParallelCheck"))
-        if(!MyBatchFrame->GetCheckParallel())
-        {
-            MyBatchFrame->OnCheckParallel(event);
-            MyBatchFrame->SetCheckboxes();
-        };
+#if !defined __WXMAC__ && !defined __WXOSX_COCOA__
+    // wxMac has not wxShutdown
     if(item==wxT("SetShutdownCheck"))
-        if(!MyBatchFrame->GetCheckShutdown())
+        if (MyBatchFrame->GetEndTask()!=Batch::SHUTDOWN)
         {
-            MyBatchFrame->OnCheckShutdown(event);
+            wxCommandEvent choiceEvent;
+            choiceEvent.SetInt(Batch::SHUTDOWN);
+            MyBatchFrame->OnChoiceEnd(choiceEvent);
             MyBatchFrame->SetCheckboxes();
         };
+#endif
     if(item==wxT("SetOverwriteCheck"))
         if(!MyBatchFrame->GetCheckOverwrite())
         {
