@@ -18,14 +18,15 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
 #include "CenterHorizontally.h"
 
 #include <vigra/impex.hxx>
+#include <vigra/copyimage.hxx>
 #include <panodata/PanoramaData.h>
 #include <nona/RemappedPanoImage.h>
 
@@ -114,8 +115,8 @@ void CenterHorizontally::centerHorizontically(PanoramaData& panorama)
                 curColOccupied = true;
             }
         }
-        if (colOccupied && (! curColOccupied) ||
-            (!colOccupied) && curColOccupied )
+        if ((colOccupied && !curColOccupied) ||
+            (!colOccupied && curColOccupied))
         {
             // change in position, save point.
             borders.push_back(h-180);
