@@ -16,8 +16,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -52,7 +52,7 @@ inline double weightHuber(double x, double sigma)
 PhotometricOptimizer::OptimData::OptimData(const PanoramaData & pano, const OptimizeVector & optvars,
                                            const std::vector<vigra_ext::PointPairRGB> & data,
                                            double mEstimatorSigma, bool symmetric,
-                                           int maxIter, AppBase::ProgressReporter & progress)
+                                           int maxIter, AppBase::ProgressDisplay* progress)
   : m_pano(pano), m_data(data), huberSigma(mEstimatorSigma), symmetricError(symmetric),
     m_maxIter(maxIter), m_progress(progress)
 {
@@ -270,12 +270,12 @@ int PhotometricOptimizer::photometricVis(double *p, double *x, int m, int n, int
     tmp[199] = 0;
     double error = sqrt(sqerror/n)*255;
     snprintf(tmp,199, "Iteration: %d, error: %f", iter, error);
-    return dat->m_progress.increaseProgress(0.0, tmp) ? 1 : 0 ;
+    return dat->m_progress->updateDisplay(std::string(tmp)) ? 1 : 0 ;
 }
 
 void PhotometricOptimizer::optimizePhotometric(PanoramaData & pano, const OptimizeVector & vars,
                                                const std::vector<vigra_ext::PointPairRGB> & correspondences,
-                                               AppBase::ProgressReporter & progress,
+                                               AppBase::ProgressDisplay* progress,
                                                double & error)
 {
 
@@ -393,7 +393,7 @@ bool CheckStrangeWB(PanoramaData & pano)
 
 void SmartPhotometricOptimizer::smartOptimizePhotometric(PanoramaData & pano, PhotometricOptimizeMode mode,
                                                           const std::vector<vigra_ext::PointPairRGB> & correspondences,
-                                                          AppBase::ProgressReporter & progress,
+                                                          AppBase::ProgressDisplay* progress,
                                                           double & error)
 {
     std::vector<SrcPanoImage> ret;
@@ -401,7 +401,7 @@ void SmartPhotometricOptimizer::smartOptimizePhotometric(PanoramaData & pano, Ph
     UIntSet images;
     fill_set(images, 0, pano.getNrOfImages()-1);
     std::vector<UIntSet> stacks = getHDRStacks(pano, images, pano.getOptions());
-    bool singleStack = (stacks.size() == 1);
+    const bool singleStack = (stacks.size() == 1);
 
     int vars = 0;
     if (mode == OPT_PHOTOMETRIC_LDR || mode == OPT_PHOTOMETRIC_LDR_WB)
@@ -418,7 +418,7 @@ void SmartPhotometricOptimizer::smartOptimizePhotometric(PanoramaData & pano, Ph
         //optimize vignetting only if there are more than 1 image stack
         // for a single stack vignetting can't be calculated by this optimization
         vars |= OPT_VIG;
-        const VariableMapVector & oldVars = pano.getVariables();
+        VariableMapVector oldVars = pano.getVariables();
         optimizePhotometric(pano, 
                             createOptVars(pano, vars, opts.colorReferenceImage),
                             correspondences, progress, error);
@@ -437,14 +437,14 @@ void SmartPhotometricOptimizer::smartOptimizePhotometric(PanoramaData & pano, Ph
     {
         vars |= OPT_WB;
     };
-    const VariableMapVector & oldVars = pano.getVariables();
+    VariableMapVector oldVars = pano.getVariables();
     optimizePhotometric(pano, 
                         createOptVars(pano, vars, opts.colorReferenceImage),
                         correspondences, progress, error);
     // now check the results
-    bool hasHighVignetting = IsHighVignetting(pano.getImage(0).getRadialVigCorrCoeff());
+    const bool hasHighVignetting = IsHighVignetting(pano.getImage(0).getRadialVigCorrCoeff());
     // @TODO check also response curve, what parameters are considered invalid?
-    bool hasHighWB = CheckStrangeWB(pano);
+    const bool hasHighWB = CheckStrangeWB(pano);
     if(hasHighVignetting)
     {
         vars &= ~OPT_VIG;
@@ -469,21 +469,14 @@ void SmartPhotometricOptimizer::smartOptimizePhotometric(PanoramaData & pano, Ph
 
 bool PhotometricOptimizer::runAlgorithm()
 {
-    // is this correct? how much progress requierd?
-    AppBase::ProgressReporter* progRep = 
-        AppBase::ProgressReporterAdaptor::newProgressReporter(getProgressDisplay(), 0.0);    
-    
     optimizePhotometric(o_panorama, 
                         o_vars, o_correspondences,
-                        *progRep, o_resultError);
-    
-    delete progRep;
+                        getProgressDisplay(), o_resultError);
     
     // optimizePhotometric does not tell us if it's cancelled
-    if(hasProgressDisplay())
+    if (getProgressDisplay()->wasCancelled())
     {
-        if(getProgressDisplay()->wasCancelled())
-            cancelAlgorithm();
+        cancelAlgorithm();
     }
     
     return wasCancelled(); // let's hope so.
@@ -491,28 +484,16 @@ bool PhotometricOptimizer::runAlgorithm()
 
 bool SmartPhotometricOptimizer::runAlgorithm()
 {
-    AppBase::ProgressReporter* progRep;
-    
-    if(hasProgressDisplay()) {
-        // is this correct? how much progress requierd?
-        progRep = new AppBase::ProgressReporterAdaptor(*getProgressDisplay(), 0.0); 
-    } else {
-        progRep = new AppBase::DummyProgressReporter();
-    }
-    
     smartOptimizePhotometric(o_panorama,
                              o_optMode,
                              o_correspondences,
-                             *progRep, o_resultError);
-    
-    delete progRep;
+                             getProgressDisplay(), o_resultError);
     
     // smartOptimizePhotometric does not tell us if it's cancelled
-    if(hasProgressDisplay())
+    if (getProgressDisplay()->wasCancelled())
     {
-        if(getProgressDisplay()->wasCancelled())
-            cancelAlgorithm();
-    }
+        cancelAlgorithm();
+    };
     
     return !wasCancelled(); // let's hope so.
 }
