@@ -16,8 +16,8 @@
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
-*  License along with this software; if not, write to the Free Software
-*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*  License along with this software. If not, see
+*  <http://www.gnu.org/licenses/>.
 *
 */
 
@@ -27,13 +27,19 @@
 #include <ctime>
 
 #include <hugin_shared.h>
+#include <hugin_config.h>
+
 #include <algorithms/PanoramaAlgorithm.h>
 
+#ifdef HAVE_CXX11
+#include <random>
+#include <functional>
+#else
 #include <boost/random.hpp>
+#endif
 #include <vigra/impex.hxx>
 #include <vigra_ext/utils.h>
-#include <vigra_ext/Pyramid.h>
-#include <appbase/ProgressReporterOld.h>
+#include <appbase/ProgressDisplay.h>
 #include <panodata/PanoramaData.h>
 
 namespace HuginBase
@@ -58,7 +64,7 @@ namespace HuginBase
         public:
             /// for compatibility deprecated
             static void extractPoints(PanoramaData& pano, std::vector<vigra::FRGBImage*> images, int nPoints,
-                                      bool randomPoints, AppBase::ProgressReporter& progress,
+                                      bool randomPoints, AppBase::ProgressDisplay* progress,
                                       std::vector<vigra_ext::PointPairRGB>& points);
         
         protected:
@@ -68,7 +74,7 @@ namespace HuginBase
                                                  vigra_ext::interp_cubic           > InterpolImg;
             
             ///
-            void sampleAndExtractPoints(AppBase::ProgressReporter& progress);
+            void sampleAndExtractPoints(AppBase::ProgressDisplay* progress);
             
             ///
             virtual void samplePoints(const std::vector<InterpolImg>& imgs,
@@ -80,7 +86,7 @@ namespace HuginBase
                                       std::vector<std::multimap<double,vigra_ext::PointPairRGB> >& radiusHist,
                                       unsigned& nGoodPoints,
                                       unsigned& nBadPoints,
-                                      AppBase::ProgressReporter& progress) =0;
+                                      AppBase::ProgressDisplay*) = 0;
             
             /** extract some random points out of the bins.
             *  This should ensure that the radius distribution is
@@ -90,7 +96,7 @@ namespace HuginBase
             static void sampleRadiusUniform(const std::vector<std::multimap<double,PointPairClass> >& radiusHist,
                                             unsigned nPoints,
                                             std::vector<PointPairClass>& selectedPoints,
-                                            AppBase::ProgressReporter& progress);
+                                            AppBase::ProgressDisplay*);
             
         public:
             ///
@@ -158,7 +164,7 @@ namespace HuginBase
                                      std::vector<std::multimap<double, PP > > & radiusHist,
                                      unsigned & nGoodPoints,
                                      unsigned & nBadPoints,
-                                     AppBase::ProgressReporter & progress);
+                                     AppBase::ProgressDisplay* progress);
             
         protected:
             ///
@@ -171,7 +177,7 @@ namespace HuginBase
                                       std::vector<std::multimap<double,vigra_ext::PointPairRGB> >& radiusHist,
                                       unsigned& nGoodPoints,
                                       unsigned& nBadPoints,
-                                      AppBase::ProgressReporter& progress)
+                                      AppBase::ProgressDisplay* progress)
             {
                 sampleAllPanoPoints(imgs,
                                     voteImgs,
@@ -217,7 +223,7 @@ namespace HuginBase
                                                float maxI,
                                                std::vector<std::multimap<double, PP > > & radiusHist,
                                                unsigned & nBadPoints,
-                                               AppBase::ProgressReporter & progress);
+                                               AppBase::ProgressDisplay* progress);
             
         protected:
             ///
@@ -230,7 +236,7 @@ namespace HuginBase
                                       std::vector<std::multimap<double,vigra_ext::PointPairRGB> >& radiusHist,
                                       unsigned& nGoodPoints,
                                       unsigned& nBadPoints,
-                                      AppBase::ProgressReporter& progress)
+                                      AppBase::ProgressDisplay* progress)
             {
                  sampleRandomPanoPoints(imgs,
                                         voteImgs,
@@ -257,19 +263,15 @@ namespace HuginBase
 //==============================================================================
 //  templated methods
 
-
-#include <boost/random.hpp>
 #include <panotools/PanoToolsInterface.h>
 
-
 namespace HuginBase {
 
-
 template<class PointPairClass>
 void PointSampler::sampleRadiusUniform(const std::vector<std::multimap<double, PointPairClass> >& radiusHist,
                                        unsigned nPoints,
                                        std::vector<PointPairClass> &selectedPoints,
-                                       AppBase::ProgressReporter & progress)
+                                       AppBase::ProgressDisplay* progress)
 {
     typedef std::multimap<double,PointPairClass> PointPairMap;
     
@@ -291,7 +293,6 @@ void PointSampler::sampleRadiusUniform(const std::vector<std::multimap<double, P
             if (i == 0)
                 break;
         }
-        progress.increaseProgress(1.0/radiusHist.size());
     }
 }
     
@@ -309,7 +310,7 @@ void AllPointSampler::sampleAllPanoPoints(const std::vector<Img> &imgs,
                                           std::vector<std::multimap<double, PP > > & radiusHist,
                                           unsigned & nGoodPoints,
                                           unsigned & nBadPoints,
-                                          AppBase::ProgressReporter& progress)
+                                          AppBase::ProgressDisplay* progress)
 {
     typedef typename Img::PixelType PixelType;
 
@@ -430,10 +431,6 @@ void AllPointSampler::sampleAllPanoPoints(const std::vector<Img> &imgs,
                 }
             }
         }
-        int h = dest.getROI().bottom() - dest.getROI().top();
-        if ((y-dest.getROI().top())%(h/10) == 0) {
-            progress.increaseProgress(0.1);
-        }
     }
 
     for(unsigned i=0; i < imgs.size(); i++) {
@@ -454,7 +451,7 @@ void RandomPointSampler::sampleRandomPanoPoints(const std::vector<Img>& imgs,
                                                 //std::vector<PP> &points,
                                                 std::vector<std::multimap<double, PP > > & radiusHist,
                                                 unsigned & nBadPoints,
-                                                AppBase::ProgressReporter & progress)
+                                                AppBase::ProgressDisplay* progress)
 {
     typedef typename Img::PixelType PixelType;
 
@@ -483,6 +480,14 @@ void RandomPointSampler::sampleRandomPanoPoints(const std::vector<Img>& imgs,
         maxr[i] = sqrt(((double)srcSize.x)*srcSize.x + ((double)srcSize.y)*srcSize.y) / 2.0;
     }
     // init random number generator
+#ifdef HAVE_CXX11
+    std::mt19937 rng(static_cast<unsigned int>(std::time(0)));
+    std::uniform_int_distribution<unsigned int> distribx(dest.getROI().left(), dest.getROI().right()-1);
+    std::uniform_int_distribution<unsigned int> distriby(dest.getROI().top(), dest.getROI().bottom()-1);
+
+    auto randX = std::bind(distribx, std::ref(rng));
+    auto randY = std::bind(distriby, std::ref(rng));
+#else
     boost::mt19937 rng;
     // start with a different seed every time.
     rng.seed(static_cast<unsigned int>(std::time(0)));
@@ -494,6 +499,7 @@ void RandomPointSampler::sampleRandomPanoPoints(const std::vector<Img>& imgs,
             randX(rng, distribx);             // glues randomness with mapping
     boost::variate_generator<boost::mt19937&, boost::uniform_int<> >
             randY(rng, distriby);             // glues randomness with mapping
+#endif
 
     double percentReported = 0.0;
     
@@ -599,19 +605,12 @@ void RandomPointSampler::sampleRandomPanoPoints(const std::vector<Img>& imgs,
                 }
             }
         }
-        double pc = (allPoints - nPoints);
-        double percentNow = (pc / allPoints) * 100.0;
-        if (percentNow - percentReported >= 10) {
-            percentReported = percentNow;
-            progress.increaseProgress(0.1);
-        }
     }
     for(unsigned i=0; i < imgs.size(); i++) {
         delete transf[i];
     }
     
     DEBUG_INFO("Point sampled: " << allPoints-nPoints)
-    progress.increaseProgress(1.0 - percentReported/100.0);
 }
 
 
