@@ -17,17 +17,24 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
 #include "utils.h"
+#include "hugin_version.h"
 
 #ifdef WIN32
+    #define NOMINMAX
     #include <sys/utime.h>
+    #include <ShlObj.h>
 #else
     #include <sys/time.h>
+    #include <cstdlib>
+    #include <unistd.h>
+    #include <sys/types.h>
+    #include <pwd.h>
 #endif
 #include <time.h>
 #include <fstream>
@@ -40,6 +47,8 @@
 #include <hugin_config.h>
 #endif
 #include <algorithm>
+#define BOOST_FILESYSTEM_VERSION 3
+#include <boost/filesystem.hpp>
 
 #ifdef __APPLE__
 #include <mach-o/dyld.h>  /* _NSGetExecutablePath */
@@ -189,6 +198,26 @@ std::string getPathPrefix(const std::string & filename)
     }
 }
 
+std::string StrTrim(const std::string& str)
+{
+    std::string s(str);
+    std::string::size_type pos = s.find_last_not_of(" \t");
+    if (pos != std::string::npos)
+    {
+        s.erase(pos + 1);
+        pos = s.find_first_not_of(" \t");
+        if (pos != std::string::npos)
+        {
+            s.erase(0, pos);
+        };
+    }
+    else
+    {
+        s.erase(s.begin(), s.end());
+    };
+    return s;
+}
+
 std::string doubleToString(double d, int digits)
 {
     char fmt[10];
@@ -224,6 +253,77 @@ std::string doubleToString(double d, int digits)
     return number;
 }
 
+bool stringToInt(const std::string& s, int& val)
+{
+    if (StrTrim(s) == "0")
+    {
+        val = 0;
+        return true;
+    };
+    int x = atoi(s.c_str());
+    if (x != 0)
+    {
+        val = x;
+        return true;
+    };
+    return false;
+};
+
+bool stringToUInt(const std::string&s, unsigned int& val)
+{
+    int x;
+    if (stringToInt(s, x))
+    {
+        if (x >= 0)
+        {
+            val = static_cast<unsigned int>(x);
+            return true;
+        };
+    };
+    return false;
+};
+
+std::vector<std::string> SplitString(const std::string& s, const std::string& sep)
+{
+    std::vector<std::string> result;
+    std::size_t pos = s.find_first_of(sep, 0);
+    std::size_t pos2 = 0;
+    while (pos != std::string::npos)
+    {
+        if (pos - pos2 > 0)
+        {
+            std::string t(s.substr(pos2, pos - pos2));
+            t=StrTrim(t);
+            if (!t.empty())
+            {
+                result.push_back(t);
+            };
+        };
+        pos2 = pos + 1;
+        pos = s.find_first_of(sep, pos2);
+    }
+    if (pos2 < s.length())
+    {
+        std::string t(s.substr(pos2));
+        t = StrTrim(t);
+        if (!t.empty())
+        {
+            result.push_back(t);
+        };
+    };
+    return result;
+};
+
+void ReplaceAll(std::string& s, const std::string& oldChar, char newChar)
+{
+    std::size_t found = s.find_first_of(oldChar);
+    while (found != std::string::npos)
+    {
+        s[found] = newChar;
+        found = s.find_first_of(oldChar, found + 1);
+    };
+};
+
     void ControlPointErrorColour(const double cperr, 
         double &r,double &g, double &b)
     {
@@ -293,8 +393,8 @@ std::string GetDataDir()
 #if _WINDOWS
     char buffer[MAX_PATH];//always use MAX_PATH for filepaths
     GetModuleFileName(NULL,buffer,sizeof(buffer));
-    std::string working_path=(buffer);
-    std::string data_path="";
+    std::string working_path(buffer);
+    std::string data_path("");
     //remove filename
     std::string::size_type pos=working_path.rfind("\\");
     if(pos!=std::string::npos)
@@ -325,6 +425,49 @@ std::string GetDataDir()
     return data_path;
 };
 
+std::string GetUserAppDataDir()
+{
+    boost::filesystem::path path;
+#ifdef _WIN32
+    char fullpath[_MAX_PATH];
+    if(SHGetFolderPath(NULL, CSIDL_APPDATA | CSIDL_FLAG_CREATE, NULL, 0, fullpath)!=S_OK)
+    {
+        return std::string();
+    };
+    path = boost::filesystem::path(fullpath);
+    path /= "hugin";
+#else
+    char *homedir = getenv("HOME");
+    struct passwd *pw;
+    if (homedir == NULL)
+    {
+        pw = getpwuid(getuid());
+        if(pw != NULL)
+        {
+            homedir = pw->pw_dir;
+        };
+    };
+    if(homedir == NULL)
+    {
+        return std::string();
+    };
+    path = boost::filesystem::path(homedir);
+    // we have already a file with name ".hugin" for our wxWidgets settings
+    // therefore we use directory ".hugindata" in homedir
+    path /= ".hugindata";
+#endif
+    if (!boost::filesystem::exists(path))
+    {
+        if (!boost::filesystem::create_directories(path))
+        {
+            std::cerr << "ERROR: Could not create destination directory: " << path.string() << std::endl
+                << "Maybe you have not sufficent rights to create this directory." << std::endl;
+            return std::string();
+        };
+    };
+    return path.string();
+};
+
 // initialization and wrapup of GPU for GPU remapping
 static GLuint GlutWindowHandle;
 
@@ -344,7 +487,7 @@ bool initGPU(int *argcp, char **argv)
         return false;
     }
 
-    std::cout << argv[0] << ": using graphics card: " << glGetString(GL_VENDOR) << " " << glGetString(GL_RENDERER) << std::endl;
+    std::cout << hugin_utils::stripPath(argv[0]) << ": using graphics card: " << glGetString(GL_VENDOR) << " " << glGetString(GL_RENDERER) << std::endl;
 
     GLboolean has_arb_fragment_shader = glewGetExtension("GL_ARB_fragment_shader");
     GLboolean has_arb_vertex_shader = glewGetExtension("GL_ARB_vertex_shader");
@@ -378,4 +521,9 @@ bool wrapupGPU()
     return true;
 }
 
+std::string GetHuginVersion()
+{
+    return std::string(DISPLAY_VERSION);
+};
+
 } //namespace
