@@ -16,8 +16,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -190,6 +190,12 @@ namespace hugin_utils
         }
     }
 
+    /** convert string to integer value, returns true, if sucessful */
+    IMPEX bool stringToInt(const std::string& s, int& val);
+
+    /** convert string to unsigned integer value, returns true, if sucessful */
+    IMPEX bool stringToUInt(const std::string& s, unsigned int& val);
+
     /** Get the path to a filename */
     IMPEX std::string getPathPrefix(const std::string & filename);
 
@@ -204,26 +210,14 @@ namespace hugin_utils
     /** remove extension of a filename */
     IMPEX std::string stripExtension(const std::string & basename);
 
-    template <typename Target, typename Source>
-    Target lexical_cast(Source arg) {
+    /** remove trailing and leading white spaces and tabs */
+    IMPEX std::string StrTrim(const std::string& str);
 
-        std::stringstream interpreter;
-
-        Target result;
-
-        if (!(interpreter << arg) ||
-            !(interpreter >> result) ||
-            !(interpreter >> std::ws).eof()) {
-
-            DEBUG_ERROR("lexical cast error");
-            // cast error.  handle it somehow
-            // boost guys throw an exception here
-        };
-
-        return result;
-
-    }; // lexical cast
+    /** split string s at given sep, returns vector of strings */
+    IMPEX std::vector<std::string> SplitString(const std::string& s, const std::string& sep);
 
+    /** replace all characters oldChar in s with newChar */
+    IMPEX void ReplaceAll(std::string& s, const std::string& oldChar, char newChar);
 
     template <class str>
     str QuoteStringInternal(const str & arg, const str & quotechar,
@@ -247,23 +241,7 @@ namespace hugin_utils
         }
         return ret;
     }
-    
-    ///
-    template <class str>
-    str replaceAll(const str& arg, const str& target, const str& replacement)
-    {
-        str ret(arg);
-        typename str::size_type pos = ret.find(target, 0);
-        
-        for ( typename str::size_type n = 0 ;  pos != str::npos ;  pos = ret.find(target, n) )
-        {
-            ret.replace(pos, target.size(), replacement);
-            n = pos + replacement.size();
-        }
-        
-        return ret;
-    }
-    
+
     IMPEX void ControlPointErrorColour(const double cperr, 
         double &r,double &g, double &b);
 
@@ -276,6 +254,11 @@ namespace hugin_utils
     /** returns the full path to the data directory */
     IMPEX std::string GetDataDir();
 
+    /** returns the directory for user specific Hugin settings, e.g. lens database
+        the directory is automatically created if it does not exists 
+        @return path, empty if path could not retrieved or not created */
+    IMPEX std::string GetUserAppDataDir();
+
     /** Try to initalise GLUT and GLEW, and create an OpenGL context for GPU stitching.
     * OpenGL extensions required by the GPU stitcher (-g option) are checked here.
     * @return true if everything went OK, false if we can't use GPU mode.
@@ -283,6 +266,8 @@ namespace hugin_utils
     IMPEX bool initGPU(int *argcp, char **argv);
     /** cleanup GPU settings */
     IMPEX bool wrapupGPU();
+    /** return a string with version numbers */
+    IMPEX std::string GetHuginVersion();
 
 } // namespace
 
