@@ -19,15 +19,20 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
 #include "ImageCache.h"
 
 #include <iostream>
+#include "hugin_config.h"
+#ifdef USE_CXX11_THREAD
+#include <thread>
+#else
 #include <boost/thread/thread.hpp>
+#endif
 #include <vigra/inspectimage.hxx>
 #include <vigra/accessor.hxx>
 #include <vigra/functorexpression.hxx>
@@ -154,6 +159,12 @@ void ImageCache::removeImage(const std::string & filename)
     } while (found);
 }
 
+std::string ImageCache::PyramidKey::toString()
+{
+    std::ostringstream s;
+    s << filename << level;
+    return s.str();
+};
 
 void ImageCache::flush()
 {
@@ -280,10 +291,6 @@ void ImageCache::softFlush()
                 break;
             }
         }
-//        m_progress->progressMessage(
-//            wxString::Format(_("Purged %d MB from image cache. Current cache usage: %d MB"),
-//                             purgedMem>>20, (usedMem - purgedMem)>>20
-//                ).c_str(),0);
         DEBUG_DEBUG("purged: " << (purgedMem>>20) << " MB, memory used for images: " << ((usedMem - purgedMem)>>20) << " MB");
 
     }
@@ -511,13 +518,13 @@ ImageCache::EntryPtr ImageCache::getImage(const std::string & filename)
         return it->second;
     } else {
         if (m_progress) {
-            m_progress->pushTask(AppBase::ProgressTask("Loading image: "+hugin_utils::stripPath(filename), "", 0));
+            m_progress->setMessage("Loading image:", hugin_utils::stripPath(filename));
         }
         
         EntryPtr e = loadImageSafely(filename);
         
         if (m_progress) {
-            m_progress->popTask();
+            m_progress->taskFinished();
         }
         
         if (!e.get())
@@ -728,7 +735,7 @@ ImageCache::EntryPtr ImageCache::getSmallImage(const std::string & filename)
     } else {
         if (m_progress)
         {
-            m_progress->pushTask(AppBase::ProgressTask("Scaling image: "+hugin_utils::stripPath(filename), "", 0));
+            m_progress->setMessage("Scaling image:", hugin_utils::stripPath(filename));
         }
         DEBUG_DEBUG("creating small image " << name );
         EntryPtr entry = getImage(filename);
@@ -738,7 +745,7 @@ ImageCache::EntryPtr ImageCache::getSmallImage(const std::string & filename)
         images[name] = small_entry;
         DEBUG_INFO ( "created small image: " << name);
         if (m_progress) {
-            m_progress->popTask();
+            m_progress->taskFinished();
         }
         return small_entry;
     }
@@ -889,7 +896,11 @@ void ImageCache::postEvent(RequestPtr request, EntryPtr entry)
         } else if (getSmallImageIfAvailable(it->first).get()) {
             // already loaded.
             // signal to anything waiting and remove from the list.
-            it->second->ready(getSmallImage(it->first), it->first, true);
+            while (!it->second->ready.empty())
+            {
+                it->second->ready.front()(getSmallImage(it->first), it->first, true);
+                it->second->ready.erase(it->second->ready.begin());
+            };
             m_smallRequests.erase(it);
         }
         it = next_it;
@@ -907,7 +918,11 @@ void ImageCache::postEvent(RequestPtr request, EntryPtr entry)
         } else if (getImageIfAvailable(it->first).get()) {
             // already loaded.
             // Signal to anything waiting.
-            it->second->ready(getImage(it->first), it->first, false);
+            while (!it->second->ready.empty())
+            {
+                it->second->ready.front()(getImage(it->first), it->first, false);
+                it->second->ready.erase(it->second->ready.begin());
+            };
             m_requests.erase(it);
         }
         it = next_it;
@@ -930,7 +945,12 @@ void ImageCache::spawnAsyncThread()
         {
             DEBUG_DEBUG("Not staring a thread to load an image, since no images are wanted.");
         } else {
+#ifdef USE_CXX11_THREAD
+            std::thread thread(loadSafely, it->second, EntryPtr());
+            thread.detach();
+#else
             boost::thread(loadSafely, it->second, EntryPtr());
+#endif
         }
     } else {
         // got a small image request, check if its larger version has loaded.
@@ -940,10 +960,20 @@ void ImageCache::spawnAsyncThread()
         {
             // the larger one is needed to generate it.
             RequestPtr request(new Request(filename, false));
+#ifdef USE_CXX11_THREAD
+            std::thread thread(loadSafely, request, EntryPtr());
+            thread.detach();
+#else
             boost::thread(loadSafely, request, EntryPtr());
+#endif
         } else {
             // we have the large image.
+#ifdef USE_CXX11_THREAD
+            std::thread thread(loadSafely, it->second, large);
+            thread.detach();
+#else
             boost::thread(loadSafely, it->second, large);
+#endif
         }
     }
     // thread should be processing the image asynchronously now.
