@@ -16,8 +16,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -48,6 +48,16 @@ unsigned int ConstImageVariableGroup::getPartNumber(unsigned int imageNr) const
     return m_image_part_numbers[imageNr];
 }
 
+UIntSetVector ConstImageVariableGroup::getPartsSet() const
+{
+    UIntSetVector result(getNumberOfParts(), HuginBase::UIntSet());
+    for (unsigned int imgNr = 0; imgNr<m_image_part_numbers.size(); ++imgNr)
+    {
+        result[m_image_part_numbers[imgNr]].insert(imgNr);
+    };
+    return result;
+}
+
 bool ConstImageVariableGroup::getVarLinkedInPart(ImageVariableEnum variable,
                                                  std::size_t part) const
 {
@@ -248,7 +258,7 @@ void ImageVariableGroup::switchParts (unsigned int imageNr, unsigned int partNr)
     setPartNumbers();
 }
 
-std::size_t ConstImageVariableGroup::getNumberOfParts()
+std::size_t ConstImageVariableGroup::getNumberOfParts() const
 {
     return m_num_parts;
 }
