@@ -16,8 +16,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -38,10 +38,7 @@
 #endif // _HSI_IGNORE_SECTION
 
 namespace HuginBase {
-    
-    
-    
-    
+
 /** Memento class for a Panorama object
 *
 *  Holds the internal state of a Panorama.
@@ -67,18 +64,12 @@ class IMPEX PanoramaMemento : public PanoramaDataMemento
         
         virtual ~PanoramaMemento();
         
-        
-    protected:
-        /** enum for supported PTScript syntax bastards */
-        //  enum PTFileFormat { PTFILE_HUGIN, PTFILE_PTGUI, PTFILE_PTA };
-        
-        /** load a PTScript file
+        /** load a Hugin file
         *
-        *  initializes the PanoramaMemento from a PTScript file
+        *  initializes the PanoramaMemento from a script file
         */
         bool loadPTScript(std::istream & i, int & ptoVersion, const std::string & prefix = "");
-    
-        
+
     private:
         enum PTParseState {
             P_NONE,
@@ -432,6 +423,13 @@ class IMPEX Panorama : public ManagedPanoramaData, public AppBase::DocumentData
         */
         virtual void updateWhiteBalance(double redFactor, double blueFactor);
 
+        /** returns the maximum exposure value difference of all images in the project */
+        const double getMaxExposureDifference() const;
+        /** return true, if the metadata indicates that the projects is a bracketet project */
+        const bool hasPossibleStacks() const;
+        /** create automatically stacks as indicated by metadata */
+        void linkPossibleStacks(bool linkPosition);
+
     // = Optimise Vector =    
         /** return the optimize settings stored inside panorama */
         const OptimizeVector & getOptimizeVector() const
@@ -676,31 +674,7 @@ class IMPEX Panorama : public ManagedPanoramaData, public AppBase::DocumentData
         /// adjust the links of the linked variables, must be called
         /// when a lens has been changed.
         void adjustVarLinks();
-
-
-        /// image addition notification
-    //    void notifyImageAdded(unsigned int imgNr);
-
-        /// image removal notification
-    //    void notifyImageRemoved(unsigned int imgNr);
-
-        /// image change notification
-    //    void notifyImageChanged(unsigned int imgNr);
-
-        
-
     private:
-
-        // data
-//        enum ProcessType { NO_PROCESS, OPTIMIZER, STITCHER };
-
-        // to run stitcher & optimizer
-//        ProcessType currentProcess;
-//        std::string optimizerExe;
-//        std::string stitcherExe;
-//        std::string PTScriptFile;
-        
-        //
         /** center the crop for given image and all linked images */
         void centerCrop(unsigned int imgNr);
         /** return the centered crop for given image */
@@ -723,12 +697,5 @@ class IMPEX Panorama : public ManagedPanoramaData, public AppBase::DocumentData
         std::set<std::string> m_ptoptimizerVarNames;
 };
 
-
-
-
 } // namespace
-
-
-
-
 #endif // _PANORAMA_H
