@@ -16,8 +16,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -121,7 +121,9 @@ class IMPEX PanoramaOptions
          */
         enum FileFormat {
             JPEG = 0,
+            JPEG_m,
             PNG,
+            PNG_m,
             TIFF,
             TIFF_m,
             TIFF_mask,
@@ -160,7 +162,8 @@ class IMPEX PanoramaOptions
             PTBLENDER_BLEND=1,
             ENBLEND_BLEND=2,
             SMARTBLEND_BLEND=3,
-            PTMASKER_BLEND=4
+            PTMASKER_BLEND=4,
+            INTERNAL_BLEND=5
         };
 
         ///
