@@ -43,8 +43,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -138,6 +138,7 @@ image_variable( ExifDate, std::string, "")
 image_variable( ExifExposureMode, int, 0)
 image_variable( ExifRedBalance, double, 1)
 image_variable( ExifBlueBalance, double, 1)
+image_variable( FileMetadata, HuginBase::FileMetaData, HuginBase::FileMetaData())
 
 #if 0
 //
