@@ -16,8 +16,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -26,6 +26,11 @@
 
 #include <vector>
 #include <functional>
+#include "hugin_config.h"
+#ifdef HAVE_CXX11
+#include <random>
+#define RANDOMGENERATOR std::mt19937
+#else
 #include <boost/version.hpp>
 #if BOOST_VERSION>104700
 #include <boost/random/taus88.hpp>
@@ -34,6 +39,7 @@
 #include <boost/random/mersenne_twister.hpp>
 #define RANDOMGENERATOR boost::mt19937
 #endif
+#endif
 
 #include <vigra/stdimage.hxx>
 #include <vigra/numerictraits.hxx>
@@ -344,8 +350,8 @@ double ResponseTransform<VTIn>::calcVigFactor(hugin_utils::FDiff2D d) const
     } else if (m_VigCorrMode & HuginBase::SrcPanoImage::VIGCORR_FLATFIELD) {
         // TODO: implement flatfield
         if (m_flatfield) {
-            int x = std::min(std::max(hugin_utils::roundi(d.x),0), m_flatfield->width()-1);;
-            int y = std::min(std::max(hugin_utils::roundi(d.y),0), m_flatfield->height()-1);;
+            int x = std::min(std::max(hugin_utils::roundi(d.x),0), static_cast<int>(m_flatfield->width()-1));
+            int y = std::min(std::max(hugin_utils::roundi(d.y),0), static_cast<int>(m_flatfield->height()-1));
             return (*m_flatfield)(x,y);
         } else {
             return 1;
