@@ -16,31 +16,29 @@
 *
 * You should have received a copy of the GNU General Public License
 * along with Panomatic; if not, write to the Free Software
-* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+* <http://www.gnu.org/licenses/>.
 */
 
 #include "PanoDetector.h"
 #include <iostream>
 #include <fstream>
 #include <sstream>
-#include <boost/foreach.hpp>
 
 #include <time.h>
 
-#include "zthread/Runnable.h"
-#include "zthread/PoolExecutor.h"
 #include "Utils.h"
 #include "Tracer.h"
 #include "hugin_base/hugin_utils/platform.h"
 
 #include <algorithms/nona/ComputeImageROI.h>
+#include <algorithms/basic/CalculateOptimalScale.h>
 #include <nona/RemappedPanoImage.h>
 #include <nona/ImageRemapper.h>
 
 //for multi row strategy
 #include <panodata/StandardImageVariableGroups.h>
-#include <PT/Panorama.h>
-#include <PT/ImageGraph.h>
+#include <panodata/Panorama.h>
+#include <algorithms/optimizer/ImageGraph.h>
 #include <algorithms/optimizer/PTOptimizer.h>
 #include <algorithms/basic/CalculateOverlap.h>
 
@@ -50,15 +48,18 @@
 #include <direct.h>
 #else
 #include <unistd.h>
-#include <hugin_config.h>
 #endif
+#include <hugin_config.h>
 
 #ifndef srandom
 #define srandom srand
 #endif
 
+#ifdef HAVE_OPENMP
+#include <omp.h>
+#endif
+
 using namespace std;
-using namespace ZThread;
 using namespace HuginBase;
 using namespace AppBase;
 using namespace HuginBase::Nona;
@@ -107,7 +108,7 @@ PanoDetector::PanoDetector() :
     _writeAllKeyPoints(false), _verbose(1),
     _sieve1Width(10), _sieve1Height(10), _sieve1Size(100),
     _kdTreeSearchSteps(200), _kdTreeSecondDistance(0.25), _ransacIters(1000), _ransacDistanceThres(50),
-    _sieve2Width(5), _sieve2Height(5),_sieve2Size(1), _test(false), _cores(hugin_utils::getCPUCount()),
+    _sieve2Width(5), _sieve2Height(5),_sieve2Size(1), _test(false), _cores(0),
     _minimumMatches(6), _linearMatchLen(1), _downscale(true), _cache(false), _cleanup(false),
     _keypath(""), _outputFile("default.pto"), _outputGiven(false), _celeste(false), _celesteThreshold(0.5), _celesteRadius(20), 
     _matchingStrategy(ALLPAIRS)
@@ -172,7 +173,9 @@ void PanoDetector::printDetails()
     {
         cout << "Automatically cache keypoints files to disc." << endl;
     };
-    cout << "Number of CPU        : " << _cores << endl << endl;
+#ifdef HAVE_OPENMP
+    cout << "Number of threads  : " << (_cores>0 ? _cores : omp_get_max_threads()) << endl << endl;
+#endif
     cout << "Input image options" << endl;
     cout << "  Downscale to half-size : " << (_downscale?"yes":"no") << endl;
     if(_celeste)
@@ -276,14 +279,21 @@ void PanoDetector::printFilenames()
     };
 };
 
+class Runnable
+{
+public:
+    Runnable() {};
+    virtual void run() = 0;
+};
+
 // definition of a runnable class for image data
 class ImgDataRunnable : public Runnable
 {
 public:
-    ImgDataRunnable(PanoDetector::ImgData& iImageData, const PanoDetector& iPanoDetector) :
+    ImgDataRunnable(PanoDetector::ImgData& iImageData, const PanoDetector& iPanoDetector) : Runnable(),
         _imgData(iImageData), _panoDetector(iPanoDetector) {};
 
-    void run()
+    virtual void run()
     {
         TRACE_IMG("Analyzing image...");
         if (!PanoDetector::AnalyzeImage(_imgData, _panoDetector))
@@ -309,7 +319,7 @@ public:
     WriteKeyPointsRunnable(PanoDetector::ImgData& iImageData, const PanoDetector& iPanoDetector) :
         _imgData(iImageData), _panoDetector(iPanoDetector) {};
 
-    void run()
+    virtual void run()
     {
         TRACE_IMG("Analyzing image...");
         if (!PanoDetector::AnalyzeImage(_imgData, _panoDetector))
@@ -334,7 +344,7 @@ public:
     LoadKeypointsDataRunnable(PanoDetector::ImgData& iImageData, const PanoDetector& iPanoDetector) :
         _imgData(iImageData), _panoDetector(iPanoDetector) {};
 
-    void run()
+    virtual void run()
     {
         TRACE_IMG("Loading keypoints...");
         PanoDetector::LoadKeypoints(_imgData, _panoDetector);
@@ -353,7 +363,7 @@ public:
     MatchDataRunnable(PanoDetector::MatchData& iMatchData, const PanoDetector& iPanoDetector) :
         _matchData(iMatchData), _panoDetector(iPanoDetector) {};
 
-    void run()
+    virtual void run()
     {
         //TRACE_PAIR("Matching...");
         if(_matchData._i1->_kp.size()>0 && _matchData._i2->_kp.size()>0)
@@ -394,6 +404,23 @@ bool PanoDetector::LoadSVMModel()
     return true;
 };
 
+typedef std::vector<Runnable*> RunnableVector;
+
+void RunQueue(std::vector<Runnable*>& queue)
+{
+#pragma omp parallel for schedule(dynamic)
+    for (int i = 0; i < queue.size(); ++i)
+    {
+        queue[i]->run();
+    };
+    // now clear queue
+    while (!queue.empty())
+    {
+        delete queue.back();
+        queue.pop_back();
+    };
+};
+
 void PanoDetector::run()
 {
     // init the random time generator
@@ -432,7 +459,8 @@ void PanoDetector::run()
             };
         };
     };
-    if(maxImageSize!=0)
+#ifdef HAVE_OPENMP
+    if (maxImageSize != 0)
     {
         unsigned long long maxCores;
         //determinded factors by testing of some projects
@@ -450,6 +478,10 @@ void PanoDetector::run()
         {
             maxCores=1;
         }
+        if (_cores == 0)
+        {
+            setCores(omp_get_max_threads());
+        }
         if(maxCores<_cores)
         {
             if(getVerbose()>0)
@@ -460,8 +492,10 @@ void PanoDetector::run()
             setCores(maxCores);
         };
     };
-    PoolExecutor aExecutor(_cores);
-    svmModel=NULL;
+    omp_set_num_threads(_cores);
+#endif
+    RunnableVector queue;
+    svmModel = NULL;
     if(_celeste)
     {
         TRACE_INFO("\nLoading Celeste model file...\n");
@@ -484,45 +518,58 @@ void PanoDetector::run()
     //running multi threading part
     std::string s=vigra::impexListExtensions();
 #endif
-    try
+    if (_keyPointsIdx.size() != 0)
     {
-        if (_keyPointsIdx.size() != 0)
+        if (_verbose > 0)
         {
-            if (_verbose > 0)
+            TRACE_INFO(endl << "--- Analyze Images ---" << endl);
+        }
+        for (unsigned int i = 0; i < _keyPointsIdx.size(); ++i)
+        {
+            queue.push_back(new WriteKeyPointsRunnable(_filesData[_keyPointsIdx[i]], *this));
+        };
+    }
+    else
+    {
+        TRACE_INFO(endl << "--- Analyze Images ---" << endl);
+        if (getMatchingStrategy() == MULTIROW)
+        {
+            // when using multirow, don't analyse stacks with linked positions
+            buildMultiRowImageSets();
+            HuginBase::UIntSet imagesToAnalyse;
+            imagesToAnalyse.insert(_image_layer.begin(), _image_layer.end());
+            for (size_t i = 0; i < _image_stacks.size(); i++)
             {
-                TRACE_INFO(endl<< "--- Analyze Images ---" << endl);
+                imagesToAnalyse.insert(_image_stacks[i].begin(), _image_stacks[i].end());
             }
-            for (unsigned int i = 0; i < _keyPointsIdx.size(); ++i)
+            for (HuginBase::UIntSet::const_iterator it = imagesToAnalyse.begin(); it != imagesToAnalyse.end(); ++it)
             {
-                aExecutor.execute(new WriteKeyPointsRunnable(_filesData[_keyPointsIdx[i]], *this));
+                if (_filesData[*it]._hasakeyfile)
+                {
+                    queue.push_back(new LoadKeypointsDataRunnable(_filesData[*it], *this));
+                }
+                else
+                {
+                    queue.push_back(new ImgDataRunnable(_filesData[*it], *this));
+                };
             };
         }
         else
         {
-            TRACE_INFO(endl<< "--- Analyze Images ---" << endl);
             for (ImgDataIt_t aB = _filesData.begin(); aB != _filesData.end(); ++aB)
             {
                 if (aB->second._hasakeyfile)
                 {
-                    aExecutor.execute(new LoadKeypointsDataRunnable(aB->second, *this));
+                    queue.push_back(new LoadKeypointsDataRunnable(aB->second, *this));
                 }
                 else
                 {
-                    aExecutor.execute(new ImgDataRunnable(aB->second, *this));
+                    queue.push_back(new ImgDataRunnable(aB->second, *this));
                 }
             }
-        }
-        aExecutor.wait();
-    }
-    catch(Synchronization_Exception& e)
-    {
-        TRACE_ERROR(e.what() << endl);
-        if(svmModel!=NULL)
-        {
-            celeste::destroySVMmodel(svmModel);
         };
-        return;
     }
+    RunQueue(queue);
 
     if(svmModel!=NULL)
     {
@@ -558,14 +605,14 @@ void PanoDetector::run()
             case LINEAR:
                 {
                     std::vector<HuginBase::UIntSet> imgPairs(_panoramaInfo->getNrOfImages());
-                    if(!match(aExecutor, imgPairs))
+                    if(!match(imgPairs))
                     {
                         return;
                     };
                 };
                 break;
             case MULTIROW:
-                if(!matchMultiRow(aExecutor))
+                if(!matchMultiRow())
                 {
                     return;
                 };
@@ -590,7 +637,7 @@ void PanoDetector::run()
                         imgMap[i]=i;
                     };
                     //and the final matching step
-                    if(!matchPrealigned(aExecutor,_panoramaInfo, connectedImages, imgMap))
+                    if(!matchPrealigned(_panoramaInfo, connectedImages, imgMap))
                     {
                         return;
                     };
@@ -625,10 +672,11 @@ void PanoDetector::run()
     };
 }
 
-bool PanoDetector::match(PoolExecutor& aExecutor, std::vector<HuginBase::UIntSet> &checkedPairs)
+bool PanoDetector::match(std::vector<HuginBase::UIntSet> &checkedPairs)
 {
     // 3. prepare matches
-    _matchesData.clear();
+    RunnableVector queue;
+    MatchData_t matchesData;
     unsigned int aLen = _filesData.size();
     if (getMatchingStrategy()==LINEAR)
     {
@@ -655,9 +703,9 @@ bool PanoDetector::match(PoolExecutor& aExecutor, std::vector<HuginBase::UIntSet
                 continue;
             };
             // create a new entry in the matches map
-            _matchesData.push_back(MatchData());
+            matchesData.push_back(MatchData());
 
-            MatchData& aM = _matchesData.back();
+            MatchData& aM = matchesData.back();
             aM._i1 = &(_filesData[i1]);
             aM._i2 = &(_filesData[i2]);
 
@@ -667,23 +715,23 @@ bool PanoDetector::match(PoolExecutor& aExecutor, std::vector<HuginBase::UIntSet
     }
     // 4. find matches
     TRACE_INFO(endl<< "--- Find pair-wise matches ---" << endl);
-    try
+    for (size_t i = 0; i < matchesData.size(); ++i)
     {
-        BOOST_FOREACH(MatchData& aMD, _matchesData)
-        aExecutor.execute(new MatchDataRunnable(aMD, *this));
-        aExecutor.wait();
-    }
-    catch(Synchronization_Exception& e)
-    {
-        TRACE_ERROR(e.what() << endl);
-        return false;
-    }
+        queue.push_back(new MatchDataRunnable(matchesData[i], *this));
+    };
+    RunQueue(queue);
 
     // Add detected matches to _panoramaInfo
-    BOOST_FOREACH(MatchData& aM, _matchesData)
-    BOOST_FOREACH(lfeat::PointMatchPtr& aPM, aM._matches)
-    _panoramaInfo->addCtrlPoint(ControlPoint(aM._i1->_number, aPM->_img1_x, aPM->_img1_y,
-                                aM._i2->_number, aPM->_img2_x, aPM->_img2_y));
+    for (size_t i = 0; i < matchesData.size(); ++i)
+    {
+        const MatchData& aM = matchesData[i];
+        for (size_t j = 0; j < aM._matches.size(); ++j)
+        {
+            const lfeat::PointMatchPtr& aPM = aM._matches[j];
+            _panoramaInfo->addCtrlPoint(ControlPoint(aM._i1->_number, aPM->_img1_x, aPM->_img1_y,
+                aM._i2->_number, aPM->_img2_x, aPM->_img2_y));
+        };
+    };
     return true;
 };
 
@@ -796,7 +844,7 @@ bool PanoDetector::loadProject()
             aImgData._projOpts.setHeight(canvasSize.height());
 
             // set roi to cover the remapped input image
-            roi = roi * scalefactor;
+            roi *= scalefactor;
             _filesData[imgNr]._detectWidth = roi.width();
             _filesData[imgNr]._detectHeight = roi.height();
             aImgData._projOpts.setROI(roi);
@@ -825,7 +873,7 @@ bool PanoDetector::loadProject()
 
 bool PanoDetector::checkLoadSuccess()
 {
-    if(_keyPointsIdx.size()!=0)
+    if(!_keyPointsIdx.empty())
     {
         for (unsigned int i = 0; i < _keyPointsIdx.size(); ++i)
         {
@@ -859,136 +907,140 @@ void PanoDetector::CleanupKeyfiles()
     };
 };
 
-struct img_ev
+struct SortVectorByExposure
 {
-    unsigned int img_nr;
-    double ev;
-};
-struct stack_img
-{
-    unsigned int layer_nr;
-    std::vector<img_ev> images;
-};
-bool sort_img_ev (img_ev i1, img_ev i2)
-{
-    return (i1.ev<i2.ev);
+    SortVectorByExposure(const HuginBase::Panorama* pano) : m_pano(pano) {};
+    bool operator()(const size_t& img1, const size_t& img2)
+    {
+        return m_pano->getImage(img1).getExposureValue() < m_pano->getImage(img2).getExposureValue();
+    }
+private:
+    const HuginBase::Panorama* m_pano;
 };
 
-bool PanoDetector::matchMultiRow(PoolExecutor& aExecutor)
+void PanoDetector::buildMultiRowImageSets()
 {
-    //step 1
-    std::vector<HuginBase::UIntSet> checkedImagePairs(_panoramaInfo->getNrOfImages());
-    std::vector<stack_img> stack_images;
-    HuginBase::StandardImageVariableGroups* variable_groups = new HuginBase::StandardImageVariableGroups(*_panoramaInfo);
-    for(unsigned int i=0; i<_panoramaInfo->getNrOfImages(); i++)
+    HuginBase::ConstStandardImageVariableGroups variable_groups(*_panoramaInfo);
+    HuginBase::UIntSetVector imageGroups = variable_groups.getStacks().getPartsSet();
+    //get image with median exposure for search with cp generator
+    for (size_t imgGroup = 0; imgGroup < imageGroups.size(); ++imgGroup)
     {
-        unsigned int stack_nr=variable_groups->getStacks().getPartNumber(i);
-        //check, if this stack is already in list
-        bool found=false;
-        unsigned int index=0;
-        for(index=0; index<stack_images.size(); index++)
+        HuginBase::UIntVector stackImages(imageGroups[imgGroup].begin(), imageGroups[imgGroup].end());
+        std::sort(stackImages.begin(), stackImages.end(), SortVectorByExposure(_panoramaInfo));
+        size_t index = 0;
+        if (_panoramaInfo->getImage(*(stackImages.begin())).getExposureValue() != _panoramaInfo->getImage(*(stackImages.rbegin())).getExposureValue())
         {
-            found=(stack_images[index].layer_nr==stack_nr);
-            if(found)
+            index = stackImages.size() / 2;
+        };
+        _image_layer.insert(stackImages[index]);
+        if (stackImages.size()>1)
+        {
+            //build list for stacks, consider only unlinked stacks
+            if (!_panoramaInfo->getImage(*(stackImages.begin())).YawisLinked())
             {
-                break;
+                _image_stacks.push_back(stackImages);
             };
         };
-        if(!found)
-        {
-            //new stack
-            stack_images.resize(stack_images.size()+1);
-            index=stack_images.size()-1;
-            //add new stack
-            stack_images[index].layer_nr=stack_nr;
-        };
-        //add new image
-        unsigned int new_image_index=stack_images[index].images.size();
-        stack_images[index].images.resize(new_image_index+1);
-        stack_images[index].images[new_image_index].img_nr=i;
-        stack_images[index].images[new_image_index].ev=_panoramaInfo->getImage(i).getExposure();
     };
-    delete variable_groups;
-    //get image with median exposure for search with cp generator
-    vector<size_t> images_layer;
-    UIntSet images_layer_set;
-    for(unsigned int i=0; i<stack_images.size(); i++)
+};
+
+bool PanoDetector::matchMultiRow()
+{
+    RunnableVector queue;
+    MatchData_t matchesData;
+    //step 1
+    std::vector<HuginBase::UIntSet> checkedImagePairs(_panoramaInfo->getNrOfImages());
+    for (size_t i = 0; i < _image_stacks.size();i++)
     {
-        std::sort(stack_images[i].images.begin(),stack_images[i].images.end(),sort_img_ev);
-        unsigned int index=0;
-        if(stack_images[i].images[0].ev!=stack_images[i].images[stack_images[i].images.size()-1].ev)
+        //build match list for stacks
+        for(unsigned int j=0; j<_image_stacks[i].size()-1; j++)
         {
-            index=stack_images[i].images.size() / 2;
-        };
-        images_layer.push_back(stack_images[i].images[index].img_nr);
-        images_layer_set.insert(stack_images[i].images[index].img_nr);
-        if(stack_images[i].images.size()>1)
-        {
-            //build match list for stacks
-            for(unsigned int j=0; j<stack_images[i].images.size()-1; j++)
+            const size_t img1 = _image_stacks[i][j];
+            const size_t img2 = _image_stacks[i][j + 1];
+            matchesData.push_back(MatchData());
+            MatchData& aM=matchesData.back();
+            if (img1 < img2)
             {
-                size_t img1=stack_images[i].images[j].img_nr;
-                size_t img2=stack_images[i].images[j+1].img_nr;
-                _matchesData.push_back(MatchData());
-                MatchData& aM=_matchesData.back();
-                aM._i1=&(_filesData[img1]);
-                aM._i2=&(_filesData[img2]);
+                aM._i1 = &(_filesData[img1]);
+                aM._i2 = &(_filesData[img2]);
+            }
+            else
+            {
+                aM._i1 = &(_filesData[img2]);
+                aM._i2 = &(_filesData[img1]);
+            };
+            checkedImagePairs[img1].insert(img2);
+            checkedImagePairs[img2].insert(img1);
+        };
+    };
+    //build match data list for image pairs
+    if(_image_layer.size()>1)
+    {
+        for (HuginBase::UIntSet::const_iterator it = _image_layer.begin(); it != _image_layer.end(); ++it)
+        {
+            const size_t img1 = *it;
+            HuginBase::UIntSet::const_iterator it2 = it;
+            it2++;
+            if (it2 != _image_layer.end())
+            {
+                const size_t img2 = *it2;
+                matchesData.push_back(MatchData());
+                MatchData& aM = matchesData.back();
+                if (img1 < img2)
+                {
+                    aM._i1 = &(_filesData[img1]);
+                    aM._i2 = &(_filesData[img2]);
+                }
+                else
+                {
+                    aM._i1 = &(_filesData[img2]);
+                    aM._i2 = &(_filesData[img1]);
+                };
                 checkedImagePairs[img1].insert(img2);
                 checkedImagePairs[img2].insert(img1);
             };
         };
     };
-    //build match data list for image pairs
-    if(images_layer.size()>1)
-    {
-        std::sort(images_layer.begin(), images_layer.end());
-        for(unsigned int i=0; i<images_layer.size()-1; i++)
-        {
-            size_t img1=images_layer[i];
-            size_t img2=images_layer[i+1];
-            _matchesData.push_back(MatchData());
-            MatchData& aM = _matchesData.back();
-            aM._i1 = &(_filesData[img1]);
-            aM._i2 = &(_filesData[img2]);
-            checkedImagePairs[img1].insert(img2);
-            checkedImagePairs[img2].insert(img1);
-        };
-    };
     TRACE_INFO(endl<< "--- Find matches ---" << endl);
-    try
+    for (size_t i = 0; i < matchesData.size(); ++i)
     {
-        BOOST_FOREACH(MatchData& aMD, _matchesData)
-        aExecutor.execute(new MatchDataRunnable(aMD, *this));
-        aExecutor.wait();
-    }
-    catch(Synchronization_Exception& e)
-    {
-        TRACE_ERROR(e.what() << endl);
-        return false;
-    }
+        queue.push_back(new MatchDataRunnable(matchesData[i], *this));
+    };
+    RunQueue(queue);
 
     // Add detected matches to _panoramaInfo
-    BOOST_FOREACH(MatchData& aM, _matchesData)
-    BOOST_FOREACH(lfeat::PointMatchPtr& aPM, aM._matches)
-    _panoramaInfo->addCtrlPoint(ControlPoint(aM._i1->_number, aPM->_img1_x, aPM->_img1_y,
-                                aM._i2->_number, aPM->_img2_x, aPM->_img2_y));
+    for (size_t i = 0; i < matchesData.size(); ++i)
+    {
+        const MatchData& aM = matchesData[i];
+        for (size_t j = 0; j < aM._matches.size(); ++j)
+        {
+            const lfeat::PointMatchPtr& aPM = aM._matches[j];
+            _panoramaInfo->addCtrlPoint(ControlPoint(aM._i1->_number, aPM->_img1_x, aPM->_img1_y,
+                aM._i2->_number, aPM->_img2_x, aPM->_img2_y));
+        };
+    };
 
     // step 2: connect all image groups
-    _matchesData.clear();
-    Panorama mediumPano=_panoramaInfo->getSubset(images_layer_set);
-    CPGraph graph;
-    createCPGraph(mediumPano, graph);
-    CPComponents comps;
-    unsigned int n = findCPComponents(graph, comps);
+    queue.clear();
+    matchesData.clear();
+    Panorama mediumPano=_panoramaInfo->getSubset(_image_layer);
+    HuginBase::CPGraph graph;
+    HuginBase::createCPGraph(mediumPano, graph);
+    HuginBase::CPComponents comps;
+    unsigned int n = HuginBase::findCPComponents(graph, comps);
     if(n>1)
     {
         vector<unsigned int> ImagesGroups;
         for(unsigned int i=0; i<n; i++)
         {
-            ImagesGroups.push_back(images_layer[*(comps[i].begin())]);
+            HuginBase::UIntSet::iterator imgIt = _image_layer.begin();
+            std::advance(imgIt, *(comps[i].begin()));
+            ImagesGroups.push_back(*imgIt);
             if(comps[i].size()>1)
             {
-                ImagesGroups.push_back(images_layer[*(comps[i].rbegin())]);
+                imgIt = _image_layer.begin();
+                std::advance(imgIt, *(comps[i].rbegin()));
+                ImagesGroups.push_back(*imgIt);
             }
         };
         for(unsigned int i=0; i<ImagesGroups.size()-1; i++)
@@ -1002,38 +1054,48 @@ bool PanoDetector::matchMultiRow(PoolExecutor& aExecutor)
                 {
                     continue;
                 };
-                _matchesData.push_back(MatchData());
-                MatchData& aM = _matchesData.back();
-                aM._i1 = &(_filesData[img1]);
-                aM._i2 = &(_filesData[img2]);
+                matchesData.push_back(MatchData());
+                MatchData& aM = matchesData.back();
+                if (img1 < img2)
+                {
+                    aM._i1 = &(_filesData[img1]);
+                    aM._i2 = &(_filesData[img2]);
+                }
+                else
+                {
+                    aM._i1 = &(_filesData[img2]);
+                    aM._i2 = &(_filesData[img1]);
+                };
                 checkedImagePairs[img1].insert(img2);
                 checkedImagePairs[img2].insert(img1);
             };
         };
         TRACE_INFO(endl<< "--- Find matches in images groups ---" << endl);
-        try
+        for (size_t i = 0; i < matchesData.size(); ++i)
         {
-            BOOST_FOREACH(MatchData& aMD, _matchesData)
-            aExecutor.execute(new MatchDataRunnable(aMD, *this));
-            aExecutor.wait();
-        }
-        catch(Synchronization_Exception& e)
+            queue.push_back(new MatchDataRunnable(matchesData[i], *this));
+        };
+        RunQueue(queue);
+
+        for (size_t i = 0; i < matchesData.size(); ++i)
         {
-            TRACE_ERROR(e.what() << endl);
-            return false;
-        }
-        BOOST_FOREACH(MatchData& aM, _matchesData)
-        BOOST_FOREACH(lfeat::PointMatchPtr& aPM, aM._matches)
-        _panoramaInfo->addCtrlPoint(ControlPoint(aM._i1->_number, aPM->_img1_x, aPM->_img1_y,
-                                    aM._i2->_number, aPM->_img2_x, aPM->_img2_y));
+            const MatchData& aM = matchesData[i];
+            for (size_t j = 0; j < aM._matches.size(); ++j)
+            {
+                const lfeat::PointMatchPtr& aPM = aM._matches[j];
+                _panoramaInfo->addCtrlPoint(ControlPoint(aM._i1->_number, aPM->_img1_x, aPM->_img1_y,
+                    aM._i2->_number, aPM->_img2_x, aPM->_img2_y));
+            };
+        };
     };
     // step 3: now connect all overlapping images
-    _matchesData.clear();
-    PT::Panorama optPano=_panoramaInfo->getSubset(images_layer_set);
-    createCPGraph(optPano, graph);
-    if(findCPComponents(graph, comps)==1)
+    queue.clear();
+    matchesData.clear();
+    HuginBase::Panorama optPano=_panoramaInfo->getSubset(_image_layer);
+    HuginBase::createCPGraph(optPano, graph);
+    if(HuginBase::findCPComponents(graph, comps)==1)
     {
-        if(images_layer.size()>2)
+        if(_image_layer.size()>2)
         {
             //reset translation parameters
             VariableMapVector varMapVec=optPano.getVariables();
@@ -1055,7 +1117,7 @@ bool PanoDetector::matchMultiRow(PoolExecutor& aExecutor)
             opts.setHeight(15000);
 
             optPano.setOptions(opts);
-            int w = optPano.calcOptimalWidth();
+            int w = hugin_utils::roundi(HuginBase::CalculateOptimalScale::calcOptimalScale(optPano) * opts.getWidth());
             opts.setWidth(w);
             opts.setHeight(w/2);
             optPano.setOptions(opts);
@@ -1108,7 +1170,8 @@ bool PanoDetector::matchMultiRow(PoolExecutor& aExecutor)
             };
 
             //now match overlapping images
-            if(!matchPrealigned(aExecutor, &optPano, checkedImagePairs, images_layer, false))
+            std::vector<size_t> imgMap(_image_layer.begin(), _image_layer.end());
+            if(!matchPrealigned(&optPano, checkedImagePairs, imgMap, false))
             {
                 return false;
             };
@@ -1116,7 +1179,7 @@ bool PanoDetector::matchMultiRow(PoolExecutor& aExecutor)
     }
     else
     {
-        if(!match(aExecutor, checkedImagePairs))
+        if(!match(checkedImagePairs))
         {
             return false;
         };
@@ -1124,8 +1187,10 @@ bool PanoDetector::matchMultiRow(PoolExecutor& aExecutor)
     return true;
 };
 
-bool PanoDetector::matchPrealigned(PoolExecutor& aExecutor, Panorama* pano, std::vector<HuginBase::UIntSet> &connectedImages, std::vector<size_t> imgMap, bool exactOverlap)
+bool PanoDetector::matchPrealigned(Panorama* pano, std::vector<HuginBase::UIntSet> &connectedImages, std::vector<size_t> imgMap, bool exactOverlap)
 {
+    RunnableVector queue;
+    MatchData_t matchesData;
     Panorama tempPano=pano->duplicate();
     if(!exactOverlap)
     {
@@ -1150,8 +1215,8 @@ bool PanoDetector::matchPrealigned(PoolExecutor& aExecutor, Panorama* pano, std:
             };
             if(overlap.getOverlap(i,j)>0)
             {
-                _matchesData.push_back(MatchData());
-                MatchData& aM = _matchesData.back();
+                matchesData.push_back(MatchData());
+                MatchData& aM = matchesData.back();
                 aM._i1 = &(_filesData[imgMap[i]]);
                 aM._i2 = &(_filesData[imgMap[j]]);
                 connectedImages[imgMap[i]].insert(imgMap[j]);
@@ -1161,23 +1226,23 @@ bool PanoDetector::matchPrealigned(PoolExecutor& aExecutor, Panorama* pano, std:
     };
 
     TRACE_INFO(endl<< "--- Find matches for overlapping images ---" << endl);
-    try
+    for (size_t i = 0; i < matchesData.size(); ++i)
     {
-        BOOST_FOREACH(MatchData& aMD, _matchesData)
-        aExecutor.execute(new MatchDataRunnable(aMD, *this));
-        aExecutor.wait();
-    }
-    catch(Synchronization_Exception& e)
-    {
-        TRACE_ERROR(e.what() << endl);
-        return false;
-    }
+        queue.push_back(new MatchDataRunnable(matchesData[i], *this));
+    };
+    RunQueue(queue);
 
     // Add detected matches to _panoramaInfo
-    BOOST_FOREACH(MatchData& aM, _matchesData)
-    BOOST_FOREACH(lfeat::PointMatchPtr& aPM, aM._matches)
-    _panoramaInfo->addCtrlPoint(ControlPoint(imgMap[aM._i1->_number], aPM->_img1_x, aPM->_img1_y,
-                                imgMap[aM._i2->_number], aPM->_img2_x, aPM->_img2_y));
+    for (size_t i = 0; i < matchesData.size(); ++i)
+    {
+        const MatchData& aM = matchesData[i];
+        for (size_t j = 0; j < aM._matches.size(); ++j)
+        {
+            const lfeat::PointMatchPtr& aPM = aM._matches[j];
+            _panoramaInfo->addCtrlPoint(ControlPoint(aM._i1->_number, aPM->_img1_x, aPM->_img1_y,
+                aM._i2->_number, aPM->_img2_x, aPM->_img2_y));
+        };
+    };
 
     return true;
 };
