@@ -15,7 +15,7 @@
 *
 * You should have received a copy of the GNU General Public License
 * along with Panomatic; if not, write to the Free Software
-* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+* <http://www.gnu.org/licenses/>.
 */
 
 #include "Utils.h"
@@ -24,38 +24,13 @@
 #endif
 #ifdef _WINDOWS
 #include <Windows.h>
+#include <algorithm>
 #elif defined __APPLE__
 #include <CoreServices/CoreServices.h>  //for gestalt
 #else
 #include <unistd.h>
 #endif
 
-int utils::getCPUCount()
-{
-#ifdef WIN32
-    SYSTEM_INFO si;
-    GetSystemInfo(&si);
-    return si.dwNumberOfProcessors;
-#elif defined(HW_NCPU) || defined(__APPLE__)
-    // BSD and OSX like system
-    int mib[2];
-    int numCPUs = 1;
-    size_t len = sizeof(numCPUs);
-
-    mib[0] = CTL_HW;
-    mib[1] = HW_NCPU;
-    sysctl(mib, 2, &numCPUs, &len, 0, 0);
-    return numCPUs;
-
-#elif defined(_SC_NPROCESSORS_ONLN)
-    // Linux and Solaris
-    long nProcessorsOnline = sysconf(_SC_NPROCESSORS_ONLN);
-    return nProcessorsOnline;
-#else
-    return 1;
-#endif
-}
-
 #ifdef _WINDOWS
 unsigned long long utils::getTotalMemory()
 {
