@@ -15,7 +15,7 @@
 *
 * You should have received a copy of the GNU General Public License
 * along with Panomatic; if not, write to the Free Software
-* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+* <http://www.gnu.org/licenses/>.
 */
 
 #ifndef __utils_h
@@ -29,14 +29,8 @@
 #include <sys/time.h>
 #endif
 
-#include <zthread/Condition.h>
-#include <zthread/FastMutex.h>
-#include <zthread/Guard.h>
-
-
-
-#define TIMETRACE(TEXT, CODE) { utils::os_TIME t1,t2; utils::os_GetTime(&t1); CODE; \
-        utils::os_GetTime(&t2); std::cout << TEXT << " took " << utils::os_TimeDiff(&t2,&t1)/1000.0 << " seconds." << std::endl; }
+#define TIMETRACE(TEXT, CODE) { std::ostringstream buf; utils::os_TIME t1,t2; utils::os_GetTime(&t1); CODE; \
+        utils::os_GetTime(&t2); buf << TEXT << " took " << utils::os_TimeDiff(&t2,&t1)/1000.0 << " seconds." << std::endl; std::cout << buf.str(); }
 
 
 namespace utils
