@@ -19,8 +19,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -34,6 +34,7 @@
 #include <limits>
 #include <iterator>
 
+#define BOOST_SPIRIT_USE_PHOENIX_V3 1
 #include <boost/spirit/version.hpp>
 #if !defined(SPIRIT_VERSION) || SPIRIT_VERSION < 0x2010
 #error "At least Spirit version 2.1 required"
@@ -50,11 +51,9 @@ namespace Parser
 //power function
 struct lazy_pow_
 {
-    template <typename X, typename Y>
-    struct result { typedef X type; };
+    typedef double result_type;
 
-    template <typename X, typename Y>
-    X operator()(X x, Y y) const
+    double operator()(double x, double y) const
     {
         return std::pow(x, y);
     }
@@ -63,11 +62,9 @@ struct lazy_pow_
 // modulus for double values
 struct lazy_mod_
 {
-    template <typename X, typename Y>
-    struct result { typedef X type; };
+    typedef double result_type;
 
-    template <typename X, typename Y>
-    X operator()(X x, Y y) const
+    double operator()(double x, double y) const
     {
         return std::fmod(x,y);
     }
@@ -76,37 +73,33 @@ struct lazy_mod_
 // if statement
 struct lazy_if_
 {
-    template <typename X, typename Y, typename Z>
-    struct result { typedef Y type; };
+    typedef double result_type;
 
-    template <typename X, typename Y, typename Z>
-    X operator()(X x, Y y, Z z) const
+    double operator()(double x, double y, double z) const
     {
-        return x ? y : z;
+        return (std::fabs(x)>1e-5) ? y : z;
     }
 };
 
 // wrapper for unary function
 struct lazy_ufunc_
 {
-    template <typename F, typename A1>
-    struct result { typedef A1 type; };
+    typedef double result_type;
 
-    template <typename F, typename A1>
-    A1 operator()(F f, A1 a1) const
+    double operator()(double (*f)(double), double a1) const
     {
         return f(a1);
     }
 };
 
 // convert rad into deg
-double deg(const double d)
+const double deg(const double d)
 {
     return d*180.0/boost::math::constants::pi<double>();
 };
 
 // convert deg into rad
-double rad(const double d)
+const double rad(const double d)
 {
     return d*boost::math::constants::pi<double>()/180;
 };
@@ -121,36 +114,36 @@ struct grammar:boost::spirit::qi::grammar<std::string::const_iterator, double(),
         constant_(const ConstantMap constMap)
         {
             this->add("pi", boost::math::constants::pi<double>());
-            if(constMap.size()>0)
+            if (constMap.size()>0)
             {
-                for(ConstantMap::const_iterator it=constMap.begin(); it!=constMap.end(); it++)
+                for (ConstantMap::const_iterator it = constMap.begin(); it != constMap.end(); it++)
                 {
-                    this->add(it->first, it->second); 
+                    this->add(it->first, it->second);
                 };
             };
         };
     };
 
     // symbol table for unary functions like "abs"
-    struct ufunc_  : boost::spirit::qi::symbols<char, double(*)(double) >
+    struct ufunc_ : boost::spirit::qi::symbols<char, double(*)(double) >
     {
         ufunc_()
         {
             this->add
-                ("abs"   , (double (*)(double)) std::abs  )
-                ("acos"  , (double (*)(double)) std::acos )
-                ("asin"  , (double (*)(double)) std::asin )
-                ("atan"  , (double (*)(double)) std::atan )
-                ("ceil"  , (double (*)(double)) std::ceil )
-                ("sin"   , (double (*)(double)) std::sin  )
-                ("cos"   , (double (*)(double)) std::cos  )
-                ("tan"   , (double (*)(double)) std::tan  )
-                ("exp"   , (double (*)(double)) std::exp  )
-                ("floor" , (double (*)(double)) std::floor)
-                ("sqrt"  , (double (*)(double)) std::sqrt )
-                ("deg"   , (double (*)(double)) deg  )
-                ("rad"   , (double (*)(double)) rad  )
-            ;
+                ("abs", (double(*)(double)) std::abs)
+                ("acos", (double(*)(double)) std::acos)
+                ("asin", (double(*)(double)) std::asin)
+                ("atan", (double(*)(double)) std::atan)
+                ("ceil", (double(*)(double)) std::ceil)
+                ("sin", (double(*)(double)) std::sin)
+                ("cos", (double(*)(double)) std::cos)
+                ("tan", (double(*)(double)) std::tan)
+                ("exp", (double(*)(double)) std::exp)
+                ("floor", (double(*)(double)) std::floor)
+                ("sqrt", (double(*)(double)) std::sqrt)
+                ("deg", (double(*)(double)) deg)
+                ("rad", (double(*)(double)) rad)
+                ;
         }
     } ufunc;
 
@@ -160,7 +153,7 @@ struct grammar:boost::spirit::qi::grammar<std::string::const_iterator, double(),
     {
         using boost::spirit::qi::real_parser;
         using boost::spirit::qi::real_policies;
-        real_parser<double,real_policies<double> > real;
+        real_parser<double, real_policies<double> > real;
 
         using boost::spirit::qi::_1;
         using boost::spirit::qi::_2;
@@ -174,65 +167,64 @@ struct grammar:boost::spirit::qi::grammar<std::string::const_iterator, double(),
         boost::phoenix::function<lazy_if_>    lazy_if;
         boost::phoenix::function<lazy_ufunc_> lazy_ufunc;
 
-        expression = 
-            (compExpression >> '\?' >> compExpression >> ':' >> compExpression) [_val = lazy_if(_1, _2, _3)]
-            | compExpression [_val=_1]
+        expression =
+            (compExpression >> '\?' >> compExpression >> ':' >> compExpression)[_val = lazy_if(_1, _2, _3)]
+            | compExpression[_val = _1]
             ;
-        
-        compExpression=
-            compTerm  [_val=_1]
-            >> * ( ("&&" >> compTerm [_val = _val && _1] )
-                  |("||" >> compTerm [_val = _val || _1] )
-                 )
+
+        compExpression =
+            compTerm[_val = _1]
+            >> *(("&&" >> compTerm[_val = _val && _1])
+            | ("||" >> compTerm[_val = _val || _1])
+            )
             ;
 
         compTerm =
-            numExpression                [_val = _1        ]
-            >>*( ( '<'  >> numExpression [_val = _val <  _1])
-                |( '>'  >> numExpression [_val = _val >  _1])
-                |( "<=" >> numExpression [_val = _val <= _1])
-                |( ">=" >> numExpression [_val = _val >= _1])
-                |( "==" >> numExpression [_val = _val == _1])
-                |( "!=" >> numExpression [_val = _val != _1])
-               )
+            numExpression[_val = _1]
+            >> *(('<' >> numExpression[_val = _val <  _1])
+            | ('>' >> numExpression[_val = _val >  _1])
+            | ("<=" >> numExpression[_val = _val <= _1])
+            | (">=" >> numExpression[_val = _val >= _1])
+            | ("==" >> numExpression[_val = _val == _1])
+            | ("!=" >> numExpression[_val = _val != _1])
+            )
             ;
 
         numExpression =
-            term                   [_val =  _1]
-            >> *(  ('+' >> term    [_val += _1])
-                |  ('-' >> term    [_val -= _1])
-                )
+            term[_val = _1]
+            >> *(('+' >> term[_val += _1])
+            | ('-' >> term[_val -= _1])
+            )
             ;
 
         term =
-            factor                 [_val =  _1]
-            >> *(  ('*' >> factor  [_val *= _1])
-                |  ('/' >> factor  [_val /= _1])
-                |  ('%' >> factor  [_val = lazy_mod(_val, _1)])
-                )
+            factor[_val = _1]
+            >> *(('*' >> factor[_val *= _1])
+            | ('/' >> factor[_val /= _1])
+            | ('%' >> factor[_val = lazy_mod(_val, _1)])
+            )
             ;
 
         factor =
-            primary                [_val =  _1]
-            >> *(  ('^' >> factor [_val = lazy_pow(_val, _1)]) )
+            primary[_val = _1]
+            >> *(('^' >> factor[_val = lazy_pow(_val, _1)]))
             ;
 
         primary =
-            real                   [_val =  _1]
-            |  '(' >> expression   [_val =  _1] >> ')'
-            |  ('-' >> primary     [_val = -_1])
-            |  ('+' >> primary     [_val =  _1])
-            |  no_case[constant]   [_val =  _1]
-            |  (no_case[ufunc] >> '(' >> expression >> ')') [_val = lazy_ufunc(_1, _2) ]
+            real[_val = _1]
+            | '(' >> expression[_val = _1] >> ')'
+            | ('-' >> primary[_val = -_1])
+            | ('+' >> primary[_val = _1])
+            | no_case[constant][_val = _1]
+            | (no_case[ufunc] >> '(' >> expression >> ')')[_val = lazy_ufunc(_1, _2)]
             ;
 
     };
 };
 
-//template <typename ParserType, typename Iterator>
-bool parse(std::string::const_iterator &iter,
+bool parse(std::string::const_iterator& iter,
            std::string::const_iterator end,
-           const grammar &g,
+           const grammar& g,
            double& result)
 {
     if(!boost::spirit::qi::phrase_parse(iter, end, g, boost::spirit::ascii::space, result))
@@ -243,14 +235,6 @@ bool parse(std::string::const_iterator &iter,
     return iter==end;
 }
 
-// the function which exposes the interface to external
-// version without pre-defined constants 
-bool ParseExpression(const std::string& expression, double& result)
-{
-    ConstantMap constants;
-    return ParseExpression(expression, result, constants);
-};
-    
 // version with pre-defined constants
 bool ParseExpression(const std::string& expression, double& result, const ConstantMap& constants)
 {
