@@ -19,8 +19,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -33,8 +33,7 @@ namespace Parser
 {
 typedef std::map<const char*, double> ConstantMap;
 
-bool ParseExpression(const std::string& expression, double& result);
-bool ParseExpression(const std::string& expression, double& result, const ConstantMap& constants);
+bool ParseExpression(const std::string& expression, double& result, const ConstantMap& constants=ConstantMap());
 
 };
 
