@@ -19,13 +19,11 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
-#include <hugin_version.h>
-
 #include <fstream>
 #include <sstream>
 #include <getopt.h>
@@ -40,7 +38,7 @@ using namespace std;
 using namespace HuginBase;
 using namespace AppBase;
 
-// check if given point x,y is inside both images img1 and img2, 
+// check if given point x,y is inside both images img1 and img2,
 // using PTools::Tranform object to prevent creating of this transform for each point
 bool CheckAndAddPoint(Panorama& pano, size_t img1, size_t img2, PTools::Transform& transform1, PTools::Transform& transform2, double x, double y)
 {
@@ -252,8 +250,8 @@ void SetGeometricControlPointsOverlap(Panorama& pano, const int minOverlap)
                 for(CPVector::const_iterator it=cp.begin(); it!=cp.end(); ++it)
                 {
                     if(((it->image1Nr==i && it->image2Nr==j) ||
-                        (it->image1Nr==j && it->image2Nr==i) ) &&
-                       (it->mode==ControlPoint::X_Y))
+                            (it->image1Nr==j && it->image2Nr==i) ) &&
+                            (it->mode==ControlPoint::X_Y))
                     {
                         hasControlPoints=true;
                         break;
@@ -294,7 +292,7 @@ void SetGeometricControlPointsOverlap(Panorama& pano, const int minOverlap)
 static void usage(const char* name)
 {
     cout << name << ": set geometric control points" << endl
-         << name << " version " << DISPLAY_VERSION << endl
+         << name << " version " << hugin_utils::GetHuginVersion() << endl
          << endl
          << "Usage:  " << name << " [options] input.pto" << endl
          << endl
@@ -352,7 +350,7 @@ int main(int argc, char* argv[])
                 };
                 break;
             case 'h':
-                usage(argv[0]);
+                usage(hugin_utils::stripPath(argv[0]).c_str());
                 return 0;
             case ':':
                 cerr <<"Option " << longOptions[optionIndex].name << " requires a parameter" << endl;
