@@ -19,13 +19,11 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
-#include <hugin_version.h>
-
 #include <fstream>
 #include <sstream>
 #include <getopt.h>
@@ -33,38 +31,35 @@
 #include <unistd.h>
 #endif
 #include <panodata/Panorama.h>
+#include <panodata/StandardImageVariableGroups.h>
 #include <lines/FindLines.h>
 #include <vigra/impex.hxx>
 #include <vigra_ext/impexalpha.hxx>
 #include <vigra/functorexpression.hxx>
 #include <vigra_ext/utils.h>
+#include <hugin_utils/openmp_lock.h>
 
 extern "C"
 {
 #include <pano13/filter.h>
 }
-#if defined _MSC_VER && _MSC_VER>=1600
-#include <ppl.h>
-#define HAS_PPL
-#endif
 
-using namespace std;
 using namespace HuginBase;
 using namespace AppBase;
 
 static void usage(const char* name)
 {
-    cout << name << ": find vertical lines in images" << endl
-         << name << " version " << DISPLAY_VERSION << endl
-         << endl
-         << "Usage:  " << name << " [options] input.pto" << endl
-         << endl
-         << "  Options:" << endl
-         << "     -o, --output=file.pto  Output Hugin PTO file. Default: <filename>_lines.pto" << endl
-         << "     -i, --image=IMGNR      Work only on given image numbers" << endl
-         << "     -l, --lines=COUNT      Save maximal COUNT lines (default: 5)" << endl
-         << "     -h, --help             Shows this help" << endl
-         << endl;
+    std::cout << name << ": find vertical lines in images" << std::endl
+              << name << " version " << hugin_utils::GetHuginVersion() << std::endl
+              << std::endl
+              << "Usage:  " << name << " [options] input.pto" << std::endl
+              << std::endl
+              << "  Options:" << std::endl
+              << "     -o, --output=file.pto  Output Hugin PTO file. Default: <filename>_lines.pto" << std::endl
+              << "     -i, --image=IMGNR      Work only on given image numbers" << std::endl
+              << "     -l, --lines=COUNT      Save maximal COUNT lines (default: 5)" << std::endl
+              << "     -h, --help             Shows this help" << std::endl
+              << std::endl;
 }
 
 // dummy panotools progress functions
@@ -85,7 +80,7 @@ static int ptinfoDlg( int command, char* argument )
  */
 // 2 versions: one for color images, the other for gray images
 template <class SrcIMG>
-void convertToUInt8(SrcIMG & src, const std::string & origType, vigra::UInt8RGBImage & dest)
+void convertToUInt8(SrcIMG& src, const std::string& origType, vigra::UInt8RGBImage& dest)
 {
     dest.resize(src.size());
     long newMax=vigra_ext::getMaxValForPixelType("UINT8");
@@ -106,12 +101,12 @@ void convertToUInt8(SrcIMG & src, const std::string & origType, vigra::UInt8RGBI
     else
     {
         vigra::transformImage(srcImageRange(src), destImage(dest),
-            vigra::functor::Arg1()*vigra::functor::Param( newMax/ vigra_ext::getMaxValForPixelType(origType)));
+                              vigra::functor::Arg1()*vigra::functor::Param( newMax/ vigra_ext::getMaxValForPixelType(origType)));
     };
 }
 
 template <class SrcIMG>
-void convertGrayToUInt8(SrcIMG & src, const std::string & origType, vigra::BImage & dest)
+void convertGrayToUInt8(SrcIMG& src, const std::string& origType, vigra::BImage& dest)
 {
     dest.resize(src.size());
     long newMax=vigra_ext::getMaxValForPixelType("UINT8");
@@ -130,12 +125,12 @@ void convertGrayToUInt8(SrcIMG & src, const std::string & origType, vigra::BImag
     else
     {
         vigra::transformImage(srcImageRange(src), destImage(dest),
-            vigra::functor::Arg1()*vigra::functor::Param( newMax/ vigra_ext::getMaxValForPixelType(origType)));
+                              vigra::functor::Arg1()*vigra::functor::Param( newMax/ vigra_ext::getMaxValForPixelType(origType)));
     };
 }
 
 template <class SrcIMG>
-vigra::BImage LoadGrayImageAndConvert(vigra::ImageImportInfo & info)
+vigra::BImage LoadGrayImageAndConvert(vigra::ImageImportInfo& info)
 {
     vigra::BImage image;
     SrcIMG imageIn(info.width(),info.height());
@@ -155,7 +150,7 @@ vigra::BImage LoadGrayImageAndConvert(vigra::ImageImportInfo & info)
 };
 
 template <class SrcIMG>
-vigra::UInt8RGBImage LoadImageAndConvert(vigra::ImageImportInfo & info)
+vigra::UInt8RGBImage LoadImageAndConvert(vigra::ImageImportInfo& info)
 {
     vigra::UInt8RGBImage image;
     SrcIMG imageIn(info.width(),info.height());
@@ -175,7 +170,7 @@ vigra::UInt8RGBImage LoadImageAndConvert(vigra::ImageImportInfo & info)
 };
 
 // loads the gray images and finds vertical lines, returns a CPVector with found vertical lines
-HuginBase::CPVector LoadGrayImageAndFindLines(vigra::ImageImportInfo info, Panorama & pano, size_t imgNr, int nrLines)
+HuginBase::CPVector LoadGrayImageAndFindLines(vigra::ImageImportInfo info, Panorama& pano, size_t imgNr, int nrLines)
 {
     vigra::BImage image;
     HuginBase::CPVector lineCp;
@@ -227,7 +222,7 @@ HuginBase::CPVector LoadGrayImageAndFindLines(vigra::ImageImportInfo info, Panor
 };
 
 // loads the color images and finds vertical lines, returns a CPVector with found vertical lines
-HuginBase::CPVector LoadImageAndFindLines(vigra::ImageImportInfo info, Panorama & pano, size_t imgNr, int nrLines)
+HuginBase::CPVector LoadImageAndFindLines(vigra::ImageImportInfo info, Panorama& pano, size_t imgNr, int nrLines)
 {
     vigra::UInt8RGBImage image;
     HuginBase::CPVector lineCp;
@@ -274,10 +269,23 @@ HuginBase::CPVector LoadImageAndFindLines(vigra::ImageImportInfo info, Panorama
     if(image.width()>0 && image.height()>0)
     {
         lineCp=HuginLines::GetVerticalLines(pano, imgNr, image, nrLines);
-    };
+    }
     return lineCp;
 };
 
+struct SortVectorByExposure
+{
+    SortVectorByExposure(const HuginBase::Panorama& pano) : m_pano(pano) {};
+    bool operator()(const size_t& img1, const size_t& img2)
+    {
+        return m_pano.getImage(img1).getExposureValue() < m_pano.getImage(img2).getExposureValue();
+    }
+private:
+    const HuginBase::Panorama& m_pano;
+};
+
+static hugin_omp::Lock lock;
+
 int main(int argc, char* argv[])
 {
     // parse arguments
@@ -296,7 +304,7 @@ int main(int argc, char* argv[])
     int c;
     int optionIndex = 0;
     int nrLines = 5;
-    string output;
+    std::string output;
     while ((c = getopt_long (argc, argv, optstring, longOptions,&optionIndex)) != -1)
     {
         switch (c)
@@ -305,14 +313,14 @@ int main(int argc, char* argv[])
                 output = optarg;
                 break;
             case 'h':
-                usage(argv[0]);
+                usage(hugin_utils::stripPath(argv[0]).c_str());
                 return 0;
             case 'i':
                 {
                     int imgNr=atoi(optarg);
                     if((imgNr==0) && (strcmp(optarg,"0")!=0))
                     {
-                        cerr << "Could not parse image number.";
+                        std::cerr << "Could not parse image number.";
                         return 1;
                     };
                     cmdlineImages.insert(imgNr);
@@ -322,12 +330,12 @@ int main(int argc, char* argv[])
                 nrLines=atoi(optarg);
                 if(nrLines<1)
                 {
-                    cerr << "Could not parse number of lines.";
+                    std::cerr << "Could not parse number of lines.";
                     return 1;
                 };
                 break;
             case ':':
-                cerr <<"Option " << longOptions[optionIndex].name << " requires a number" << endl;
+                std::cerr <<"Option " << longOptions[optionIndex].name << " requires a number" << std::endl;
                 return 1;
                 break;
             case '?':
@@ -339,49 +347,68 @@ int main(int argc, char* argv[])
 
     if (argc - optind != 1)
     {
-        cout << "Warning: " << argv[0] << " can only work on one project file at one time" << endl << endl;
-        usage(argv[0]);
+        std::cout << "Warning: " << argv[0] << " can only work on one project file at one time" << std::endl << std::endl;
+        usage(hugin_utils::stripPath(argv[0]).c_str());
         return 1;
     };
 
-    string input=argv[optind];
+    std::string input=argv[optind];
     // read panorama
     Panorama pano;
-    ifstream prjfile(input.c_str());
+    std::ifstream prjfile(input.c_str());
     if (!prjfile.good())
     {
-        cerr << "could not open script : " << input << endl;
+        std::cerr << "could not open script : " << input << std::endl;
         return 1;
     }
     pano.setFilePrefix(hugin_utils::getPathPrefix(input));
     DocumentData::ReadWriteError err = pano.readData(prjfile);
     if (err != DocumentData::SUCCESSFUL)
     {
-        cerr << "error while parsing panos tool script: " << input << endl;
-        cerr << "DocumentData::ReadWriteError code: " << err << endl;
+        std::cerr << "error while parsing panos tool script: " << input << std::endl;
+        std::cerr << "DocumentData::ReadWriteError code: " << err << std::endl;
         return 1;
     }
 
     if(pano.getNrOfImages()==0)
     {
-        cerr << "error: project file does not contains any image" << endl;
-        cerr << "aborting processing" << endl;
+        std::cerr << "error: project file does not contains any image" << std::endl;
+        std::cerr << "aborting processing" << std::endl;
         return 1;
     };
 
     std::vector<size_t> imagesToProcess;
     if(cmdlineImages.size()==0)
     {
-        //no image given, process all
-        for(size_t i=0;i<pano.getNrOfImages();i++)
+        //no image given, process one image of each stack
+        HuginBase::ConstStandardImageVariableGroups variable_groups(pano);
+        HuginBase::UIntSetVector imageGroups = variable_groups.getStacks().getPartsSet();
+        //get image with median exposure for search for lines
+        for (size_t imgGroup = 0; imgGroup < imageGroups.size(); ++imgGroup)
         {
-            imagesToProcess.push_back(i);
+            HuginBase::UIntVector stackImages(imageGroups[imgGroup].begin(), imageGroups[imgGroup].end());
+            if (pano.getImage(stackImages[0]).YawisLinked())
+            {
+                // position is linked, take only image with median exposure for search
+                std::sort(stackImages.begin(), stackImages.end(), SortVectorByExposure(pano));
+                size_t index = 0;
+                if (pano.getImage(*(stackImages.begin())).getExposureValue() != pano.getImage(*(stackImages.rbegin())).getExposureValue())
+                {
+                    index = stackImages.size() / 2;
+                };
+                imagesToProcess.push_back(stackImages[index]);
+            }
+            else
+            {
+                // stacks are unlinked, search all images
+                std::copy(stackImages.begin(), stackImages.end(), std::back_inserter(imagesToProcess));
+            };
         };
     }
     else
     {
         //check, if given image numbers are valid
-        for(UIntSet::const_iterator it=cmdlineImages.begin();it!=cmdlineImages.end();++it)
+        for(UIntSet::const_iterator it=cmdlineImages.begin(); it!=cmdlineImages.end(); ++it)
         {
             if((*it)>=0 && (*it)<pano.getNrOfImages())
             {
@@ -392,29 +419,29 @@ int main(int argc, char* argv[])
 
     if(imagesToProcess.size()==0)
     {
-        cerr << "No image to process found" << endl << "Stopping processing" << endl;
+        std::cerr << "No image to process found" << std::endl << "Stopping processing" << std::endl;
         return 1;
     };
 
     PT_setProgressFcn(ptProgress);
     PT_setInfoDlgFcn(ptinfoDlg);
 
-    cout << argv[0] << " is searching for vertical lines" << endl;
+    std::cout << hugin_utils::stripPath(argv[0]) << " is searching for vertical lines" << std::endl;
 #if _WINDOWS
     //multi threading of image loading results sometime in a race condition
     //try to prevent this by initialisation of codecManager before
     //running multi threading part
     std::string s=vigra::impexListExtensions();
 #endif
-#ifdef HAS_PPL
+
     size_t nrCPS=pano.getNrOfCtrlPoints();
-    Concurrency::parallel_for<size_t>(0,imagesToProcess.size(),[&pano,imagesToProcess,nrLines](size_t i)
-#else
-    for(size_t i=0;i<imagesToProcess.size();i++)
-#endif
+    #pragma omp parallel for schedule(dynamic)
+    for(int i=0; i < imagesToProcess.size(); ++i)
     {
         unsigned int imgNr=imagesToProcess[i];
-        cout << "Working on image " << pano.getImage(imgNr).getFilename() << endl;
+        std::ostringstream buf;
+        buf << "Working on image " << pano.getImage(imgNr).getFilename() << std::endl;
+        std::cout << buf.str();
         // now load and process all images
         vigra::ImageImportInfo info(pano.getImage(imgNr).getFilename().c_str());
         HuginBase::CPVector foundLines;
@@ -431,27 +458,22 @@ int main(int argc, char* argv[])
             }
             else
             {
-                std::cerr << "Image " << pano.getImage(imgNr).getFilename().c_str() << " has " 
-                    << info.numBands() << " channels." << std::endl
-                    << "Linefind works only with grayscale or color images." << std::endl
-                    << "Skipping image." << std::endl;
+                std::cerr << "Image " << pano.getImage(imgNr).getFilename().c_str() << " has "
+                          << info.numBands() << " channels." << std::endl
+                          << "Linefind works only with grayscale or color images." << std::endl
+                          << "Skipping image." << std::endl;
             };
         };
-#ifndef HAS_PPL
-        cout << "Found " << foundLines.size() << " vertical lines" << endl;
-#endif
         if(foundLines.size()>0)
         {
             for(CPVector::const_iterator cpIt=foundLines.begin(); cpIt!=foundLines.end(); ++cpIt)
             {
+                hugin_omp::ScopedLock sl(lock);
                 pano.addCtrlPoint(*cpIt);
             };
         };
     }
-#ifdef HAS_PPL
-    );
-    cout << endl << "Found " << pano.getNrOfCtrlPoints() - nrCPS << " vertical lines" << endl << endl;
-#endif
+    std::cout << std::endl << "Found " << pano.getNrOfCtrlPoints() - nrCPS << " vertical lines" << std::endl << std::endl;
 
     //write output
     UIntSet imgs;
@@ -461,9 +483,9 @@ int main(int argc, char* argv[])
     {
         output=input.substr(0,input.length()-4).append("_lines.pto");
     }
-    ofstream of(output.c_str());
+    std::ofstream of(output.c_str());
     pano.printPanoramaScript(of, pano.getOptimizeVector(), pano.getOptions(), imgs, false, hugin_utils::getPathPrefix(input));
 
-    cout << endl << "Written output to " << output << endl;
+    std::cout << std::endl << "Written output to " << output << std::endl;
     return 0;
 }
