@@ -19,13 +19,11 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
-#include <hugin_version.h>
-
 #include <fstream>
 #include <getopt.h>
 #ifdef _WINDOWS
@@ -52,7 +50,7 @@ using namespace HuginBase;
 static void usage(const char* name)
 {
     cout << name << ": generate project file from images" << endl
-         << name << " version " << DISPLAY_VERSION << endl
+         << name << " version " << hugin_utils::GetHuginVersion() << endl
          << endl
          << "Usage:  " << name << " [options] image1 [...]" << endl
          << endl
@@ -63,50 +61,16 @@ static void usage(const char* name)
          << "     -c, --crop=left,right,top,bottom        Sets the crop of input" << endl
          << "                            images (especially for fisheye lenses)" << endl
          << "     -s, --stacklength=INT  Number of images in stack" << endl
-         << "                            (default: 1, no stacks)" << endl
+         << "                            (default: automatic detection)" << endl
          << "     -l, --linkstacks       Link image positions in stacks" << endl
          << "     --distortion           Try to load distortion information from" << endl
-         << "                            lensfun database" << endl
+         << "                            lens database" << endl
          << "     --vignetting           Try to load vignetting information from" << endl
-         << "                            lensfun database" << endl
+         << "                            lens database" << endl
          << "     -h, --help             Shows this help" << endl
          << endl;
 }
 
-void InitLensDB()
-{
-#ifdef _WINDOWS
-    char buffer[MAX_PATH];//always use MAX_PATH for filepaths
-    GetModuleFileName(NULL,buffer,sizeof(buffer));
-    string working_path=string(buffer);
-    //remove filename
-    std::string::size_type pos=working_path.rfind("\\");
-    if(pos!=std::string::npos)
-    {
-        working_path.erase(pos);
-        //remove last dir: should be bin
-        pos=working_path.rfind("\\");
-        if(pos!=std::string::npos)
-        {
-            working_path.erase(pos);
-            //append path delimiter and path
-            working_path.append("\\share\\lensfun\\");
-            HuginBase::LensDB::LensDB::GetSingleton().SetMainDBPath(working_path);
-        }
-    }
-#elif defined MAC_SELF_CONTAINED_BUNDLE
-    char path[PATH_MAX + 1];
-    uint32_t size = sizeof(path);
-    string install_path_lensfun("");
-    if (_NSGetExecutablePath(path, &size) == 0)
-    {
-        install_path_lensfun=dirname(path);
-        install_path_lensfun.append("/../Resources/lensfun/");
-        HuginBase::LensDB::LensDB::GetSingleton().SetMainDBPath(install_path_lensfun);
-    }
-#endif
-};
-
 int main(int argc, char* argv[])
 {
     // parse arguments
@@ -132,7 +96,7 @@ int main(int argc, char* argv[])
     string output;
     int projection=-1;
     float fov=-1;
-    int stackLength=1;
+    int stackLength=0;
     bool linkStacks=false;
     vigra::Rect2D cropRect(0,0,0,0);
     bool loadDistortion=false;
@@ -145,7 +109,7 @@ int main(int argc, char* argv[])
                 output = optarg;
                 break;
             case 'h':
-                usage(argv[0]);
+                usage(hugin_utils::stripPath(argv[0]).c_str());
                 return 0;
             case 'p':
                 {
@@ -196,9 +160,9 @@ int main(int argc, char* argv[])
                 break;
             case 's':
                 stackLength=atoi(optarg);
-                if(stackLength<1)
+                if ((stackLength == 0) && (strcmp(optarg, "0") != 0))
                 {
-                    cerr << "Could not parse stack length." << endl;
+                    cerr << "Could not parse stack length.";
                     return 1;
                 };
                 break;
@@ -224,7 +188,7 @@ int main(int argc, char* argv[])
 
     if (argc - optind < 1)
     {
-        usage(argv[0]);
+        usage(hugin_utils::stripPath(argv[0]).c_str());
         return 1;
     };
 
@@ -289,13 +253,8 @@ int main(int argc, char* argv[])
     //sort filenames
     sort(filelist.begin(),filelist.end(),doj::alphanum_less());
 
-    if(projection<0)
-    {
-        InitLensDB();
-    };
-
     Panorama pano;
-    for(size_t i=0; i<filelist.size();i++)
+    for(size_t i=0; i<filelist.size(); i++)
     {
         SrcPanoImage srcImage;
         cout << "Reading " << filelist[i] << "..." << endl;
@@ -310,7 +269,15 @@ int main(int argc, char* argv[])
                 continue;
             }
             srcImage.setSize(info.size());
-            std::string pixelType=info.getPixelType();
+            // check for black/white images
+            const std::string pixelType=info.getPixelType();
+            if (pixelType == "BILEVEL")
+            {
+                cerr << "ERROR: Image " << filelist[i] << " is a black/white images." << endl
+                    << "       This is not supported. Convert to grayscale image and try again." << endl
+                    << "       Skipping this image." << endl;
+                continue;
+            }
             if((pixelType=="UINT8") || (pixelType=="UINT16") || (pixelType=="INT16"))
             {
                 srcImage.setResponseType(HuginBase::SrcPanoImage::RESPONSE_EMOR);
@@ -320,7 +287,7 @@ int main(int argc, char* argv[])
                 srcImage.setResponseType(HuginBase::SrcPanoImage::RESPONSE_LINEAR);
             };
         }
-        catch(std::exception & e)
+        catch(std::exception& e)
         {
             cerr << "ERROR: caught exception: " << e.what() << endl;
             cerr << "Could not read image information for file " << filelist[i] << endl;
@@ -329,7 +296,7 @@ int main(int argc, char* argv[])
         };
 
         srcImage.readEXIF();
-        srcImage.applyEXIFValues();
+        bool fovOk=srcImage.applyEXIFValues();
         if(projection>=0)
         {
             srcImage.setProjection((HuginBase::BaseSrcPanoImage::Projection)projection);
@@ -349,7 +316,7 @@ int main(int argc, char* argv[])
         else
         {
             //set plausible default value if they could not read from exif
-            if(srcImage.getExifFocalLength()==0 || srcImage.getCropFactor()==0)
+            if(!fovOk)
             {
                 cout << "\tNo value for field of view found in EXIF data. " << endl
                      << "\tAssuming a HFOV of 50 degrees. " << endl;
@@ -374,22 +341,22 @@ int main(int argc, char* argv[])
         {
             if(srcImage.readDistortionFromDB())
             {
-                cout << "\tRead distortion data from lensfun database." << endl;
+                cout << "\tRead distortion data from lens database." << endl;
             }
             else
             {
-                cout << "\tNo valid distortion data found in lensfun database." << endl;
+                cout << "\tNo valid distortion data found in lens database." << endl;
             };
         };
         if(loadVignetting)
         {
             if(srcImage.readVignettingFromDB())
             {
-                cout << "\tRead vignetting data from lensfun database." << endl;
+                cout << "\tRead vignetting data from lens database." << endl;
             }
             else
             {
-                cout << "\tNo valid vignetting data found in lensfun database." << endl;
+                cout << "\tNo valid vignetting data found in lens database." << endl;
             };
         };
 
@@ -419,18 +386,18 @@ int main(int argc, char* argv[])
         StandardImageVariableGroups variable_groups(pano);
         ImageVariableGroup& lenses = variable_groups.getLenses();
 
-        for(size_t i=1;i<pano.getNrOfImages();i++)
+        for(size_t i=1; i<pano.getNrOfImages(); i++)
         {
             int image=-1;
-            const SrcPanoImage & srcImg=pano.getImage(i);
-            for(size_t j=0;j<i;j++)
+            const SrcPanoImage& srcImg=pano.getImage(i);
+            for(size_t j=0; j<i; j++)
             {
-                const SrcPanoImage & compareImg=pano.getImage(j);
+                const SrcPanoImage& compareImg=pano.getImage(j);
                 if(srcImg.getHFOV()==compareImg.getHFOV() &&
-                    srcImg.getProjection()==compareImg.getProjection() &&
-                    srcImg.getExifModel()==compareImg.getExifModel() &&
-                    srcImg.getExifMake()==compareImg.getExifMake() &&
-                    srcImg.getSize()==compareImg.getSize())
+                        srcImg.getProjection()==compareImg.getProjection() &&
+                        srcImg.getExifModel()==compareImg.getExifModel() &&
+                        srcImg.getExifMake()==compareImg.getExifMake() &&
+                        srcImg.getSize()==compareImg.getSize())
                 {
                     image=j;
                     break;
@@ -451,44 +418,61 @@ int main(int argc, char* argv[])
             };
         };
         cout << endl << "Assigned " << lenses.getNumberOfParts() << " lenses." << endl;
-        if(lenses.getNumberOfParts()>1 && stackLength>1)
+        if(lenses.getNumberOfParts()>1 && stackLength!=1)
         {
             cout << "Project contains more than one lens, but you requested to assign" << endl
                  << "stacks. This is not supported. Therefore stacks will not be" << endl
                  << "assigned." << endl << endl;
             stackLength=1;
         };
-    };
 
-    //link stacks
-    if(pano.getNrOfImages()>1 && stackLength>1)
-    {
-        stackLength=std::min<int>(stackLength,pano.getNrOfImages());
-        int stackCount=pano.getNrOfImages() / stackLength;
-        if(pano.getNrOfImages() % stackLength > 0)
+        if (stackLength == 0)
         {
-            stackCount++;
-        };
-        if(stackCount<pano.getNrOfImages())
-        {
-            for(size_t stackNr=0;stackNr<stackCount;stackNr++)
+            // automatic detection
+            if (pano.hasPossibleStacks())
             {
-                size_t firstImgStack=stackNr*stackLength;
-                for(size_t i=0;i<stackLength;i++)
+                pano.linkPossibleStacks(linkStacks);
+            };
+        }
+        else
+        {
+            if (stackLength > 1)
+            {
+                stackLength = std::min<int>(stackLength, pano.getNrOfImages());
+                int stackCount = pano.getNrOfImages() / stackLength;
+                if (pano.getNrOfImages() % stackLength > 0)
                 {
-                    if(firstImgStack+i<pano.getNrOfImages())
+                    stackCount++;
+                };
+                if (stackCount < pano.getNrOfImages())
+                {
+                    for (size_t stackNr = 0; stackNr < stackCount; stackNr++)
                     {
-                        pano.linkImageVariableStack(firstImgStack,firstImgStack+i);
-                        if(linkStacks)
+                        size_t firstImgStack = stackNr*stackLength;
+                        for (size_t i = 0; i < stackLength; i++)
                         {
-                            pano.linkImageVariableYaw(firstImgStack,firstImgStack+i);
-                            pano.linkImageVariablePitch(firstImgStack,firstImgStack+i);
-                            pano.linkImageVariableRoll(firstImgStack,firstImgStack+i);
+                            if (firstImgStack + i < pano.getNrOfImages())
+                            {
+                                pano.linkImageVariableStack(firstImgStack, firstImgStack + i);
+                                if (linkStacks)
+                                {
+                                    pano.linkImageVariableYaw(firstImgStack, firstImgStack + i);
+                                    pano.linkImageVariablePitch(firstImgStack, firstImgStack + i);
+                                    pano.linkImageVariableRoll(firstImgStack, firstImgStack + i);
+                                };
+                            };
                         };
                     };
                 };
             };
-            cout << "Assigned " << stackCount << " stacks." << endl;
+        };
+
+        variable_groups.update();
+        const size_t stackCount = variable_groups.getStacks().getNumberOfParts();
+        if (stackCount != pano.getNrOfImages())
+        {
+            std::cout << "Assigned " << stackCount << " stacks: " << std::endl
+                << "\t" << (linkStacks ? "Linking position of images in stacks" : "Use individual positions of images in stacks") << std::endl;
         };
     };
 
