@@ -19,13 +19,11 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
-#include <hugin_version.h>
-
 #include <fstream>
 #include <sstream>
 #include <getopt.h>
@@ -36,9 +34,7 @@
 #include <panodata/ImageVariableTranslate.h>
 #include <panodata/ImageVariableGroup.h>
 #include <panodata/StandardImageVariableGroups.h>
-#include <boost/algorithm/string.hpp>
-#include <boost/regex.hpp>
-#include <boost/lexical_cast.hpp>
+#include "hugin_utils/utils.h"
 #include "ParseExp.h"
 
 using namespace std;
@@ -51,150 +47,91 @@ struct ParseVar
     int imgNr;
     std::string expression;
     bool removeOpt;
-    ParseVar(): varname(""), imgNr(-1), expression(""), removeOpt(false){};
+    ParseVar(): varname(""), imgNr(-1), expression(""), removeOpt(false) {};
 };
 
 typedef std::vector<ParseVar> ParseVarVec;
 
+bool ParseVarNumber(const std::string&s, ParseVar& var)
+{
+    std::size_t pos = s.find_first_of("0123456789");
+    std::string varName;
+    if (pos == std::string::npos)
+    {
+        varName = s;
+        var.imgNr = -1;
+    }
+    else
+    {
+        if (pos == 0)
+        {
+            return false;
+        };
+        varName = s.substr(0, pos);
+        if (!hugin_utils::stringToInt(s.substr(pos, s.length() - pos), var.imgNr))
+        {
+            return false;
+        };
+    }
+#define image_variable( name, type, default_value ) \
+    if (HuginBase::PTOVariableConverterFor##name::checkApplicability(varName))\
+    {\
+        var.varname=varName;\
+        return true;\
+    };
+#include "panodata/image_variables.h"
+#undef image_variable
+    return false;
+};
+
 // parse a single variable and put result in struct ParseVar
-void ParseSingleOptVar(ParseVarVec& varVec, std::string s)
+void ParseSingleOptVar(ParseVarVec& varVec, const std::string& s)
 {
-    boost::regex reg("([!]?)([a-zA-Z]{1,3})(\\d*?)");
-    boost::smatch matches;
-    if(boost::regex_match(s, matches, reg))
+    // parse following regex ([!]?)([a-zA-Z]{1,3})(\\d*?) 
+    std::string tempString(s);
+    ParseVar var;
+    var.removeOpt = (tempString[0] == '!');
+    if (var.removeOpt)
     {
-        if(matches.size()==4)
-        {
-            ParseVar var;
-            std::string temp(matches[1].first, matches[1].second);
-            var.removeOpt=(temp=="!");
-            //check if variable name is valid
-            temp=std::string(matches[2].first, matches[2].second);
-            bool validVarname=false;
-#define image_variable( name, type, default_value ) \
-            if (HuginBase::PTOVariableConverterFor##name::checkApplicability(temp))\
-            {\
-                validVarname=true;\
-            };
-#include "panodata/image_variables.h"
-#undef image_variable
-            if(!validVarname)
-            {
-                return;
-            };
-            var.varname=temp;
-            // now parse (optional) image number
-            temp=std::string(matches[3].first, matches[3].second);
-            if(temp.length()>0)
-            {
-                try
-                {
-                    var.imgNr=boost::lexical_cast<int>(temp);
-                }
-                catch (boost::bad_lexical_cast)
-                {
-                    var.imgNr=-1;
-                };
-            };
-            varVec.push_back(var);
-        };
+        tempString.erase(0, 1);
+    };
+    if (ParseVarNumber(tempString, var))
+    {
+        varVec.push_back(var);
     };
 };
 
-void ParseSingleLinkVar(ParseVarVec& varVec, std::string s)
+void ParseSingleLinkVar(ParseVarVec& varVec, const std::string& s)
 {
-    boost::regex reg("([a-zA-Z]{1,3})(\\d+?)");
-    boost::smatch matches;
-    if(boost::regex_match(s, matches, reg))
+    // parse following regex ([a-zA-Z]{1,3})(\\d+?)
+    ParseVar var;
+    if (ParseVarNumber(s, var))
     {
-        if(matches.size()==3)
-        {
-            ParseVar var;
-            //check if variable name is valid
-            std::string temp(matches[1].first, matches[1].second);
-            bool validVarname=false;
-#define image_variable( name, type, default_value ) \
-            if (HuginBase::PTOVariableConverterFor##name::checkApplicability(temp))\
-            {\
-                validVarname=true;\
-            };
-#include "panodata/image_variables.h"
-#undef image_variable
-            if(!validVarname)
-            {
-                return;
-            };
-            var.varname=temp;
-            // now parse image number
-            temp=std::string(matches[2].first, matches[2].second);
-            try
-            {
-                var.imgNr=boost::lexical_cast<int>(temp);
-            }
-            catch (boost::bad_lexical_cast)
-            {
-                var.imgNr=-1;
-                return;
-            };
-            varVec.push_back(var);
-        };
+        varVec.push_back(var);
     };
 };
 
-void ParseSingleVar(ParseVarVec& varVec, std::string s)
+void ParseSingleVar(ParseVarVec& varVec, const std::string& s)
 {
-    boost::regex reg("([a-zA-Z]{1,3})(\\d*?)=(.*)");
-    boost::smatch matches;
-    if(boost::regex_match(s, matches, reg))
+    // parse following regex ([a-zA-Z]{1,3})(\\d*?)=(.*)
+    const std::size_t pos = s.find_first_of("=", 0);
+    if (pos != std::string::npos && pos > 0 && pos < s.length() - 1)
     {
-        if(matches.size()==4)
+        ParseVar var;
+        const std::string tempString(s.substr(0, pos));
+        if (ParseVarNumber(tempString, var))
         {
-            ParseVar var;
-            std::string temp(matches[1].first, matches[1].second);
-            //check if variable name is valid
-            bool validVarname=false;
-#define image_variable( name, type, default_value ) \
-            if (HuginBase::PTOVariableConverterFor##name::checkApplicability(temp))\
-            {\
-                validVarname=true;\
-            };
-#include "panodata/image_variables.h"
-#undef image_variable
-            if(!validVarname)
-            {
-                return;
-            };
-            var.varname=temp;
-            // now parse (optional) image number
-            temp=std::string(matches[2].first, matches[2].second);
-            if(temp.length()>0)
-            {
-                try
-                {
-                    var.imgNr=boost::lexical_cast<int>(temp);
-                }
-                catch (boost::bad_lexical_cast)
-                {
-                    var.imgNr=-1;
-                };
-            };
-            // read expression
-            var.expression=std::string(matches[3].first, matches[3].second);
-            if(var.expression.length()==0)
-            {
-                return;
-            };
+            var.expression = s.substr(pos + 1, s.length() - pos - 1);
             varVec.push_back(var);
         };
     };
 };
 
 //parse complete variables string
-void ParseVariableString(ParseVarVec& parseVec, std::string input, void (*func)(ParseVarVec&, std::string))
+void ParseVariableString(ParseVarVec& parseVec, const std::string& input, void (*func)(ParseVarVec&, const std::string&))
 {
-    std::vector<std::string> splitResult;
-    boost::algorithm::split(splitResult, input, boost::algorithm::is_any_of(", "), boost::algorithm::token_compress_on);
-    for(size_t i=0; i<splitResult.size();i++)
+    std::vector<std::string> splitResult = hugin_utils::SplitString(input, ", ");
+    for(size_t i=0; i<splitResult.size(); i++)
     {
         (*func)(parseVec, splitResult[i]);
     };
@@ -204,8 +141,8 @@ void ParseVariableString(ParseVarVec& parseVec, std::string input, void (*func)(
 // does some additional checking:
 //   1. don't add y,p,r for anchor image
 //   2. handle vignetting and EMoR parameters as group
-void AddToOptVec(HuginBase::OptimizeVector& optVec, std::string varname, size_t imgNr, 
-    std::set<size_t> refImgs, bool linkRefImgsYaw, bool linkRefImgsPitch, bool linkRefImgsRoll, std::vector<std::set<std::string> > groupedVars)
+void AddToOptVec(HuginBase::OptimizeVector& optVec, std::string varname, size_t imgNr,
+                 std::set<size_t> refImgs, bool linkRefImgsYaw, bool linkRefImgsPitch, bool linkRefImgsRoll, std::vector<std::set<std::string> > groupedVars)
 {
     if(varname=="y")
     {
@@ -288,17 +225,17 @@ void UnLinkVars(Panorama& pano, ParseVarVec parseVec, bool link)
         {
             continue;
         };
-        
+
         //convert to ImageVariableGroup::IVE_name format
         std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> variables;
 #define image_variable( name, type, default_value ) \
-        if (HuginBase::PTOVariableConverterFor##name::checkApplicability(parseVec[i].varname))\
-        {\
-            variables.insert(HuginBase::ImageVariableGroup::IVE_##name);\
-        };
+    if (HuginBase::PTOVariableConverterFor##name::checkApplicability(parseVec[i].varname))\
+    {\
+        variables.insert(HuginBase::ImageVariableGroup::IVE_##name);\
+    };
 #include "panodata/image_variables.h"
 #undef image_variable
-        
+
         if(!variables.empty())
         {
             //lens variable
@@ -355,7 +292,7 @@ void UnLinkVars(Panorama& pano, ParseVarVec parseVec, bool link)
     };
 };
 
-bool UpdateSingleVar(Panorama &pano, ParseVar parseVar, size_t imgNr)
+bool UpdateSingleVar(Panorama& pano, ParseVar parseVar, size_t imgNr)
 {
     double val=pano.getImage(imgNr).getVar(parseVar.varname);
     Parser::ConstantMap constMap;
@@ -380,7 +317,7 @@ bool UpdateSingleVar(Panorama &pano, ParseVar parseVar, size_t imgNr)
 static void usage(const char* name)
 {
     cout << name << ": change image variables inside pto files" << endl
-         << name << " version " << DISPLAY_VERSION << endl
+         << name << " version " << hugin_utils::GetHuginVersion() << endl
          << endl
          << "Usage:  " << name << " [options] --opt|--link|--unlink|--set varlist input.pto" << endl
          << endl
@@ -394,7 +331,7 @@ static void usage(const char* name)
          << "                            Examples:" << endl
          << "           --opt y,p,r        Optimize yaw, pitch and roll of all images" << endl
          << "                              (special treatment for anchor image applies)" << endl
-         << "           --opt v0,b2        Optimize hfov of image 0 and barrel distortion" << endl    
+         << "           --opt v0,b2        Optimize hfov of image 0 and barrel distortion" << endl
          << "                              of image 2" << endl
          << "           --opt v,!v0        Optimize field of view for all images except" << endl
          << "                              for the first image" << endl
@@ -467,7 +404,7 @@ int main(int argc, char* argv[])
                 output = optarg;
                 break;
             case 'h':
-                usage(argv[0]);
+                usage(hugin_utils::stripPath(argv[0]).c_str());
                 return 0;
             case SWITCH_OPT:
                 ParseVariableString(optVars, std::string(optarg), ParseSingleOptVar);
@@ -490,7 +427,7 @@ int main(int argc, char* argv[])
                         contents << ifs.rdbuf();
                         ifs.close();
                         string s(contents.str());
-                        boost::algorithm::replace_all(s, "\n", ",");
+                        hugin_utils::ReplaceAll(s, "\n", ',');
                         ParseVariableString(setVars, s, ParseSingleVar);
                     }
                     else
@@ -599,19 +536,19 @@ int main(int argc, char* argv[])
                     // now remember linked variables
                     const HuginBase::SrcPanoImage& img1=pano.getImage(j);
 #define image_variable( name, type, default_value ) \
-                    if (HuginBase::PTOVariableConverterFor##name::checkApplicability(setVars[i].varname))\
-                    {\
-                        if(img1.name##isLinked())\
-                        {\
-                            for(size_t k=j+1; k<pano.getNrOfImages(); k++)\
-                            {\
-                                if(img1.name##isLinkedWith(pano.getImage(k)))\
-                                {\
-                                    updatedImgs.insert(k);\
-                                }\
-                            };\
-                        };\
-                    };
+    if (HuginBase::PTOVariableConverterFor##name::checkApplicability(setVars[i].varname))\
+    {\
+        if(img1.name##isLinked())\
+        {\
+            for(size_t k=j+1; k<pano.getNrOfImages(); k++)\
+            {\
+                if(img1.name##isLinkedWith(pano.getImage(k)))\
+                {\
+                    updatedImgs.insert(k);\
+                }\
+            };\
+        };\
+    };
 #include "panodata/image_variables.h"
 #undef image_variable
                 };
